@@ -1,6 +1,6 @@
-/* $Id: e2p_config.c 706 2007-11-06 09:13:06Z tpgww $
+/* $Id: e2p_config.c 1947 2009-10-01 08:49:48Z tpgww $
 
-Copyright (C) 2006-2007 tooar <tooar@gmx.net>
+Copyright (C) 2006-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -29,16 +29,20 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "e2_dialog.h"
 #include "e2_task.h"
 #include "e2_filetype.h"
+#ifdef E2_MOUSECUSTOM
+# include "e2_mousebinding.h"
+#endif
 
 #define ANAME "config"
 #ifndef UPGRADE_PNAME
 #define UPGRADE_PNAME "e2p_upgrade.so"
 #endif
-//NOTE this needs to be the minimum actionable version in the
-//upgrade plugin or else that will just install defaults
+//NOTE this needs to be the minimum actionable version in the upgrade plugin
+//or else that will just install defaults
 #ifndef OLDEST_UPGRADE
-#define OLDEST_UPGRADE "0.2.0"
+#define OLDEST_UPGRADE "0.4.1.3"
 #endif
+#define OLDEST_IMPORT "0.4.0"
 
 typedef struct _E2P_ConfigData
 {
@@ -55,7 +59,10 @@ typedef enum
 	ALL_P = 0,
 	NONTREE_P, ALLTREE_P, CUSTOM_P,
 	MARKS_P, FILETYPES_P,
-	BINDINGS_P, ALIASES_P,
+	ALIASES_P, KEYS_P,
+#ifdef E2_MOUSECUSTOM
+	BUTTONS_P, GESTURES_P,
+#endif
 	PLUGINS_P, MENU_P, CUSTMENU_P,
 	PANEBAR1_P, PANEBAR2_P, TASKBAR_P, CMDBAR_P,
 	MAX_FLAGS	//no. of entries in the array
@@ -69,8 +76,12 @@ typedef enum
 	E2PC_CUSTOM    = 1 << CUSTOM_P,
 	E2PC_MARKS     = 1 << MARKS_P,
 	E2PC_FILETYPES = 1 << FILETYPES_P,
-	E2PC_BINDINGS  = 1 << BINDINGS_P,
 	E2PC_ALIASES   = 1 << ALIASES_P,
+	E2PC_KEYS      = 1 << KEYS_P,
+#ifdef E2_MOUSECUSTOM
+	E2PC_BUTTONS   = 1 << BUTTONS_P,
+	E2PC_GESTURES  = 1 << GESTURES_P,
+#endif
 	E2PC_PLUGINS   = 1 << PLUGINS_P,
 	E2PC_MENU      = 1 << MENU_P,
 	E2PC_CUSTMENU  = 1 << CUSTMENU_P,
@@ -80,9 +91,16 @@ typedef enum
 	E2PC_CMDBAR    = 1 << CMDBAR_P,
 } bitflag_t;
 
-#define E2PC_ALLTREEMASK E2PC_FILETYPES | E2PC_BINDINGS | E2PC_ALIASES \
+#ifdef E2_MOUSECUSTOM
+#define E2PC_ALLTREEMASK E2PC_FILETYPES | E2PC_ALIASES | E2PC_KEYS \
+	| E2PC_BUTTONS | E2PC_GESTURES \
 	| E2PC_PLUGINS | E2PC_MENU | E2PC_CUSTMENU | E2PC_PANEBAR1 \
 	| E2PC_PANEBAR2 | E2PC_TASKBAR | E2PC_CMDBAR
+#else
+#define E2PC_ALLTREEMASK E2PC_FILETYPES | E2PC_ALIASES | E2PC_KEYS \
+	| E2PC_PLUGINS | E2PC_MENU | E2PC_CUSTMENU | E2PC_PANEBAR1 \
+	| E2PC_PANEBAR2 | E2PC_TASKBAR | E2PC_CMDBAR
+#endif
 //for determining index of some specific tree
 #define FIRSTTREE_P MARKS_P
 
@@ -91,8 +109,12 @@ static gchar *set_private_names [] =
 {
 	"bookmarks",
 	"filetypes",
-	"keybindings",
 	"command-aliases",
+	"keybindings",
+#ifdef E2_MOUSECUSTOM
+	"mousebuttons",
+	"mousedrags",
+#endif
 	"plugins",
 	"context-menu",
 	"custom-menus",
@@ -199,20 +221,20 @@ static void _e2pc_toggle_cb (GtkWidget *widget, gpointer flagnum)
 @param f enumerated value of flag to be associated with the button
 @param state T/F initial state of the toggle
 @param label  translated string for the button label
-@param callback callback function to be connected
+@param callback "toggled" signal callback function to be connected
 
 @return the button widget
 */
 /*static GtkWidget *_e2pc_create_option_button (GtkWidget *container,
-	flag_t f, gboolean state, gchar *label, gpointer callback_fn)
+	flag_t f, gboolean state, gchar *label,
+	void (*callback)(GtkToggleButton*,gpointer))
 {
 	GtkWidget *btn = gtk_check_button_new_with_mnemonic (label);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (btn), state);
-	g_signal_connect (G_OBJECT (btn), "toggled", G_CALLBACK (callback_fn),
+	g_signal_connect (G_OBJECT (btn), "toggled", G_CALLBACK (callback),
 		(gpointer) f);  //CHECKME = correct value?
 	gtk_container_add (GTK_CONTAINER (container), btn);
-	gtk_widget_show (btn);
-	return (btn);
+	return btn;
 } */
 /**
 @brief create and show a check button in a specified container
@@ -227,15 +249,12 @@ static void _e2pc_toggle_cb (GtkWidget *widget, gpointer flagnum)
 static GtkWidget *_e2pc_create_check_button (GtkWidget *container,
 	flag_t f, gboolean state, gchar *label)
 {
-//	GtkWidget *btn = _e2pc_create_option_button
-//		(container, f, state, label, _e2pc_toggle_cb);
 	GtkWidget *btn = gtk_check_button_new_with_mnemonic (label);
 	g_signal_connect (G_OBJECT (btn), "toggled", G_CALLBACK (_e2pc_toggle_cb),
 		(gpointer) f);  //CHECKME = correct value?
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (btn), state);
 	gtk_container_add (GTK_CONTAINER (container), btn);
-	gtk_widget_show (btn);
-	return (btn);
+	return btn;
 }
 /**
 @brief create and show a radio btn in a specified container
@@ -258,8 +277,7 @@ static GtkWidget *_e2pc_create_radio_button (GtkWidget *container,
 	g_signal_connect (G_OBJECT (btn), "toggled", G_CALLBACK (_e2pc_toggle_cb),
 		(gpointer) f);
 	gtk_container_add (GTK_CONTAINER (container), btn);
-	gtk_widget_show (btn);
-	return (btn);
+	return btn;
 }
 /**
 @brief create and show a radio btn in a specified container
@@ -282,8 +300,7 @@ static GtkWidget *_e2pc_create_radio_grouped_button (GtkWidget *container, GtkWi
 		(gpointer) f);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (btn), state);
   gtk_container_add (GTK_CONTAINER (container), btn);
-  gtk_widget_show (btn);
-  return (btn);
+  return btn;
 }
 /**
 @brief check whether a tree option is to be imported
@@ -299,7 +316,7 @@ static gboolean _e2pc_match_tree (gchar *setname)
 			i < treeset_names->len;
 			i++, iterator++)
 	{
-		if (g_str_equal (*iterator, setname))
+		if (!strcmp (*iterator, setname))
 		{
 			g_ptr_array_remove_index_fast (treeset_names, i);
 			return TRUE;
@@ -333,7 +350,7 @@ static void _e2pc_filter_options (gchar *contents, bitflag_t flags)
 		lsplit = g_strsplit (line, "=", 2);
 		if (lsplit[1] != NULL)
 		{
-			if (!g_str_equal (lsplit[1], "<")) //not a tree set
+			if (strcmp (lsplit[1], "<")) //not a tree set
 			{
 				if (flags & (E2PC_NONTREE | E2PC_ALL))
 				{
@@ -397,7 +414,7 @@ static void _e2pc_filter_options (gchar *contents, bitflag_t flags)
 
 @return
 */
-static void _e2pc_import_cb (GtkWidget *button, E2P_ConfigData *rt)
+static void _e2pc_import_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	//convert import flags to bitflags that we can play with
 	bitflag_t import_flags = 0;
@@ -422,10 +439,16 @@ static void _e2pc_import_cb (GtkWidget *button, E2P_ConfigData *rt)
 		g_ptr_array_add (treeset_names, (gpointer) set_private_names[MARKS_P - FIRSTTREE_P]);
 	if (import_flags & E2PC_FILETYPES)
 		g_ptr_array_add (treeset_names, (gpointer) set_private_names[FILETYPES_P - FIRSTTREE_P]);
-	if (import_flags & E2PC_BINDINGS)
-		g_ptr_array_add (treeset_names, (gpointer) set_private_names[BINDINGS_P - FIRSTTREE_P]);
 	if (import_flags & E2PC_ALIASES)
 		g_ptr_array_add (treeset_names, (gpointer) set_private_names[ALIASES_P - FIRSTTREE_P]);
+	if (import_flags & E2PC_KEYS)
+		g_ptr_array_add (treeset_names, (gpointer) set_private_names[KEYS_P - FIRSTTREE_P]);
+#ifdef E2_MOUSECUSTOM
+	if (import_flags & E2PC_BUTTONS)
+		g_ptr_array_add (treeset_names, (gpointer) set_private_names[BUTTONS_P - FIRSTTREE_P]);
+	if (import_flags & E2PC_GESTURES)
+		g_ptr_array_add (treeset_names, (gpointer) set_private_names[GESTURES_P - FIRSTTREE_P]);
+#endif
 	if (import_flags & E2PC_PLUGINS)
 		g_ptr_array_add (treeset_names, (gpointer) set_private_names[PLUGINS_P - FIRSTTREE_P]);
 	if (import_flags & E2PC_MENU)
@@ -451,7 +474,7 @@ static void _e2pc_import_cb (GtkWidget *button, E2P_ConfigData *rt)
 #endif
 	gpointer contents;
 	//get file
-read_file:
+//read_file:
 #ifdef E2_VFS
 	if (e2_fs_get_file_contents (&ddata, &contents, NULL, TRUE E2_ERR_NONE()))
 #else
@@ -476,28 +499,33 @@ read_file:
 					upgrade = (strcmp (sq, VERSION RELEASE) < 0);
 					if (upgrade)
 					{
-						gchar *command;
-						gchar *sed = g_find_program_in_path ("sed");
-						if (sed != NULL)
+						if (strcmp (sq, OLDEST_IMPORT) >= 0)
 						{
-							if (strcmp (sq,"0.1.6.3") < 0 && (import_flags & (E2PC_ALLTREEMASK)))
+							//do any interim content-transforms here
+/*							gchar *command;
+							gchar *sed = g_find_program_in_path ("sed");
+							if (sed != NULL)
 							{
-								//re-configure all tree-option name lines
-								//this format change is no longer supported in the upgrade plugin
-								command = g_strconcat (
-								"cp -f ", realpath, " ", realpath, ".save;",
-								sed, " -e 's/^<\\(.*\\)/\\1=</'",
-								 " -e 's/^\\(\t*\\)[\\]</\\1</' ",
-								realpath, ".save >", realpath, NULL);
-								system (command);
-								g_free (command);
-								g_free (sed);
-								g_free (sq);
-								if (realpath != filepath)
-									g_free (realpath);
-								g_free (contents);	//need free() if file buffer allocated by malloc()
-								goto read_file;
+								if (strcmp (sq,"0.1.6.3") < 0 && (import_flags & (E2PC_ALLTREEMASK)))
+								{
+									//re-configure all tree-option name lines
+									//this format change is no longer supported in the upgrade plugin
+									command = g_strconcat (
+									"cp -f ", realpath, " ", realpath, ".save;",
+									sed, " -e 's/^<\\(.*\\)/\\1=</'",
+									 " -e 's/^\\(\t*\\)[\\]</\\1</' ",
+									realpath, ".save >", realpath, NULL);
+									system (command);
+									g_free (command);
+									g_free (sed);
+									g_free (sq);
+									if (realpath != filepath)
+										g_free (realpath);
+									g_free (contents);	//need free() if file buffer allocated by malloc()
+									goto read_file;
+								}
 							}
+*/
 						}
 						else
 						{
@@ -530,9 +558,9 @@ read_file:
 				gchar *path1 = g_build_filename (e2_cl_options.config_dir,
 					default_config_file, NULL);
 				gchar *local1 = F_FILENAME_TO_LOCALE (path1);
-				gchar *path2 = g_strconcat (default_config_file, "-before-import", NULL);
-				gchar *path3 = g_build_filename (e2_cl_options.config_dir, path2, NULL);
-				gchar *local2 = F_FILENAME_TO_LOCALE (path3);
+				gchar *name2 = g_strconcat (default_config_file, "-before-import", NULL);
+				gchar *path2 = g_build_filename (e2_cl_options.config_dir, name2, NULL);
+				gchar *local2 = F_FILENAME_TO_LOCALE (path2);
 				gchar *savepath = e2_utils_get_tempname (local2);
 				gdk_threads_leave ();	//downstream errors invoke local mutex locking
 #ifdef E2_VFS
@@ -544,11 +572,11 @@ read_file:
 #endif
 				gdk_threads_enter ();
 				g_free (path1);
+				F_FREE (local1, path1);
+				g_free (name2);
 				g_free (path2);
-				g_free (path3);
+				F_FREE (local2, path2);
 				g_free (savepath);
-				F_FREE (local1);
-				F_FREE (local2);
 				//save updated config file
 				e2_option_file_write (NULL);
 				//do any upgrades needed
@@ -584,8 +612,14 @@ read_file:
 			//parts of that
 			app.rebuild_enabled = FALSE;	//block recursion
 
-			if (import_flags & E2PC_BINDINGS)
+			if (import_flags & E2PC_KEYS)
 				e2_keybinding_clean ();
+#ifdef E2_MOUSECUSTOM
+			if (import_flags & E2PC_BUTTONS)
+				e2_mousebinding_clean ();
+			if (import_flags & E2PC_GESTURES)
+				e2_mousegesture_clean ();
+#endif
 			if (import_flags & E2PC_FILETYPES)
 				g_hash_table_destroy (app.filetypes);
 
@@ -600,7 +634,7 @@ read_file:
 						if (member->data != NULL)
 						{
 
-							if (! g_str_equal(((Plugin *)member->data)->signature, ANAME VERSION))
+							if (strcmp(((Plugin *)member->data)->signature, ANAME VERSION))
 							//FIXME handle plugins that are not allowed to be unloaded
 							{
 								e2_plugins_unload1 ((Plugin *)member->data, TRUE);
@@ -622,7 +656,7 @@ read_file:
 //				e2_option_file_read ();
 //			}
 
-		//	e2_option_tree_install_defaults ();
+//			e2_option_tree_install_defaults ();
 
 			if (import_flags & E2PC_PLUGINS)
 				e2_plugins_load_all (); // load plugins (if any) CHECKME what if this plugin is reloaded ?
@@ -631,16 +665,23 @@ read_file:
 			e2_pane_create_option_data (&app.pane1);
 			e2_pane_create_option_data (&app.pane2);
 
-			e2_toolbar_initialise (E2_BAR_PANE1);
-			e2_toolbar_initialise (E2_BAR_PANE2);
-			e2_toolbar_initialise (E2_BAR_TASK);
-			e2_toolbar_initialise (E2_BAR_COMMAND);
-
-		//	if (recreate)
+//			if (recreate)
 				e2_window_recreate (&app.window); //this also recreates key bindings
-		//	else
-		//		e2_keybinding_register_all ();
+/*			else
+			{
+				e2_toolbar_create (&app.pane1.toolbar);
+				e2_toolbar_rebadge (&app.pane1.toolbar);
+				e2_toolbar_create (&app.pane2.toolbar);
+				e2_toolbar_rebadge (&app.pane2.toolbar);
+				e2_toolbar_create (&app.toolbar);
+				e2_toolbar_create (&app.commandbar);
 
+				e2_keybinding_register_all ();
+#ifdef E2_MOUSECUSTOM
+				e2_mousebinding_register_all ();
+#endif
+			}
+*/
 			if (import_flags & E2PC_FILETYPES)
 				e2_filetype_add_all ();
 
@@ -654,7 +695,7 @@ read_file:
 	}
 	if (*filepath == '\0')
 		g_free (realpath);
-	F_FREE (localpath);
+	F_FREE (localpath, realpath);
 	g_ptr_array_free (treeset_names, TRUE);
 }
 /**
@@ -665,12 +706,12 @@ read_file:
 
 @return
 */
-static void _e2pc_select_config_cb (GtkWidget *button, E2P_ConfigData *rt)
+static void _e2pc_select_config_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
 		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_OPEN,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_OK, GTK_RESPONSE_OK,
+		GTK_STOCK_OPEN, GTK_RESPONSE_OK,
 		NULL);
 
 	e2_dialog_setup_chooser (dialog,
@@ -682,7 +723,7 @@ static void _e2pc_select_config_cb (GtkWidget *button, E2P_ConfigData *rt)
 		GTK_RESPONSE_OK);	//default response
 
 	gint response;
-	while ((response = gtk_dialog_run (GTK_DIALOG (dialog))) == E2_RESPONSE_USER1)
+	while ((response = e2_dialog_run_simple (dialog, app.main_window)) == E2_RESPONSE_USER1)
 	{}
 
 	if (response == GTK_RESPONSE_OK)
@@ -691,7 +732,7 @@ static void _e2pc_select_config_cb (GtkWidget *button, E2P_ConfigData *rt)
 		gchar *openpath = F_FILENAME_FROM_LOCALE (local);
 		gtk_entry_set_text (GTK_ENTRY (rt->open_entry), openpath);
 		g_free (local);
-		F_FREE (openpath);
+		F_FREE (openpath, local);
 	}
 	gtk_widget_destroy (dialog);
 }
@@ -703,7 +744,7 @@ static void _e2pc_select_config_cb (GtkWidget *button, E2P_ConfigData *rt)
 
 @return
 */
-static void _e2pc_save_cb (GtkWidget *button, E2P_ConfigData *rt)
+static void _e2pc_save_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	const gchar *savepath = gtk_entry_get_text (GTK_ENTRY (rt->save_entry));
 	if (*savepath != '\0')
@@ -719,7 +760,15 @@ static void _e2pc_save_cb (GtkWidget *button, E2P_ConfigData *rt)
 			if (e2_fs_access2 (dlocal E2_ERR_NONE()) == 0)
 #endif
 			{
-				DialogButtons choice = e2_dialog_ow_check (NULL, dlocal, NONE);
+				gdk_threads_leave ();
+				DialogButtons choice = e2_dialog_ow_check (NULL,
+#ifdef E2_VFS
+					&ddata,
+#else
+					dlocal,
+#endif
+					NONE);
+				gdk_threads_enter ();
 				if (choice != OK)
 				{
 					g_free (dlocal);
@@ -739,12 +788,12 @@ static void _e2pc_save_cb (GtkWidget *button, E2P_ConfigData *rt)
 
 @return
 */
-static void _e2pc_saveas_cb (GtkWidget *button, E2P_ConfigData *rt)
+static void _e2pc_saveas_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
 		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_SAVE,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_OK, GTK_RESPONSE_OK,
+		GTK_STOCK_SAVE, GTK_RESPONSE_OK,
 		NULL);
 
 	e2_dialog_setup_chooser (dialog,
@@ -756,7 +805,7 @@ static void _e2pc_saveas_cb (GtkWidget *button, E2P_ConfigData *rt)
 		GTK_RESPONSE_OK);	//default response
 
 	gint response;
-	while ((response = gtk_dialog_run (GTK_DIALOG (dialog))) == E2_RESPONSE_USER1)
+	while ((response = e2_dialog_run_simple (dialog, app.main_window)) == E2_RESPONSE_USER1)
 	{}
 
 	if (response == GTK_RESPONSE_OK)
@@ -774,7 +823,15 @@ static void _e2pc_saveas_cb (GtkWidget *button, E2P_ConfigData *rt)
 		&& e2_fs_access2 (local E2_ERR_NONE()) == 0)
 #endif
 		{
-			DialogButtons choice = e2_dialog_ow_check (NULL, local, NONE);
+			gdk_threads_leave ();
+			DialogButtons choice = e2_dialog_ow_check (NULL,
+#ifdef E2_VFS
+				&ddata,
+#else
+				local,
+#endif
+				NONE);
+			gdk_threads_enter ();
 			if (choice != OK)
 			{
 				gtk_widget_destroy (dialog);
@@ -786,19 +843,19 @@ static void _e2pc_saveas_cb (GtkWidget *button, E2P_ConfigData *rt)
 		gchar *savepath = F_FILENAME_FROM_LOCALE (local);
 		gtk_entry_set_text (GTK_ENTRY (rt->save_entry), savepath);
 		g_free (local);
-		F_FREE (savepath);
+		F_FREE (savepath, local);
 	}
 	gtk_widget_destroy (dialog);
 }
 /**
 @brief select directory containing custom icons to be used
 
-@param widget clicked button, UNUSED
+@param button clicked button, UNUSED
 @param rt dialog runtime struct
 
 @return
 */
-static void _e2pc_select_icondir_cb (GtkWidget *widget, E2P_ConfigData *rt)
+static void _e2pc_select_icondir_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
 		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
@@ -815,7 +872,7 @@ static void _e2pc_select_icondir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 		GTK_RESPONSE_OK);	//default response
 
 	gint response;
-	while ((response = gtk_dialog_run (GTK_DIALOG (dialog))) == E2_RESPONSE_USER1)
+	while ((response = e2_dialog_run_simple (dialog, app.main_window)) == E2_RESPONSE_USER1)
 	{}
 
 	if (response == GTK_RESPONSE_OK)
@@ -825,7 +882,7 @@ static void _e2pc_select_icondir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 //		{
 			gchar *opendir = F_FILENAME_FROM_LOCALE (local);
 			gtk_entry_set_text (GTK_ENTRY (rt->icondir_entry), opendir);
-			F_FREE (opendir);
+			F_FREE (opendir, local);
 //		}
 		g_free (local);
 	}
@@ -834,19 +891,19 @@ static void _e2pc_select_icondir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 /**
 @brief apply icon directory
 
-@param widget clicked button, UNUSED
+@param button clicked button, UNUSED
 @param rt dialog runtime struct
 
 @return
 */
-static void _e2pc_apply_icondir_cb (GtkWidget *widget, E2P_ConfigData *rt)
+static void _e2pc_apply_icondir_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	gchar *utfpath = g_strdup (gtk_entry_get_text (GTK_ENTRY (rt->icondir_entry)));
 	if (g_str_has_suffix (utfpath, G_DIR_SEPARATOR_S))
 		*(utfpath + strlen (utfpath) - sizeof(gchar)) = '\0';
 
 	gchar *localpath = F_FILENAME_TO_LOCALE (utfpath);
-	if (g_str_equal (localpath, ICON_DIR))
+	if (!strcmp (localpath, ICON_DIR))
 		e2_option_bool_set ("use-icon-dir", FALSE);
 	else
 	{
@@ -855,23 +912,23 @@ static void _e2pc_apply_icondir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 		e2_option_str_set_direct (set, utfpath);
 		e2_toolbar_recreate_all ();
 	}
-	F_FREE (localpath);
 	g_free (utfpath);
+	F_FREE (localpath, utfpath);
 }
 /**
 @brief select directory in which to save custom icons now in use
 
-@param widget clicked button, UNUSED
+@param button clicked button, UNUSED
 @param rt dialog runtime struct
 
 @return
 */
-static void _e2pc_select_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
+static void _e2pc_select_iconsavedir_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
 		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_OK, GTK_RESPONSE_OK,
+		GTK_STOCK_SAVE, GTK_RESPONSE_OK,
 		NULL);
 
 	e2_dialog_setup_chooser (dialog,
@@ -883,7 +940,7 @@ static void _e2pc_select_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 		GTK_RESPONSE_OK);	//default response
 
 	gint response;
-	while ((response = gtk_dialog_run (GTK_DIALOG (dialog))) == E2_RESPONSE_USER1)
+	while ((response = e2_dialog_run_simple (dialog, app.main_window)) == E2_RESPONSE_USER1)
 	{}
 
 	if (response == GTK_RESPONSE_OK)
@@ -901,7 +958,15 @@ static void _e2pc_select_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 			&& e2_fs_access2 (local E2_ERR_NONE()) == 0)
 #endif
 		{
-			DialogButtons choice = e2_dialog_ow_check (NULL, local, NONE);
+			gdk_threads_leave ();
+			DialogButtons choice = e2_dialog_ow_check (NULL,
+#ifdef E2_VFS
+				&ddata,
+#else
+				local,
+#endif
+				NONE);
+			gdk_threads_enter ();
 			if (choice != OK)
 			{
 				gtk_widget_destroy (dialog);
@@ -913,19 +978,19 @@ static void _e2pc_select_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 		gchar *openpath = F_FILENAME_FROM_LOCALE (local);
 		gtk_entry_set_text (GTK_ENTRY (rt->iconsavedir_entry), openpath);
 		g_free (local);
-		F_FREE (openpath);
+		F_FREE (openpath, local);
 	}
 	gtk_widget_destroy (dialog);
 }
 /**
 @brief copy icons to specified directory
 If the dir is not absolute, curr_view->dir is prepended
-@param widget clicked button, UNUSED
+@param button clicked button, UNUSED
 @param rt dialog runtime struct
 
 @return
 */
-static void _e2pc_apply_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
+static void _e2pc_apply_iconsavedir_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	const gchar *path;
 	gchar *slocal, *dest, *dlocal;
@@ -953,7 +1018,20 @@ static void _e2pc_apply_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 #else
 		&& e2_fs_access2 (dlocal E2_ERR_NONE()) == 0)
 #endif
-		result = e2_dialog_ow_check (slocal, dlocal, NONE);
+	{
+#ifdef E2_VFS
+		VPATH sdata = { slocal, NULL };
+#endif
+		gdk_threads_leave ();
+		result = e2_dialog_ow_check (
+#ifdef E2_VFS
+			&sdata, &ddata,
+#else
+			slocal, dlocal,
+#endif
+			NONE);
+		gdk_threads_enter ();
+	}
 	else
 		result = OK;
 
@@ -973,44 +1051,7 @@ static void _e2pc_apply_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 
 	g_free (slocal);
 	g_free (dest);
-	F_FREE (dlocal);
-}
-/**
-@brief cleanup after cancel button is pressed
-
-@param widget clicked button, UNUSED
-@param rt dialog runtime struct
-
-@return
-*/
-static void _e2pc_quit_cb (GtkWidget *widget, E2P_ConfigData *rt)
-{
-	printd (DEBUG, "Config plugin dialog quit cb");
-	gtk_widget_destroy (rt->dialog);
-	DEALLOCATE (E2P_ConfigData, rt);
-//	gtk_widget_grab_focus (curr_view->treeview);
-}
-
-/**
-@brief  create and show notbook page with config export widgets
-
-@param dialog the dialog where the response was triggered
-@param response the number assigned to the widget that triggered the callback
-@param rt dialog runtime struct
-
-@return
-*/
-static void	_e2pc_response_cb (GtkDialog *dialog, gint response, E2P_ConfigData *rt)
-{
-	printd (DEBUG, "Config plugin dialog response cb");
-	switch (response)
-	{
-		case GTK_RESPONSE_CLOSE:
-			_e2pc_quit_cb (NULL, rt);
-			break;
-		default:
-			break;
-	}
+	F_FREE (dlocal, dest);
 }
 
   /***************************/
@@ -1028,14 +1069,13 @@ static void	_e2pc_response_cb (GtkDialog *dialog, gint response, E2P_ConfigData
 static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 {
 	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox);
 #ifdef E2_ASSISTED
 	GtkWidget *label =
 #endif
 	e2_widget_add_label (vbox, _("Save configuration data in"),
 		0.5, 0.5, FALSE, E2_PADDING);
 
-	const gchar *savepath;
+	const gchar *savedir;
 	gchar *local = F_FILENAME_TO_LOCALE (e2_cl_options.config_dir);
 	//default export to config dir if it's usable
 #ifdef E2_VFS
@@ -1044,13 +1084,13 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 #else
 	if (e2_fs_is_dir3 (local E2_ERR_NONE()) && !e2_fs_access (local, R_OK | W_OK | X_OK E2_ERR_NONE()))
 #endif
-		savepath = e2_cl_options.config_dir;
+		savedir = e2_cl_options.config_dir;
 	else
-		savepath = g_get_home_dir ();
-	F_FREE (local);
+		savedir = g_get_home_dir ();
+	F_FREE (local, e2_cl_options.config_dir);
 
-	gchar *savename = g_build_filename (savepath, default_config_file, NULL);
-	local = F_FILENAME_TO_LOCALE (savename);
+	gchar *savepath = g_build_filename (savedir, default_config_file, NULL);
+	local = F_FILENAME_TO_LOCALE (savepath);
 	gchar *temppath, *tempext, *local2;
 	guint i = 0;
 	while (TRUE)
@@ -1059,10 +1099,10 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 		local2 = F_FILENAME_TO_LOCALE (tempext);
 		temppath = e2_utils_strcat (local, local2);
 		g_free (tempext);
-		F_FREE (local2);
+		F_FREE (local2, tempext);
 		E2_ERR_DECLARE
 #ifdef E2_VFS
-		ddata.localpath = temppath;
+		ddata.path = temppath;
 		if (e2_fs_access2 (&ddata E2_ERR_PTR())	//checkme need for vfs ?
 #else
 		if (e2_fs_access2 (temppath E2_ERR_PTR())	//checkme need for vfs ?
@@ -1076,21 +1116,22 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 		g_free (temppath);
 		i++;
 	}
-	F_FREE (local);
-	g_free (savename);
+	g_free (savepath);
+	F_FREE (local, savepath);
+
 	savepath = F_FILENAME_FROM_LOCALE (temppath);
-	if (savepath != temppath)
-		g_free (temppath);
 
 	rt->save_entry = e2_widget_add_entry (vbox, (gchar *) savepath, TRUE, TRUE);
 #ifdef E2_ASSISTED
-	e2_widget_set_label_relations (GTK_LABEL (label), rt->save_entry);
+	e2_widget_set_label_relations (label, rt->save_entry);
 #endif
 	gtk_widget_set_size_request (rt->save_entry, 400, -1);
 
+	g_free (temppath);
+	F_FREE (savepath, temppath);
+
 	GtkWidget *hbox = gtk_hbutton_box_new ();
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, E2_PADDING);
-	gtk_widget_show (hbox);
 
 	gtk_box_set_spacing (GTK_BOX (hbox), E2_PADDING_LARGE);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
@@ -1105,7 +1146,6 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 	GtkWidget *
 #endif
 	label = gtk_label_new (_("export"));
-	gtk_widget_show (label);
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);
 }
 /**
@@ -1119,7 +1159,6 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 {
 	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox);
 #ifdef E2_ASSISTED
 	GtkWidget *label =
 #endif
@@ -1138,19 +1177,18 @@ static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 		openpath = e2_cl_options.config_dir;
 	else
 		openpath = g_get_home_dir ();
-	F_FREE (local);
+	F_FREE (local, e2_cl_options.config_dir);
 
 	gchar *openname = g_build_filename (openpath, default_config_file, NULL);
 
 	rt->open_entry = e2_widget_add_entry (vbox, (gchar *) openname, TRUE, TRUE);
 #ifdef E2_ASSISTED
-	e2_widget_set_label_relations (GTK_LABEL (label), rt->open_entry);
+	e2_widget_set_label_relations (label, rt->open_entry);
 #endif
 	gtk_widget_set_size_request (rt->open_entry, 400, -1);
 
 	GtkWidget *hbox = gtk_hbutton_box_new ();
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, E2_PADDING);
-	gtk_widget_show (hbox);
 
 	gtk_box_set_spacing (GTK_BOX (hbox), E2_PADDING_LARGE);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
@@ -1179,10 +1217,8 @@ static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 		FALSE, _("_specific group option(s)"));
 	rt->expander = gtk_expander_new_with_mnemonic (_("_groups"));
 	gtk_box_pack_start (GTK_BOX (vbox), rt->expander, FALSE, FALSE, 0);
-	gtk_widget_show (rt->expander);
 	GtkWidget *expvbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (rt->expander), expvbox);
-	gtk_widget_show (expvbox);
 	hbox = e2_widget_add_box (expvbox, TRUE, E2_PADDING_SMALL,
 		FALSE, TRUE, E2_PADDING);
 	gchar *labeltxt = _e2pc_get_setlabel (PANEBAR1_P);
@@ -1209,12 +1245,22 @@ static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 	g_free (labeltxt);
 	hbox = e2_widget_add_box (expvbox, TRUE, E2_PADDING_SMALL,
 		FALSE, TRUE, E2_PADDING);
-	labeltxt = _e2pc_get_setlabel (BINDINGS_P);
-	_e2pc_create_check_button (hbox, BINDINGS_P, FALSE, labeltxt);
-	g_free (labeltxt);
 	labeltxt = _e2pc_get_setlabel (ALIASES_P);
 	_e2pc_create_check_button (hbox, ALIASES_P, FALSE, labeltxt);
 	g_free (labeltxt);
+	labeltxt = _e2pc_get_setlabel (KEYS_P);
+	_e2pc_create_check_button (hbox, KEYS_P, FALSE, labeltxt);
+	g_free (labeltxt);
+#ifdef E2_MOUSECUSTOM
+	hbox = e2_widget_add_box (expvbox, TRUE, E2_PADDING_SMALL,
+		FALSE, TRUE, E2_PADDING);
+	labeltxt = _e2pc_get_setlabel (BUTTONS_P);
+	_e2pc_create_check_button (hbox, BUTTONS_P, FALSE, labeltxt);
+	g_free (labeltxt);
+	labeltxt = _e2pc_get_setlabel (GESTURES_P);
+	_e2pc_create_check_button (hbox, GESTURES_P, FALSE, labeltxt);
+	g_free (labeltxt);
+#endif
 	hbox = e2_widget_add_box (expvbox, TRUE, E2_PADDING_SMALL,
 		FALSE, TRUE, E2_PADDING);
 	labeltxt = _e2pc_get_setlabel (MENU_P);
@@ -1233,7 +1279,6 @@ static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 	GtkWidget *
 #endif
 	label = gtk_label_new (_("import"));
-	gtk_widget_show (label);
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);
 }
 /**
@@ -1247,7 +1292,6 @@ static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 {
 	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox);
 #ifdef E2_ASSISTED
 	GtkWidget *label =
 #endif
@@ -1270,14 +1314,13 @@ static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 
 	rt->icondir_entry = e2_widget_add_entry (vbox, openpath, TRUE, TRUE);
 #ifdef E2_ASSISTED
-	e2_widget_set_label_relations (GTK_LABEL (label), rt->icondir_entry);
+	e2_widget_set_label_relations (label, rt->icondir_entry);
 #endif
 	g_free (openpath);
 	gtk_widget_set_size_request (rt->icondir_entry, 400, -1);
 
 	GtkWidget *hbox = gtk_hbutton_box_new ();
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, E2_PADDING);
-	gtk_widget_show (hbox);
 
 	gtk_box_set_spacing (GTK_BOX (hbox), E2_PADDING_LARGE);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
@@ -1301,14 +1344,13 @@ static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 		_("icons"), NULL);
 	rt->iconsavedir_entry = e2_widget_add_entry (vbox, openpath, TRUE, TRUE);
 #ifdef E2_ASSISTED
-	e2_widget_set_label_relations (GTK_LABEL (label), rt->iconsavedir_entry);
+	e2_widget_set_label_relations (label, rt->iconsavedir_entry);
 #endif
 	g_free (openpath);
 	gtk_widget_set_size_request (rt->iconsavedir_entry, 400, -1);
 
 	hbox = gtk_hbutton_box_new ();
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, E2_PADDING);
-	gtk_widget_show (hbox);
 
 	gtk_box_set_spacing (GTK_BOX (hbox), E2_PADDING_LARGE);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
@@ -1321,7 +1363,6 @@ static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 	gtk_box_pack_start (GTK_BOX (hbox), btn, FALSE, FALSE, 0);
 
 	GtkWidget *pagelabel = gtk_label_new (_("icons"));
-	gtk_widget_show (pagelabel);
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, pagelabel);
 }
 /**
@@ -1334,31 +1375,29 @@ static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 */
 static gboolean _e2p_config_dialog_create (gpointer from, E2_ActionRuntime *art)
 {
-	E2P_ConfigData *rt = ALLOCATE (E2P_ConfigData);
-	CHECKALLOCATEDWARN (rt, return FALSE;)
-	srt = rt;	//copy for some funcs that don't get rt as an argument
-	//create dialog
-	rt->dialog = e2_dialog_create (NULL, NULL, _("manage configuration data"),
-	_e2pc_response_cb, rt);
+	E2P_ConfigData rt;
+
+	rt.dialog = e2_dialog_create (NULL, NULL, _("manage configuration data"),
+		DUMMY_RESPONSE_CB, NULL);
+	GtkWidget *vbox =
+#ifdef USE_GTK2_14
+		gtk_dialog_get_content_area (GTK_DIALOG (rt.dialog));
+#else
+		GTK_DIALOG (rt.dialog)->vbox;
+#endif
 	//populate it with widgets
-	GtkWidget *vbox = GTK_DIALOG (rt->dialog)->vbox;
 	GtkWidget *notebook = e2_widget_add_notebook
 		(vbox, TRUE, 0, NULL, NULL);
-	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
+	_e2pc_make_export_tab (notebook, &rt);	//page 0
+	_e2pc_make_import_tab (notebook, &rt);	//page 1
+	_e2pc_make_icons_tab (notebook, &rt);	//page 2
 
-	_e2pc_make_export_tab (notebook, rt);	//page 0
-	_e2pc_make_import_tab (notebook, rt);	//page 1
-	_e2pc_make_icons_tab (notebook, rt);	//page 2
-	gtk_widget_show (notebook);
+	e2_dialog_set_negative_response (rt.dialog, GTK_RESPONSE_CLOSE);
+	//block until user selects
+	e2_dialog_show (rt.dialog, app.main_window,
+		E2_DIALOG_BLOCKED /*| E2_DIALOG_DONT_SHOW_ALL*/ | E2_DIALOG_FREE,
+		&E2_BUTTON_CLOSE, NULL);	//FIXME close btn is not suitable here
 
-	//add action buttons in the order that they will appear
-	e2_dialog_add_defined_button (rt->dialog, &E2_BUTTON_CLOSE);
-
-	e2_dialog_set_negative_response (rt->dialog, GTK_RESPONSE_CLOSE);
-
-	e2_dialog_setup (rt->dialog, app.main_window);
-	gtk_widget_show (rt->dialog);
-	gtk_main ();
 	return TRUE;
 }
 
@@ -1377,15 +1416,17 @@ gboolean init_plugin (Plugin *p)
   p->signature = ANAME VERSION;
   p->menu_name = _("_Configure..");
   p->description = _("Export or import configuration data");
-  p->icon = "plugin_config_"E2IP".png";
+  p->icon = "plugin_config"E2ICONTB;
 
   if (p->action == NULL)
   {
-	//no need to free this
-	gchar *action_name = g_strconcat (_A(2),".",aname,NULL);
-	p->action = e2_plugins_action_register
-	  (action_name, E2_ACTION_TYPE_ITEM, _e2p_config_dialog_create, NULL, FALSE, 0, NULL);
-	return TRUE;
+	//don't free name string here
+	E2_Action plugact =
+	{g_strconcat (_A(3),".",aname,NULL),_e2p_config_dialog_create,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+	p->action = e2_plugins_action_register (&plugact);
+	if (p->action != NULL)
+		return TRUE;
+	g_free (plugact.name);
   }
   return FALSE;
 }
@@ -1398,8 +1439,9 @@ gboolean init_plugin (Plugin *p)
 */
 gboolean clean_plugin (Plugin *p)
 {
-  gchar *action_name = g_strconcat (_A(2),".",aname,NULL);
+  gchar *action_name = g_strconcat (_A(3),".",aname,NULL);
   gboolean ret = e2_plugins_action_unregister (action_name);
   g_free (action_name);
   return ret;
 }
+
