@@ -1,6 +1,6 @@
-/* $Id: e2p_du.c 585 2007-08-09 07:34:42Z tpgww $
+/* $Id: e2p_du.c 1835 2009-09-17 00:35:44Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark
 
 This file is part of emelFM2.
@@ -129,7 +129,7 @@ static gpointer _e2p_du_all (E2_OutputTabRuntime *starttab)
 #ifdef E2_VFS
 	ddata.spacedata = curr_view->spacedata;
 #endif
-	curr_local = F_FILENAME_TO_LOCALE (curr_view->dir);
+	curr_local = D_FILENAME_TO_LOCALE (curr_view->dir);//always dup, to avoid dirchange race
 
 	base = e2_fileview_get_selected_local (curr_view);
 	for (tmp = base; tmp != NULL; tmp = tmp->next)
@@ -139,7 +139,7 @@ static gpointer _e2p_du_all (E2_OutputTabRuntime *starttab)
 		local_path = e2_utils_strcat (curr_local, info->filename);
 		//if (!
 #ifdef E2_VFS
-		ddata.localpath = local_path;
+		ddata.path = local_path;
 		e2_fs_tw (&ddata, _e2p_du_twcb, cbdata, -1, E2TW_PHYS E2_ERR_NONE());
 #else
 		e2_fs_tw (local_path, _e2p_du_twcb, cbdata, -1, E2TW_PHYS E2_ERR_NONE());
@@ -150,7 +150,7 @@ static gpointer _e2p_du_all (E2_OutputTabRuntime *starttab)
 		//}
 		g_free (local_path);
 	}
-	F_FREE (curr_local);
+	g_free (curr_local);
 	total = cbdata->total;
 	files = cbdata->files;
 	dirs = cbdata->dirs;
@@ -269,14 +269,17 @@ init_plugin (Plugin *p)
 	p->signature = ANAME VERSION;
 	p->menu_name = _("_Disk usage");
 	p->description = _("Calculate the disk usage of selected item(s)");
-	p->icon = "plugin_"ANAME"_"E2IP".png";  //icon file pathname
+	p->icon = "plugin_"ANAME E2ICONTB;  //icon file pathname
 
 	if (p->action == NULL)
 	{
-		gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
-		p->action = e2_plugins_action_register
-		(action_name, E2_ACTION_TYPE_ITEM, _e2p_du, NULL, FALSE, 0, NULL);
-		return TRUE;
+		//don't free name string here
+		E2_Action plugact =
+		{g_strconcat (_A(6),".",aname,NULL),_e2p_du,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+		p->action = e2_plugins_action_register (&plugact);
+		if G_LIKELY((p->action != NULL))
+			return TRUE;
+		g_free (plugact.name);
 	}
 	return FALSE;
 }
@@ -289,7 +292,7 @@ init_plugin (Plugin *p)
 */
 gboolean clean_plugin (Plugin *p)
 {
-  gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
+  gchar *action_name = g_strconcat (_A(6),".",aname,NULL);
   gboolean ret = e2_plugins_action_unregister (action_name);
   g_free (action_name);
   return ret;
