@@ -1,6 +1,6 @@
-/* $Id: e2p_glob.c 799 2008-02-03 23:11:42Z tpgww $
+/* $Id: e2p_glob.c 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark
 
 This file is part of emelFM2.
@@ -102,7 +102,8 @@ static gchar *date_format [5] =
 /**
 @brief decide whether an item should be selected, on the basis of its name
 
-Item name string is localised
+Item name string is localised. Obviously, @a info is expected to survive any
+refresh that may occur.
 
 @param info data structure for item being checked
 @param data data structure for view being processed
@@ -123,17 +124,17 @@ static gboolean _e2p_glob_match_name (FileInfo *info, E2_FltDlgData *data)
 		save = *s;
 		*s = '\0';
 		while (p[0] == ' ')
-			p++;
+			p += sizeof (gchar);
 		if (p[0] == '!')
 		{
 			negated = !data->name_filter.invert_mask;
-			p++;
+			p += sizeof (gchar);
 		}
 		else
 		{
 			negated = data->name_filter.invert_mask;
 			if (p[0] == '\\' && p[1] == '!')
-				p++;
+				p += sizeof (gchar);
 		}
 		if (!positive_check)
 			positive_check = !negated;
@@ -150,14 +151,14 @@ static gboolean _e2p_glob_match_name (FileInfo *info, E2_FltDlgData *data)
 		*s = save;
 		if (matched && negated)
 		{
-		  F_FREE (utf);
+		  F_FREE (utf, info->filename);
 		  return FALSE;
 		}
 		if (matched && !negated)
 		  result = TRUE;	//but keep looking for any later exclude
 		//if neither negated nor matched, we don't change result
 
-		NCHR(s);	//in case the matched char is bigger than 1 byte
+		s += sizeof (gchar);	//pass the ','
 		p = s;
 	}
 	//check the last (or only) pattern
@@ -193,7 +194,7 @@ static gboolean _e2p_glob_match_name (FileInfo *info, E2_FltDlgData *data)
 	else if (negated && !positive_check)
 		result = TRUE;
 
-	F_FREE (utf);
+	F_FREE (utf, info->filename);
 	return result;
 }
 /**
@@ -282,7 +283,7 @@ static void _e2p_glob_response_cb (GtkDialog *dialog, gint response,
 {
 	switch (response)
 	{
-		case GTK_RESPONSE_OK:
+		case E2_RESPONSE_APPLY:
 		{
 			const gchar *s;
 //#warning ignore compiler warnings about unitialized usage of choices.*
@@ -290,7 +291,12 @@ static void _e2p_glob_response_cb (GtkDialog *dialog, gint response,
 			//assignment for complier-warning prevention only
 			memset (&choices, '\0', sizeof (E2_FltDlgData));
 			//get all the relevant parameters from the dialog
-			choices.name_filter.use = GTK_TOGGLE_BUTTON (data->usename_check)->active;
+			choices.name_filter.use =
+#ifdef USE_GTK2_14
+				gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->usename_check));
+#else
+				GTK_TOGGLE_BUTTON (data->usename_check)->active;
+#endif
 			if (choices.name_filter.use)
 			{
 				//filter pattern is utf
@@ -306,11 +312,26 @@ static void _e2p_glob_response_cb (GtkDialog *dialog, gint response,
 						g_free (previous_pattern);
 					previous_pattern = g_strdup (s);
 					choices.name_filter.pattern = s;
-					choices.name_filter.invert_mask = GTK_TOGGLE_BUTTON (data->invert_check)->active;
-					choices.name_filter.case_sensitive = GTK_TOGGLE_BUTTON (data->case_sensitive_check)->active;
+					choices.name_filter.invert_mask =
+#ifdef USE_GTK2_14
+						gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->invert_check));
+#else
+						GTK_TOGGLE_BUTTON (data->invert_check)->active;
+#endif
+					choices.name_filter.case_sensitive =
+#ifdef USE_GTK2_14
+						gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->case_sensitive_check));
+#else
+						GTK_TOGGLE_BUTTON (data->case_sensitive_check)->active;
+#endif
 				}
 			}
-			choices.size_filter.use = GTK_TOGGLE_BUTTON (data->usesize_check)->active;
+			choices.size_filter.use =
+#ifdef USE_GTK2_14
+				gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->usesize_check));
+#else
+				GTK_TOGGLE_BUTTON (data->usesize_check)->active;
+#endif
 			if (choices.size_filter.use)
 			{
 				s = gtk_entry_get_text (GTK_ENTRY (data->size_entry));
@@ -322,21 +343,31 @@ static void _e2p_glob_response_cb (GtkDialog *dialog, gint response,
 				choices.size_filter.op = gtk_combo_box_get_active
 					(GTK_COMBO_BOX (data->sizop_combo));	//0=1st entry index, =
 			}
-			choices.date_filter.use = GTK_TOGGLE_BUTTON (data->usedate_check)->active;
+			choices.date_filter.use =
+#ifdef USE_GTK2_14
+				gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->usedate_check));
+#else
+				GTK_TOGGLE_BUTTON (data->usedate_check)->active;
+#endif
 			if (choices.date_filter.use)
 			{
-				struct tm tm_time;
-				gchar *strf_withtime = g_strconcat (date_format [date_index] , " %T", NULL);
-				gchar *date_string = g_strdup_printf ("%s 00:00:00",
-					gtk_entry_get_text (GTK_ENTRY (data->date_entry)));
-				strptime (date_string, strf_withtime, &tm_time);
-				g_free (strf_withtime);
-				g_free (date_string);
-				choices.date_filter.time = (time_t) mktime (&tm_time);
-				gint index = gtk_combo_box_get_active (GTK_COMBO_BOX (data->datop_combo));
-				//these are tricks consequent on the order of combo entries
-				choices.date_filter.time_type = index / 2;
-				choices.date_filter.op = index % 2;
+				gchar date_buf[25];
+				choices.date_filter.time = e2_date_filter_dialog_get_time
+					(gtk_entry_get_text(GTK_ENTRY (data->date_entry)),
+					date_buf, sizeof(date_buf), date_index);
+				if (choices.date_filter.time != (time_t)-1)
+				{
+					gtk_entry_set_text (GTK_ENTRY (data->date_entry), date_buf);
+					gint index = gtk_combo_box_get_active (GTK_COMBO_BOX (data->datop_combo));
+					//these are tricks consequent on the order of combo entries
+					choices.date_filter.time_type = index / 2;
+					choices.date_filter.op = index % 2;
+				}
+				else
+				{
+					//FIXME warning to user
+					choices.date_filter.use = FALSE;
+				}
 			}
 			if (choices.name_filter.use || choices.size_filter.use || choices.date_filter.use)
 			{
@@ -347,17 +378,8 @@ static void _e2p_glob_response_cb (GtkDialog *dialog, gint response,
 					FileInfo *info;
 					e2_filelist_disable_refresh ();
 					e2_window_set_cursor (GDK_WATCH);
-					//wait until any current recreation is finished
-					while (TRUE)
-					{
-						LISTS_LOCK
-						gboolean busy = curr_view->listcontrols.refresh_working
-									 || curr_view->listcontrols.cd_working;
-						LISTS_UNLOCK
-						if (!busy)
-							break;
-						usleep (100000);
-					}
+					//wait until any current re-creation is finished
+					WAIT_FOR_REFRESH(curr_view)
 
 					GtkTreeSelection *sel = curr_view->selection;
 					gtk_tree_selection_unselect_all (sel);	//start with clean slate
@@ -382,13 +404,10 @@ static void _e2p_glob_response_cb (GtkDialog *dialog, gint response,
 				}
 			}
 		}
-		break;
+			break;
 		default:
-		break;
+			break;
 	}
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-	gtk_main_quit ();
-	return;
 }
 /**
 @brief create and run selection-filter dialog
@@ -401,24 +420,36 @@ static void _e2p_glob_response_cb (GtkDialog *dialog, gint response,
 static gboolean _e2p_glob (gpointer from, E2_ActionRuntime *art)
 {
 	GtkWidget *dialog;
-	GtkWidget *vbox, *hbox, *table;
+	GtkWidget *dialog_vbox, *hbox, *table;
 	E2_FltDlgRuntime data;
 	gint index;
 	gchar *utf;
 
-	dialog = e2_dialog_create (NULL, _("Select items:"),
-		_("selection filter"), _e2p_glob_response_cb, &data);
-	vbox = GTK_DIALOG (dialog)->vbox;
+	e2_filelist_disable_refresh ();	//don't want any info to be disrupted
+
+	E2_PaneRuntime *rt = e2_pane_get_runtime (from, art->data, NULL);
+	dialog = e2_dialog_create (GTK_STOCK_INDEX, _("Select items:"),
+		_("selection filter"),
+		(ResponseFunc)_e2p_glob_response_cb, &data);
+	dialog_vbox =
+#ifdef USE_GTK2_14
+		gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+#else
+		GTK_DIALOG (dialog)->vbox;
+#endif
 	//the name-related things ...
-	hbox = e2_widget_add_box (vbox, TRUE, 0, FALSE, FALSE, 0);
+	hbox = e2_widget_add_box (dialog_vbox, TRUE, 0, FALSE, FALSE, 0);
 	data.usename_check = e2_button_add_toggle
 		(hbox, TRUE, FALSE, NULL, NULL, FALSE, 0, NULL, &data);
+#ifdef E2_ASSISTED
+	GtkWidget *label =
+#endif
 	e2_widget_add_mid_label (hbox, _("Named like"), 0.0, FALSE, 0);
 	data.pattern_entry = e2_widget_add_entry (hbox, "", TRUE, FALSE);
-//#ifdef E2_ASSISTED
-//	e2_widget_set_label_relations (GTK_LABEL (label), data.pattern_entry);
-//#endif
-	FileInfo *info = e2_fileview_get_selected_first_local (curr_view, FALSE);
+#ifdef E2_ASSISTED
+	e2_widget_set_label_relations (label, data.pattern_entry);
+#endif
+	FileInfo *info = e2_fileview_get_selected_first_local (&rt->view, FALSE);
 	if (info != NULL)
 	{
 		gchar *text, *s;
@@ -430,29 +461,25 @@ static gboolean _e2p_glob (gpointer from, E2_ActionRuntime *art)
 		gtk_entry_set_text (GTK_ENTRY(data.pattern_entry), text);
 		if (text != utf)
 			g_free (text);
-		F_FREE (utf);
+		F_FREE (utf, info->filename);
 	}
 	else if (previous_pattern != NULL)
 		gtk_entry_set_text (GTK_ENTRY(data.pattern_entry), previous_pattern);
 
 	e2_widget_add_mid_label (hbox, _("example: *.c,*.h"), 0.0, FALSE, 0);
 
-	table = e2_widget_add_table (vbox, 1, 3, TRUE, TRUE, 0);
+	table = e2_widget_add_table (dialog_vbox, 1, 3, TRUE, TRUE, 0);
 	data.invert_check = e2_button_add_toggle_to_table (table, _("_Invert"),
 		FALSE, NULL, NULL, 1, 2, 0, 1);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		data.invert_check, _("Select items that DO NOT match the given mask"));
+	e2_widget_set_safetip (data.invert_check,
+		_("Select items that DO NOT match the given mask"));
 	data.case_sensitive_check = e2_button_add_toggle_to_table (table, _("Case _sensitive"),
 		TRUE, NULL, NULL, 2, 3, 0, 1);
 
-	e2_widget_add_separator (vbox, TRUE, 1);
+	e2_widget_add_separator (dialog_vbox, TRUE, 1);
 
 	//the size-related things ...
-	hbox = e2_widget_add_box (vbox, TRUE, 0, FALSE, FALSE, 0);
+	hbox = e2_widget_add_box (dialog_vbox, TRUE, 0, FALSE, FALSE, 0);
 	data.usesize_check = e2_button_add_toggle (hbox, TRUE, FALSE, NULL, NULL, FALSE, 0, NULL, &data);
 	data.sizop_combo = e2_combobox_add (hbox, FALSE, 0, NULL, NULL, NULL,
 	  E2_COMBOBOX_MENU_STYLE);
@@ -492,9 +519,9 @@ static gboolean _e2p_glob (gpointer from, E2_ActionRuntime *art)
 
 	data.size_entry = e2_widget_add_entry (hbox, size_string, TRUE, FALSE);
 //	gtk_widget_set_size_request (data.size_entry, 100, 30);
-//#ifdef E2_ASSISTED
-//	e2_widget_set_label_relations (GTK_LABEL (label), data.size_entry);
-//#endif
+#ifdef E2_ASSISTED
+	e2_widget_set_label_relations (data.sizop_combo, data.size_entry);
+#endif
 
 	data.units_combo = e2_combobox_add (hbox, FALSE, 0, NULL, NULL, NULL,
 	  E2_COMBOBOX_MENU_STYLE);
@@ -502,10 +529,10 @@ static gboolean _e2p_glob (gpointer from, E2_ActionRuntime *art)
 	e2_combobox_append_history_counted (data.units_combo, 3, size_names);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (data.units_combo), index);
 
-	e2_widget_add_separator (vbox, TRUE, 1);
+	e2_widget_add_separator (dialog_vbox, TRUE, 1);
 
 	//the date-related things ...
-	hbox = e2_widget_add_box (vbox, TRUE, 0, FALSE, FALSE, 0);
+	hbox = e2_widget_add_box (dialog_vbox, TRUE, 0, FALSE, FALSE, 0);
 	data.usedate_check = e2_button_add_toggle
 		(hbox, TRUE, FALSE, NULL, NULL, FALSE, 0, NULL, &data);
 	data.datop_combo = e2_combobox_add (hbox, FALSE, 0, NULL, NULL, NULL,
@@ -537,18 +564,20 @@ static gboolean _e2p_glob (gpointer from, E2_ActionRuntime *art)
 	if (info != NULL)
 		g_free (utf);
 //	gtk_widget_set_size_request (data.date_entry, 120, 30);
-//#ifdef E2_ASSISTED
-//	e2_widget_set_label_relations (GTK_LABEL (label), data.date_entry);
-//#endif
+#ifdef E2_ASSISTED
+	e2_widget_set_label_relations (data.datop_combo, data.date_entry);
+#endif
 	//start at the name entry
 	gtk_editable_select_region (GTK_EDITABLE(data.pattern_entry), 0, -1);
 	gtk_widget_grab_focus (data.pattern_entry);
 
+	e2_filelist_enable_refresh ();
+
 //	e2_dialog_resize (dialog, 1.3);	//really, we only want to make it wider ..
-	//non-modal dialog
-	e2_dialog_show (dialog, app.main_window, 0,
-		&E2_BUTTON_CANCEL, &E2_BUTTON_OK, NULL);
-	gtk_main ();
+	//block until the user selects
+	e2_dialog_show (dialog, app.main_window, E2_DIALOG_BLOCKED | E2_DIALOG_FREE,
+		&E2_BUTTON_CANCEL, &E2_BUTTON_APPLY, NULL);
+
 	return TRUE;
 }
 
@@ -569,15 +598,17 @@ gboolean init_plugin (Plugin *p)
 	p->signature = ANAME VERSION;
 	p->menu_name = _("_Glob..");
 	p->description = _("Select items matching a specified pattern");
-	p->icon = "plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate
+	p->icon = "plugin_"ANAME E2ICONTB;  //use icon file pathname if appropriate
 
 	if (p->action == NULL)
 	{
-		//no need to free this
-		gchar *action_name = g_strconcat (_A(10),".",aname,NULL);
-		p->action = e2_plugins_action_register
-			(action_name, E2_ACTION_TYPE_ITEM, _e2p_glob, NULL, FALSE, 0, NULL);
-		return TRUE;
+		//don't free name string here
+		E2_Action plugact =
+		{g_strconcat (_A(7),".",aname,NULL),_e2p_glob,TRUE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+		p->action = e2_plugins_action_register (&plugact);
+		if G_LIKELY((p->action != NULL))
+			return TRUE;
+		g_free (plugact.name);
 	}
 	return FALSE;
 }
@@ -590,10 +621,12 @@ gboolean init_plugin (Plugin *p)
 */
 gboolean clean_plugin (Plugin *p)
 {
-	gchar *action_name = g_strconcat (_A(10),".",aname,NULL);
+	gchar *action_name = g_strconcat (_A(7),".",aname,NULL);
 	gboolean ret = e2_plugins_action_unregister (action_name);
 	g_free (action_name);
 	if (previous_pattern != NULL)
 		g_free (previous_pattern);
 	return ret;
 }
+
+
