@@ -1,4 +1,4 @@
-/* $Id: e2p_mvbar.c 773 2007-11-30 21:19:14Z tpgww $
+/* $Id: e2p_mvbar.c 1917 2009-09-27 09:30:57Z tpgww $
 **************************************
 
 Emelmovebar - a plugin for emelFM2
@@ -34,6 +34,7 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include <string.h>
 #include <pthread.h>
 #include <signal.h>
+#include <sys/shm.h>
 #include "e2_plugins.h"
 #include "e2_dialog.h"
 #include "e2_option.h"
@@ -49,14 +50,9 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 //#define MAX_MSG 220
 //interval (usec) between progress window updates
 //for items < and > 10MB resepcively
-#define MIN_UPDATE_INTERVAL 100000
-#define MAX_UPDATE_INTERVAL 200000
+//#define MIN_UPDATE_INTERVAL 100000
+//#define MAX_UPDATE_INTERVAL 200000
 
-//include things to support pause and resume
-#ifndef IS_PAUSABLE
-#define IS_PAUSABLE
-#endif
-#ifdef IS_PAUSABLE
 typedef enum
 {
 	E2_BARTASK_STOPPED   = 1,
@@ -65,7 +61,6 @@ typedef enum
 	E2_BARTASK_COMPLETED = 1 << 3,
 	E2_BARTASK_SUCCEEDED = 1 << 4,
 } E2_BarFlags;
-#endif
 
 typedef struct _E2_BarData
 {
@@ -75,35 +70,20 @@ typedef struct _E2_BarData
 
 typedef struct _E2_ProgressData
 {
-	pthread_mutex_t mutex;  //protects access to these data
-	pthread_cond_t cond;   //signals change
 	VPATH *dlocal;	//includes localised destination path (maybe temp) of item moved
 	guint64 done_size;
-	guint64 refresh_interval;
 } E2_ProgressData;
 
-typedef struct _E2_ActionData
-{
-//	pthread_mutex_t mutex;  //protects access to these data
-	E2_FileTaskMode flags;	//flags for features of current copy
-	VPATH *slocal;	//includes localised source path of item copied
-	VPATH *dlocal;	//includes localised destination path of item copied
-	gboolean completed;	//set TRUE when the backend task is completed (valid or not)
-	gboolean result;	//value returned by backend task function
-} E2_ActionData;
-
 typedef struct _E2_BarWindowData
 {
 	GtkWidget *dialog;
 	GtkWidget *label;
-//	GtkWidget *label2;
 	GtkWidget *progbar;
-#ifdef IS_PAUSABLE
 	GtkWidget *pause_btn;
 	GtkWidget *resume_btn;
 	GtkWidget *stop_btn;
-	E2_BarFlags flags;
-#endif
+	E2_BarFlags bflags;
+	E2_MainLoop *loop;
 } E2_BarWindowData;
 
 static gboolean _e2p_mvbarQ (E2_ActionTaskData *qed);
@@ -143,178 +123,104 @@ _e2p_mvbar_twcb (VPATH *localpath, const struct stat *statptr,
 	return E2TW_CONTINUE;
 }
 /**
-@brief 'shorten' string @a string if it is longer than allowed
-@a dots is set to "..." if @a string is too long, or else ""
-
-@param string utf string which may need to be shortened
-@param dots pointer to store for utf string which will preceed @a string, when it is displayed
-@param string_diff pointer to store for the no. of bytes to be omitted from the start of @a string, when it is displayed
+@brief determine how much progress has been made on the current item
+This provides data for progressive updates during the course of moving an item.
+BGL will be open (though no UI change here)
+@param data pointer to struct with data which is needed here
 
 @return
 */
-/*static void
-_e2p_mvbar_shorten (gchar *string, gchar **dots, gint *string_diff)
-{
-	if (strlen (string) > MAX_CHAR)
-	{
-		//nominally we will drop this many bytes off the start of string
-		gint trunc = strlen (string) - MAX_CHAR;
-		//but make sure we don't cut off in the middle of a char
-		gchar *c = g_utf8_find_next_char (string+trunc-1, NULL);
-		*string_diff = c - string;
-		*dots = "...";
-	}
-	else
-	{
-		*string_diff = 0;
-		*dots = "";
-	}
-	return;
-} */
-/**
-@brief thread function to determine how much progress has been made on the current item
-
-This provides data for progressive updates during the course of moving an item.
-The thread doesn't end of its own accord, but continues until it is cancelled
-by the main thread
-BGL will be open
-
-@param data pointer to struct with data which is needed here
-
-@return NULL (never happens)
-*/
-static void *
+static void
 _e2p_mvbar_progress (E2_ProgressData *data)
 {
 	gchar *localpath;
 	struct stat sb;
 #ifdef E2_VFS
 	VPATH ddata;
-#endif
-//	printd (DEBUG, "monitor thread underway");
-
-	e2_utils_block_thread_signals ();	//block all allowed signals to this thread
-
-#ifdef E2_VFS
 	ddata.spacedata = data->dlocal->spacedata;
 #endif
-	while (TRUE)
+	localpath = VPSTR (data->dlocal);
+
+	//determine how much of current item has been moved
+	E2_BarData pdata;
+	pdata.totalsize = 0;
+#ifdef E2_VFS
+	ddata.path = localpath;
+	if (!e2_fs_lstat (&ddata, &sb E2_ERR_NONE()))
+#else
+	if (!e2_fs_lstat (localpath, &sb E2_ERR_NONE()))
+#endif
 	{
-		pthread_testcancel ();	//has a cancel-instruction been issued ?
-		//prevent the main thread from setting the cancel signal now
-		if (pthread_mutex_lock (&(data->mutex)))
+		if (S_ISDIR (sb.st_mode))
 		{
-			printd (WARN, "Lock error!");
+			//if (!
+#ifdef E2_VFS
+			e2_fs_tw (&ddata,
+#else
+			e2_fs_tw (localpath,
+#endif
+				_e2p_mvbar_twcb, &pdata, -1, E2TW_PHYS E2_ERR_NONE());
+			//)
+			//{
+				//FIXME handle error
+			//}
 		}
-		//get a local copy of the present item's destination path
-		//(that will be NULL if the action thread has finished already)
-		if (data->dlocal == NULL)
-			localpath = NULL;
 		else
-			localpath = g_strdup (VPSTR (data->dlocal));
-
-		if (pthread_mutex_unlock (&(data->mutex)))
-		{
-			printd (WARN,"Unlock error!");
-		}
-
-		if (localpath != NULL)
-		{	//we're still underway
-			//determine how much of current item has been moved
-			E2_BarData pdata;
-			pdata.totalsize = 0;
-//			pdata.count = 0;
-#ifdef E2_VFS
-			ddata.localpath = localpath;
-			if (!e2_fs_lstat (&ddata, &sb E2_ERR_NONE()))
-#else
-			if (!e2_fs_lstat (localpath, &sb E2_ERR_NONE()))
-#endif
-			{
-				if (S_ISDIR (sb.st_mode))
-				{
-					//if (!
-#ifdef E2_VFS
-					e2_fs_tw (&ddata,
-#else
-					e2_fs_tw (localpath,
-#endif
-					_e2p_mvbar_twcb, &pdata, -1, E2TW_PHYS E2_ERR_NONE());
-					//)
-					//{
-						//FIXME handle error
-					//}
-				}
-				else
-//			{
-				pdata.totalsize = sb.st_size;
-//				pdata.count = 1;
-//			}
-			}
-			g_free (localpath);
-
-			if (pthread_mutex_lock (&(data->mutex)))
-			{
-				printd (WARN, "Lock error!\n");
-			}
-			//update size data for the main thread to use
-			data->done_size = pdata.totalsize;
-//			printd (DEBUG, "progress is %li", pdata.totalsize);
-
-			if (pthread_cond_signal (&data->cond))
-			{
-				printd (WARN,"Signal error!");
-			}
-
-			if (pthread_mutex_unlock (&(data->mutex)))
-			{
-				printd (WARN,"Unlock error!");
-			}
-		}
-		//usleep is another cancellation point
-		usleep (data->refresh_interval);	//delay between updates
+			pdata.totalsize = sb.st_size;
 	}
-	return NULL;
+
+	data->done_size = pdata.totalsize;
+//	printd (DEBUG, "progress is %llu", pdata.totalsize);
 }
 /**
-@brief perform the move
-
-This is the thread function which performs the move task.
-The thread simply exits when completed
-
-@param data pointer to struct with data which is needed here
-
-@return NULL (irrelevant pointer)
+@brief cleanup after this task's thread is aborted
+@param pid pointer to store for pid of child process doing the copying
+@return
 */
-static void *
-_e2p_mvbar_action (E2_ActionData *data)
+static void
+_e2p_mvbar_abort_clean_process (pid_t *pid)
 {
-//	printd (DEBUG, "action thread underway for %s", data->slocal);
-	e2_utils_block_thread_signals ();	//block all allowed signals to this thread
-
-	data->result = e2_task_backend_move	//no background-move option in this context
-		(data->slocal, data->dlocal);
-	data->completed = TRUE;
-	return NULL;
+	if (*pid != 0)
+		kill (*pid, SIGSTOP);
+	//FIXME cleanup shared memory, temp file
+}
+/*static void
+_e2p_mvbar_abort_clean_memory (gboolean **shared)
+{
+	if (*shared != NULL && shmdt (*shared) == 0)
+		shmctl (shmid, IPC_RMID, NULL);
+}
+*/
+/**
+@brief cleanup after this task's thread is aborted
+@param dialog the progress dialog
+@return
+*/
+static void
+_e2p_mvbar_abort_clean_dialog (GtkWidget **dialog)
+{
+	if (GTK_IS_DIALOG (*dialog))
+	{
+		gdk_threads_enter ();
+		gtk_widget_destroy (*dialog);
+		gdk_threads_leave();
+	}
 }
 /**
-@brief update progress window details, and move item @a src to @a dest
+@brief move item @a slocal to @a dlocal, while updating progress-window details
 This is called with BGL open
 @param slocal path of item to be moved, localised string
 @param dlocal new path of moved item, localised string
 @param realmove TRUE if this is a move between devices or partititions
-@param breakflag pointer to store for abort-flag
 @param flags bitflags indicating task parameters
 @param bdata pointer to bar data struct holding progress data
 @param tdata pointer to bar data struct holding totals data
 @param wdata pointer to info-window data struct
 
-@return
+@return enumerator of user's request, if any, to abort
 */
-static void _e2p_mvbar_exec (VPATH *slocal, VPATH *dlocal, gboolean realmove,
-#ifndef IS_PAUSABLE
-	gboolean *breakflag,
-#endif
+static DialogButtons
+_e2p_mvbar_exec (VPATH *slocal, VPATH *dlocal, gboolean realmove,
 	E2_FileTaskMode flags,
 	E2_BarData *bdata, E2_BarData *tdata, E2_BarWindowData *wdata)
 {
@@ -333,75 +239,57 @@ static void _e2p_mvbar_exec (VPATH *slocal, VPATH *dlocal, gboolean realmove,
 		//FIXME handle error
 	//}
 
-    //create action thread
 	//work with a temp name so that the backend doesn't create its own
 	//(which would prevent destination-monitoring)
 	gchar *templocal = e2_utils_get_tempname (VPSTR (dlocal));
 #ifdef E2_VFS
 	VPATH tempdata = { templocal, dlocal->spacedata };
-	E2_ActionData a_data = { flags, slocal , &tempdata, FALSE, FALSE };
-#else
-	E2_ActionData a_data = { flags, slocal, templocal, FALSE, FALSE };
 #endif
-	pthread_t action_thread_id;
-	gint status = pthread_create (&action_thread_id, NULL,
-		(void *) _e2p_mvbar_action, &a_data);
-	if (status != 0)
-	{
-		printd (WARN,"action-thread-create error!");
-//		if (errno == EAGAIN)
-//			printd (WARN, "not enough system resources");
-		g_free (templocal);
-		return;
+	key_t key = ftok (templocal, pthread_self ());
+	gint shmid = shmget (key, 2*sizeof(gboolean), 0600 | IPC_CREAT);
+//	gboolean *results = NULL;
+//	pthread_cleanup_push ((gpointer)_e2p_mvbar_abort_clean_memory, &results); FIXME
+	gboolean *results = shmat (shmid, NULL, 0);
+	results[0] = FALSE;	//result-value returned by backend task function
+	results[1] = FALSE;	//set TRUE when the backend task is completed (valid or not)
+	pid_t pid = fork ();
+	if (pid == 0)
+	{	//child
+		//CHECKME BGL mutex here ?
+		//threads enter/leave don't hang, at least, but any message printed here
+		//generates a SIGCHILD but no actual message ?
+		results[0] = e2_task_backend_move (slocal,
+#ifdef E2_VFS
+			&tempdata);
+#else
+			templocal);
+#endif
+		results[1] = TRUE;	//finished now
+		_exit (0);
 	}
-
-	//wait for task to initiate, at least
+	if (pid < 0)
+	{	//fork error
+		printd (WARN,"action-process-create error!");
+		g_free (templocal);
+		//cleanup shared memory
+		if (shmdt (results) == 0)
+			shmctl (shmid, IPC_RMID, NULL);
+		return CANCEL;
+	}
+	//parent process
+	//for very quick operation, don't bother with anything fancy
 	guint waitup;
 	if (!realmove)
 		waitup = 1000;  //shorter delay if the move is logical (same device)
 	else
 		waitup = 50000;  //longer delay if the move needs to be physical
-	g_usleep (waitup);
+	usleep (waitup);
 
-	//create progress monitor, if task not finished already
-	//NOTE since we're processing 1 item at a time, this monitoring
-	//could probably be handled just as well in the main thread
-	if (!a_data.completed)
-	{
-		//create monitor thread
-		E2_ProgressData m_data;
-		pthread_mutex_init (&(m_data.mutex), NULL);
-		pthread_cond_init (&(m_data.cond), NULL);
-		//get the temp name now in use for moving the item
-#ifdef E2_VFS
-		m_data.dlocal = &tempdata;
-#else
-		m_data.dlocal = templocal;
-#endif
-		m_data.done_size = 0;
-		//rough approach to setting the reporting interval (usec)
-//		m_data.refresh_interval = pdata.totalsize * 10;
-//		if (m_data.refresh_interval > MAX_UPDATE_INTERVAL)
-		if (pdata.totalsize < 10000000)
-			m_data.refresh_interval = MIN_UPDATE_INTERVAL;
-		else
-			m_data.refresh_interval = MAX_UPDATE_INTERVAL;	//with a cap
-		pthread_t mon_thread_id;
-		status = pthread_create (&mon_thread_id, NULL,
-			(gpointer) _e2p_mvbar_progress, &m_data);
-		if (status != 0)
-		{
-			printd (WARN,"monitor-thread-create error!");
-			g_free (templocal);
-			return;
-		}
-
-		if (!GTK_WIDGET_VISIBLE (wdata->dialog))
-		{
-			gdk_threads_enter ();
-			gtk_widget_show (wdata->dialog);
-			gdk_threads_leave ();
-		}
+	if (!results[1])
+	{	//task not finished already, do progress reporting
+		//to reduce race-risk, this push should be before the fork, but in that case
+		//we can't pop in case of fork error
+		pthread_cleanup_push ((gpointer)_e2p_mvbar_abort_clean_process, &pid);
 
 		gchar *shortsrc = e2_utils_str_shorten (src, MAX_CHAR, E2_DOTS_START);
 		gchar *shortdest = e2_utils_str_shorten (dest_dir, MAX_CHAR, E2_DOTS_START);
@@ -412,6 +300,12 @@ static void _e2p_mvbar_exec (VPATH *slocal, VPATH *dlocal, gboolean realmove,
 			shortsrc, shortdest, num1, num2);
 		gdk_threads_enter ();
 		gtk_label_set_text (GTK_LABEL (wdata->label), labeltext);
+#ifdef USE_GTK2_18
+		if (!gtk_widget_get_visible (wdata->dialog))
+#else
+		if (!GTK_WIDGET_VISIBLE (wdata->dialog))
+#endif
+			gtk_widget_show (wdata->dialog);
 		gdk_threads_leave ();
 		g_free (shortsrc);
 		g_free (shortdest);
@@ -421,57 +315,49 @@ static void _e2p_mvbar_exec (VPATH *slocal, VPATH *dlocal, gboolean realmove,
 
 		gchar *progress_format = _("%.2f MB of %.2f MB  (%.0f\%%)");
 		gfloat fraction;
-
-		//update progress when the monitor thread reports
 		guint64 progress;
-		while (!a_data.completed)	//loop until the action thread is completed
-		{
-			if (pthread_mutex_lock (&(m_data.mutex)))
-			{
-				printd (WARN, "Lock error!");
-			}
-			while (m_data.done_size == 0)
-			{
-				status = pthread_cond_wait (&(m_data.cond), &(m_data.mutex));
-			}
-			if (status != 0)
-			{
-				printd (WARN,"Wait error!");
-			}
 
-			progress = m_data.done_size + bdata->totalsize;
-			m_data.done_size = 0;	//reset
-
-			if (pthread_mutex_unlock (&(m_data.mutex)))
-			{
-				printd (WARN,"Unlock error!");
-			}
-
-#ifdef IS_PAUSABLE
-			if (wdata->flags & E2_BARTASK_STOPPED)	//user wants to abort
+		E2_ProgressData m_data;
+#ifdef E2_VFS
+		m_data.dlocal = &tempdata;
 #else
-			if (*breakflag) //user wants to abort
+		m_data.dlocal = templocal;
 #endif
+		m_data.done_size = 0;
+/*		//rough approach to setting the reporting interval (usec)
+		guint refresh_interval;
+		if (pdata.totalsize < 10000000)
+			refresh_interval = MIN_UPDATE_INTERVAL;
+		else
+			refresh_interval = MAX_UPDATE_INTERVAL;	//with a cap
+*/
+		while (!results[1])	//loop until the action is completed
+		{
+			if (wdata->bflags & E2_BARTASK_STOPPED)	//user wants to abort
 			{
-				if (pthread_cancel (mon_thread_id))
-				{
-					printd (WARN,"Thread cancel error!\n");
-				}
-				//cancel the task thread
-				if (pthread_cancel (action_thread_id))
-				{
-					printd (WARN,"Thread cancel error!\n");
-				}
-				//cleanup anything part-moved
+				//cancel the task process
+				kill (pid, SIGKILL);
+				//CHECKME properly abort the whole process
+				gdk_threads_enter ();
+				gtk_widget_destroy (wdata->dialog);
+				gdk_threads_leave ();
+				//cleanup anything part-copied
 #ifdef E2_VFS
 				e2_task_backend_delete (&tempdata);
 #else
 				e2_task_backend_delete (templocal);
 #endif
 				g_free (templocal);
-				return;
+				//FIXME cleanup cancellation push(es)
+				//cleanup shared memory
+				if (shmdt (results) == 0)
+					shmctl (shmid, IPC_RMID, NULL);
+				return NO_TO_ALL;
 			}
 
+			_e2p_mvbar_progress (&m_data);
+			progress = m_data.done_size + bdata->totalsize;
+
 			fraction = (gdouble) progress / tdata->totalsize;
 			//deal with rounding errors
 			if (fraction > 1.0)
@@ -482,23 +368,42 @@ static void _e2p_mvbar_exec (VPATH *slocal, VPATH *dlocal, gboolean realmove,
 			gdk_threads_enter ();
 			gtk_progress_bar_set_text (GTK_PROGRESS_BAR (wdata->progbar), progresstext);
 			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (wdata->progbar), fraction);
-			//CHECKME some sort of wait here ??
 			gdk_threads_leave ();
-		}
 
-		//immediate signal to monitor thread to stop processing
-		if (pthread_mutex_lock (&(m_data.mutex)))
-		{
-			printd (WARN, "Lock error!");
-		}
-		m_data.dlocal = NULL;
-		if (pthread_mutex_unlock (&(m_data.mutex)))
-		{
-			printd (WARN,"Unlock error!");
+			if (wdata->bflags & E2_BARTASK_PAUSEREQ //pause was requested
+				&& GTK_IS_DIALOG (wdata->dialog) &&
+#ifdef USE_GTK2_18
+				gtk_widget_get_visible (wdata->dialog))
+#else
+				GTK_WIDGET_VISIBLE (wdata->dialog))
+#endif
+			{
+				wdata->bflags &= ~E2_BARTASK_PAUSEREQ;
+				wdata->loop = e2_main_loop_new (FALSE);
+				if (wdata->loop != NULL)
+				{
+					kill (pid, SIGSTOP);
+					wdata->bflags |= E2_BARTASK_PAUSED;
+					e2_filelist_enable_refresh ();
+					pthread_cleanup_push ((gpointer)gdk_threads_leave, NULL);
+					gdk_threads_enter ();
+					e2_main_loop_run (wdata->loop);
+					pthread_cleanup_pop (1);
+					kill (pid, SIGCONT);	//don't get to here if stopped or aborted
+				}
+			}
+			else
+				usleep (100000);	//no need to hog CPU by refreshing too fast (cancellation point)
 		}
 
 		//show the full-time score ASAP
-		if (a_data.result) //move succeeded
+		if (results[0]	//move succeeded
+			&& GTK_IS_WIDGET (wdata->dialog) &&
+#ifdef USE_GTK2_18
+			gtk_widget_get_visible (wdata->dialog))
+#else
+			GTK_WIDGET_VISIBLE (wdata->dialog))
+#endif
 		{
 			progress = pdata.totalsize + bdata->totalsize;
 			g_snprintf (progresstext, sizeof (progresstext), progress_format,
@@ -508,179 +413,104 @@ static void _e2p_mvbar_exec (VPATH *slocal, VPATH *dlocal, gboolean realmove,
 			gtk_progress_bar_set_text (GTK_PROGRESS_BAR (wdata->progbar), progresstext);
 			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (wdata->progbar),
 				(gdouble) progress / tdata->totalsize);
-			//CHECKME some sort of wait here ??
 			gdk_threads_leave ();
 			//update progressive total
 			bdata->totalsize = progress;
 		}
-
-		//cancel the monitor thread
-		if (pthread_cancel (mon_thread_id))
-		{
-			printd (WARN,"Thread cancel error!\n");
-		}
-		//make sure things are finished before we move on
-		pthread_join (action_thread_id, NULL);
-		pthread_join (mon_thread_id, NULL);
-
 		//CHECKME some sort of wait here ??
 		usleep (100000);	//small delay to (sort of) show the completion status ?
 		//need to slow down ...
 		//this approach is empirical, needs to be validated for various systems
 //		if (realmove && bdata->count > 300 && bdata->count%10 == 0)
 //		g_usleep (50000);	//small delay to allow filesystem to complete its business
-#ifdef IS_PAUSABLE
 //		if (!(wdata->flags & E2_BARTASK_PAUSED))
-#endif
-//		gtk_widget_hide (wdata->dialog);
+//			gtk_widget_hide (wdata->dialog);
+		pthread_cleanup_pop (0);
 	}
 	else
 		bdata->totalsize += pdata.totalsize;
 
-	if (a_data.result) //move succeeded
-	{
+	if (results[0]) //move succeeded
+		e2_task_backend_rename (
 #ifdef E2_VFS
-		e2_task_backend_rename (&tempdata, dlocal);
+		&tempdata, dlocal);
 #else
-		e2_task_backend_rename (templocal, dlocal);
+		templocal, dlocal);
 #endif
-		//update the progressive totals
-//		bdata->count += pdata.count;
-/* final display now done earlier
-		bdata->count++;
-		bdata->totalsize += pdata.totalsize;
-		fraction =  (gdouble) bdata->totalsize / tdata->totalsize;
-		//deal with rounding errors
-		if (fraction > 1.0)
-			fraction = 1.0;
-		//and show them
-		g_snprintf (progresstext, sizeof (progresstext), progress_format,
-			bdata->totalsize / 1048576.0, tdata->totalsize / 1048576.0,
-			fraction * 100.0);
-		gdk_threads_enter ();
-		gtk_label_set_text (GTK_LABEL (wdata->label2), progresstext);
-//		gtk_progress_bar_set_text (GTK_PROGRESS_BAR (wdata->progbar), progresstext);
-		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (wdata->progbar), fraction);
-		//CHECKME some sort of wait here ??
-		gdk_threads_leave ();
-*/
-	}
 	else
+		 //cleanup anything part-moved
+		e2_task_backend_delete (
 #ifdef E2_VFS
-		e2_task_backend_delete (&tempdata); //cleanup anything part-moved
+		&tempdata);
 #else
-		e2_task_backend_delete (templocal); //cleanup anything part-moved
+		templocal);
 #endif
 
+//	pthread_cleanup_pop (0);
 	g_free (templocal);
+	//cleanup shared memory
+	if (shmdt (results) == 0)
+		shmctl (shmid, IPC_RMID, NULL);
 
-#ifdef IS_PAUSABLE
-	if (wdata->flags & E2_BARTASK_PAUSEREQ)
-	{	//if it was paused, the dialog must be still visible
-		wdata->flags &= ~E2_BARTASK_PAUSEREQ;
-		wdata->flags |= E2_BARTASK_PAUSED;
-		e2_filelist_enable_refresh ();
-		gdk_threads_enter ();
-		gtk_main ();
-		gdk_threads_leave ();
-	}
-#endif
-	return;
+	return OK;
 }
-#ifdef IS_PAUSABLE
 /**
-@brief callback for stop-button press and window-close event
+@brief "response" signal callback for progress dialog
+@param dialog UNUSED the dialog where the response was generated
+@param response the response returned from the dialog
+@param wdata pointer to data struct for the dialog
 
-@param widget the activated widget which triggered the callback
-@param flag pointer to store for flag which will abort the process when set to TRUE;
-
-@return TRUE always
+@return
 */
-static gboolean
-_e2p_mvbar_break_cb (GtkWidget *widget, E2_BarWindowData *wdata)
+static void
+_e2p_mvbar_response_cb (GtkDialog *dialog, gint response, E2_BarWindowData *wdata)
 {
-	wdata->flags |= E2_BARTASK_STOPPED;
-	//handle paused moving
-	if (wdata->flags & E2_BARTASK_PAUSED)
+	switch (response)
 	{
-		wdata->flags &= ~E2_BARTASK_PAUSED;
-		e2_filelist_disable_refresh ();
-		gtk_main_quit ();
-		//FIXME = cleanups
-		// pthread_cancel ();
+		case E2_RESPONSE_USER1:	//pause
+			if ((wdata->bflags & E2_BARTASK_PAUSED) == 0)
+			{	//not paused already
+				gtk_widget_set_sensitive (wdata->pause_btn, FALSE);
+				gtk_widget_set_sensitive (wdata->resume_btn, TRUE);
+				gtk_widget_grab_focus (wdata->resume_btn);
+				wdata->bflags |= E2_BARTASK_PAUSEREQ;
+				//actual pause is started near end of _e2p_cpbar_exec()
+			}
+			break;
+		case E2_RESPONSE_USER2:	//resume
+			wdata->bflags &= ~E2_BARTASK_PAUSEREQ;	//insurance
+			if (wdata->bflags & E2_BARTASK_PAUSED)
+			{
+				gtk_widget_set_sensitive (wdata->resume_btn, FALSE);
+				gtk_widget_set_sensitive (wdata->pause_btn, TRUE);
+				gtk_widget_grab_focus (wdata->pause_btn);
+				wdata->bflags &= ~E2_BARTASK_PAUSED;
+				e2_filelist_disable_refresh ();
+				e2_main_loop_quit (wdata->loop);
+				wdata->loop = NULL;
+			}
+			break;
+		case E2_RESPONSE_NOTOALL:	//cancel
+		case GTK_RESPONSE_NONE:	//dialog hidden or destroyed
+		case GTK_RESPONSE_DELETE_EVENT:
+			wdata->bflags |= E2_BARTASK_STOPPED;
+			wdata->bflags &= ~E2_BARTASK_PAUSEREQ;	//insurance
+			//handle paused copying
+			if (wdata->bflags & E2_BARTASK_PAUSED)
+			{
+				wdata->bflags &= ~E2_BARTASK_PAUSED;
+				e2_filelist_disable_refresh ();
+				e2_main_loop_quit (wdata->loop);
+				wdata->loop = NULL;
+			}
+		default:
+			break;
 	}
-
-	return TRUE;
 }
 /**
-@brief callback for pause-button press
-
-@param widget the button widget which triggered the callback
-@param resbutton pointer to paired button widget
-
-@return TRUE always
-*/
-static gboolean
-_e2p_mvbar_pause_cb (GtkWidget *widget, E2_BarWindowData *wdata)
-{
-	gtk_widget_set_sensitive (widget, FALSE);
-	gtk_widget_set_sensitive (wdata->resume_btn, TRUE);
-	gtk_widget_grab_focus (wdata->resume_btn);
-	wdata->flags |= E2_BARTASK_PAUSEREQ;
-	//actual pause is started in the monitor loop
-	return TRUE;
-}
-/**
-@brief callback for resume-button press
-
-@param widget the button widget which triggered the callback
-@param resbutton pointer to paired button widget
-
-@return TRUE unless the process is not paused
-*/
-static gboolean
-_e2p_mvbar_resume_cb (GtkWidget *widget, E2_BarWindowData *wdata)
-{
-	if (!(wdata->flags & E2_BARTASK_PAUSED))
-		return FALSE;
-	gtk_widget_set_sensitive (widget, FALSE);
-	gtk_widget_set_sensitive (wdata->pause_btn, TRUE);
-	gtk_widget_grab_focus (wdata->pause_btn);
-	wdata->flags &= ~E2_BARTASK_PAUSED;
-	e2_filelist_disable_refresh ();
-	gtk_main_quit ();
-	return TRUE;
-}
-#else //ndef IS_PAUSABLE
-/**
-@brief callback for stop-button press and window-close event
-
-@param widget the activated widget which triggered the callback
-@param flag pointer to store for flag which will abort the process when set to TRUE;
-
-@return TRUE always
-*/
-static gboolean
-_e2p_mvbar_break_cb (GtkWidget *widget,
-#ifdef IS_PAUSABLE
-	E2_BarFlags *flags
-#else
-	gboolean *flag
-#endif
-	)
-{
-#ifdef IS_PAUSABLE
-	*flags |= E2_BARTASK_STOPPED;
-#else
-	*flag = TRUE;
-#endif
-	return TRUE;
-}
-#endif
-/**
 @brief mvbar action
 This sets up progress display window, and processes each selected item
-
+@art ->data may contain pointerised flags indicating how the copy is to be done
 @param from the button, menu item etc which was activated
 @param art action runtime data
 
@@ -696,7 +526,7 @@ static gboolean
 _e2p_mvbarQ (E2_ActionTaskData *qed)
 {
 	//handle case of specific data instead of selection??
-	if (g_str_equal (qed->currdir, qed->othrdir))
+	if (!strcmp (qed->currdir, qed->othrdir))
 	{
 		//display some message ??
 		return FALSE;
@@ -704,8 +534,7 @@ _e2p_mvbarQ (E2_ActionTaskData *qed)
 	E2_ERR_DECLARE
 	//FIXME change dir permission if possible ?
 #ifdef E2_VFS
-	VPATH sdata;
-	sdata.localpath = qed->currdir;
+	VPATH sdata = { qed->currdir, qed->currspace };
 	VPATH ddata = { qed->othrdir, qed->othrspace };
 	if (e2_fs_access (&ddata, W_OK E2_ERR_PTR()))
 #else
@@ -723,7 +552,7 @@ _e2p_mvbarQ (E2_ActionTaskData *qed)
 	}
 
 	GPtrArray *names = qed->names;
-	GtkWidget *vbox;	//, *hbox;
+	GtkWidget *dialog_vbox;	//, *hbox;
 	GString *src = g_string_sized_new (1024);
 	GString *dest = g_string_sized_new (1024);
 
@@ -761,54 +590,55 @@ _e2p_mvbarQ (E2_ActionTaskData *qed)
 
 	//setup the information window
 	E2_BarWindowData windowdata;
-#ifdef IS_PAUSABLE
-	windowdata.flags = 0;
-#else
-	gboolean breakflag = FALSE;	//this is for aborting the process
-#endif
-	windowdata.dialog = e2_dialog_create (NULL, NULL, _("moving"), NULL, NULL);
+	windowdata.bflags = 0;
+	windowdata.loop = NULL;	//no pause yet
+	windowdata.dialog = NULL;	//setup for abort
+	pthread_cleanup_push ((gpointer)_e2p_mvbar_abort_clean_dialog, &windowdata.dialog);
+	windowdata.dialog = e2_dialog_create (NULL, NULL, _("moving"),
+		(ResponseFunc)_e2p_mvbar_response_cb, &windowdata);
+
+	gdk_threads_enter ();
 	e2_dialog_setup (windowdata.dialog, app.main_window);
-//CHECKME permit Esc-key aborting ?
-#ifdef IS_PAUSABLE
-	g_signal_connect (G_OBJECT (windowdata.dialog), "delete-event",
-		G_CALLBACK (_e2p_mvbar_break_cb), &windowdata);
-#else
-	g_signal_connect (G_OBJECT (windowdata.dialog), "delete-event",
-		G_CALLBACK (_e2p_mvbar_break_cb), &breakflag);
-#endif
+	gdk_threads_leave ();
+
 	gtk_dialog_set_has_separator (GTK_DIALOG (windowdata.dialog), FALSE);
 
-	vbox = GTK_DIALOG (windowdata.dialog)->vbox;
-	windowdata.label = e2_widget_add_mid_label (vbox, "", 0.0, FALSE, 0);
+	dialog_vbox =
+#ifdef USE_GTK2_14
+		gtk_dialog_get_content_area (GTK_DIALOG (windowdata.dialog));
+#else
+		GTK_DIALOG (windowdata.dialog)->vbox;
+#endif
+	windowdata.label = e2_widget_add_mid_label (dialog_vbox, "", 0.0, FALSE, 0);
 
-//	hbox = e2_widget_add_box (vbox, FALSE, 0, TRUE, FALSE, 0);
+//	hbox = e2_widget_add_box (dialog_vbox, FALSE, 0, TRUE, FALSE, 0);
 //	windowdata.label2 = e2_widget_add_mid_label (hbox, "", 0.5, TRUE, 0);
 
 	windowdata.progbar = gtk_progress_bar_new ();
-	gtk_box_pack_start (GTK_BOX (vbox), windowdata.progbar, TRUE, TRUE, E2_PADDING_LARGE);
-	gtk_widget_show (windowdata.progbar);
-#ifdef IS_PAUSABLE
-	//buttons in reverse order
-	windowdata.resume_btn =
-	e2_dialog_add_undefined_button_custom (windowdata.dialog, FALSE,
-		E2_RESPONSE_USER1,_("_Resume"), GTK_STOCK_MEDIA_PLAY,
-		_("Resume moving after pause"), _e2p_mvbar_resume_cb, &windowdata);
+	gtk_box_pack_start (GTK_BOX (dialog_vbox), windowdata.progbar, TRUE, TRUE, E2_PADDING_LARGE);
+
+	//buttons
+	E2_Button local_btn = { _("_Pause"), GTK_STOCK_MEDIA_PAUSE,
+		//_("Suspend moving"), E2_BTN_TIPPED, E2_BTN_TIPPED,
+		NULL, 0, 0,
+		E2_RESPONSE_USER1 };
+	windowdata.pause_btn = e2_dialog_add_defined_button (windowdata.dialog, &local_btn);
+
+	local_btn.label = _("_Resume");
+	local_btn.stock = GTK_STOCK_MEDIA_PLAY;
+	//local_btn.tip = _("Resume moving after pause");
+	local_btn.response = E2_RESPONSE_USER2;
+	windowdata.resume_btn = e2_dialog_add_defined_button (windowdata.dialog, &local_btn);
 	//this one is disabled for now
 	gtk_widget_set_sensitive (windowdata.resume_btn, FALSE);
-	windowdata.pause_btn =
-	e2_dialog_add_undefined_button_custom (windowdata.dialog, FALSE,
-		E2_RESPONSE_USER2,_("_Pause"), GTK_STOCK_MEDIA_PAUSE,
-		_("Suspend moving, after the current item"), _e2p_mvbar_pause_cb,
-		&windowdata);
-	windowdata.stop_btn =
-	e2_dialog_add_undefined_button_custom (windowdata.dialog, TRUE,
-		E2_RESPONSE_NOTOALL,_("_Stop"), GTK_STOCK_STOP,
-		_("Abort the moving"), _e2p_mvbar_break_cb, &windowdata);
-#else
-	e2_dialog_add_undefined_button_custom (windowdata.dialog, TRUE,
-		E2_RESPONSE_NOTOALL,_("_Stop"), GTK_STOCK_STOP,
-		_("Abort the moving"), _e2p_mvbar_break_cb, &breakflag);
-#endif
+
+	local_btn = E2_BUTTON_CANCEL;
+	local_btn.showflags |= E2_BTN_DEFAULT;
+	windowdata.stop_btn = e2_dialog_add_defined_button (windowdata.dialog, &local_btn);
+	//show everything but the dialog itself
+	gtk_widget_show_all (dialog_vbox);
+
+	e2_dialog_set_negative_response (windowdata.dialog, E2_BUTTON_CANCEL.response);
 
 	//accumulate non-recursed count and total size of src item(s)
 	E2_BarData totaldata = { 0,0 };
@@ -818,15 +648,16 @@ _e2p_mvbarQ (E2_ActionTaskData *qed)
 	{
 		g_string_printf (src, "%s%s", qed->currdir, (*iterator)->filename);  //separator comes with dir
 #ifdef E2_VFS
-		sdata.localpath = src->str;
+		sdata.path = src->str;
 #endif
 		//if (!
+			e2_fs_tw (
 #ifdef E2_VFS
-		e2_fs_tw (&sdata, _e2p_mvbar_twcb, &totaldata, -1,
+			&sdata,
 #else
-		e2_fs_tw (src->str, _e2p_mvbar_twcb, &totaldata, -1,
+			src->str,
 #endif
-			E2TW_PHYS E2_ERR_NONE());
+			_e2p_mvbar_twcb, &totaldata, -1, E2TW_PHYS E2_ERR_NONE());
 		//)
 		//{
 			//FIXME handle error
@@ -847,47 +678,44 @@ _e2p_mvbarQ (E2_ActionTaskData *qed)
 
 	for (count=0; count < names->len; count++, iterator++)
 	{
-#ifdef IS_PAUSABLE
-		if (windowdata.flags & E2_BARTASK_STOPPED)
-#else
-		if (breakflag)
-#endif
+		if (windowdata.bflags & E2_BARTASK_STOPPED)
 			break;	//user pressed stop btn or closed info window
 		//src_dir, dest_dir have trailing "/"
 		g_string_printf (src, "%s%s", qed->currdir, (*iterator)->filename);  //separator comes with dir
 		g_string_printf (dest, "%s%s", qed->othrdir, (*iterator)->filename);
 
+		DialogButtons result;
 #ifdef E2_VFS
-		sdata.localpath = src->str;
-		ddata.localpath = dest->str;
+		sdata.path = src->str;
+		ddata.path = dest->str;
 		if (check && e2_fs_access2 (&ddata E2_ERR_NONE()) == 0)
 #else
 		if (check && e2_fs_access2 (dest->str E2_ERR_NONE()) == 0)
 #endif
 		{
-			e2_filelist_enable_refresh ();  //allow updates while we wait
-			gdk_threads_enter ();
 			*qed->status = E2_TASK_PAUSED;
-			DialogButtons result = e2_dialog_ow_check (src->str, dest->str, extras);
+			result = e2_dialog_ow_check (
+#ifdef E2_VFS
+				&sdata, &ddata,
+#else
+				src->str, dest->str,
+#endif
+				extras);
 			*qed->status = E2_TASK_RUNNING;
-			gdk_threads_leave ();
-			e2_filelist_disable_refresh ();
 			switch (result)
 			{
 				case YES_TO_ALL:
 					check = FALSE;
 				case OK:
+					result = _e2p_mvbar_exec (
 #ifdef E2_VFS
-					_e2p_mvbar_exec (&sdata, &ddata,
+					&sdata, &ddata,
 #else
-					_e2p_mvbar_exec (src->str, dest->str,
+					src->str, dest->str,
 #endif
-						realmove,
-#ifndef IS_PAUSABLE
-						&breakflag,
-#endif
-						GPOINTER_TO_INT (qed->action->data),
-						&progressdata, &totaldata, &windowdata);
+					realmove,
+					GPOINTER_TO_INT (qed->action->data),
+					&progressdata, &totaldata, &windowdata);
 					break;
 				case CANCEL:
 					break;
@@ -895,31 +723,27 @@ _e2p_mvbarQ (E2_ActionTaskData *qed)
 					result = NO_TO_ALL;
 					break;
 			}
-			if (result == NO_TO_ALL)
-			{
-				break;
-			}
 		}
 		else  //no overwrite, or don't care
 		{
+			result = _e2p_mvbar_exec (
 #ifdef E2_VFS
-			_e2p_mvbar_exec (&sdata, &ddata,
+			&sdata, &ddata,
 #else
-			_e2p_mvbar_exec (src->str, dest->str,
+			src->str, dest->str,
 #endif
-				realmove,
-#ifndef IS_PAUSABLE
-				&breakflag,
-#endif
-				GPOINTER_TO_INT (qed->action->data),
-				&progressdata, &totaldata, &windowdata);
+			realmove,
+			GPOINTER_TO_INT (qed->action->data),
+			&progressdata, &totaldata, &windowdata);
+		}
+		if (result == NO_TO_ALL)
+		{
+			break;
 		}
 		progressdata.count++;
 	}
-	gdk_threads_enter ();
-	gtk_widget_destroy (windowdata.dialog);
-	gdk_threads_leave ();
 
+	pthread_cleanup_pop (1);	//always cleanup any dialog
 	g_string_free (src, TRUE);
 	g_string_free (dest, TRUE);
 #ifdef E2_FAM
@@ -950,15 +774,17 @@ init_plugin (Plugin *p)
 	p->signature = ANAME VERSION;
 	p->menu_name = _("_Move");
 	p->description = _("Move selected item(s), with displayed progress details");
-	p->icon = "plugin_move_"E2IP".png";  //use icon file pathname if appropriate
+	p->icon = "plugin_move"E2ICONTB;  //use icon file pathname if appropriate
 
 	if (p->action == NULL)
 	{
-		//no need to free this
-		gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
-		p->action = e2_plugins_action_register
-		(action_name, E2_ACTION_TYPE_ITEM, _e2p_mvbar, NULL, FALSE, 0, NULL);
-		return TRUE;
+		//don't free name string here
+		E2_Action plugact =
+		{g_strconcat (_A(6),".",aname,NULL),_e2p_mvbar,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+		p->action = e2_plugins_action_register (&plugact);
+		if G_LIKELY((p->action != NULL))
+			return TRUE;
+		g_free (plugact.name);
 	}
 	return FALSE;
 }
@@ -971,10 +797,8 @@ init_plugin (Plugin *p)
 */
 gboolean clean_plugin (Plugin *p)
 {
-  gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
+  gchar *action_name = g_strconcat (_A(6),".",aname,NULL);
   gboolean ret = e2_plugins_action_unregister (action_name);
   g_free (action_name);
   return ret;
 }
-
-#undef IS_PAUSABLE
