@@ -1,6 +1,6 @@
-/* $Id: e2p_names_clip.c 596 2007-08-12 02:44:13Z tpgww $
+/* $Id: e2p_names_clip.c 1835 2009-09-17 00:35:44Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark
 
 This file is part of emelFM2.
@@ -40,17 +40,29 @@ static gboolean _e2p_copy_to_clipboard (gpointer from, E2_ActionRuntime *art)
 {
 	GString *text;
 	gchar *names;
-	GdkModifierType state;
-	if (gtk_get_current_event_state (&state)
-		&& (state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK)))
+	gboolean full =
+			(ACTION_BUTTON (art,1) && ACTION_MASK (art,GDK_SHIFT_MASK))
+		 || (art->data != NULL && strstr ((gchar*)art->data, _("shift")) != NULL);
+	gboolean lined =
+			(ACTION_BUTTON (art,1) && ACTION_MASK (art,GDK_CONTROL_MASK))
+		 || (art->data != NULL && strstr ((gchar*)art->data, _("ctrl")) != NULL);
+	//art was probably packed during context-menu creation - which is too early
+	//for pressing mod-keys
+	if ((!full || !lined) /*&& (GTK_IS_BUTTON (from) || GTK_IS_MENU_ITEM (from))*/)
+	{
+		GdkModifierType state = e2_utils_get_modifiers ();
+		full = full || (state & GDK_SHIFT_MASK);
+		lined = lined || (state & GDK_CONTROL_MASK);
+	}
+
+	if (full || lined)
 	{
 		//get quoted list of selected-item-names, to facilitate their separation
 		names = e2_utils_expand_macros ("%f", NULL);
 		if (names == NULL)
 			return FALSE;
 
-		gboolean full = (state & GDK_SHIFT_MASK);
-		gchar sep = (state & GDK_CONTROL_MASK) ? '\n' : ' ';
+		gchar sep = (lined) ? '\n' : ' ';
 
 		gchar **split = g_strsplit (names, "\"", -1);
 		gchar **tmp = split;
@@ -108,15 +120,17 @@ gboolean init_plugin (Plugin *p)
   p->signature = ANAME VERSION;
   p->menu_name = _("Copy _names");
   p->description = _("Copy path or name of each selected item to the clipboard");
-  p->icon = "plugin_clip_"E2IP".png";  //use icon file pathname if appropriate
+  p->icon = "plugin_clip"E2ICONTB;  //use icon file pathname if appropriate
 
   if (p->action == NULL)
   {
-	//no need to free this
-	gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
-	p->action = e2_plugins_action_register
-	  (action_name, E2_ACTION_TYPE_ITEM, _e2p_copy_to_clipboard, NULL, FALSE, 0, NULL);
-	return TRUE;
+	//don't free name string here
+	E2_Action plugact =
+	{g_strconcat (_A(6),".",aname,NULL),_e2p_copy_to_clipboard,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+	p->action = e2_plugins_action_register (&plugact);
+	if G_LIKELY((p->action != NULL))
+		return TRUE;
+	g_free (plugact.name);
   }
   return FALSE;
 }
@@ -129,7 +143,7 @@ gboolean init_plugin (Plugin *p)
 */
 gboolean clean_plugin (Plugin *p)
 {
-  gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
+  gchar *action_name = g_strconcat (_A(6),".",aname,NULL);
   gboolean ret = e2_plugins_action_unregister (action_name);
   g_free (action_name);
   return ret;
