@@ -1,6 +1,6 @@
-/* $Id: e2p_pack.c 706 2007-11-06 09:13:06Z tpgww $
+/* $Id: e2p_pack.c 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark
 
 This file is part of emelFM2.
@@ -31,8 +31,8 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "e2_command.h"
 #include "e2_task.h"
 #include "e2_filelist.h"
-//same enum as in unpack plugin
-enum { TAR_GZ, TAR_BZ2, TAR, ZIP, Z7Z, RAR, ARJ, ZOO, MAXTYPES };
+//same enum as in unpack plugin, though it need not be so
+enum { TAR_GZ, TAR_BZ2, TAR_LZMA, TAR, /*DEB, RPM, */ZIP, Z7Z, RAR, ARJ, ZOO, MAXTYPES };
 
 typedef struct _E2_PackDlgRunTime
 {
@@ -45,34 +45,50 @@ typedef struct _E2_PackDlgRunTime
 static gint pkg_type = TAR_GZ;	//in theory, we should protect this by mutex
 
 //these extension strings are in same order as enum
-//CHECKME do these really need to be translated ?
 static gchar *ext_str [MAXTYPES] =
 {
-	N_(".tar.gz"),
-	N_(".tar.bz2"),
-	N_(".tar"),
-	N_(".zip"),
-	N_(".7z"),
-	N_(".rar"),
-	N_(".arj"),
-	N_(".zoo")
+	".tar.gz",
+	".tar.bz2",
+	".tar.lzma",
+	".tar",
+//	".deb",
+//	".rpm",
+	".zip",
+	".7z",
+	".rar",
+	".arj",
+	".zoo"
 };
 
 static gboolean _e2p_packQ (E2_ActionTaskData *qed);
 
+/**
+@brief implement user's choice to pack the archive
+BGL on
+*/
 static void _e2p_pack_yes (E2_PackDlgRunTime *rt)
 {
 	static gchar *cmd_str [MAXTYPES] =
 	{	//these command strings are in same order as enum
-		//NOTE that %%f is converted to %f when the command is constructed with g_strdup_printf ()
-		">tar cvf - %%f | gzip - > \"%s\"",	//run in separate shell
-		">tar cvf - %%f | bzip2 - > \"%s\"",	//ditto
-		"tar cvf \"%s\" %%f",
-		"zip -r \"%s\" %%f",
-		"7za a -t7z \"%s\" %%f",
-		"rar a -r -ol \"%s\" %%f",
-		"arj a -a1 -r \"%s\" %%f",
-		"zoo ahP \"%s\" %%f"
+		//NOTE: %%f is converted to %f when the command is constructed with g_strdup_printf ()
+		">tar cvf - %%f | gzip - > %s",	//run in separate shell
+		">tar cvf - %%f | bzip2 - > %s",//ditto default block size -9 is not significantly better, maybe use -3 instead ?
+		">tar cvf - %%f | lzma - > %s", //ditto default compresssion -7, maybe use -2 instead ?
+		"tar cvf %s %%f",
+//		FIXME deb pack command
+//		FIXME rpm "pack" command does not exist. rpmbuild ?
+//		"zip -r %s %%f", adds to any existing archive
+		">zip -r - %%f > %s",
+
+//		"7za a -t7z %s %%f", adds to any existing archive
+		">7za a -t7z -so %%f > %s",	//CHECKME
+
+		"rar a -r -ol -tl %s %%f",
+
+//		"arj a -a1 -r %s %%f",
+		"arj a -a -r -s -2s %s %%f", //CHECKME
+
+		"zoo ahP %s %%f"
 	};
 /*
 compress ANSI files: 7za a -tzip archive.zip file1 file2 ... fileN
@@ -108,7 +124,15 @@ decompress UNICODE: 7za x archive.7z -odirname -aoa
 #endif
 				{	//same-named item exists already
 					//FIXME some apps allow addition to existing archive
-					DialogButtons result = e2_dialog_ow_check (NULL, dlocal, NONE);
+					gdk_threads_leave ();
+					DialogButtons result = e2_dialog_ow_check (NULL,
+#ifdef E2_VFS
+						&ddata,
+#else
+						dlocal,
+#endif
+						NONE);
+					gdk_threads_enter ();
 					if (result != OK)
 					{
 						flag = FALSE;	//signal that we will not proceed
@@ -116,24 +140,27 @@ decompress UNICODE: 7za x archive.7z -odirname -aoa
 					}
 				}
 				g_free (utf);
-				F_FREE (dlocal);
+				F_FREE (dlocal, utf);
 			}
 		}
 
 		if (flag)
 		{
-			gchar *command =  g_strdup_printf (cmd_str [pkg_type], full_name);
+//tag E2_BADQUOTES
+			gchar *qp = e2_utils_quote_string (full_name);
+			gchar *command =  g_strdup_printf (cmd_str [pkg_type], qp);
+			g_free (qp);
 //			e2_filelist_disable_refresh ();	//prevent changes to selected item data
 //			gint res =
+			e2_command_run (command, E2_COMMAND_RANGE_DEFAULT, rt->dialog
 #ifdef E2_COMMANDQ
-			e2_command_run (command, E2_COMMAND_RANGE_DEFAULT, FALSE);
-#else
-			e2_command_run (command, E2_COMMAND_RANGE_DEFAULT);
+			, FALSE
 #endif
+			);
 //			e2_filelist_enable_refresh (); //probably nothing reported yet, to trigger a refresh
 //			flag = (res == 0);
-			g_free (command);
 			g_free (full_name);
+			g_free (command);
 		}
 	}
 	else
@@ -153,7 +180,7 @@ static void _e2p_pack_response_cb (GtkDialog *dialog, gint response,
 {
 	switch (response)
 	{
-		case GTK_RESPONSE_OK:
+		case E2_RESPONSE_APPLY:
 			gtk_widget_hide (rt->dialog);
 			_e2p_pack_yes (rt);
 			break;
@@ -173,7 +200,7 @@ static void _e2p_pack_response_cb (GtkDialog *dialog, gint response,
 */
 static void _e2p_pack_activated_cb (GtkEntry *entry, E2_PackDlgRunTime *rt)
 {
-	_e2p_pack_response_cb (GTK_DIALOG (rt->dialog), GTK_RESPONSE_OK, rt);
+	_e2p_pack_response_cb (GTK_DIALOG (rt->dialog), E2_RESPONSE_APPLY, rt);
 }
 /**
 @brief create and run dialog
@@ -192,47 +219,37 @@ static gboolean _e2p_packQ (E2_ActionTaskData *qed)
 	E2_PackDlgRunTime *rt = ALLOCATE (E2_PackDlgRunTime);
 	CHECKALLOCATEDWARNT (rt, return FALSE;)
 
-	rt->dialog = e2_dialog_create (NULL, _("Filename:"),
-		_("archive creation"), _e2p_pack_response_cb, rt);
+	rt->dialog = e2_dialog_create (NULL, _("Filename:"), _("archive creation"),
+		(ResponseFunc)_e2p_pack_response_cb, rt);
 
 	GtkWidget *hbox = g_object_get_data (G_OBJECT (rt->dialog), "e2-dialog-hbox");
 
 	GPtrArray *names = qed->names;
 	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
 	gchar *suggested_name = F_FILENAME_FROM_LOCALE ((*iterator)->filename);
+
 	rt->filename_entry = e2_widget_add_entry (hbox, suggested_name, TRUE, TRUE);
-	F_FREE (suggested_name);
+	F_FREE (suggested_name, (*iterator)->filename);
 #ifdef E2_ASSISTED
 	GtkWidget *label = (GtkWidget *) g_object_get_data (G_OBJECT (rt->dialog),
 		"e2-dialog-label");
-	e2_widget_set_label_relations (GTK_LABEL (label), rt->filename_entry);
+	e2_widget_set_label_relations (label, rt->filename_entry);
 #endif
-
-	rt->pkgtype_combo = e2_combobox_add (hbox, TRUE, 0, NULL, NULL, NULL,
-		E2_COMBOBOX_MENU_STYLE);
 	g_signal_connect (G_OBJECT (rt->filename_entry), "activate",
 		G_CALLBACK (_e2p_pack_activated_cb), rt);
 
-	//these package name strings are in same order as enum
-	//CHECKME do these really need to be translated ?
-/*	gchar *type_choices [MAXTYPES] = { _(".tar.gz"),  //index 0
-		_(".tar.bz2"), _(".tar"), _(".zip"), _(".7z"), _(".rar"),
-		_(".arj") _(".zoo")};  //index 6
-	e2_combobox_append_history_counted (rt->pkgtype_combo, MAXTYPES, type_choices);
-*/
-	guint i;
-	for (i = 0; i < MAXTYPES; i++)
-		gtk_combo_box_append_text (GTK_COMBO_BOX (rt->pkgtype_combo),
-			gettext (ext_str[i]));
+	rt->pkgtype_combo = e2_combobox_add (hbox, FALSE, 0, NULL, NULL, NULL,
+		E2_COMBOBOX_MENU_STYLE);
+	//these applied package-extension strings are in same order as enum
+	e2_combobox_append_history_counted (rt->pkgtype_combo, MAXTYPES, ext_str);
 
 	gtk_combo_box_set_active (GTK_COMBO_BOX (rt->pkgtype_combo), pkg_type);
 
 	rt->curr_dir = D_FILENAME_FROM_LOCALE (qed->currdir);
 
-	gdk_threads_enter ();
-	e2_dialog_show (rt->dialog, app.main_window, 0, &E2_BUTTON_CANCEL,
-		&E2_BUTTON_OK, NULL);
-	gdk_threads_leave ();
+	e2_dialog_show (rt->dialog, app.main_window, E2_DIALOG_CLOSELOCK,
+		&E2_BUTTON_CANCEL, &E2_BUTTON_APPLY, NULL);
+
 	return TRUE;
 }
 
@@ -253,15 +270,17 @@ gboolean init_plugin (Plugin *p)
 	p->signature = ANAME VERSION;
 	p->menu_name = _("_Pack..");
 	p->description = _("Build an archive containing the selected item(s)");
-	p->icon = "plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate
+	p->icon = "plugin_"ANAME E2ICONTB;  //use icon file pathname if appropriate
 
 	if (p->action == NULL)
 	{
-		//no need to free this
-		gchar *action_name = g_strconcat (_A(5),".", aname, NULL);
-		p->action = e2_plugins_action_register
-			(action_name, E2_ACTION_TYPE_ITEM, _e2p_pack, NULL, FALSE, 0, NULL);
-		return TRUE;
+		//don't free name string here
+		E2_Action plugact =
+		{g_strconcat (_A(6),".", aname, NULL),_e2p_pack,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+		p->action = e2_plugins_action_register (&plugact);
+		if G_LIKELY((p->action != NULL))
+			return TRUE;
+		g_free (plugact.name);
 	}
 	return FALSE;
 }
@@ -274,7 +293,7 @@ gboolean init_plugin (Plugin *p)
 */
 gboolean clean_plugin (Plugin *p)
 {
-	gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
+	gchar *action_name = g_strconcat (_A(6),".",aname,NULL);
 	gboolean ret = e2_plugins_action_unregister (action_name);
 	g_free (action_name);
 	return ret;
