@@ -1,6 +1,6 @@
-/* $Id: e2p_times.c 700 2007-11-05 08:55:39Z tpgww $
+/* $Id: e2p_times.c 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2006-2007 tooar <tooar@gmx.net>
+Copyright (C) 2006-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -193,10 +193,10 @@ static E2_TwResult _e2_task_twcb_touch (VPATH *localpath,
 				dirfix = member->data;
 				if (dirfix != NULL)
 				{
-					if (g_str_equal (dirfix->path, localpath))
+					if (!strcmp (dirfix->path, localpath))
 					{
 #ifdef E2_VFS
-						ddata.localpath = dirfix->path;
+						ddata.path = dirfix->path;
 						if (!_e2pt_touch1 (&ddata, statptr, user_data))
 #else
 						if (!_e2pt_touch1 (dirfix->path, statptr, user_data))
@@ -342,7 +342,7 @@ static gboolean _e2p_touch (VPATH *localpath, E2_TouchData *data, gboolean recur
 				{
 					E2P_DirEnt *dirfix = member->data;
 #ifdef E2_VFS
-					ddata.localpath = dirfix->path;
+					ddata.path = dirfix->path;
 					if (e2_fs_lstat (&ddata, &statbuf E2_ERR_NONE())  //no link pass-thru
 						|| !_e2pt_touch1 (&ddata, &statbuf, data))
 #else
@@ -404,8 +404,20 @@ static gboolean _e2p_times_parse_time (GtkWidget *current, GtkWidget *date_combo
 	struct tm tm;
 	//clear result structure
 	memset (&tm, '\0', sizeof (tm));
-	const gchar *date = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (date_combo)->child));
-	const gchar *time = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (time_combo)->child));
+	const gchar *date = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+		GTK_ENTRY (gtk_bin_get_child (GTK_BIN (date_combo)))
+#else
+		GTK_ENTRY (GTK_BIN (date_combo)->child)
+#endif
+	);
+	const gchar *time = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+		GTK_ENTRY (gtk_bin_get_child (GTK_BIN (time_combo)))
+#else
+		GTK_ENTRY (GTK_BIN (time_combo)->child)
+#endif
+	);
 	const gchar *currdate = gtk_label_get_text (GTK_LABEL (current));
 	const gchar *currtime = strchr (currdate, ' ') + 1;	//ascii scanning for space char
 	const gchar *cp;
@@ -413,14 +425,14 @@ static gboolean _e2p_times_parse_time (GtkWidget *current, GtkWidget *date_combo
 	if (*date != '\0')
 	{
 		if (*time != '\0')
-			newvalue = g_strdup_printf ("%s %s", date, time);
+			newvalue = g_strconcat (date, " ", time, NULL);
 		else
-			newvalue = g_strdup_printf ("%s %s", date, currtime);
+			newvalue = g_strconcat (date, " ", currtime, NULL);
 	}
 	else if (*time != '\0')
 	{
 		gchar *freeme = g_strndup (currdate, currtime - currdate - 1);
-		newvalue = g_strdup_printf ("%s %s", freeme, time);
+		newvalue = g_strconcat (freeme, " ", time, NULL);
 		g_free (freeme);
 	}
 	else
@@ -446,7 +458,7 @@ static gboolean _e2p_times_parse_time (GtkWidget *current, GtkWidget *date_combo
 	return (*newdt != (time_t) -1);
 }
 /**
-@brief handle ok or apply-to-all button press
+@brief handle apply or apply-to-all button press
 Converts all entered date-time strings to calendar values
 If there's a problem, it may set a flag to produce a cancel
 response for the dialog as a whole
@@ -456,7 +468,7 @@ response for the dialog as a whole
 
 @return
 */
-static void _e2p_times_ok_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
+static void _e2p_times_apply_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 {
 	if (rt->permission)
 	{
@@ -465,7 +477,13 @@ static void _e2p_times_ok_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 //		GtkWidget *entry;
 		const gchar *entrytext;
 		//store any valid new dates/times
-		if (GTK_TOGGLE_BUTTON (rt->set_mtime_button)->active)
+		if (
+#ifdef USE_GTK2_14
+			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->set_mtime_button))
+#else
+			GTK_TOGGLE_BUTTON (rt->set_mtime_button)->active
+#endif
+		)
 		{
 			success = _e2p_times_parse_time (rt->mcurrent, rt->mdate_combo,
 				rt->mtime_combo, &rt->data->mtime);
@@ -473,19 +491,41 @@ static void _e2p_times_ok_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 			{
 /*as dialog is destroyed each iteration, no point in deferring backup until later
 				//update combo history
-				entry = GTK_BIN (rt->mdate_combo)->child;
+				entry =
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (rt->mdate_combo));
+#else
+				GTK_BIN (rt->mdate_combo)->child;
+#endif
 				entrytext = gtk_entry_get_text (GTK_ENTRY (entry));
 				if (*entrytext != '\0')
 					e2_combobox_activated_cb (entry, NULL);
-				entry = GTK_BIN (rt->mtime_combo)->child;
+				entry =
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (rt->mtime_combo));
+#else
+				GTK_BIN (rt->mtime_combo)->child;
+#endif
 				entrytext = gtk_entry_get_text (GTK_ENTRY (entry));
 				if (*entrytext != '\0')
 					e2_combobox_activated_cb (entry, NULL);
 */
-				entrytext = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->mdate_combo)->child));
+				entrytext = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+					GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->mdate_combo)))
+#else
+					GTK_ENTRY (GTK_BIN (rt->mdate_combo)->child)
+#endif
+				);
 				if (*entrytext != '\0')
 					e2_list_update_history (entrytext, &mdate_history, NULL, 0, FALSE);
-				entrytext = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->mtime_combo)->child));
+				entrytext = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+					GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->mtime_combo)))
+#else
+					GTK_ENTRY (GTK_BIN (rt->mtime_combo)->child)
+#endif
+				);
 				if (*entrytext != '\0')
 					e2_list_update_history (entrytext, &mtime_history, NULL, 0, FALSE);
 			}
@@ -495,26 +535,54 @@ static void _e2p_times_ok_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 		else
 			rt->data->mtime = (time_t) -1;
 
-		if (success && GTK_TOGGLE_BUTTON (rt->set_atime_button)->active)
+		if (success &&
+#ifdef USE_GTK2_14
+			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->set_atime_button))
+#else
+			GTK_TOGGLE_BUTTON (rt->set_atime_button)->active
+#endif
+		)
 		{
 			success = _e2p_times_parse_time (rt->acurrent, rt->adate_combo,
 				rt->atime_combo, &rt->data->atime);
 			if (success)
 			{
 /*				//update combo history
-				entry = GTK_BIN (rt->adate_combo)->child;
+				entry =
+#ifdef USE_GTK2_14
+					gtk_bin_get_child (GTK_BIN (rt->adate_combo));
+#else
+					GTK_BIN (rt->adate_combo)->child;
+#endif
 				entrytext = gtk_entry_get_text (GTK_ENTRY (entry));
 				if (*entrytext != '\0')
 					e2_combobox_activated_cb (entry, NULL);
-				entry = GTK_BIN (rt->atime_combo)->child;
+				entry =
+#ifdef USE_GTK2_14
+					gtk_bin_get_child (GTK_BIN (rt->atime_combo));
+#else
+					GTK_BIN (rt->atime_combo)->child;
+#endif
 				entrytext = gtk_entry_get_text (GTK_ENTRY (entry));
 				if (*entrytext != '\0')
 					e2_combobox_activated_cb (entry, NULL);
 */
-				entrytext = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->adate_combo)->child));
+				entrytext = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+					GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->adate_combo)))
+#else
+					GTK_ENTRY (GTK_BIN (rt->adate_combo)->child)
+#endif
+				);
 				if (*entrytext != '\0')
 					e2_list_update_history (entrytext, &adate_history, NULL, 0, FALSE);
-				entrytext = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->atime_combo)->child));
+				entrytext = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+					GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->atime_combo)))
+#else
+					GTK_ENTRY (GTK_BIN (rt->atime_combo)->child)
+#endif
+				);
 				if (*entrytext != '\0')
 					e2_list_update_history (entrytext, &atime_history, NULL, 0, FALSE);
 			}
@@ -524,7 +592,13 @@ static void _e2p_times_ok_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 		else
 			rt->data->atime = (time_t) -1;
 
-		if (success && GTK_TOGGLE_BUTTON (rt->set_ctime_button)->active)
+		if (success &&
+#ifdef USE_GTK2_14
+			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->set_ctime_button))
+#else
+			GTK_TOGGLE_BUTTON (rt->set_ctime_button)->active
+#endif
+		)
 		{
 			success = _e2p_times_parse_time (rt->ccurrent, rt->cdate_combo,
 				rt->ctime_combo, &rt->data->ctime);
@@ -533,8 +607,8 @@ static void _e2p_times_ok_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 				gchar *prompt =
 				_("Changing 'ctime' requires temporary changes to the system clock."
 				  " That is normally unwise, as typically, other things rely on"
-				  " system time. Click 'ok' to proceed.");
-				DialogButtons choice = e2_dialog_warning (prompt);
+				  " system time.");
+				DialogButtons choice = e2_dialog_warning (prompt, NULL);
 				if (choice != OK)
 				{
 					success = FALSE;
@@ -547,19 +621,41 @@ static void _e2p_times_ok_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 			if (success)
 			{
 /*				//update combo history
-				entry = GTK_BIN (rt->cdate_combo)->child;
+				entry =
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (rt->cdate_combo));
+#else
+				GTK_BIN (rt->cdate_combo)->child;
+#endif
 				entrytext = gtk_entry_get_text (GTK_ENTRY (entry));
 				if (*entrytext != '\0')
 					e2_combobox_activated_cb (entry, NULL);
-				entry = GTK_BIN (rt->ctime_combo)->child;
+				entry =
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (rt->ctime_combo));
+#else
+				GTK_BIN (rt->ctime_combo)->child;
+#endif
 				entrytext = gtk_entry_get_text (GTK_ENTRY (entry));
 				if (*entrytext != '\0')
 					e2_combobox_activated_cb (entry, NULL);
 */
-				entrytext = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->cdate_combo)->child));
+				entrytext = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+					GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->cdate_combo)))
+#else
+					GTK_ENTRY (GTK_BIN (rt->cdate_combo)->child)
+#endif
+				);
 				if (*entrytext != '\0')
 					e2_list_update_history (entrytext, &cdate_history, NULL, 0, FALSE);
-				entrytext = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->ctime_combo)->child));
+				entrytext = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+					GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->ctime_combo)))
+#else
+					GTK_ENTRY (GTK_BIN (rt->ctime_combo)->child)
+#endif
+				);
 				if (*entrytext != '\0')
 					e2_list_update_history (entrytext, &ctime_history, NULL, 0, FALSE);
 			}
@@ -577,7 +673,12 @@ static void _e2p_times_ok_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 			{
 				rt->real_choice = OK; //the entered choice stands confirmed
 				if (rt->recurse_button != NULL)
-					*rt->recurse = GTK_TOGGLE_BUTTON (rt->recurse_button)->active;
+					*rt->recurse =
+#ifdef USE_GTK2_14
+						gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->recurse_button));
+#else
+						GTK_TOGGLE_BUTTON (rt->recurse_button)->active;
+#endif
 			}
 //			return;
 		}
@@ -592,44 +693,31 @@ static void _e2p_times_ok_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 	}
 }
 /**
-@brief dialog response callback - make response available to main loop
-In general, the response of the clicked button is made available.
-However if the ok/yesall cb generates an error, the return code
-set there will be used instead
-
-@param dialog the dialog from which the response was generated
-@param response the response returned by the dialog
-@param dialog_result pointer to store for @a response or error cancel
-
-@return
-*/
-/*static void _e2p_times_response_cb (GtkDialog *dialog, gint response,
-	DialogButtons *dialog_result)
-{
-//	printd (DEBUG, "times dialog response cb, response is %d", response);
-	e2_dialog_response_decode_cb (dialog, response, dialog_result);
-}
-*/
-/**
 @brief create comboboxentry widget for the dialog
 @param history history list for the widget (static for the session)
 @return the widget
 */
 static GtkWidget *_e2p_times_create_combo (GList *history)
 {
-	GtkWidget *combo = e2_combobox_get (NULL, NULL, &history,
+	GtkWidget *combo = e2_combobox_get ((ActivateFunc)NULL, NULL, &history,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_NO_AUTO_HISTORY |
 		E2_COMBOBOX_FOCUS_ON_CHANGE | E2_COMBOBOX_CYCLE_HISTORY);
 	//show the last-used value, if any
 	if (e2_combobox_has_history (GTK_COMBO_BOX (combo)))
 		e2_combobox_set_active (combo, 0);
-//	gtk_entry_set_activates_default (GTK_ENTRY (GTK_BIN (combo)->child), TRUE);
+//	gtk_entry_set_activates_default (GTK_ENTRY
+//#ifdef USE_GTK2_14
+//	(gtk_bin_get_child (GTK_BIN (combo))),
+//#else
+//	(GTK_BIN (combo)->child),
+//#endif
+//	TRUE);
 //	gtk_widget_set_size_request (combo,  40, -1);
 	return combo;
 }
 /**
 @brief create and run time-change dialog
-
+This expects BGL off.
 @param local_dir path of dir containing item to process, localised string with trailer
 @param spacedata pointer to space data for active pane
 @param info pointer to selection data struct for the item being processed
@@ -686,10 +774,13 @@ static DialogButtons _e2p_times_dialog_run (gchar *local_dir,
 	rt.data = data;
 	rt.recurse = recurse_ret;
 
-	times_dialog = e2_dialog_create (NULL, NULL, _("times"),
-//		_e2p_times_response_cb, &choice);
-		e2_dialog_response_decode_cb, &choice);
-	dialog_vbox = GTK_DIALOG (times_dialog)->vbox;
+	times_dialog = e2_dialog_create (NULL, NULL, _("times"), DUMMY_RESPONSE_CB, NULL);
+	dialog_vbox =
+#ifdef USE_GTK2_14
+		gtk_dialog_get_content_area (GTK_DIALOG (times_dialog));
+#else
+		GTK_DIALOG (times_dialog)->vbox;
+#endif
 	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), E2_PADDING);
 
 	hbox = gtk_hbox_new (FALSE, 0);
@@ -698,13 +789,11 @@ static DialogButtons _e2p_times_dialog_run (gchar *local_dir,
 	gchar *utf = F_DISPLAYNAME_FROM_LOCALE (info->filename);
 	gchar *label_text = g_strdup_printf ("%s: <b>%s</b>", labeltype, utf);
 	e2_widget_add_mid_label (hbox, label_text, 0, TRUE, E2_PADDING); //L, R padding
-	gtk_widget_show (hbox);
-	F_FREE (utf);
+	F_FREE (utf, info->filename);
 	g_free (label_text);
 
 	sub_vbox = gtk_vbox_new (FALSE, E2_PADDING);
 	gtk_box_pack_start (GTK_BOX (dialog_vbox), sub_vbox, TRUE, TRUE, E2_PADDING_LARGE);
-	gtk_widget_show (sub_vbox);
 
 	gint rows = (thisis_dir) ? 5 : 4;
 	table = e2_widget_add_table (sub_vbox, rows, 5, FALSE, TRUE, E2_PADDING);  //4 rows, 5 cols, !homogen, expand
@@ -773,7 +862,7 @@ static DialogButtons _e2p_times_dialog_run (gchar *local_dir,
 		gtk_widget_set_sensitive (rt.set_ctime_button, FALSE);
 		gtk_widget_set_sensitive (rt.recurse_button, FALSE);
 
-		e2_ownership_dialog_warn (dialog_vbox);
+		e2_dialog_setup_auth (dialog_vbox);
 	}
 	else if (getuid () != 0)
 	{
@@ -782,34 +871,42 @@ static DialogButtons _e2p_times_dialog_run (gchar *local_dir,
 		gtk_widget_set_sensitive (rt.cdate_combo, FALSE);
 		gtk_widget_set_sensitive (rt.set_ctime_button, FALSE);
 	}
+
+	//block until user responds
+	E2_DialogFlags flags = E2_DIALOG_BLOCKED;
+
 	// add buttons in the order that they will appear
 	GtkWidget *btn;
+	E2_Button no_btn;
 	if (multi)
 	{
 		e2_dialog_set_negative_response (times_dialog, E2_RESPONSE_NOTOALL);
-		e2_dialog_add_defined_button (times_dialog, &E2_BUTTON_NOTOALL);
-		btn = e2_dialog_add_button_custom (times_dialog, FALSE,
-				&E2_BUTTON_APPLYTOALL, NULL, _e2p_times_ok_cb, &rt);
+		e2_dialog_add_defined_button (times_dialog, &E2_BUTTON_CANCEL);
+		btn = e2_dialog_add_custom_button (times_dialog, &E2_BUTTON_APPLYTOALL,
+			FALSE, E2_BUTTON_APPLYTOALL.tip, _e2p_times_apply_cb, &rt);
 		if (!rt.permission)
 			gtk_widget_set_sensitive (btn, FALSE);
+		e2_button_derive (&no_btn, &E2_BUTTON_NO, BTN_NO_SKIP);
+		flags |= E2_DIALOG_MULTI;
 	}
-	if (rt.permission || multi)
+	else
 	{
-		btn = e2_dialog_add_defined_button (times_dialog, &E2_BUTTON_CANCEL);
-		if (!rt.permission)
-			gtk_widget_set_sensitive (btn, FALSE);
+		e2_dialog_set_negative_response (times_dialog, GTK_RESPONSE_NO);
+		e2_button_derive (&no_btn, &E2_BUTTON_NO, BTN_NO_CANCEL);
 	}
 
-	e2_dialog_add_button_custom (times_dialog, TRUE, &E2_BUTTON_OK,
-	NULL, _e2p_times_ok_cb, &rt);
-
-	e2_dialog_setup (times_dialog, app.main_window);
+	e2_dialog_add_defined_button (times_dialog, &no_btn);
+	btn = e2_dialog_add_custom_button (times_dialog, &E2_BUTTON_APPLY, TRUE,
+		NULL, _e2p_times_apply_cb, &rt);
+	if (!rt.permission)
+		gtk_widget_set_sensitive (btn, FALSE);
 
 	gdk_threads_enter ();
+
+	e2_dialog_setup (times_dialog, app.main_window);
 	do
 	{
-		e2_dialog_run (times_dialog, NULL, 0);
-		gtk_main ();
+		choice = e2_dialog_run (times_dialog, NULL, flags);
 	} while (rt.real_choice == NO); //after date/time parse error, keep trying
 /*as dialog is destroyed each iteration, no gain in deferring this from the
   response callback
@@ -878,7 +975,7 @@ static gboolean _e2p_task_timesQ (E2_ActionTaskData *qed)
 		//".." entries filtered when names compiled
 		g_string_printf (path, "%s%s", curr_local, (*iterator)->filename); //separator comes with dir
 #ifdef E2_VFS
-		sdata.localpath = path->str;
+		sdata.path = path->str;
 #endif
 		if (all)
 		{
@@ -892,7 +989,7 @@ static gboolean _e2p_task_timesQ (E2_ActionTaskData *qed)
 			if (!permission)
 			{
 #ifdef E2_VFS
-				sdata.localpath = (*iterator)->filename;
+				sdata.path = (*iterator)->filename;
 #endif
 				e2_fs_error_simple (
 					_("You do not have authority to change time(s) for %s"),
@@ -957,7 +1054,7 @@ static gboolean _e2p_task_timesQ (E2_ActionTaskData *qed)
 #  ifdef E2_VFS
 			if (_e2p_touch (&sdata, &data, recurse))
 			{
-				sdata.localpath = curr_local;
+				sdata.path = curr_local;
 				//make the file-list refresher notice successful change
 				e2_fs_utime (&sdata, NULL E2_ERR_NONE());
 			}
@@ -978,7 +1075,6 @@ static gboolean _e2p_task_timesQ (E2_ActionTaskData *qed)
 			break;
 	}
 	g_string_free (path, TRUE);
-	F_FREE (curr_local);
 
 	//FIXME E2_INCLIST - just do other view if it's same
 	e2_window_clear_status_message ();
@@ -1006,15 +1102,17 @@ gboolean init_plugin (Plugin *p)
 	p->signature = ANAME VERSION;
 	p->menu_name = _("Change _times..");
 	p->description = _("Change any of the time properties of selected items");
-	p->icon = "plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate
+	p->icon = "plugin_timeset"E2ICONTB;  //use icon file pathname if appropriate
 
 	if (p->action == NULL)
 	{
-		//no need to free this
-		gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
-		p->action = e2_plugins_action_register (action_name, E2_ACTION_TYPE_ITEM,
-			_e2p_task_times, NULL, FALSE, 0, NULL);
-		return TRUE;
+		//don't free name string here
+		E2_Action plugact =
+		{g_strconcat (_A(6),".",aname,NULL),_e2p_task_times,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+		p->action = e2_plugins_action_register (&plugact);
+		if G_LIKELY((p->action != NULL))
+			return TRUE;
+		g_free (plugact.name);
 	}
 	return FALSE;
 }
@@ -1027,7 +1125,7 @@ gboolean init_plugin (Plugin *p)
 */
 gboolean clean_plugin (Plugin *p)
 {
-	gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
+	gchar *action_name = g_strconcat (_A(6),".",aname,NULL);
 	gboolean ret = e2_plugins_action_unregister (action_name);
 	g_free (action_name);
 	if (ret)
