@@ -1,6 +1,6 @@
-/* $Id: e2p_view.c 598 2007-08-12 21:41:11Z tpgww $
+/* $Id: e2p_view.c 1835 2009-09-17 00:35:44Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark
 
 This file is part of emelFM2.
@@ -82,17 +82,19 @@ gboolean init_plugin (Plugin *p)
   p->signature = ANAME VERSION;
   p->menu_name = _("_View");
   p->description = g_strdup_printf (_("Open the first selected item with the %s text-file viewer"), PROGNAME);
-//  p->icon = "plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate
+//  p->icon = "plugin_"ANAME E2ICONTB;  //use icon file pathname if appropriate
 //  p->icon = "";  add icon file pathname if appropriate
   p->cleanflags = E2P_CLEANTIP;
 
   if (p->action == NULL)
   {
-	//no need to free this
-	gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
-	p->action = e2_plugins_action_register
-	  (action_name, E2_ACTION_TYPE_ITEM, _e2p_view, NULL, FALSE, 0, NULL);
-	return TRUE;
+	//don't free name string here
+	E2_Action plugact =
+	{g_strconcat (_A(6),".",aname,NULL),_e2p_view,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+	p->action = e2_plugins_action_register (&plugact);
+	if G_LIKELY((p->action != NULL))
+		return TRUE;
+	g_free (plugact.name);
   }
   return FALSE;
 }
@@ -105,7 +107,7 @@ gboolean init_plugin (Plugin *p)
 */
 gboolean clean_plugin (Plugin *p)
 {
-  gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
+  gchar *action_name = g_strconcat (_A(6),".",aname,NULL);
   gboolean ret = e2_plugins_action_unregister (action_name);
   g_free (action_name);
   return ret;
