@@ -1,6 +1,6 @@
-/* $Id: e2p_track.c 682 2007-10-28 05:33:18Z tpgww $
+/* $Id: e2p_track.c 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2006-2007 tooar <tooar@gmx.net>
+Copyright (C) 2006-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -161,29 +161,41 @@ static void _e2p_track_choose_response_cb (GtkDialog *dialog, gint response,
 		if (local != NULL)
 		{	//no conversion error
 			gchar *utf = F_FILENAME_FROM_LOCALE (local);
-			gtk_entry_set_text (GTK_ENTRY (GTK_BIN (rt->query_combo)->child), utf);
+			gtk_entry_set_text (
+#ifdef USE_GTK2_14
+				GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->query_combo))),
+#else
+				GTK_ENTRY (GTK_BIN (rt->query_combo)->child),
+#endif
+				utf);
 			g_free (local);
-			F_FREE (utf);
+			F_FREE (utf, local);
 		}
 	}
 	gtk_widget_destroy (GTK_WIDGET (dialog));
-	gtk_widget_grab_focus (GTK_BIN (rt->query_combo)->child);
+	gtk_widget_grab_focus (
+#ifdef USE_GTK2_14
+		gtk_bin_get_child (GTK_BIN (rt->query_combo))
+#else
+		GTK_BIN (rt->query_combo)->child
+#endif
+	);
 }
 /**
-@brief bring up a system find-file window to choose a plugin
+@brief bring up a system find-file window to choose a query
 
-@param button activated widget, UNUSED
+@param button UNUSED activated widget
 @param rt ptr to dialog data struct
 
 @return
 */
-static void _e2p_track_choose_rdf_cb (GtkWidget *button, E2_TrackDlgData *rt)
+static void _e2p_track_choose_rdf_cb (GtkButton *button, E2_TrackDlgData *rt)
 {
 	//no need for vfs support here
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
 		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_OPEN,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_OK, GTK_RESPONSE_OK,
+		GTK_STOCK_OPEN, GTK_RESPONSE_OK,
 		NULL);
 
 	e2_dialog_setup_chooser (dialog,
@@ -200,20 +212,29 @@ static void _e2p_track_choose_rdf_cb (GtkWidget *button, E2_TrackDlgData *rt)
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
 
 /*	//hide the dialog's standard 'open' button
-	GtkContainer *bbox = GTK_CONTAINER (GTK_DIALOG (dialog)->action_area);
+	GtkContainer *bbox = GTK_CONTAINER (
+#ifdef USE_GTK2_14
+		gtk_dialog_get_action_area (GTK_DIALOG (dialog));
+#else
+		GTK_DIALOG (dialog)->action_area);
+#endif
 	GList* children = gtk_container_get_children (bbox);
 	GtkWidget *btn = children->data;
 	gtk_widget_hide (btn);
 	g_list_free (children);
 	//add 2 buttons that we want
+	E2_Button no_btn;
+	e2_button_derive (&no_btn, &E2_BUTTON_NO, BTN_NO_DEFAULT);
+
+	e2_dialog_add_defined_button (dialog, &E2_BUTTON_?APPLY);
+	e2_dialog_add_defined_button (dialog, &no_btn);
 	e2_dialog_add_defined_button (dialog, &E2_BUTTON_APPLY);
-	e2_dialog_add_defined_button (dialog, &E2_BUTTON_OK);
 */
 	g_signal_connect (G_OBJECT (dialog), "response",
 		G_CALLBACK (_e2p_track_choose_response_cb), rt);
 
 	e2_dialog_setup (dialog, rt->dialog);
-	gtk_widget_show (dialog);
+	gtk_widget_show_all (dialog);
 }
 /*
  //get files with service (Documents, Music, Images, Videos, Text, Development, Other)
@@ -241,25 +262,42 @@ This is called from a callback, BGL closed
 */
 static void _e2p_track_list_results (E2_TrackDlgData *rt)
 {
-	GtkWidget *entry = GTK_BIN (rt->query_combo)->child;
+	GtkWidget *entry =
+#ifdef USE_GTK2_14
+		gtk_bin_get_child (GTK_BIN (rt->query_combo));
+#else
+		GTK_BIN (rt->query_combo)->child;
+#endif
 	const gchar *find = gtk_entry_get_text (GTK_ENTRY (entry));
 	gchar *command;
 	gint result;
 
-	if ((GTK_TOGGLE_BUTTON (rt->type_radio))->active)	//search for service
+	if (
+#ifdef USE_GTK2_14
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->type_radio))
+#else
+		(GTK_TOGGLE_BUTTON (rt->type_radio))->active
+#endif
+	)	//search for service
 	{
 		gint type = gtk_combo_box_get_active (GTK_COMBO_BOX (rt->service_combo));
 		if (type == -1)
 			return;
 		if (*find == '\0'
-			|| g_str_equal (find, "*")
-			|| g_str_equal (find, _("all")))
+			|| !strcmp (find, "*")
+			|| !strcmp (find, _("all")))
 			command = g_strdup_printf ("tracker-files -s %s", cmd_str[service_index]);
 		else
 			command = g_strdup_printf ("tracker-search -s %s %s",
 				cmd_str[service_index], find);
 	}
-	else if ((GTK_TOGGLE_BUTTON (rt->type_radio2))->active)	//search for mimetype
+	else if (
+#ifdef USE_GTK2_14
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->type_radio2))
+#else
+		(GTK_TOGGLE_BUTTON (rt->type_radio2))->active
+#endif
+	)	//search for mimetype
 	{
 		if (*find == '\0')
 		{
@@ -278,11 +316,11 @@ static void _e2p_track_list_results (E2_TrackDlgData *rt)
 		command = g_strdup_printf ("tracker-query %s", find);
 	}
 
+	result = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT, rt->dialog
 #ifdef E2_COMMANDQ
-	result = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT, TRUE);
-#else
-	result = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT);
+	, TRUE
 #endif
+	);
 	if (result == 0)
 		e2_output_print_end (&app.tab, FALSE);
 
@@ -304,11 +342,22 @@ static void _e2p_track_response_cb (GtkDialog *dialog, gint response,
 	{
 		case E2_RESPONSE_USER1:	//show help on using tracker
 			e2_utils_show_help ("tracker plugin"); //no translation unless help doc is translated
-			gtk_widget_grab_focus (GTK_BIN (rt->query_combo)->child);
+			gtk_widget_grab_focus (
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (rt->query_combo))
+#else
+				GTK_BIN (rt->query_combo)->child
+#endif
+			);
 			break;
 		case E2_RESPONSE_USER2:	//clear query entry
 		{
-			GtkWidget *entry = GTK_BIN (rt->query_combo)->child;
+			GtkWidget *entry =
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (rt->query_combo));
+#else
+				GTK_BIN (rt->query_combo)->child;
+#endif
 			gtk_entry_set_text (GTK_ENTRY (entry), "");
 			gtk_widget_grab_focus (entry);
 		}
@@ -316,7 +365,12 @@ static void _e2p_track_response_cb (GtkDialog *dialog, gint response,
 		case E2_RESPONSE_FIND:
 			//in case the user aborts during the search, update history stuff now
 			service_index = gtk_combo_box_get_active (GTK_COMBO_BOX (rt->service_combo));
-			GtkWidget *entry = GTK_BIN (rt->query_combo)->child;
+			GtkWidget *entry =
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (rt->query_combo));
+#else
+				GTK_BIN (rt->query_combo)->child;
+#endif
 			const gchar *find = gtk_entry_get_text (GTK_ENTRY (entry));
 			if (*find != '\0')
 				e2_list_update_history (find, &query_history, NULL, 30, FALSE);
@@ -353,10 +407,16 @@ static gboolean _e2p_track (gpointer from, E2_ActionRuntime *art)
 	CHECKALLOCATEDWARN (rt, return FALSE; );
 
 	rt->dialog = e2_dialog_create (NULL, NULL, _("tracker query"),
-		_e2p_track_response_cb, rt);
+		(ResponseFunc)_e2p_track_response_cb, rt);
 
-	GtkWidget *hbox = e2_widget_add_box ((GTK_DIALOG (rt->dialog))->vbox, TRUE,
-		E2_PADDING, FALSE, FALSE, E2_PADDING_SMALL);
+	GtkWidget *dialog_vbox =
+#ifdef USE_GTK2_14
+		gtk_dialog_get_content_area (GTK_DIALOG (rt->dialog));
+#else
+		(GTK_DIALOG (rt->dialog))->vbox;
+#endif
+	GtkWidget *hbox = e2_widget_add_box (dialog_vbox, TRUE, E2_PADDING,
+		FALSE, FALSE, E2_PADDING_SMALL);
 //	rt->type_radio = e2_button_add_radio (hbox, NULL, NULL, FALSE, 0, NULL, NULL);
 //	e2_widget_add_label (hbox, _("Search for"), 0.0, 0.5, FALSE, 0);
 	rt->type_radio = e2_button_add_radio (hbox, _("_Search for"), NULL,
@@ -369,67 +429,52 @@ static gboolean _e2p_track (gpointer from, E2_ActionRuntime *art)
 		gtk_combo_box_append_text (GTK_COMBO_BOX (rt->service_combo),
 			gettext (object_names[i]));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (rt->service_combo), service_index);
-//USELESS? gtk_widget_set_tooltip_text (
-//	rt->service_combo, _("Select information service"));
+//USELESS? 	e2_widget_set_safetip (rt->service_combo, _("Select information service"));
 	e2_widget_add_label (hbox, _("which match:"), 0.0, 0.5, FALSE, 0);
-	gtk_widget_show (hbox);
 
-	hbox = e2_widget_add_box ((GTK_DIALOG (rt->dialog))->vbox, TRUE,
-		E2_PADDING, FALSE, FALSE, E2_PADDING_SMALL);
+	hbox = e2_widget_add_box (dialog_vbox, TRUE, E2_PADDING, FALSE, FALSE,
+		E2_PADDING_SMALL);
 	GSList *group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (rt->type_radio));
 //	e2_button_add_radio (hbox, NULL, group, FALSE, 0, NULL, NULL);
 //	e2_widget_add_label (hbox, _("Search for items whose _mimetype is any of:"), 0.0, 0.5, FALSE, 0);
 	rt->type_radio2 = e2_button_add_radio (hbox,
 		_("Search for items whose _mimetype is any of:"), group,
 		FALSE, FALSE, 0, NULL, NULL);
-	gtk_widget_show (hbox);
 
-	hbox = e2_widget_add_box ((GTK_DIALOG (rt->dialog))->vbox, TRUE,
-		E2_PADDING, FALSE, FALSE, E2_PADDING_SMALL);
+	hbox = e2_widget_add_box (dialog_vbox, TRUE, E2_PADDING, FALSE, FALSE,
+		E2_PADDING_SMALL);
 //	group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (rt->type_radio));
 	e2_button_add_radio (hbox, _("Search for items using this rdf query:"), group,
 		FALSE, FALSE, 0, NULL, NULL);
 	e2_button_add (hbox, FALSE, E2_PADDING_LARGE, _("_Select"), GTK_STOCK_OPEN,
 			_("Open query-selection dialog"), _e2p_track_choose_rdf_cb, rt);
-	gtk_widget_show (hbox);
 
-	rt->query_combo = e2_combobox_add ((GTK_DIALOG (rt->dialog))->vbox, FALSE,
-		E2_PADDING, _e2p_track_query_activated_cb, rt, &query_history,
+	rt->query_combo = e2_combobox_add (dialog_vbox, FALSE, E2_PADDING,
+		(ActivateFunc)_e2p_track_query_activated_cb, rt, &query_history,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE | E2_COMBOBOX_CYCLE_HISTORY);
 
-	gtk_widget_show (rt->query_combo);
-
-	GtkWidget *btn = e2_dialog_add_undefined_button (rt->dialog, GTK_STOCK_HELP,
+	GtkWidget *btn = e2_dialog_add_simple_button (rt->dialog, GTK_STOCK_HELP,
 		_("_Help"), E2_RESPONSE_USER1);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		btn, _("Get help on constructing queries"));
+	e2_widget_set_safetip (btn, _("Get help on constructing queries"));
 
-	btn = e2_dialog_add_undefined_button (rt->dialog, GTK_STOCK_CLEAR,
+	btn = e2_dialog_add_simple_button (rt->dialog, GTK_STOCK_CLEAR,
 		_("C_lear"), E2_RESPONSE_USER2);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		btn, _("Clear the current query"));
+	e2_widget_set_safetip (btn, _("Clear the current query"));
 
-	btn = e2_dialog_add_undefined_button (rt->dialog, GTK_STOCK_FIND,
+	btn = e2_dialog_add_simple_button (rt->dialog, GTK_STOCK_FIND,
 		_("_Find"), E2_RESPONSE_FIND);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		btn, _("Initiate a query using currently-specified criteria"));
+	e2_widget_set_safetip (btn,
+		_("Initiate a query using currently-specified criteria"));
 
 	e2_dialog_set_negative_response (rt->dialog, GTK_RESPONSE_CLOSE);
 	e2_dialog_show (rt->dialog, app.main_window, 0, &E2_BUTTON_CLOSE, NULL);
-	GtkWidget *entry = GTK_BIN (rt->query_combo)->child;
-	gtk_widget_grab_focus (entry);
+	gtk_widget_grab_focus (
+#ifdef USE_GTK2_14
+		gtk_bin_get_child (GTK_BIN (rt->query_combo))
+#else
+		GTK_BIN (rt->query_combo)->child
+#endif
+	);
 
 	return TRUE;
 }
@@ -451,19 +496,22 @@ gboolean init_plugin (Plugin *p)
 	p->signature = ANAME VERSION;
 	p->menu_name = _("_Track..");
 	p->description = _("Find items in the tracker database");
-	p->icon = "plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate
+	p->icon = "plugin_"ANAME""E2ICONTB;  //use icon file pathname if appropriate
 
 	if (p->action == NULL)
 	{
-		//no need to free this
-		gchar *action_name = g_strconcat (_A(1),".",aname,NULL);
-		p->action = e2_plugins_action_register
-			(action_name, E2_ACTION_TYPE_ITEM, _e2p_track, NULL, FALSE, 0, NULL);
-
-		e2_cache_int_register ("track-plugin-type", &service_index, 0);
-		e2_cache_list_register ("track-plugin-history", &query_history);
-
-		return TRUE;
+		//don't free name string here
+		E2_Action plugact =
+		{g_strconcat (_A(1),".",aname,NULL),_e2p_track,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+		p->action = e2_plugins_action_register (&plugact);
+		if G_LIKELY((p->action != NULL))
+		{
+			e2_cache_int_register ("track-plugin-type", &service_index, 0);
+			e2_cache_list_register ("track-plugin-history", &query_history);
+			return TRUE;
+		}
+		else
+			g_free (plugact.name);
 	}
 	return FALSE;
 }
