@@ -1,6 +1,6 @@
-/* $Id: e2_complete.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_complete.h 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -32,35 +32,29 @@ This is the header file for the string auto completion functions.
 
 typedef enum
 {
-	E2_COMPLETE_FLAG_DONT_ESCAPE = 1<<0,
-	E2_COMPLETE_FLAG_STOP = 1<<1,
-	E2_COMPLETE_FLAG_ALL = 1<<2,
-	E2_COMPLETE_FLAG_FILES = 1<<3,
-	E2_COMPLETE_FLAG_DIRS = 1<<4,
-	E2_COMPLETE_FLAG_EXEC = 1<<5,
-	E2_COMPLETE_FLAG_PATH = 1<<6,
+	E2_COMPLETE_FLAG_DONT_ESCAPE = 1,	//unused
+	E2_COMPLETE_FLAG_STOP    = 1<<1,
+	E2_COMPLETE_FLAG_ALL     = 1<<2,
+	E2_COMPLETE_FLAG_FILES   = 1<<3,
+	E2_COMPLETE_FLAG_DIRS    = 1<<4,
+	E2_COMPLETE_FLAG_EXEC    = 1<<5,
+	E2_COMPLETE_FLAG_PATH    = 1<<6,
 	E2_COMPLETE_FLAG_HISTORY = 1<<7,
-	E2_COMPLETE_FLAG_MOUNT = 1<<8,
-	E2_COMPLETE_FLAG_SHELL = 1<<9,
+	E2_COMPLETE_FLAG_MOUNT   = 1<<8,
+	E2_COMPLETE_FLAG_SHELL   = 1<<9,
 } E2_CompleteFlags;
 
 typedef struct _E2_CompleteRuntime
 {
 	guint priority;
 	E2_CompleteFlags flags;
-//	gchar *name;	//the category of completion to which this method applies
-//	gchar *group;	//"default"
-//	gchar *desc; IRRELEVANT, NO UI PRESENCE
-//	gchar *tip;	IRRELEVANT, NO UI PRESENCE
-	gpointer func;	//the funtion which implements the competion method
+	//the funtion which implements the completion method
+	guint (*func)(gchar*,gchar*,gint,GList**,E2_CompleteFlags*,gpointer,guint);
 	gpointer data;	//available for method-specific data usage
 } E2_CompleteRuntime;
 
-gint e2_complete_str (gchar **complete, gint *pos, GList **found,
+guint e2_complete_str (gchar **complete, gint *pos, GList **found,
 	E2_CompleteFlags flags, guint pane);
-void e2_complete_register_method (guint priority, E2_CompleteFlags flags,
-	//const gchar *name, gchar *group, gchar *desc, gchar *tip,
-	gpointer func, gpointer data);
 void e2_complete_init (void);
 void e2_complete_clear (void);
 
@@ -68,13 +62,11 @@ void e2_complete_clear (void);
  /***** functions from e2_complete__*.c files *****/
 /*************************************************/
 
-gint e2_complete_files (gchar *line, gchar *word, gint pos, GList **found,
-	E2_CompleteFlags *flags, gpointer data, guint pane);
-gint e2_complete_path (gchar *line, gchar *word, gint pos, GList **found,
-	E2_CompleteFlags *flags, gpointer data);
+guint e2_complete_path (gchar *line, gchar *word, gint pos, GList **found,
+	E2_CompleteFlags *flags, gpointer data, guint unused);
 #ifdef E2_FS_MOUNTABLE
-gint e2_complete_mount (gchar *line, gchar *word, gint pos, GList **found,
-	E2_CompleteFlags *flags, gpointer data);
+guint e2_complete_mount (gchar *line, gchar *word, gint pos, GList **found,
+	E2_CompleteFlags *flags, gpointer data, guint unused);
 gboolean e2_complete_mount_menu_create (gpointer from, E2_ActionRuntime *art);
 void e2_complete_actions_register (void);
 #endif
