@@ -1,6 +1,6 @@
-/* $Id: e2_complete__mount.c 638 2007-09-05 03:09:44Z tpgww $
+/* $Id: e2_complete__mount.c 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2005-2007 tooar <tooar@gmx.net>
+Copyright (C) 2005-2009 tooar <tooar@emelfm2.net>
 Portions copyright (C) 2004 Florian Zaehringer <flo.zaehringer@web.de>
 
 This file is part of emelFM2.
@@ -35,7 +35,7 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 @param points list of mountpoint strings to be scanned
 @param word utf-8 string with string to be completed ("mount" or start of mountpoint path)
-@param found sore for pointer to list to hold matching items from @a points
+@param found sore for pointer to list to hold matching items from @a points, must be initialized by caller
 
 @return the number of matches found
 */
@@ -45,7 +45,7 @@ static guint _e2_complete_mount_find (GList *points, gchar *word, GList **found)
 	GList *member;
 	guint ret = 0;
 	gboolean all = FALSE; //the mountpoint was started in the command
-	if ((match = strstr (word, "mount")) != NULL)	//_I(
+	if ((match = strstr (word, E2_MOUNTCOMMAND)) != NULL)	//_I(
 	{
 		match = e2_utils_find_whitespace (match);
 		if (match == NULL)
@@ -126,27 +126,31 @@ will be empty
 @param found store for pointer to list of matching items (utf-8 paths)
 @param flags pointer to bit-flags indicating type of match
 @param data discretionary data pointer for this method
+@param unused parameter needed for other complete funcs
 
 @return the number of successful matches
 */
-gint e2_complete_mount (gchar *line, gchar *word, gint pos, GList **found,
-	E2_CompleteFlags *flags, gpointer data)
+guint e2_complete_mount (gchar *line, gchar *word, gint pos, GList **found,
+	E2_CompleteFlags *flags, gpointer data, guint unused)
 {
 	GList *candidates;
 	//make sure this is an (un)mount command
 	gchar *tmp = g_utf8_offset_to_pointer (line, pos);
 	gint byte_pos = (tmp - line);
-	if (g_strstr_len (line, byte_pos, "mount") == NULL)	//_I( ??
+	if (g_strstr_len (line, byte_pos, E2_MOUNTCOMMAND) == NULL)
 		return 0;
 
-	gint ret = 0;
-	*flags |= E2_COMPLETE_FLAG_STOP;
+	//after mountpoints completion, do not try any other other category
+//	*flags |= E2_COMPLETE_FLAG_STOP;
+
 //	uid = getuid (); needed if checking permissions
 
 	if (g_strstr_len (line, byte_pos, "umount") != NULL)	//_I( ??
 		candidates = e2_fs_mount_get_mounts_list ();
 	else
 		candidates = e2_fs_mount_get_mountable_list ();
+
+	guint ret;
 	if (candidates != NULL)
 	{
 		ret = _e2_complete_mount_find (candidates, word, found);
@@ -158,4 +162,3 @@ gint e2_complete_mount (gchar *line, gchar *word, gint pos, GList **found,
 	return ret;
 }
 #endif	//def E2_FS_MOUNTABLE
-
