@@ -1,6 +1,6 @@
-/* $Id: e2_complete__path.c 846 2008-04-04 22:32:40Z tpgww $
+/* $Id: e2_complete__path.c 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -68,7 +68,7 @@ static gboolean _e2_complete_path_drcb_check_dir (VPATH *parent,
 			*found = g_list_append (*found, temp);
 		}
 	}
-	F_FREE (utf);
+	F_FREE (utf, itemname);
 	return TRUE;
 }
 /**
@@ -83,7 +83,7 @@ Downstream code expects BGL to be closed on arrival here
 
 @return the number of successful matches
 */
-static gint _e2_complete_path_check_dir (VPATH *localpath, gchar *word, gchar *prefix,
+static guint _e2_complete_path_check_dir (VPATH *localpath, gchar *word, gchar *prefix,
 	GList **found)
 {
 	printd (DEBUG, "check_dir (dir:%s,word:%s,found:_,prefix:%s)", localpath, word, prefix);
@@ -115,14 +115,15 @@ Otherwise the scan iterates over $PATH.
 @param line utf-8 string containing the whole line to be completed
 @param word utf-8 string, copy of part of @a line with the 'word' to be completed
 @param pos 0-based index of cursor position (characters, not bytes) in @a line
-@param found pointer to list to record matching items
+@param found pointer to list to record matching items, must be initialised by caller
 @param flags pointer to bit-flags indicating type of match (dirs or files)
 @param data discretionary data pointer for this method
+@param unused parameter needed for other complete funcs
 
 @return the number of successful matches
 */
-gint e2_complete_path (gchar *line, gchar *word, gint pos, GList **found,
-	E2_CompleteFlags *flags, gpointer data)
+guint e2_complete_path (gchar *line, gchar *word, gint pos, GList **found,
+	E2_CompleteFlags *flags, gpointer data, guint unused)
 {
 	printd (DEBUG, "complete_path (line:%s,word:%s,pos:%d,found:_,flags:%d,data:_)",
 		line, word, pos, *flags);
@@ -133,12 +134,11 @@ gint e2_complete_path (gchar *line, gchar *word, gint pos, GList **found,
 WRONG ! //the word to complete needs to be the whole line
 	//(i.e. no ' ' in the line prior to the cursor position)
 	//or else we can't match a path
-	if (!g_str_equal (word, line))
+	if (strcmp (word, line))
 	//error message in this case?
 		return 0;
 */
-	*found = NULL;
-	gint ret;
+	guint ret;
 #ifdef E2_VFS
 	VPATH ddata;
 #endif
@@ -146,18 +146,23 @@ WRONG ! //the word to complete needs to be the whole line
 	if (strchr (word, G_DIR_SEPARATOR) != NULL) //string-to-complete has path separator
 	{
 		gchar *p1, *p2, *localpath;
-		p1 = g_utf8_next_char (strrchr (word, G_DIR_SEPARATOR));	//the 'real' word to complete
+		p1 = strrchr (word, G_DIR_SEPARATOR) + sizeof(gchar);	//the 'real' word to complete
 		p2 = g_strndup (word, p1 - word);	//prefix for matches
 		if (word[0] == G_DIR_SEPARATOR)
 		{	//word is an absolute path string
 			localpath = F_FILENAME_TO_LOCALE (p2);
 #ifdef E2_VFS
-			ddata.localpath = localpath;
+			ddata.path = localpath;
 			ddata.spacedata = curr_view->spacedata;
-			ret = _e2_complete_path_check_dir (&ddata, p1, p2, found);
-#else
-			ret = _e2_complete_path_check_dir (localpath, p1, p2, found);
 #endif
+			ret = _e2_complete_path_check_dir (
+#ifdef E2_VFS
+				&ddata,
+#else
+				localpath,
+#endif
+				p1, p2, found);
+			F_FREE (localpath, p2);
 		}
 		else	//a relative-path string
 		{
@@ -165,17 +170,22 @@ WRONG ! //the word to complete needs to be the whole line
 //E2_VFSTMPOK
 			localpath = F_FILENAME_TO_LOCALE (p3);
 #ifdef E2_VFS
-			ddata.localpath = localpath;
+			ddata.path = localpath;
 			ddata.spacedata = curr_view->spacedata;
-			ret = _e2_complete_path_check_dir (&ddata, p1, p2, found);
-#else
-			ret = _e2_complete_path_check_dir (localpath, p1, p2, found);
 #endif
+			ret = _e2_complete_path_check_dir (
+#ifdef E2_VFS
+				&ddata,
+#else
+				localpath,
+#endif
+				p1, p2, found);
 			g_free (p3);
+			F_FREE (localpath, p3);
 		}
 		g_free (p2);
-		F_FREE (localpath);
-		*flags |= E2_COMPLETE_FLAG_STOP; //with a path, never keep on looking
+		//after checking for completing items in path, never try other categories
+//		*flags |= E2_COMPLETE_FLAG_STOP;
 	}
 	else //no separator
 	{	//look everywhere in $PATH
@@ -190,7 +200,7 @@ WRONG ! //the word to complete needs to be the whole line
 			while (paths[i] != NULL)
 			{
 #ifdef E2_VFS
-				ddata.localpath = paths[i];
+				ddata.path = paths[i];
 				ddata.spacedata = NULL;	//$PATH items must be local
 				ret += _e2_complete_path_check_dir (&ddata, word, NULL, found);
 #else
@@ -200,8 +210,8 @@ WRONG ! //the word to complete needs to be the whole line
 			}
 			g_strfreev (paths);
 		}
-		if (ret > 0)
-			*flags |= E2_COMPLETE_FLAG_STOP;
+//		if (ret > 0)
+//			*flags |= E2_COMPLETE_FLAG_STOP;
 	}
 
 	return ret;
