@@ -1,6 +1,6 @@
-/* $Id: e2_command.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_command.h 1663 2009-05-09 09:59:37Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -66,18 +66,21 @@ typedef struct _E2_CommandTaskData
 	VOLATILE gint exit;	//command exit code, 0 = success
 } E2_CommandTaskData;
 
+void e2_command_block_childsignal (void);
+void e2_command_unblock_childsignal (void);
 gboolean e2_command_clear_pending (gpointer from, E2_ActionRuntime *art);
 gboolean e2_command_find_process (guint pid);
-gint e2_command_run_at (gchar *command, const gchar *workdir, E2_CommandRange range);
+gint e2_command_run_at (gchar *command, const gchar *workdir,
+	E2_CommandRange range, gpointer from);
 //gint e2_command_run (gchar *command, E2_CommandRange range);
-#define e2_command_run(c,r) e2_command_run_at(c,NULL,r)
+#define e2_command_run(c,r,f) e2_command_run_at(c,NULL,r,f)
 guint e2_command_count_running_tasks (gboolean countcmds, gboolean countpaused);
 void e2_command_retab_children (E2_OutputTabRuntime *currenttab,
 	E2_OutputTabRuntime *replacetab);
 void e2_command_retab2_children (E2_OutputTabRuntime *currenttab,
 	E2_OutputTabRuntime *replacetab);
 gboolean e2_command_kill_child (guint pid);
-const gchar *e2_command_get_variable_value (gchar *var_name);
+const gchar *e2_command_get_variable_value (gchar *var_name, const gchar **tailstore);
 void e2_command_output_help (void);
 void e2_command_actions_register (void);
 void e2_command_options_register (void);
