@@ -1,7 +1,7 @@
-/* $Id: e2_cache.c 851 2008-04-07 10:34:28Z tpgww $
+/* $Id: e2_cache.c 1853 2009-09-21 08:12:21Z tpgww $
 
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 Copyright (C) 2004 Florian Zaehringer (flo.zaehringer@web.de)
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -25,6 +25,9 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 This file contains functions for e2's cache system: eg. to register
 and unregister caches, and to write and read them to/from disk.
+Custom-icon cacheing is handled here. We do not use GtkIconFactory etc as that's
+mainly designed for "fixed" UI and is too much hassle to work with emelFM2
+runtime-configuration
 */
 /**
 \page cache cacheing of emelFM2 runtime data
@@ -53,7 +56,7 @@ static GList *unknown_list;
 */
 static gint _e2_cache_str_compare (E2_Cache *cache_element, const gchar *name)
 {
-	return !g_str_equal (name, cache_element->name);
+	return strcmp (name, cache_element->name);
 }
 
 #ifdef E2_IMAGECACHE
@@ -637,8 +640,8 @@ void e2_cache_unregister (gchar *name)
 			GList *stringlist;
 			if (cache->sync_func != NULL)
 			{	//custom conversion to list for backup
-				GList* (*fun) (gpointer, gpointer) = cache->sync_func;
-				stringlist = fun (*(gpointer *)cache->data, cache->sync_data);
+				GList* (*func) (gpointer, gpointer) = cache->sync_func;
+				stringlist = func (*(gpointer *)cache->data, cache->sync_data);
 			}
 			else
 			{
@@ -654,26 +657,26 @@ void e2_cache_unregister (gchar *name)
 		case E2_CACHE_TYPE_LIST:
 			if (cache->sync_func != NULL)
 			{
-				void (*fun) (gpointer) = cache->sync_func;
-				fun (cache->sync_data);
+				void (*func) (gpointer) = cache->sync_func;
+				func (cache->sync_data);
 			}
 #ifndef E2_VFS
 			GList *
 #endif
 			stringlist = NULL;
-			if ((cache->data != NULL) && (*cache->data != NULL))
+			if (cache->data != NULL && *cache->data != NULL)
 			{
-				GList *node;
-				for (node = (GList *) *cache->data; node != NULL; node = node->next)
-					stringlist = g_list_append (stringlist, g_strdup ((gchar *) node->data));
+				//just steal the existing list
+				stringlist = (GList *) *cache->data;
+				*cache->data = NULL;	//no further use elsewhere
 			}
-			cache->data = (gpointer) stringlist; //backup records list start, not a pointer address
+			cache->data = (gpointer) stringlist;
 			break;
 		case E2_CACHE_TYPE_ARRAY:
 			if (cache->sync_func != NULL)
 			{
-				void (*fun) (gpointer) = cache->sync_func;
-				fun (cache->sync_data);
+				void (*func) (gpointer) = cache->sync_func;
+				func (cache->sync_data);
 			}
 			GString *merged = g_string_new ("");
 			gint *values = (gint *) cache->data;
@@ -922,11 +925,20 @@ extern gint real_height;
 					break;
 			}
 		}
+
+		//fs rename operation may be not reliably atomic (ext4)
+		gint fdesc = fileno (f);
+		if (fdesc == -1)
+			goto error_handler;
+		if (e2_fs_writeflush (fdesc) != 0)
+			goto error_handler;
+
 		e2_fs_close_stream (f);
+
 #ifdef E2_VFS
-		tdata.localpath = tempname;
+		tdata.path = tempname;
 		tdata.spacedata = NULL;
-		ddata.localpath = local;
+		ddata.path = local;
 		ddata.spacedata = NULL;
 #endif
 		gdk_threads_leave ();	//downstream error messages invoke local mutex management
@@ -957,7 +969,7 @@ error_handler:
 	e2_output_print_error (msg, TRUE);
 cleanup:
 	g_free (utfpath);
-	F_FREE (local);
+	F_FREE (local, utfpath);
 	g_free (tempname);
 }
 /**
@@ -1076,7 +1088,7 @@ static void _e2_cache_file_read (gchar *fn)
 	}
 
 	g_free (filepath);
-	F_FREE (localpath);
+	F_FREE (localpath, filepath);
 }
 /**
 @brief initialize the cache
