@@ -1,6 +1,6 @@
-/* $Id: e2_cl_option.c 708 2007-11-09 21:30:03Z tpgww $
+/* $Id: e2_cl_option.c 1663 2009-05-09 09:59:37Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -174,24 +174,24 @@ void e2_cl_option_process (gint argc, gchar *argv[])
 				case 'c':
 					g_free (e2_cl_options.config_dir);
 					if (g_path_is_absolute (optarg))
-#ifdef E2_FILES_UTF8ONLY
-						convert = g_strdup (optarg);
-#else
+//#ifdef E2_FILES_UTF8ONLY
+//						convert = g_strdup (optarg);
+//#else
 						convert = optarg;
-#endif
+//#endif
 					else
 					{
 						freeme = g_get_current_dir ();
 						convert = g_build_filename (freeme, optarg, NULL);
 						g_free (freeme);
 					}
-#ifdef E2_FILES_UTF8ONLY
-					e2_cl_options.config_dir = convert;
-#else
+//#ifdef E2_FILES_UTF8ONLY
+//					e2_cl_options.config_dir = convert;
+//#else
 					e2_cl_options.config_dir = e2_utf8_filename_from_locale (convert);
 					if (convert != optarg)
 						g_free (convert);
-#endif
+//#endif
 					printd (DEBUG, "setting config directory '%s'", e2_cl_options.config_dir);
 					break;
 #ifdef DEBUG_MESSAGES
@@ -229,24 +229,24 @@ void e2_cl_option_process (gint argc, gchar *argv[])
 				case 't':
 					g_free (e2_cl_options.trash_dir);
 					if (g_path_is_absolute (optarg))
-#ifdef E2_FILES_UTF8ONLY
-						convert = g_strdup (optarg);
-#else
+//#ifdef E2_FILES_UTF8ONLY
+//						convert = g_strdup (optarg);
+//#else
 						convert = optarg;
-#endif
+//#endif
 					else
 					{
 						freeme = g_get_current_dir ();
 						convert = g_build_filename (freeme, optarg, NULL);
 						g_free (freeme);
 					}
-#ifdef E2_FILES_UTF8ONLY
-					e2_cl_options.trash_dir = convert;
-#else
+//#ifdef E2_FILES_UTF8ONLY
+//					e2_cl_options.trash_dir = convert;
+//#else
 					e2_cl_options.trash_dir = e2_utf8_filename_from_locale (convert);
 					if (convert != optarg)
 						g_free (convert);
-#endif
+//#endif
 					printd (DEBUG, "setting trash directory '%s'", e2_cl_options.trash_dir);
 					break;
 				default:
@@ -308,13 +308,13 @@ void e2_cl_option_process (gint argc, gchar *argv[])
 		usedir = g_getenv ("XDG_CONFIG_HOME");
 		if (usedir == NULL || *usedir == '\0')
 			usedir = g_get_user_config_dir ();
-#ifdef E2_FILES_UTF8ONLY
-		e2_cl_options.config_dir = g_build_filename (usedir, BINNAME, NULL);
-#else
+//#ifdef E2_FILES_UTF8ONLY
+//		e2_cl_options.config_dir = g_build_filename (usedir, BINNAME, NULL);
+//#else
 		freeme = g_build_filename (usedir, BINNAME, NULL);
 		e2_cl_options.config_dir =  e2_utf8_filename_from_locale (freeme);
 		g_free (freeme);
-#endif
+//#endif
 	}
 	//setup local default trash dir (any others done elsewhere)
 	if (e2_cl_options.trash_dir == NULL)
@@ -322,13 +322,13 @@ void e2_cl_option_process (gint argc, gchar *argv[])
 		usedir = g_getenv ("XDG_DATA_HOME");
 		if (usedir == NULL || *usedir == '\0')
 			usedir = g_get_user_data_dir ();
-#ifdef E2_FILES_UTF8ONLY
-		e2_cl_options.trash_dir = g_build_filename (usedir, "Trash", "files", NULL);
-#else
-		freeme = g_build_filename (usedir, "Trash", "files", NULL);
+//#ifdef E2_FILES_UTF8ONLY
+//		e2_cl_options.trash_dir = g_build_filename (usedir, "Trash", NULL);
+//#else
+		freeme = g_build_filename (usedir, "Trash", NULL);
 		e2_cl_options.trash_dir = e2_utf8_filename_from_locale (freeme);
 		g_free (freeme);
-#endif
+//#endif
 	}
 
 	const gchar **encodings;
@@ -338,13 +338,16 @@ void e2_cl_option_process (gint argc, gchar *argv[])
 		if (encodings[0] != NULL && *encodings[0] != '\0')
 			e2_cl_options.encoding = g_strdup (encodings[0]);
 		else
-			e2_cl_options.encoding = g_strdup ("ISO-8859-1");
+			e2_cl_options.encoding = g_strdup ("ISO-8859-15");
 	}
 	if (e2_cl_options.fallback_encoding == NULL)
 	{
 		if (encodings[1] != NULL && *encodings[1] != '\0')
 			e2_cl_options.fallback_encoding = g_strdup (encodings[1]);
+		else if (strcmp (e2_cl_options.encoding,"ISO-8859-1") == 0
+			  || strcmp (e2_cl_options.encoding,"ISO-8859-15") == 0)
+			e2_cl_options.fallback_encoding = g_strdup ("C");
 		else
-			e2_cl_options.fallback_encoding = g_strdup ("ISO-8859-1");
+			e2_cl_options.fallback_encoding = g_strdup ("ISO-8859-15");
 	}
 }
