@@ -1,6 +1,6 @@
-/* $Id: e2_option_bool.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_option_bool.h 1955 2009-10-02 04:02:01Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -21,8 +21,6 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #ifndef __E2_OPTION_BOOL_H__
 #define __E2_OPTION_BOOL_H__
 
-GtkWidget *e2_option_bool_add_menu_widget (GtkWidget *controller,
-	GtkWidget *menu, E2_OptionSet *set, gpointer func, gpointer data);
 E2_OptionSet *e2_option_bool_register (gchar *name, gchar *group, gchar *desc,
 	gchar *tip, gchar *depends, gboolean def, E2_OptionFlags flags);
 gboolean e2_option_bool_get (gchar *option);
