@@ -1,6 +1,6 @@
-/* $Id: e2_option_int.c 682 2007-10-28 05:33:18Z tpgww $
+/* $Id: e2_option_int.c 1955 2009-10-02 04:02:01Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -34,14 +34,10 @@ void e2_option_int_add_widget (GtkWidget *dialog, GtkWidget *box,
 	gtk_size_group_add_widget (size_group, label);
 	gdouble max_percent = set->ex.num.max / 100.0;
 	GtkAdjustment *spinner_adj = (GtkAdjustment *) gtk_adjustment_new
-		(set->ival, set->ex.num.min, set->ex.num.max, 1.0, max_percent, max_percent);
+		((gdouble)set->ival, (gdouble)set->ex.num.min, (gdouble)set->ex.num.max,
+		1.0, max_percent, 0.0);
 	set->widget = gtk_spin_button_new (spinner_adj, 1.0, 0);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		event, set->tip);
+	e2_widget_set_safetip (event, set->tip);
 	gtk_box_pack_start (GTK_BOX (hbox), set->widget, FALSE, FALSE, E2_PADDING);
 }
 
