@@ -1,6 +1,6 @@
-/* $Id: e2_option_sel.c 682 2007-10-28 05:33:18Z tpgww $
+/* $Id: e2_option_sel.c 1955 2009-10-02 04:02:01Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -22,7 +22,7 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "e2_option.h"
 #include "e2_option_sel.h"
 
-static void _e2_option_sel_activated_cb (GtkWidget *menu_item, E2_OptionSet *set)
+static void _e2_option_sel_activated_cb (GtkMenuItem *menu_item, E2_OptionSet *set)
 {
 	GtkWidget *controller = g_object_get_data (G_OBJECT (menu_item),
 		"e2-controller-widget");
@@ -35,8 +35,15 @@ static void _e2_option_sel_activated_cb (GtkWidget *menu_item, E2_OptionSet *set
 	gint index = 2 - g_slist_index (group, menu_item);
 	e2_option_sel_set_direct (set, index);
 }
+/**
+@brief conform a check-menu item state to the corresponding option value
+This is a hook function
+@param data menu index
+@param submenu widget containing item to be updated
 
-static gboolean _e2_option_sel_value_changed_cb2 (gpointer data, GtkWidget *submenu)
+@return TRUE always, so func is not delisted after run
+*/
+static gboolean _e2_option_sel_value_changed_hook (gpointer data, GtkWidget *submenu)
 {
 	GtkWidget *controller = g_object_get_data (G_OBJECT (submenu),
 		"e2-controller-widget");
@@ -54,15 +61,26 @@ static gboolean _e2_option_sel_value_changed_cb2 (gpointer data, GtkWidget *subm
  /***** public *****/
 /******************/
 
-GtkWidget *e2_option_sel_add_menu_widget (GtkWidget *controller,
-	GtkWidget *menu, E2_OptionSet *set, gpointer func, gpointer data)
+/**
+@brief create a sub-menu of radio items reflecting the alternatives in @a set
+Args for @a func should be (GtkMenuItem*,some pointer)
+@param controller
+@param menu the parent menu for the new submenu
+@param set data for the set whose values are to be presented in the submenu
+@param func callback for "activate" signal on each radio-menu-item
+@param data data to be supplied to @a func
+
+@return the submenu widget
+*/
+GtkWidget *e2_option_sel_add_menu_widget (GtkWidget *controller, GtkWidget *menu,
+	E2_OptionSet *set, void (*func)(), gpointer data)
 {
 	//note set->tip not effective for an item with a submenu
 	GtkWidget *submenu = e2_menu_add_submenu (menu, set->desc, NULL);
 	g_object_set_data (G_OBJECT (submenu), "e2-controller-widget", controller);
 	//this will update the menu in case the option value changes
 	e2_option_attach_value_changed_simple (set, submenu,
-		_e2_option_sel_value_changed_cb2, submenu);
+		(HookFunc)_e2_option_sel_value_changed_hook, submenu);
 
 	GSList *group = NULL;
 	gint i = 0;
@@ -71,12 +89,7 @@ GtkWidget *e2_option_sel_add_menu_widget (GtkWidget *controller,
 		GtkWidget *item = e2_menu_add_radio (submenu, &group,
 			(gchar *)set->ex.sel.def[i], (e2_option_int_get_direct (set) == i),
 			func, data);
-#ifdef USE_GTK2_12TIPS
-		gtk_widget_set_tooltip_text (
-#else
-		e2_widget_set_tooltip (NULL,
-#endif
-			item, set->tip);
+		e2_widget_set_safetip (item, set->tip);
 		g_object_set_data (G_OBJECT (item), "e2-controller-widget", controller);
 		g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (_e2_option_sel_activated_cb), set);
@@ -99,12 +112,7 @@ void e2_option_sel_add_widget (GtkWidget *dialog, GtkWidget *box,
 		E2_COMBOBOX_MENU_STYLE);
 	e2_combobox_append_history_strv (combo, set->ex.sel.def);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), set->ival);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		event, set->tip);
+	e2_widget_set_safetip (event, set->tip);
 	set->widget = combo;
 	gtk_widget_show_all (event);
 }
