@@ -1,6 +1,6 @@
-/* $Id: e2_option_sel.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_option_sel.h 1955 2009-10-02 04:02:01Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -24,8 +24,8 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "emelfm2.h"
 #include "e2_option.h"
 
-GtkWidget *e2_option_sel_add_menu_widget (GtkWidget *controller,
-	GtkWidget *menu, E2_OptionSet *set, gpointer func, gpointer data);
+GtkWidget *e2_option_sel_add_menu_widget (GtkWidget *controller, GtkWidget *menu,
+	E2_OptionSet *set, void (*func)(), gpointer data); //args GtkMenuItem*,pointer
 void e2_option_sel_add_widget (GtkWidget *dialog, GtkWidget *box,
 	GtkSizeGroup *size_group, E2_OptionSet *set);
 E2_OptionSet *e2_option_sel_register (gchar *name, gchar *group, gchar *desc,
