@@ -1,6 +1,6 @@
-/* $Id: e2_option_tree.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_option_tree.h 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -30,12 +30,18 @@ typedef enum
 	E2_OPTION_TREE_TYPE_STR,
 	E2_OPTION_TREE_TYPE_ICON,
 	E2_OPTION_TREE_TYPE_SEL,
-	E2_OPTION_TREE_TYPE_KEY
+	E2_OPTION_TREE_TYPE_KEY,
+#ifdef E2_MOUSECUSTOM
+	E2_OPTION_TREE_TYPE_BUTTON,
+# ifdef E2_PTRGESTURES
+	E2_OPTION_TREE_TYPE_GESTURE,
+# endif
+#endif
 } E2_OptionTreeType;
 
-//these are flags that may be set for a tree-option
-//column when that column is added to the tree option
-//stored in glist at respective ((set->ex.tree.columns)->data)->flags
+//these are flags that may be set for a tree-option column when that column is
+//added to the tree option data, stored in GList at respective
+//((set->ex.tree.columns)->data)->flags
 typedef enum
 {
 	E2_OPTION_TREE_COL_NOT_EDITABLE = 1<< 0,
@@ -65,7 +71,7 @@ typedef struct _E2_OptionTreeColumn
 
 void e2_option_tree_flag_change (E2_OptionSet *set);
 void e2_option_tree_adjust_buttons (GtkTreeView *view, gboolean value);
-void e2_option_tree_add_widget (GtkWidget *parent, GtkWidget *box,
+void e2_option_tree_add_widget (GtkWidget *parent, gboolean single, GtkWidget *box,
 	E2_OptionSet *set);
 E2_OptionSet *e2_option_tree_register (gchar *name, gchar *group, gchar *desc,
 	gchar *depends, gpointer selection_check_func, gpointer draggable_check_func,
@@ -75,6 +81,7 @@ void e2_option_tree_add_column (E2_OptionSet *set, const gchar *name,
 	E2_OptionTreeColFlags flags,
 	gpointer visible_check_func, gpointer visible_check_data);
 void e2_option_tree_create_store (E2_OptionSet *set);
+void e2_option_tree_connect_mnemonics (GtkWidget *button_box);
 void e2_option_tree_revert (gchar *name);
 E2_OptionSet *e2_option_tree_get (gchar *option);
 void e2_option_tree_add_simple (gchar *option, GtkTreeIter *iter, gint n_options,
@@ -93,8 +100,12 @@ void e2_option_tree_set (gchar *option, GtkTreeIter *iter, gint col,
 void e2_option_tree_backup (E2_OptionSet *set);
 void e2_option_tree_unbackup (E2_OptionSet *set, gboolean revert);
 void e2_option_tree_restore_all (void);
-void e2_option_tree_prepare_defaults (E2_OptionSet *set, void *func);
-void e2_option_tree_setup_defaults (E2_OptionSet *set, gchar *first, ...);
+void e2_option_tree_prepare_defaults (E2_OptionSet *set, void (*func)(E2_OptionSet*));
+void e2_option_tree_setup_defaults (E2_OptionSet *set, gchar *first, ...)
+#ifdef G_GNUC_NULL_TERMINATED
+    G_GNUC_NULL_TERMINATED
+#endif
+;
 void e2_option_tree_install_defaults (void);
 gboolean e2_option_tree_set_from_array (gchar *name, gchar *f[], gint *index,
 	GtkTreeIter *root_iter);
