@@ -1,6 +1,6 @@
-/* $Id: e2_option_tree_context_menu.c 868 2008-05-06 04:42:09Z tpgww $
+/* $Id: e2_option_tree_context_menu.c 1936 2009-09-29 23:53:50Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 Portions copyright (C) 2004 Florian Zaehringer <flo.zaehringer@web.de>
 
 This file is part of emelFM2.
@@ -35,32 +35,6 @@ This file is #included in e2_option_tree.c
  /***** private *****/
 /*******************/
 
-/**
-@brief set popup menu position
-
-This function is supplied when calling gtk_menu_popup(), to position
-the displayed menu.
-set @a push_in to TRUE for menu completely inside the screen,
-FALSE for menu clamped to screen size
-
-@param menu UNUSED the GtkMenu to be positioned
-@param x	place to store gint representing the menu left
-@param y  place to store gint representing the menu top
-@param push_in place to store pushin flag
-@param treeview where the menu is to be shown
-
-@return
-*/
-static void _e2_option_tree_menu_set_position (GtkMenu *menu,
-	gint *x, gint *y, gboolean *push_in, GtkWidget *treeview)
-{
-	gint left, top;
-	gtk_window_get_position (GTK_WINDOW (config_dialog), &left, &top);
-	GtkAllocation alloc = treeview->allocation;
-	*x = left + alloc.x + alloc.width;
-	*y = top + alloc.y + alloc.height/2;
-	*push_in = FALSE;
-}
 /**
 @brief create treeview signature
 
@@ -80,7 +54,7 @@ static const gchar *_e2_option_tree_menu_get_signature (GtkTreeView *treeview)
 	E2_ToolbarData **thisbar = app.bars;
 	while (*thisbar != NULL)
 	{
-		if (g_str_equal ((*thisbar)->name, set->name))
+		if (!strcmp ((*thisbar)->name, set->name))
 			return "toolbar";
 		thisbar++;
 	}
@@ -94,14 +68,14 @@ static const gchar *_e2_option_tree_menu_get_signature (GtkTreeView *treeview)
 
 @return
 */
-static void _e2_option_tree_expand_all_cb (GtkWidget *widget, GtkTreeView *treeview)
+static void _e2_option_tree_expand_all_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 {
-	gtk_widget_set_sensitive (widget, FALSE);
+	gtk_widget_set_sensitive (GTK_WIDGET (widget), FALSE);
 	//get the paired item
 	gpointer m = g_object_get_data (G_OBJECT (widget), "e2-menu-pair");
 	gtk_widget_set_sensitive (GTK_WIDGET (m), TRUE);
 
-	e2_tree_expand_all_cb (widget, treeview);
+	gtk_tree_view_expand_all (treeview);
 }
 /**
 @brief tree collapse-all callback
@@ -111,14 +85,14 @@ static void _e2_option_tree_expand_all_cb (GtkWidget *widget, GtkTreeView *treev
 
 @return
 */
-static void _e2_option_tree_collapse_all_cb (GtkWidget *widget, GtkTreeView *treeview)
+static void _e2_option_tree_collapse_all_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 {
-	gtk_widget_set_sensitive (widget, FALSE);
+	gtk_widget_set_sensitive (GTK_WIDGET (widget), FALSE);
 	//get the paired button
 	gpointer m = g_object_get_data (G_OBJECT (widget), "e2-menu-pair");
 	gtk_widget_set_sensitive (GTK_WIDGET (m), TRUE);
 
-	e2_tree_collapse_all_cb (widget, treeview);
+	gtk_tree_view_collapse_all (treeview);
 }
 /**
 @brief context-menu copy callback
@@ -133,12 +107,12 @@ each with a path string and a whole row contents, are added
 to a hash table.
 The table may contain entries for more than 1 treeview
 
-@param menu_item UNUSED the selected menu widget
+@param menu_item UNUSED the activated menu widget
 @param treeview where the action is to occur
 
 @return
 */
-static void _e2_option_tree_menu_copy_cb (GtkWidget *menu_item, GtkTreeView *treeview)
+static void _e2_option_tree_menu_copy_cb (GtkMenuItem *menu_item, GtkTreeView *treeview)
 {
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
 	if (gtk_tree_selection_count_selected_rows (sel) > 0)
@@ -158,12 +132,12 @@ paste is appropriate
 Pasted row(s) are placed after the selected row in the
 destination treeview, as child(ren) if the path depth is appropriate
 
-@param menu_item the selected menu widget
+@param menu_item the activated menu widget
 @param treeview widget where the action is to occur
 
 @return
 */
-static void _e2_option_tree_menu_paste_cb (GtkWidget *menu_item, GtkTreeView *treeview)
+static void _e2_option_tree_menu_paste_cb (GtkMenuItem *menu_item, GtkTreeView *treeview)
 {
 	//we only paste after the 1st selected row
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
@@ -185,12 +159,12 @@ static void _e2_option_tree_menu_paste_cb (GtkWidget *menu_item, GtkTreeView *tr
 /**
 @brief context-menu cut callback
 
-@param menu_item the selected menu widget
+@param menu_item the activated menu widget
 @param treeview widget where the action is to occur
 
 @return
 */
-static void _e2_option_tree_menu_cut_cb (GtkWidget *menu_item, GtkTreeView *treeview)
+static void _e2_option_tree_menu_cut_cb (GtkMenuItem *menu_item, GtkTreeView *treeview)
 {
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
 	if (gtk_tree_selection_count_selected_rows (sel) > 0)
@@ -203,17 +177,17 @@ static void _e2_option_tree_menu_cut_cb (GtkWidget *menu_item, GtkTreeView *tree
 	}
 }
 /**
-@brief add treeview context menu items
+@brief add items to treev-option context menu
 
-@param treeview where the action is to occur
 @param menu the menu widget to which the items are to be added
+@param treeview where the action is to occur
 @param flags indicator of menu items to be created
 @param set pointer to data struct for the option that is being edited
 
 @return
 */
-static void _e2_option_tree_menu_items_add (GtkTreeView *treeview,
-	GtkWidget *menu, E2_TreeContextMenuFlags flags, E2_OptionSet *set)
+static void _e2_option_tree_populate_menu (GtkWidget *menu,
+	GtkTreeView *treeview, E2_TreeContextMenuFlags flags, E2_OptionSet *set)
 {
 	E2_Nontet *n = e2_utils_nontet_new ();
 	g_object_set_data_full (G_OBJECT (menu), "e2-config-menu", n,
@@ -254,18 +228,18 @@ static void _e2_option_tree_menu_items_add (GtkTreeView *treeview,
 		e2_menu_add_separator (menu); //more items added in the calling function
 	}
 	n->f = e2_menu_add (menu, _("_Add"), GTK_STOCK_ADD,
-		_("Add a row after the current one"), _e2_option_tree_add_below_cb, set);
+		_("Add a row after the current one"), (void(*)()) _e2_option_tree_add_below_cb, set);
 
 	if (flags & E2_TREE_CONTEXT_EXP_COL)
 		n->g = e2_menu_add (menu, _("Add c_hild"), GTK_STOCK_INDENT,
-			_("Add a child to the selected row"), _e2_option_tree_add_child_cb, set);
+			_("Add a child to the selected row"), (void(*)()) _e2_option_tree_add_child_cb, set);
 
 	e2_menu_add_separator (menu);
 	n->h = e2_menu_add (menu, _("_Up"), GTK_STOCK_GO_UP,
-		_("Move selected row up"), _e2_option_tree_move_up_cb, set);
+		_("Move selected row up"), (void(*)()) _e2_option_tree_move_up_cb, set);
 
 	n->i = e2_menu_add (menu, _("_Down"), GTK_STOCK_GO_DOWN,
-		_("Move selected row down"), _e2_option_tree_move_down_cb, set);
+		_("Move selected row down"), (void(*)()) _e2_option_tree_move_down_cb, set);
 
 }
 
@@ -300,41 +274,61 @@ void e2_option_tree_menu_set_sensitive (GtkWidget *menu, GtkWidget *treeview)
 		gtk_widget_set_sensitive (m->c, state);
 	}
 	//now adjust other menu items to conform with the page buttons
-	E2_Sextet *s2 = g_object_get_data (G_OBJECT (treeview), "e2-config-buttons");
-	if (s2 == NULL)
+	E2_Sextet *st = g_object_get_data (G_OBJECT (treeview), "e2-config-buttons");
+	if (st == NULL)
 		return;
-	if (s2->b != NULL)	//up
+	if (st->b != NULL)	//up
 	{
-		state = GTK_WIDGET_SENSITIVE(s2->b);
+#ifdef USE_GTK2_18
+		state = gtk_widget_get_sensitive(st->b);
+#else
+		state = GTK_WIDGET_SENSITIVE(st->b);
+#endif
 		if (m->h != NULL)	//up
 			gtk_widget_set_sensitive (m->h, state);
 	}
-	if (s2->c != NULL)	//down
+	if (st->c != NULL)	//down
 	{
-		state = GTK_WIDGET_SENSITIVE(s2->c);
+#ifdef USE_GTK2_18
+		state = gtk_widget_get_sensitive(st->c);
+#else
+		state = GTK_WIDGET_SENSITIVE(st->c);
+#endif
 		if (m->i != NULL)	//down
 			gtk_widget_set_sensitive (m->i, state);
 	}
-	if (s2->d != NULL)	//remove
+	if (st->d != NULL)	//add
 	{
-		state = GTK_WIDGET_SENSITIVE(s2->d);
+#ifdef USE_GTK2_18
+		state = gtk_widget_get_sensitive(st->d);
+#else
+		state = GTK_WIDGET_SENSITIVE(st->d);
+#endif
+		if (m->f != NULL)	//add
+			gtk_widget_set_sensitive (m->f, state);
+	}
+	if (st->e != NULL)	//add child
+	{
+#ifdef USE_GTK2_18
+		state = gtk_widget_get_sensitive(st->e);
+#else
+		state = GTK_WIDGET_SENSITIVE(st->e);
+#endif
+		if (m->g != NULL)	//add child
+			gtk_widget_set_sensitive (m->g, state);
+	}
+	if (st->f != NULL)	//remove
+	{
+#ifdef USE_GTK2_18
+		state = gtk_widget_get_sensitive(st->f);
+#else
+		state = GTK_WIDGET_SENSITIVE(st->f);
+#endif
 		if (m->a != NULL)	//cut
 			gtk_widget_set_sensitive (m->a, state);
 		if (m->b != NULL)	//copy
 			gtk_widget_set_sensitive (m->b, state);
 	}
-	if (s2->e != NULL)	//add
-	{
-		state = GTK_WIDGET_SENSITIVE(s2->e);
-		if (m->f != NULL)	//add
-			gtk_widget_set_sensitive (m->f, state);
-		}
-	if (s2->f != NULL)	//add child
-	{
-		state = GTK_WIDGET_SENSITIVE(s2->f);
-		if (m->g != NULL)	//add child
-			gtk_widget_set_sensitive (m->g, state);
-	}
 }
 /**
 @brief free memory used by tree-option context menu hash table entry
