@@ -1,6 +1,6 @@
-/* $Id: e2_config_dialog.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_config_dialog.h 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -39,15 +39,17 @@ typedef struct _E2_ConfigDialogRuntime
 	GHashTable *opthash;	//sizegroups hash
 } E2_ConfigDialogRuntime;
 
+void e2_confdlg_menu_set_position (GtkMenu *menu,
+	gint *x, gint *y, gboolean *push_in, GtkWidget *treeview);
 gboolean e2_confdlg_key_press_cb (GtkWidget *widget,
 	GdkEventKey *event, gpointer data);
 void e2_config_dialog_create (gchar *page);
 void e2_config_dialog_setup_labels (void);
 void e2_config_dialog_actions_register (void);
 
-void e2_confdlg_choose_plugin_cb (GtkWidget *widget, E2_OptionSet *set);
+void e2_confdlg_choose_plugin_cb (GtkButton *button, E2_OptionSet *set);
 #ifdef E2_RAINBOW
-void e2_confdlg_extcolorpick_cb (GtkWidget *widget, E2_OptionSet *set);
+void e2_confdlg_extcolorpick_cb (GtkButton *button, E2_OptionSet *set);
 #endif
 
 // things for single-page config dialogs
@@ -56,10 +58,10 @@ typedef struct _E2_SpecificConfDialogRuntime
 {
 	GtkWidget *dialog;
 	E2_OptionSet *set;
-	void (*apply_function) ();
+	void (*apply_function)(void);
 } E2_SpecificConfDialogRuntime;
 
-E2_SpecificConfDialogRuntime *e2_config_dialog_single
-	(gchar *set_name, void *response_cb, gboolean showit);
+E2_SpecificConfDialogRuntime *e2_config_dialog_single (gchar *set_name,
+	void (*apply_function)(void), gboolean showit);
 
 #endif //ndef __E2_CONFIG_DIALOG_H__
