@@ -1,6 +1,6 @@
-/* $Id: e2_file_info_dialog.c 863 2008-05-05 22:55:54Z tpgww $
+/* $Id: e2_file_info_dialog.c 1830 2009-09-05 22:30:33Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -26,6 +26,10 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include <time.h>
 #include <pwd.h>
 #include <grp.h>
+#ifdef E2_MAGIC
+# include <magic.h>
+# include <dlfcn.h>
+#endif
 #include "e2_dialog.h"
 
 //max length of the file type string
@@ -37,6 +41,7 @@ typedef struct _E2_FileInfoRuntime
 	gchar *localpath; //path of item being displayed
 	GtkWidget *description;
 	GtkWidget *target;
+	GtkWidget *mime;
 	GtkWidget *size;
 	GtkWidget *user;
 	GtkWidget *group;
@@ -49,112 +54,134 @@ typedef struct _E2_FileInfoRuntime
 /**
 @brief get description and mimetype of @a filepath, using the 'file' command
 @a readable_type and/or @a mime_type are set to NULL if nothing valid is available
-@param filepath localised path string of item to be processed
+@param localpath data for item to be processed, including ulocalised path string
 @param readable_type store for pointer to determined description string
 @param mime_type store for pointer to determined mime (and for text files, charset too) string
 
 @return TRUE if both description and mume commands are executed successfully (which happens if file options are bad)
 */
-static gboolean _e2_file_info_dialog_get_file_type (VPATH *filepath,
+static gboolean _e2_file_info_dialog_get_file_type (VPATH *localpath,
 	gchar **readable_type, gchar **mime_type)
 {
 #ifdef E2_VFS
-	if (e2_fs_item_is_mounted (filepath))
+	if (e2_fs_item_is_mounted (localpath))
 	{
 #endif
-	size_t bsize = 512;
-#ifdef USE_GLIB2_10
-	gchar *buf = (gchar *) g_slice_alloc ((gulong) bsize);
-#else
-	gchar *buf = (gchar *) g_try_malloc ((gulong) bsize);
-#endif
-	CHECKALLOCATEDWARNT (buf, );
-	if (buf == NULL)
-	{
-		*readable_type = NULL;
-		*mime_type = NULL;
-		return FALSE;
-	}
-	//CHECKME -b option ok ? -i option to get mimetype
-	gchar *command = g_strdup_printf ("file -bhnprs \"%s\"", VPSTR (filepath));
-	E2_FILE *pipe = e2_fs_open_pipe (command);
-	g_free (command);
-	if (pipe == NULL)
-	{
-		*readable_type = NULL;
-		*mime_type = NULL;
-		return FALSE;
-	}
+#ifdef E2_MAGIC
+		MagicIface ifc;
 
-# ifdef __USE_GNU
-	ssize_t bytes_read = getdelim (&buf, &bsize, '\n', pipe);
-	if (bytes_read > 1)	//not an empty line
-	{
-		*(buf+bytes_read-1) = '\0';	//strip trailing \n
-# else //getdelim() not available
-	gchar *strtmp = fgets (buf, bsize, pipe);
-	if (strtmp != NULL && *strtmp != '\n')
-	{
-		*(buf+strlen(buf)-1) = '\0';
-# endif
-		*readable_type = g_strdup (buf);
-	}
-	else
-	{
-		*readable_type = NULL;	//NOTE this is really an error, but returns TRUE
-	}
-	e2_fs_pipe_close (pipe);
-	//get mimetype (and for text files, encoding type) string
-	//we use -i option to get encoding for text files, instead of just --mime-type,
-	//because separate usage of  --mime-encoding does not work properly
-	command = g_strdup_printf ("file -bhnpri \"%s\"", VPSTR (filepath));
-	pipe = e2_fs_open_pipe (command);
-	g_free (command);
-	if (pipe == NULL)
-	{
-		if (*readable_type != NULL)
+		if (e2_utils_fill_magic_iface (&ifc))
 		{
-			g_free (*readable_type);
-			*readable_type = NULL;
+			magic_t handle = ifc.open (
+				MAGIC_PRESERVE_ATIME | MAGIC_RAW | MAGIC_ERROR | MAGIC_DEVICES);
+			if (handle != NULL)
+			{
+				gchar *results;
+				const gchar *msg;
+
+				ifc.load (handle, NULL); //load failure will result in NULL msg
+				msg = ifc.file (handle, VPCSTR (localpath));
+				if (msg != NULL)
+				{
+					results = g_strdup (msg);
+					g_strstrip (results);
+					*readable_type = results;
+				}
+				else
+				{
+					msg = ifc.error (handle);
+					if (msg != NULL)
+						e2_output_print_error ((gchar *)msg, FALSE);
+				}
+
+				*mime_type = e2_utils_get_mimetype (localpath);
+
+				if (*mime_type != NULL && g_str_has_prefix (*mime_type, "text/"))
+				{
+					//try to get encoding as well
+					ifc.setflags (handle, MAGIC_PRESERVE_ATIME | MAGIC_RAW | MAGIC_ERROR
+						| MAGIC_MIME_TYPE | MAGIC_MIME_ENCODING);
+					msg = ifc.file (handle, VPCSTR (localpath));
+					if (msg != NULL)
+					{
+						results = g_strdup (msg);
+						g_strstrip (results);	//get rid of \n etc
+						if (*(gchar *)results != '\0')
+						{
+							g_free (*mime_type);
+							*mime_type = results;
+						}
+						else
+							g_free (results);
+					}
+					else
+					{
+						msg = ifc.error (handle);
+						if (msg != NULL)
+							e2_output_print_error ((gchar *)msg, FALSE);
+					}
+				}
+
+				ifc.close (handle);
+				return TRUE;
+			}
+			dlclose (ifc.libhandle);
 		}
+//		e2_output_print_error ("?", FALSE);
+		*readable_type = NULL;
 		*mime_type = NULL;
 		return FALSE;
-	}
+#else
+//tag E2_BADQUOTES
+		gchar *qp = e2_utils_quote_string (VPCSTR (localpath));
+		gchar *command = e2_utils_strcat ("file -bhnprs ", qp);
+		gpointer results;
+		if (!e2_fs_get_command_output (command, &results))
+		{
+			g_free (qp);
+			g_free (command);
+			*readable_type = NULL;
+			*mime_type = NULL;
+			return FALSE;
+		}
+		g_free (command);
+		g_strstrip ((gchar *)results);	//get rid of \n etc
+		*readable_type = results;
 
-# ifdef __USE_GNU
-	bytes_read = getdelim (&buf, &bsize, '\n', pipe);
-	if (bytes_read > 1)	//not an empty line
-	{
-		*(buf+bytes_read-1) = '\0';	//strip trailing \n
-# else //getdelim() not available
-	strtmp = fgets (buf, bsize, pipe);
-	if (strtmp != NULL && *strtmp != '\n')
-	{
-		*(buf+strlen(buf)-1) = '\0';
-# endif
-		*mime_type = g_strdup (buf);
-	}
-	else
-	{
-		*mime_type = NULL;
-	}
-	e2_fs_pipe_close (pipe);
-# ifdef USE_GLIB2_10
-	g_slice_free1 (bsize, buf);
-# else
-	g_free (buf);
-# endif
-	return TRUE;
+		*mime_type = e2_utils_get_mimetype (localpath);
+
+		if (*mime_type != NULL && g_str_has_prefix (*mime_type, "text/"))
+		{
+			//try to get encoding as well
+			command = e2_utils_strcat ("file -bhnpri ", qp);
+			if (e2_fs_get_command_output (command, &results))
+			{
+				g_strstrip ((gchar *)results);	//get rid of \n etc
+				if (*(gchar *)results != '\0')
+				{
+					g_free (*mime_type);
+					*mime_type = results;
+				}
+				else
+					g_free (results);
+			}
+			g_free (command);
+		}
+		g_free (qp);
+
+		return TRUE;
+#endif
 #ifdef E2_VFS
 	}
 	else	//not a local item
 	{
 # ifdef E2_VFSTMP
 		FIXME
-# endif
+# else
 		*readable_type = NULL;
 		*mime_type = NULL;
 		return FALSE;
+# endif
 	}
 #endif
 }
@@ -167,9 +194,9 @@ set @a push_in to TRUE for menu completely inside the screen,
 FALSE for menu clamped to screen size
 
 @param menu the GtkMenu to be positioned
-@param x	place to store gint representing the menu left
-@param y  place to store gint representing the menu top
-@param push_in place to store pushin flag
+@param x pointer to store for left-position of @a menu
+@param y pointer to store for top-position of @a menu
+@param push_in pointer to store for pushin flag
 @param dialog the dialog widget in focus when the menu key was pressed
 
 @return
@@ -179,7 +206,12 @@ static void _e2_file_info_dialog_set_menu_position (GtkWidget *menu,
 {
 	gint left, top;
 	gtk_window_get_position (GTK_WINDOW (dialog), &left, &top);
-	GtkAllocation alloc = dialog->allocation;
+	GtkAllocation alloc;
+#ifdef USE_GTK2_18
+	gtk_widget_get_allocation (dialog, &alloc);
+#else
+	alloc = dialog->allocation;
+#endif
 	*x = left + alloc.x + alloc.width/2;
 	*y = top + alloc.y + alloc.height/2;
 	*push_in = FALSE;
@@ -187,12 +219,12 @@ static void _e2_file_info_dialog_set_menu_position (GtkWidget *menu,
 /**
 @brief copy dialog data to clipboard
 
-@param menuitem UNUSED the selected widget
+@param menuitem UNUSED the activated widget
 @param rt runtime struct to work on
 
 @return
 */
-static void _e2_file_info_dialog_copy_cb (GtkWidget *menuitem,
+static void _e2_file_info_dialog_copy_cb (GtkMenuItem *menuitem,
 	E2_FileInfoRuntime *rt)
 {
 	//some format massaging is needed ...
@@ -210,27 +242,42 @@ static void _e2_file_info_dialog_copy_cb (GtkWidget *menuitem,
 		typelabel = g_strconcat (_("Type:"), " ", NULL);
 
 	GString *text = g_string_sized_new (512);
-	g_string_printf (text,
-		"%s %s\n" "%s%s%s\n" "%s %s\n" "%s %s\n""%s %s\n" "%s %s\n" "%s %s\n" "%s %s\n" "%s %s\n",
-		_("Item:"), utf,
-		typelabel, type, target,
+	g_string_printf (text, "%s %s\n" "%s%s%s\n",
+		_("Item:"), utf, typelabel, type, target);
+	if (rt->mime != NULL)
+	{
+		//the mime string may have 2 lines, if so the 2nd is encoding
+		const gchar *mime = gtk_label_get_text (GTK_LABEL(rt->mime));
+		const gchar *s = strchr (mime, '\n');
+		if (s == NULL)
+			g_string_append_printf (text, "%s %s\n", _("Mime:"), mime);
+		else
+		{
+			gchar *freeme = g_strndup (mime, s - mime);
+			g_string_append_printf (text, "%s %s\n" "%s %s\n",
+				_("Mime:"), freeme, _("Encoding:"), s + sizeof (gchar));
+			g_free (freeme);
+		}
+	}
+	g_string_append_printf (text,
+ 		"%s %s\n" "%s %s\n" "%s %s\n" "%s %s\n" "%s %s\n" "%s %s\n" "%s %s\n",
 		_("Size:"), gtk_label_get_text (GTK_LABEL(rt->size)),
 		_("User:"), gtk_label_get_text (GTK_LABEL(rt->user)),
 		_("Group:"), gtk_label_get_text (GTK_LABEL(rt->group)),
 		_("Permissions:"), gtk_label_get_text (GTK_LABEL(rt->permissions)),
 		_("Accessed:"), gtk_label_get_text (GTK_LABEL(rt->accessdate)),
-		_("Modified:"), gtk_label_get_text (GTK_LABEL(rt->moddate)),
-		_("Changed:"), gtk_label_get_text (GTK_LABEL(rt->changedate))
+		_("Content modified:"), gtk_label_get_text (GTK_LABEL(rt->moddate)),
+		_("Property changed:"), gtk_label_get_text (GTK_LABEL(rt->changedate))
 	);
 	GtkClipboard *cb = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
 	gtk_clipboard_set_text (cb, text->str, text->len);
-	F_FREE (utf);
+	F_FREE (utf, rt->localpath);
 	if (*typelabel != '\0')
 		g_free (typelabel);
 	g_string_free (text, TRUE);
 }
 /**
-@brief construct and show dialog context menu
+@brief construct and pop up destroyable context-menu for @a dialog
 
 @param textview the textview widget where the click happened
 @param event_button which mouse button was clicked (0 for a menu key)
@@ -240,17 +287,17 @@ static void _e2_file_info_dialog_copy_cb (GtkWidget *menuitem,
 @return
 */
 static void _e2_file_info_dialog_show_context_menu (GtkWidget *dialog,
-	guint event_button, gint event_time, E2_FileInfoRuntime *rt)
+	guint event_button, guint32 event_time, E2_FileInfoRuntime *rt)
 {
 	GtkWidget *menu = gtk_menu_new ();
 	e2_menu_add (menu, _("_Copy"), GTK_STOCK_COPY,
 		_("Copy displayed data"), _e2_file_info_dialog_copy_cb, rt);
 	g_signal_connect (G_OBJECT (menu), "selection-done",
-		G_CALLBACK (e2_menu_destroy_cb), NULL);
+		G_CALLBACK (e2_menu_selection_done_cb), NULL);
 	if (event_button == 0)
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
 			(GtkMenuPositionFunc) _e2_file_info_dialog_set_menu_position,
-			rt->dialog, event_button, event_time);
+			rt->dialog, 0, event_time);
 	else
 		//this was a button-3 click
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
@@ -283,7 +330,11 @@ static gboolean _e2_file_info_dialog_popup_menu_cb (GtkWidget *dialog,
 static gboolean _e2_file_info_dialog_button_press_cb (GtkWidget *dialog,
 	GdkEventButton *event, E2_FileInfoRuntime *rt)
 {
-	if (event->button == 3)
+	if (event->button == 3
+#ifdef E2_MOUSECUSTOM
+		&& (event->state & E2_MODIFIER_MASK) == 0
+#endif
+		)
 	{
 		_e2_file_info_dialog_show_context_menu (dialog, 3, event->time, rt);
 		return TRUE;
@@ -291,29 +342,8 @@ static gboolean _e2_file_info_dialog_button_press_cb (GtkWidget *dialog,
 	return FALSE;
 }
 /**
-@brief dialog response callback
-
-@param dialog the permissions-dialog where the response was triggered
-@param response the response for the clicked button
-@param result pointer to store for the dialog return value
-
-@return
-*/
-static void _e2_file_info_dialog_response_cb (GtkDialog *dialog, gint response,
-	DialogButtons *result)
-{
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-	if (response == GTK_RESPONSE_CLOSE)	//not covered in the general decoder
-	{
-		*result = CANCEL;
-		gtk_main_quit ();
-	}
-	else
-		e2_dialog_response_decode_cb (dialog, response, result);
-}
-/**
 @brief create and run file info dialog
-
+BGL expected to be off/open on arrival
 @param localpath localised path string of item to be processed
 @param multi TRUE when the dialog needs buttons consistent with >1 item being processed
 
@@ -321,7 +351,6 @@ static void _e2_file_info_dialog_response_cb (GtkDialog *dialog, gint response,
 */
 DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 {
-	GtkWidget *dialog;
 	GtkWidget *dialog_vbox;
 	GtkWidget *table;
 	GString *label_text = g_string_sized_new (NAME_MAX + 32);  //this is plenty big for all uses here
@@ -333,17 +362,29 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 	struct passwd *pw_buf;
 	struct group *grp_buf;
 	E2_FileInfoRuntime rt;
-	DialogButtons choice;
+
+//tag DEBUGfreeze;
+	//maybe locking gtk out of X during dialog setup will help
+	//CHECKME effects of child-signals received due to file-command pipe ?
+//	gdk_threads_enter ();
 
 	if (e2_fs_lstat (localpath, &statbuf E2_ERR_NONE()))
-//explain to user ??
+	{
+//tag DEBUGfreeze;
+		gdk_threads_leave ();
+		//CHECKME explain to user ?
 		return CANCEL;
+	}
 
 	//must prevent the default dialog response, it stuffs up Q cleanups
-	rt.dialog =
-	dialog = e2_dialog_create (NULL, NULL, _("file info"),
-		_e2_file_info_dialog_response_cb, &choice);
-	dialog_vbox = GTK_DIALOG (dialog)->vbox;
+	rt.dialog = e2_dialog_create (NULL, NULL, _("file info"),
+		DUMMY_RESPONSE_CB, NULL);
+	dialog_vbox =
+#ifdef USE_GTK2_14
+		gtk_dialog_get_content_area (GTK_DIALOG (rt.dialog));
+#else
+		GTK_DIALOG (rt.dialog)->vbox;
+#endif
 	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), E2_PADDING);
 
 	rt.localpath = VPSTR (localpath);	//store data for copying
@@ -445,8 +486,8 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 		{
 			g_string_printf (label_text, "%s %s", _("Type:"), s2);
 			//CHECKME escape label ?
-			rt.description =
-			e2_widget_add_mid_label (dialog_vbox, label_text->str, 0, TRUE, 0);
+			rt.description = e2_widget_add_mid_label (dialog_vbox,
+				label_text->str, 0, TRUE, 0);
 		}
 		if (readable != NULL)
 		{
@@ -505,9 +546,9 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 	{
 		gchar target[PATH_MAX];
 		gchar *filename = g_strdup (VPSTR (localpath));
-		gint len = strlen (filename);
-		if (filename[len-1] == G_DIR_SEPARATOR)
-			filename[len-1] = '\0';
+		gint len = strlen (filename) - sizeof(gchar);
+		if (len > 0 && filename[len] == G_DIR_SEPARATOR)
+			filename[len] = '\0';
 #ifdef E2_VFS
 		VPATH ddata = { filename, localpath->spacedata };
 		len = e2_fs_readlink (&ddata, target, sizeof (target) E2_ERR_NONE());
@@ -516,17 +557,17 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 #endif
 		if (len > 0)
 		{
-			target[len] = '\0';
 			s1 = F_DISPLAYNAME_FROM_LOCALE (target);
 			s2 = g_markup_escape_text (s1, -1);
 			label_text = g_string_assign (label_text, " ");
 			g_string_append_printf (label_text, _("to %s"), s2);
-			F_FREE (s1);
+			F_FREE (s1, target);
 			g_free (s2);
-			//statting of relative targets fails, so ...
+			//stat() fails for relative targets, so ...
 			if (g_str_has_prefix (target, "."G_DIR_SEPARATOR_S))
 			{
-				memcpy (target, (target + 2 * sizeof (gchar)), len);
+				memcpy (target, (target + 2 * sizeof (gchar)), len - 1);
+				len -= 2;
 			}
 			if (g_str_has_prefix (target, ".."G_DIR_SEPARATOR_S))
 			{
@@ -536,13 +577,13 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 #else
 				s2 = e2_utils_translate_relative_path (curr_view->dir, s1);
 #endif
-				F_FREE (s1);
+				F_FREE (s1, target);
 				s1 = F_FILENAME_TO_LOCALE (s2);
 				g_strlcpy (target, s1, sizeof(target));
 				//the translation adds a trailing separator, which lstat assumes to be a dir match ..
 				len = strlen (target);
 				target[len-1] = '\0';
-				F_FREE (s1);
+				F_FREE (s1, s2);
 				g_free (s2);
 			}
 			//FIXME get intelligent absolute path
@@ -552,41 +593,45 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 #ifdef E2_VFSTMP
 				local = ; FIXME
 #else
-				local = F_FILENAME_TO_LOCALE (curr_view->dir);
+				local = D_FILENAME_TO_LOCALE (curr_view->dir); //always dup, to avoid dirchange race
 #endif
 				gchar *t2 = g_build_filename (local, target, NULL);
 				g_strlcpy (target, t2, sizeof (target));
-				F_FREE (local);
+				g_free (local);
 				g_free (t2);
 			}
 			E2_ERR_DECLARE
 			//check the real target, without looking thru chained links
 			struct stat statbuf2;	//don't disturb the size etc data for the link itelf
 #ifdef E2_VFS
-			ddata.localpath = target;
+			ddata.path = target;
 			if (e2_fs_lstat (&ddata, &statbuf2 E2_ERR_PTR()) && E2_ERR_IS (ENOENT))
 #else
 			if (e2_fs_lstat (target, &statbuf2 E2_ERR_PTR()) && E2_ERR_IS (ENOENT))
 #endif
 				g_string_append_printf (label_text, "\n %s",_("(which is <b>missing</b>)"));
-			else if (g_str_equal (localpath, target))
+			else if (!strcmp (localpath, target))
 				g_string_append_printf (label_text, "\n %s",_("(which is <b>itself</b>)"));
 			else
 			{
 				if (e2_fs_walk_link (&filename E2_ERR_NONE()))
 				{
 					printd (DEBUG, "final target is %s", filename);
-					if (!g_str_equal (filename, target))
+					if (strcmp (filename, target))
 					{
-						s1 = F_FILENAME_FROM_LOCALE (filename);
-#ifdef E2_VFSTMP
-						//FIXME which CWD for the baseline
+						s1 = D_FILENAME_TO_LOCALE (curr_view->dir); //always dup, to avoid dirchange race
+#ifdef E2_VFS
+						VPATH data2 = { s1, localpath->spacedata };
+						ddata.path = filename;
+						s2 = e2_utils_create_relative_path (&ddata, &data2);
 #else
-						s2 = e2_utils_create_relative_path (s1, curr_view->dir);
+						s2 = e2_utils_create_relative_path (filename, s1);
 #endif
+						g_free (s1);
+						s1 = F_FILENAME_FROM_LOCALE (s2);
 						label_text = g_string_append (label_text, "\n ");
-						g_string_append_printf (label_text, _("and ultimately to %s"), s2);
-						F_FREE (s1);
+						g_string_append_printf (label_text, _("and ultimately to %s"), s1);
+						F_FREE (s1, s2);
 						g_free (s2);
 					}
 				}
@@ -594,12 +639,12 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 
 			E2_ERR_CLEAR
 
-			rt.target =
-			e2_widget_add_mid_label (dialog_vbox, label_text->str, 0, TRUE, 0);
+			rt.target = e2_widget_add_mid_label (dialog_vbox,
+				label_text->str, 0, TRUE, 0);
 		}
 		else
-			rt.target =
-			e2_widget_add_mid_label (dialog_vbox, _("(Cannot resolve the link)"), 0, TRUE, 0);
+			rt.target = e2_widget_add_mid_label (dialog_vbox,
+				_("(Cannot resolve the link)"), 0, TRUE, 0);
 
 		g_free (filename);
 	}
@@ -634,13 +679,15 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 			s2 = mime;
 		}
 		e2_widget_add_mid_label_to_table (table, s1, 0, 0,1,0,1);
-		e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,0,1);
+		rt.mime = e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,0,1);
 		if (s2 != mime)
 		{
 			g_free (s1);
 			g_free (s2);
 		}
 	}
+	else
+		rt.mime = NULL;
 
 	e2_widget_add_mid_label_to_table (table, _("Size:"), 0, 0,1,1,2);
 
@@ -685,7 +732,7 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 	rt.group =
 	e2_widget_add_mid_label_to_table (table, label_text->str, 0, 1,2,4,5);
 	gchar *perm_string = e2_fs_get_perm_string (statbuf.st_mode);
-	g_string_assign (label_text, g_utf8_next_char (perm_string));	//skip the "type code" in 1st byte of string
+	label_text = g_string_assign (label_text, g_utf8_next_char (perm_string));	//skip the "type code" in 1st byte of string
 	g_free (perm_string);
 	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 6), ' ');
 	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 3), ' ');
@@ -703,67 +750,60 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 	e2_widget_add_mid_label_to_table (table, _("Accessed:"), 0, 0,1,7,8);
 	rt.accessdate =
 	e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,7,8);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		rt.accessdate, _("Item was last opened, run, read etc"));
+	e2_widget_set_safetip (rt.accessdate, _("Item was last opened, run, read etc"));
 	g_free (s2);
 
 	tm_ptr = localtime (&statbuf.st_mtime);
 	strftime (date_string, sizeof (date_string), s1, tm_ptr);
 	s2 = e2_utf8_from_locale (date_string);
-	e2_widget_add_mid_label_to_table (table, _("Modified:"), 0, 0,1,8,9);
+	e2_widget_add_mid_label_to_table (table, _("Content modified:"), 0, 0,1,8,9);
 	rt.moddate =
 	e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,8,9);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		rt.moddate, _("Content of the item was last altered"));
+	e2_widget_set_safetip (rt.moddate, _("Item content was last changed"));
 	g_free (s2);
 
 	tm_ptr = localtime (&statbuf.st_ctime);
 	strftime (date_string, sizeof (date_string), s1, tm_ptr);
 	s2 = e2_utf8_from_locale (date_string);
-	e2_widget_add_mid_label_to_table (table, _("Changed:"), 0, 0,1,9,10);
+	e2_widget_add_mid_label_to_table (table, _("Property changed:"), 0, 0,1,9,10);
 	rt.changedate =
 	e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,9,10);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		rt.changedate, _("Property of the item (name, permission, owner etc) was last altered"));
+	e2_widget_set_safetip (rt.changedate,
+		_("Item name, permission, owner, etc was last changed"));
 	g_free (s2);
 
 	g_string_free (label_text,TRUE);
 	if (mime != NULL)
 		g_free (mime);
 
+//tag DEBUGfreeze2;
+//	gdk_threads_leave ();
+
+	//block until user responds
+	E2_DialogFlags flags = E2_DIALOG_BLOCKED | E2_DIALOG_CLOSELOCK | E2_DIALOG_FREE;
+
+	E2_Button local_btn;
 	if (multi)
 	{
-		E2_BUTTON_NOTOALL.showflags &= ~E2_BTN_DEFAULT;
-		e2_dialog_add_defined_button (dialog, &E2_BUTTON_NOTOALL);
-		e2_dialog_set_negative_response (dialog, E2_RESPONSE_NOTOALL);
+		e2_dialog_set_negative_response (rt.dialog, E2_RESPONSE_NOTOALL);
+		local_btn = E2_BUTTON_CANCEL;
+		local_btn.showflags &= ~E2_BTN_DEFAULT;
+		e2_dialog_add_defined_button (rt.dialog, &local_btn);
+		flags |= E2_DIALOG_MULTI;
 	}
-	E2_BUTTON_CLOSE.showflags |= E2_BTN_DEFAULT;
-	e2_dialog_add_defined_button (dialog, &E2_BUTTON_CLOSE);
+	local_btn = E2_BUTTON_CLOSE;
+	local_btn.showflags |= E2_BTN_DEFAULT;
+	e2_dialog_add_defined_button (rt.dialog, &local_btn);
 	//enable menu
-	g_signal_connect (G_OBJECT (dialog), "popup-menu",
+	g_signal_connect (G_OBJECT (rt.dialog), "popup-menu",
 		G_CALLBACK (_e2_file_info_dialog_popup_menu_cb), &rt);
-	g_signal_connect (G_OBJECT (dialog), "button-press-event",
+	g_signal_connect (G_OBJECT (rt.dialog), "button-press-event",
 		G_CALLBACK (_e2_file_info_dialog_button_press_cb), &rt);
 
-	e2_dialog_setup (dialog, app.main_window);
-
+//tag DEBUGfreeze
 	gdk_threads_enter ();
-	e2_dialog_run (dialog, app.main_window, 0);
-	gtk_window_present (GTK_WINDOW (dialog));	//sometimes, dialog is not focused
-	gtk_main ();
+	e2_dialog_setup (rt.dialog, app.main_window);
 	gdk_threads_leave ();
 
-	return choice;
+	return (e2_dialog_run (rt.dialog, NULL, flags));
 }
