@@ -1,6 +1,6 @@
-/* $Id: e2_filetype_dialog.c 854 2008-04-19 11:45:33Z tpgww $
+/* $Id: e2_filetype_dialog.c 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -93,7 +93,7 @@ static void _e2_edftdlg_save_current_category (E2_FileTypeDlgRuntime *rt)
 		{  // extension or command loop = level 2
 			cfgchildren = gtk_tree_model_iter_n_children (cfgmdl, &cfg_iter2);
 			gtk_tree_model_get (cfgmdl, &cfg_iter2, 1, &extension, -1);
-			if (g_str_equal (extension, _C(13)))  //extensions node found
+			if (!strcmp (extension, _C(13)))  //extensions node found
 			{
 				g_free (extension);
 				mdl = GTK_TREE_MODEL (rt->stores[E2_EXTPANE]);
@@ -145,7 +145,7 @@ static void _e2_edftdlg_save_current_category (E2_FileTypeDlgRuntime *rt)
 					}
 				}
 			}
-			else if (g_str_equal (extension, _C(6)))  //commands node found
+			else if (!strcmp (extension, _C(6)))  //commands node found
 			{
 				g_free (extension);
 				mdl = GTK_TREE_MODEL (rt->stores[E2_CMDPANE]);
@@ -261,7 +261,7 @@ static void _e2_edftdlg_update_category (GtkTreePath *path,
 				gtk_tree_model_get (cfgmdl, &iter2, 1, &extension, -1);
 				if (gtk_tree_model_iter_children (cfgmdl, &iter3, &iter2))
 				{
-					if (g_str_equal (extension, _C(13)))  //extensions node found
+					if (!strcmp (extension, _C(13)))  //extensions node found
 					{
 						g_free (extension);
 						do
@@ -280,7 +280,7 @@ static void _e2_edftdlg_update_category (GtkTreePath *path,
 							g_free (extension);
 						} while (gtk_tree_model_iter_next (cfgmdl, &iter3));
 					}
-					else if (g_str_equal (extension, _C(6)))  //commands node found
+					else if (!strcmp (extension, _C(6)))  //commands node found
 					{
 						g_free (extension);
 						do
@@ -380,12 +380,12 @@ static gboolean _e2_edftdlg_find_selected
 		return (gtk_tree_selection_get_selected (sel, &mdl, iter));
 
 	gboolean retval;
-	GList *items = gtk_tree_selection_get_selected_rows (sel, &mdl);
-	if (items != NULL)
+	GList *selpaths = gtk_tree_selection_get_selected_rows (sel, &mdl);
+	if (selpaths != NULL)
 	{
-		retval = gtk_tree_model_get_iter (mdl, iter, (GtkTreePath *)items->data);
-		g_list_foreach (items, (GFunc) gtk_tree_path_free, NULL);
-		g_list_free (items);
+		retval = gtk_tree_model_get_iter (mdl, iter, (GtkTreePath *)selpaths->data);
+		g_list_foreach (selpaths, (GFunc) gtk_tree_path_free, NULL);
+		g_list_free (selpaths);
 	}
 	else
 		retval = FALSE;
@@ -486,13 +486,7 @@ static void _e2_edftdlg_response_cb (GtkDialog *dialog, gint response,
 	GtkTreeIter iter;
 	switch (response)
 	{
-		case GTK_RESPONSE_APPLY:
-			_e2_edftdlg_save_current_category (rt); //this might be dirty
-			e2_option_tree_unbackup (rt->typeset, FALSE);	//get rid of old backup data
-			e2_filetype_apply_allnew ();	//convert config data to runtime format
-			e2_option_tree_backup (rt->typeset);	//get fresh backup data
-			break;
-		case GTK_RESPONSE_OK:
+		case E2_RESPONSE_APPLY:
 			_e2_edftdlg_save_current_category (rt); //this might be dirty
 			e2_option_tree_unbackup (rt->typeset, FALSE);	//get rid of old backup data
 			g_hash_table_destroy (rt->buffer_hash);
@@ -502,6 +496,12 @@ static void _e2_edftdlg_response_cb (GtkDialog *dialog, gint response,
 			e2_filetype_apply_allnew ();
 			gtk_widget_grab_focus (curr_view->treeview);
 			break;
+		case E2_RESPONSE_MORE:
+			_e2_edftdlg_save_current_category (rt); //this might be dirty
+			e2_option_tree_unbackup (rt->typeset, FALSE);	//get rid of old backup data
+			e2_filetype_apply_allnew ();	//convert config data to runtime format
+			e2_option_tree_backup (rt->typeset);	//get fresh backup data
+			break;
 		case E2_RESPONSE_USER1:
 			//move-up button pressed
 			if (_e2_edftdlg_find_selected (rt, &iter))
@@ -582,12 +582,12 @@ static void _e2_edftdlg_response_cb (GtkDialog *dialog, gint response,
 /**
 series of context-menu callback functions
 
-@param item the selected menu item widget
+@param item the activated menu item widget
 @param rt pointer to dialog data struct
 
 @return
 */
-static void _e2_edftdlg_menu_copy_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt)
+static void _e2_edftdlg_menu_copy_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
 	GtkTreeView *treeview = GTK_TREE_VIEW (rt->views[rt->currentID]);
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
@@ -600,7 +600,7 @@ static void _e2_edftdlg_menu_copy_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt
 		g_hash_table_replace (rt->buffer_hash, id_string, copied);
 	}
 }
-static void _e2_edftdlg_menu_cut_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt)
+static void _e2_edftdlg_menu_cut_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
 	GtkTreeView *treeview = GTK_TREE_VIEW (rt->views[rt->currentID]);
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
@@ -610,7 +610,7 @@ static void _e2_edftdlg_menu_cut_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt)
 		e2_tree_delete (treeview);
 	}
 }
-static void _e2_edftdlg_menu_paste_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt)
+static void _e2_edftdlg_menu_paste_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
 	//as a special case, if the store is empty, setup to add the data
 	gboolean empty = (gtk_tree_model_iter_n_children
@@ -634,15 +634,15 @@ static void _e2_edftdlg_menu_paste_cb (GtkWidget *item, E2_FileTypeDlgRuntime *r
 	if (empty)
 		e2_tree_delete (treeview);
 }
-static void _e2_edftdlg_menu_add_below_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt)
+static void _e2_edftdlg_menu_add_below_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
 	_e2_edftdlg_insert_row (rt);
 }
-static void _e2_edftdlg_menu_move_up_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt)
+static void _e2_edftdlg_menu_move_up_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
 	_e2_edftdlg_response_cb (GTK_DIALOG (filetypes_dialog), E2_RESPONSE_USER1, rt);
 }
-static void _e2_edftdlg_menu_move_down_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt)
+static void _e2_edftdlg_menu_move_down_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
 	_e2_edftdlg_response_cb (GTK_DIALOG (filetypes_dialog), E2_RESPONSE_USER2, rt);
 }
@@ -667,13 +667,18 @@ static void _e2_edftdlg_set_menu_position (GtkMenu *menu,
 {
 	gint left, top;
 	gtk_window_get_position (GTK_WINDOW (filetypes_dialog), &left, &top);
-	GtkAllocation alloc = treeview->allocation;
+	GtkAllocation alloc;
+#ifdef USE_GTK2_18
+	gtk_widget_get_allocation (treeview, &alloc);
+#else
+	alloc = treeview->allocation;
+#endif
 	*x = left+ alloc.x + alloc.width/2;
 	*y = top + alloc.y +alloc.height/2 - 30;
 	*push_in = FALSE;
 }
 /**
-@brief create treeview context menu
+@brief construct and pop up destroyable context-menu for the filetype-dialog
 
 @param treeview widget where the action is to occur
 @param button the clicked mouse button, or 0 for menu key
@@ -707,6 +712,9 @@ static void _e2_edftdlg_menu_create (GtkWidget *treeview,
 	e2_menu_add (menu, _("_Down"), GTK_STOCK_GO_DOWN,
 		_("Move selected row down"), _e2_edftdlg_menu_move_down_cb, rt);
 
+	g_signal_connect (G_OBJECT (menu), "selection-done",
+		G_CALLBACK (e2_menu_selection_done_cb), NULL);
+
 	if (button == 0)
 		//this was a menu-key press
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
@@ -739,11 +747,15 @@ static gboolean _e2_edftdlg_popup_menu_cb (GtkWidget *treeview,
 
 @return FALSE
 */
-//FIXME find a way to cancel editing of a renderer that is clicked again
 static gboolean _e2_edftdlg_button_press_cb (GtkWidget *treeview,
 	GdkEventButton *event, E2_FileTypeDlgRuntime *rt)
 {
-	if (event->button == 1 && event->type == GDK_BUTTON_PRESS)
+//FIXME find a way to cancel editing of a renderer that is clicked again
+	if (event->button == 1 && event->type == GDK_BUTTON_PRESS //no multi-clicks
+#ifdef E2_MOUSECUSTOM
+		&& (event->state & E2_MODIFIER_MASK) == 0
+#endif
+		)
 	{
 		GtkTreePath *path;
 		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview),
@@ -764,7 +776,11 @@ static gboolean _e2_edftdlg_button_press_cb (GtkWidget *treeview,
 		else
 			rt->currentID = E2_CMDPANE;
 	}
-	else if (event->button == 3)
+	else if (event->button == 3
+#ifdef E2_MOUSECUSTOM
+			&& (event->state & E2_MODIFIER_MASK) == 0
+#endif
+		)
 	{
 		_e2_edftdlg_menu_create (treeview, 3, event->time, rt);
 		return TRUE;
@@ -832,24 +848,29 @@ void e2_filetype_dialog_edit_create (gchar *open_category)
 		g_free, (GDestroyNotify) e2_option_tree_menu_hash_clean);
 
 	filetypes_dialog = e2_dialog_create (NULL, NULL, _("edit filetypes"),
-		_e2_edftdlg_response_cb, rt);
+		(ResponseFunc)_e2_edftdlg_response_cb, rt);
 
 	e2_option_connect (filetypes_dialog, FALSE);
 
 	//main scrolled window
-	GtkWidget *sw = e2_widget_get_sw (GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC,
-		GTK_SHADOW_IN);
-//	gtk_scrolled_window_set_placement (GTK_SCROLLED_WINDOW (sw),
-//		e2_option_int_get ("scrollbar-position"));
-	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (filetypes_dialog)->vbox), sw);
+	GtkWidget *sw = e2_widget_add_sw (
+#ifdef USE_GTK2_14
+		gtk_dialog_get_content_area (GTK_DIALOG (filetypes_dialog)),
+#else
+		GTK_DIALOG (filetypes_dialog)->vbox,
+#endif
+		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC, TRUE, E2_PADDING_SMALL);
 	gtk_widget_set_size_request (sw, 0, 400);
-	gtk_widget_show (sw);
 
 	GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), hbox);
-	gtk_viewport_set_shadow_type (GTK_VIEWPORT (GTK_BIN (sw)->child),
+	gtk_viewport_set_shadow_type (
+#ifdef USE_GTK2_14
+		GTK_VIEWPORT (gtk_bin_get_child (GTK_BIN (sw))),
+#else
+		GTK_VIEWPORT (GTK_BIN (sw)->child),
+#endif
 		GTK_SHADOW_NONE);
-	gtk_widget_show (hbox);
 
 	//categories scrolled window
 	GtkWidget *cat_sw = e2_widget_get_sw (GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC,
@@ -881,7 +902,6 @@ void e2_filetype_dialog_edit_create (gchar *open_category)
 #endif
 	gtk_container_add (GTK_CONTAINER (cat_sw), rt->views[E2_CATPANE]);
 	gtk_widget_set_size_request (cat_sw, 150, 0);
-	gtk_widget_show_all (cat_sw);
 
 	//extensions scrolled window
 	GtkWidget *ext_sw = e2_widget_get_sw (GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC,
@@ -903,7 +923,6 @@ void e2_filetype_dialog_edit_create (gchar *open_category)
 		(GTK_TREE_VIEW (rt->views[E2_EXTPANE]), -1, _("Extensions"), renderer, "text", 0, NULL);
 	gtk_container_add (GTK_CONTAINER (ext_sw), rt->views[E2_EXTPANE]);
 	gtk_widget_set_size_request (ext_sw, 100, 0);
-	gtk_widget_show_all (ext_sw);
 	GtkTreeSelection *selection = gtk_tree_view_get_selection
 		(GTK_TREE_VIEW (rt->views[E2_EXTPANE]));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
@@ -940,7 +959,6 @@ void e2_filetype_dialog_edit_create (gchar *open_category)
 
 	gtk_container_add (GTK_CONTAINER (cmd_sw), rt->views[E2_CMDPANE]);
 	gtk_widget_set_size_request (cmd_sw, 400, 0);
-	gtk_widget_show_all (cmd_sw);
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (rt->views[E2_CMDPANE]));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 
@@ -998,30 +1016,30 @@ void e2_filetype_dialog_edit_create (gchar *open_category)
 		G_CALLBACK (_e2_edftdlg_focus_in_cb), rt);
 
 	//add buttons, custom and standard
-	E2_Button buttontype;
-	buttontype.label = _("_Up");
-	buttontype.stock = GTK_STOCK_GO_UP;
-	buttontype.response = E2_RESPONSE_USER1;
-	e2_dialog_add_button_custom (filetypes_dialog, FALSE, &buttontype,
-		_("Move the selected row one place up"), NULL, NULL);
+	E2_Button local_btn = { _("_Up"), GTK_STOCK_GO_UP,
+		_("Move the selected row one place up"),
+		E2_BTN_TIPPED, E2_BTN_TIPPED, E2_RESPONSE_USER1 };
+	e2_dialog_add_defined_button (filetypes_dialog, &local_btn);
 
-	buttontype.label = _("_Down");
-	buttontype.stock = GTK_STOCK_GO_DOWN;
-	buttontype.response = E2_RESPONSE_USER2;
-	e2_dialog_add_button_custom (filetypes_dialog, FALSE, &buttontype,
-		_("Move the selected row one place down"), NULL, NULL);
+	local_btn.label = _("_Down");
+	local_btn.stock = GTK_STOCK_GO_DOWN;
+	local_btn.tip = _("Move the selected row one place down");
+	local_btn.response = E2_RESPONSE_USER2;
+	e2_dialog_add_defined_button (filetypes_dialog, &local_btn);
 
-	buttontype.label = _("Add a_fter");
-	buttontype.stock = GTK_STOCK_ADD;
-	buttontype.response = E2_RESPONSE_USER3;
-	e2_dialog_add_button_custom (filetypes_dialog, FALSE, &buttontype,
-		_("Add a row after the currently selected one"), NULL, NULL);
+	local_btn.label = _("_Add");
+	local_btn.stock = GTK_STOCK_ADD;
+	local_btn.tip = _("Add a row after the currently selected one");
+	local_btn.response = E2_RESPONSE_USER3;
+	e2_dialog_add_defined_button (filetypes_dialog, &local_btn);
 
-	E2_BUTTON_CANCEL.showflags |= E2_BTN_DEFAULT;
-	E2_BUTTON_OK.showflags &= ~E2_BTN_DEFAULT;
+//	e2_button_derive (&local_btn, &E2_BUTTON_NO, BTN_NO_CANCEL);
+	local_btn = E2_BUTTON_DISCARD;
+	local_btn.showflags |= E2_BTN_DEFAULT;
+
+	E2_BUTTON_APPLY.showflags &= ~E2_BTN_DEFAULT;	//CHECKME local copy ?
 	e2_dialog_show (filetypes_dialog, app.main_window, 0,
-		&E2_BUTTON_REMOVE, &E2_BUTTON_CANCEL,
-		&E2_BUTTON_APPLY, &E2_BUTTON_OK, NULL);
+		&E2_BUTTON_REMOVE, &E2_BUTTON_MORE, &local_btn, &E2_BUTTON_APPLY, NULL);
 }
 
 #endif	//ndef OLDFTDLG
@@ -1034,8 +1052,7 @@ typedef struct _E2_FileTypeDlg2Runtime
 {
 	GtkWidget *view_btn;
 	GtkWidget *open_btn;
-	gchar *utfpath;	//utf8 absolute path of item
-	gint processed;	//store for return-code
+	gchar *utfpath;	//UTF8 absolute path of item
 } E2_FileTypeDlg2Runtime;
 
 /**
@@ -1043,84 +1060,21 @@ typedef struct _E2_FileTypeDlg2Runtime
 
 @param dialog the dialog where the response was triggered
 @param response the number assigned the activated widget
-@param view rt data for the dialog
+@param store pointer to store for enumof user's choice
 
 @return
 */
 static void _e2_filetype_dialog_response_cb (GtkDialog *dialog, gint response,
-	E2_FileTypeDlg2Runtime *rt)
+	gint *store)
 {
 	printd (DEBUG, "response_cb (dialog:_,response:%d,rt:_)", response);
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-	gchar *action;
-	//remember this when rt is cleared at end of dialog func, and handle any whitespace
-	gchar *item = g_strconcat ("\"", rt->utfpath, "\"", NULL);
-	//send back "handled" type flag to caller
-	switch (response)
-	{
-		case E2_RESPONSE_USER1:	//edit
-		case E2_RESPONSE_USER2:	//view
-		case E2_RESPONSE_USER3:	//open with
-		case 124:	//exec
-			rt->processed = E2_TYPE_HANDLED;
-			break;
-		case GTK_RESPONSE_ACCEPT:	//add
-		case 125:	//open with default (back in the calling code)
-			rt->processed = E2_TYPE_NOTHANDLED;
-			break;
-		default:	//this includes GTK_RESPONSE_DELETE_EVENT
-			rt->processed = E2_TYPE_CANCELLED;
-			break;
-	}
-	gtk_main_quit ();	//allow the dialog creation function to return
-
-	switch (response)
-	{
-/* these callbacks handle a specific filename, so are usable for output
-   pane selections as well as filelist selections
-   item is a quouted absolute utf-8 path string */
-		case GTK_RESPONSE_ACCEPT:	//add
-			//open the filetypes config dialog with no extension selected
-			e2_filetype_config_show (NULL);
-			break;
-		case E2_RESPONSE_USER1:	//edit
-			action = g_strconcat (_A(5),".",_A(39), NULL);
-			gdk_threads_leave ();	//open BGL
-			e2_action_run_simple_from (action, item, NULL);
-			gdk_threads_enter ();
-			g_free (action);
-			break;
-		case E2_RESPONSE_USER2:	//view
-			action = g_strconcat (_A(5),".",_A(101), NULL);
-			gdk_threads_leave ();	//open BGL
-			e2_action_run_simple_from (action, item, NULL);
-			gdk_threads_enter ();
-			g_free (action);
-			break;
-		case E2_RESPONSE_USER3:	//open with
-			action = g_strconcat (_A(5),".",_A(62), NULL);
-			gdk_threads_leave ();	//open BGL
-			e2_action_run_simple_from (action, item, NULL);
-			gdk_threads_enter ();
-			g_free (action);
-			break;
-		case 124:	//execute
-#ifdef E2_COMMANDQ
-			e2_command_run (item, E2_COMMAND_RANGE_DEFAULT, FALSE);
-#else
-			e2_command_run (item, E2_COMMAND_RANGE_DEFAULT);
-#endif
-			break;
-		default:
-			break;
-	}
-	gtk_widget_grab_focus (curr_view->treeview);
-	g_free (item);
+	*store = response;
 }
 /**
 @brief create and show dialog for handling unknown or ambiguous filetype
 @a newtype should be TRUE when @a filename does have an extension
 @a newtype and @a ambig should not both be TRUE
+Assumes BGL is closed upon arrival
 @param localpath absolute or relative path of item to be processed, localised string
 @param text TRUE to include view or edit option in the dialog
 @param ambig TRUE to enable choice to execute or open item
@@ -1131,13 +1085,13 @@ static void _e2_filetype_dialog_response_cb (GtkDialog *dialog, gint response,
 gint e2_filetype_dialog_create (VPATH *localpath, gboolean text, gboolean ambig,
 	gboolean newtype)
 {
+	gint response;
 	gint retval;
 	gboolean addpath;
 	gchar *usepath, *dir, *base, *title, *public, *message;
 	struct stat sb;
 
-	E2_FileTypeDlg2Runtime *rt = ALLOCATE (E2_FileTypeDlg2Runtime);
-	CHECKALLOCATEDWARN (rt, return);
+	E2_FileTypeDlg2Runtime rt;
 
 	usepath = VPSTR (localpath);
 //	printd (DEBUG, "e2_filetype_dialog_create: %s", usepath);
@@ -1153,15 +1107,15 @@ gint e2_filetype_dialog_create (VPATH *localpath, gboolean text, gboolean ambig,
 			usepath = e2_utils_dircat (curr_view, usepath, TRUE);
 	}
 
-	rt->utfpath = D_FILENAME_FROM_LOCALE (usepath);
+	rt.utfpath = D_FILENAME_FROM_LOCALE (usepath);
 	title = (ambig) ? _("ambiguous filetype") : _("unrecognised filetype");
-	public = g_markup_escape_text (rt->utfpath, -1);
+	public = g_markup_escape_text (rt.utfpath, -1);
 	dir = g_path_get_dirname (public);
 	base = g_path_get_basename (public);
 	message = g_strdup_printf (
 		_("What would you like to do with\n<b>%s</b>\nin %s ?"), base, dir);
-	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
-		message, title, _e2_filetype_dialog_response_cb, rt);
+	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION, message, title,
+		(ResponseFunc)_e2_filetype_dialog_response_cb, &response);
 	g_free (public);
 	g_free (dir);
 	g_free (base);
@@ -1169,7 +1123,7 @@ gint e2_filetype_dialog_create (VPATH *localpath, gboolean text, gboolean ambig,
 
 	//can always add to category, even if item does not presently exist
 	if (newtype && !ambig)
-		e2_dialog_add_undefined_button_custom (dialog, FALSE,
+		e2_dialog_add_custom_button_full (dialog, FALSE,
 		GTK_RESPONSE_ACCEPT, _("_Add.."), "gtk-preferences",
 		_("Create a new filetype for this extension, or add it to an existing filetype"),
 		NULL, NULL);
@@ -1184,35 +1138,94 @@ gint e2_filetype_dialog_create (VPATH *localpath, gboolean text, gboolean ambig,
 		if (S_ISREG (sb.st_mode))
 		{
 			if (sb.st_size == 0)
-				rt->view_btn = e2_dialog_add_undefined_button_custom (dialog, FALSE,
-					E2_RESPONSE_USER1, _("_Edit"), "edit_"E2IP".png",
+				rt.view_btn = e2_dialog_add_custom_button_full (dialog, FALSE,
+					E2_RESPONSE_USER1, _("_Edit"), GTK_STOCK_EDIT,
 					_("Edit the file"), NULL, NULL);
 			else if (text)
-				rt->view_btn = e2_dialog_add_undefined_button_custom (dialog, FALSE,
-					E2_RESPONSE_USER2, _("_View"), "view_"E2IP".png",
+				rt.view_btn = e2_dialog_add_custom_button_full (dialog, FALSE,
+					E2_RESPONSE_USER2, _("_View"), "view"E2ICONTB,
 					_("Read the file"), NULL, NULL);
 		}
 		if (ambig)
 		{
-			e2_dialog_add_undefined_button_custom (dialog, FALSE,
-				124, _("_Run"), GTK_STOCK_EXECUTE,
+			e2_dialog_add_custom_button_full (dialog, FALSE,
+				E2_RESPONSE_USER5, _("_Run"), GTK_STOCK_EXECUTE,
 					_("Execute the item"), NULL, NULL);
-			rt->open_btn = e2_dialog_add_undefined_button_custom (dialog, TRUE,
-				125, _("_Open"), GTK_STOCK_OPEN,	//FIXME better icon
+			rt.open_btn = e2_dialog_add_custom_button_full (dialog, TRUE,
+				E2_RESPONSE_USER6, _("_Open"), GTK_STOCK_OPEN,	//FIXME better icon
 					_("Open with the default application"), NULL, NULL);
 		}
 		else
-			rt->open_btn = e2_dialog_add_undefined_button_custom (dialog, TRUE,
-				E2_RESPONSE_USER3, _("_Open.."), "open_with_"E2IP".png",
+			rt.open_btn = e2_dialog_add_custom_button_full (dialog, TRUE,
+				E2_RESPONSE_USER3, _("_Open.."), "open_with"E2ICONTB,
 					_("Enter a command with which to open the file"), NULL, NULL);
-		gtk_widget_grab_focus (rt->open_btn);
+		gtk_widget_grab_focus (rt.open_btn);
 	}
 
-	e2_dialog_show (dialog, NULL, 0, &E2_BUTTON_CANCEL, NULL);
-	gtk_main ();	//block until user selects something
+	//block until user selects something
+	e2_dialog_show (dialog, NULL, E2_DIALOG_BLOCKED, &E2_BUTTON_CLOSE, NULL);
+
 	if (addpath)
 		g_free (usepath);
-	retval = rt->processed;	//get code for what was selected
-	DEALLOCATE (E2_FileTypeDlg2Runtime, rt);
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+	gtk_widget_grab_focus (curr_view->treeview);
+
+	gchar *action;
+//tag E2_BADQUOTES CHECKME used for action arg
+	gchar *item = e2_utils_quote_string (rt.utfpath);
+	switch (response)
+	{
+	/* these callbacks handle a specific filename, so are usable for output
+	   pane selections as well as filelist selections
+	   item is a quouted absolute UTF-8 path string */
+		case GTK_RESPONSE_ACCEPT:	//add
+			//open the filetypes config dialog with no extension selected
+			e2_filetype_config_show (NULL);
+			break;
+		case E2_RESPONSE_USER1:	//edit
+			action = g_strconcat (_A(6),".",_A(43), NULL);
+			e2_action_run_simple_from (action, item, dialog);
+			g_free (action);
+			break;
+		case E2_RESPONSE_USER2:	//view
+			action = g_strconcat (_A(6),".",_A(106), NULL);
+			e2_action_run_simple_from (action, item, dialog);
+			g_free (action);
+			break;
+		case E2_RESPONSE_USER3:	//open with
+			action = g_strconcat (_A(6),".",_A(66), NULL);	//Q'd action
+			e2_action_run_simple_from (action, item, dialog);	//don't know if user cancelled this
+			g_free (action);
+			break;
+		case E2_RESPONSE_USER5:	//execute
+			//CHECKME no quotes in this case ?
+			e2_command_run (item, E2_COMMAND_RANGE_DEFAULT, app.main_window //no better valid widget now
+#ifdef E2_COMMANDQ
+			, FALSE
+#endif
+			);
+		default:
+			break;
+	}
+	g_free (item);
+
+	//send back "handled" type flag to caller
+	switch (response)
+	{
+		case E2_RESPONSE_USER1:	//edit
+		case E2_RESPONSE_USER2:	//view
+		case E2_RESPONSE_USER3:	//open with
+		case E2_RESPONSE_USER5:	//exec
+			retval = E2_TYPE_HANDLED;
+			break;
+		case GTK_RESPONSE_ACCEPT:	//add
+		case E2_RESPONSE_USER6:	//open with default (back in the calling code)
+			retval = E2_TYPE_NOTHANDLED;
+			break;
+		default:	//this includes GTK_RESPONSE_DELETE_EVENT
+			retval = E2_TYPE_CANCELLED;
+			break;
+	}
 	return retval;
 }
