@@ -1,6 +1,6 @@
-/* $Id: e2_mkdir_dialog.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_mkdir_dialog.h 1725 2009-06-21 04:16:16Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -30,12 +30,12 @@ typedef struct _E2_MkdirDialogRuntime
 	GtkWidget *scrolled;
 	GtkWidget *combo;
 	GtkWidget *info_expander;
-//	GtkWidget *info_box;
+	GtkWidget *info_box;
 	GtkWidget *info_label;
 	GtkWidget *info_label2;
 	GtkWidget *menu;
 //	GtkWidget *create_btn;
-	gchar *path; //utf-8 encoded
+	gchar *path; //UTF-8 encoded
 	GList *history;
 	gint history_cur;
 	gboolean creation_possible;
