@@ -1,6 +1,6 @@
-/* $Id: e2_name_filter_dialog.c 748 2007-11-22 22:04:33Z tpgww $
+/* $Id: e2_name_filter_dialog.c 1987 2009-10-14 00:41:33Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -38,9 +38,12 @@ typedef struct _E2_NmFltDlgData
 //	GtkWidget *menu_item;	//copy of pointer to clicked filters menu item
 //	gboolean itemstate;	//initial state of menu_item
 //	gboolean blocked = FALSE;	//flag to prevent toggles when not wanted
-	ViewInfo *view;
+	ViewInfo *view;		//data for filtered file pane
 } E2_NmFltDlgData;
 
+static void _e2_name_filter_dialog_response_cb (GtkDialog *dialog,
+	gint response, E2_NmFltDlgData *rt);
+
 /*static void _e2_name_filter_dialog_item_toggle_cb (void)
 {
   gulong handler = g_signal_handler_find (menu_item, G_SIGNAL_MATCH_FUNC,
@@ -65,8 +68,18 @@ static void _e2_name_filter_dialog_ok (E2_NmFltDlgData *data)
 //	data->view->name_filter.patternptr = g_strdup (view->name_filter.pattern);
 	data->view->name_filter.patternptr = g_strdup (s);
 
-	data->view->name_filter.invert_mask = GTK_TOGGLE_BUTTON (data->invert_check)->active;
-	data->view->name_filter.case_sensitive = GTK_TOGGLE_BUTTON (data->case_sensitive_check)->active;
+	data->view->name_filter.invert_mask =
+#ifdef USE_GTK2_14
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->invert_check));
+#else
+		GTK_TOGGLE_BUTTON (data->invert_check)->active;
+#endif
+	data->view->name_filter.case_sensitive =
+#ifdef USE_GTK2_14
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->case_sensitive_check));
+#else
+		GTK_TOGGLE_BUTTON (data->case_sensitive_check)->active;
+#endif
 //    if (! itemstate)
 	 //we're staying filtered, so re-toggle the menu item
 //	  _e2_name_filter_dialog_item_toggle_cb ();
@@ -82,48 +95,56 @@ static void _e2_name_filter_dialog_ok (E2_NmFltDlgData *data)
   }
 }
 /**
+@brief cleanup during the destruction of the view related to a dialog
+@param object UNUSED the view-related object being destroyed
+@param rt pointer to data struct for the dialog
+@return
+*/
+static void _e2_name_filter_dialog_destroy_cb (GtkObject *object,
+	E2_NmFltDlgData *rt)
+{
+	g_signal_handlers_disconnect_by_func ((gpointer)rt->dialog,
+		_e2_name_filter_dialog_response_cb, rt); //no double-handling
+	_e2_name_filter_dialog_response_cb (GTK_DIALOG (rt->dialog), 0, rt);
+}
+/**
 @brief dialog response callback
 
 @param dialog the permissions-dialog
 @param response the response for the clicked button
-@param data pointer to dialog data struct
+@param rt pointer to dialog data struct
 
 @return
 */
 static void _e2_name_filter_dialog_response_cb (GtkDialog *dialog,
-	gint response, E2_NmFltDlgData *data)
+	gint response, E2_NmFltDlgData *rt)
 {
 	switch (response)
 	{
-		case GTK_RESPONSE_OK:
-			gtk_widget_hide (data->dialog);
-			_e2_name_filter_dialog_ok (data);
-		  	//prevent toggle when dialog is destroyed
-//			data->blocked = TRUE;
-		  break;
+		case E2_RESPONSE_APPLY:
+			gtk_widget_hide (rt->dialog);
+			_e2_name_filter_dialog_ok (rt);
+			break;
 		case E2_RESPONSE_REMOVE:
-		  //update the cached copy of the state
-		  data-> view->name_filter.active = FALSE;
-		  gtk_widget_hide (data->dialog);
-		  //show the results
-		  e2_toolbar_toggle_filter_button (data->view);
-		  e2_fileview_refilter_list (data->view);
+			//update the cached copy of the state
+			rt->view->name_filter.active = FALSE;
+			gtk_widget_hide (rt->dialog);
+			//show the results
+			e2_toolbar_toggle_filter_button (rt->view);
+			e2_fileview_refilter_list (rt->view);
 #ifdef E2_STATUS_DEMAND
-		  if (data->view == curr_view)
-			e2_window_update_status_bar (NULL);
+			if (rt->view == curr_view)
+				e2_window_update_status_bar (NULL);
 #endif
-		  //prevent toggle when dialog is destroyed
-//		  data->blocked = TRUE;
-		  break;
-//		case GTK_RESPONSE_CANCEL:
+			break;
 		default:
-		  //the menu item will be re-toggled when dialog is destroyed
-		break;
+			//the menu item will be re-toggled when dialog is destroyed
+			break;
 	}
-	gtk_widget_destroy (data->dialog);
-	DEALLOCATE (E2_NmFltDlgData, data);
-//	gtk_main_quit ();
-	return;
+	g_signal_handlers_disconnect_by_func ((gpointer)rt->view->treeview,
+		_e2_name_filter_dialog_destroy_cb, rt);
+	gtk_widget_destroy (rt->dialog);
+	DEALLOCATE (E2_NmFltDlgData, rt);
 }
 /**
 @brief handle Return keypresses in the size entry
@@ -135,21 +156,22 @@ static void _e2_name_filter_dialog_activated_cb (GtkEntry *entry,
 	E2_NmFltDlgData *data)
 {
 	_e2_name_filter_dialog_response_cb (GTK_DIALOG (data->dialog),
-		GTK_RESPONSE_OK, data);
+		E2_RESPONSE_APPLY, data);
 }
 /**
 @brief create file name filter dialog
 
-The state of @a item when it arrives here is opposite to that
-shown in the menu, when clicked
+The state of @a item when it arrives here is opposite to that shown in the menu,
+when clicked
 
-@param item the clicked checkbox widget in the fillters menu
+@param item the activated item in the filters menu
 @param view data structure for the view to which the file list belongs
 
 @return
 */
-void e2_name_filter_dialog_create_cb (GtkWidget *item, ViewInfo *view)
+void e2_name_filter_dialog_create_cb (GtkCheckMenuItem *item, ViewInfo *view)
 {
+	E2_Button no_btn;
 	E2_NmFltDlgData *rt = ALLOCATE (E2_NmFltDlgData);
 	CHECKALLOCATEDWARN (rt, return;);
 	rt->view = view;
@@ -159,41 +181,44 @@ void e2_name_filter_dialog_create_cb (GtkWidget *item, ViewInfo *view)
 //	rt->itemstate = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM (item));
 
 	rt->dialog = e2_dialog_create (NULL, _("Display only the items named like:"),
-		_("name filter"), _e2_name_filter_dialog_response_cb, rt);
-	GtkWidget *vbox = GTK_DIALOG (rt->dialog)->vbox;
-
-	rt->pattern_entry = e2_widget_add_entry (vbox, view->name_filter.patternptr, FALSE, FALSE);
+		_("name filter"), (ResponseFunc)_e2_name_filter_dialog_response_cb, rt);
+	GtkWidget *dialog_vbox =
+#ifdef USE_GTK2_14
+		gtk_dialog_get_content_area (GTK_DIALOG (rt->dialog));
+#else
+		GTK_DIALOG (rt->dialog)->vbox;
+#endif
+	rt->pattern_entry = e2_widget_add_entry (dialog_vbox, view->name_filter.patternptr,
+		FALSE, FALSE);
 #ifdef E2_ASSISTED
 	GtkWidget *label = (GtkWidget *) g_object_get_data (G_OBJECT (rt->dialog),
 		"e2-dialog-label");
-	e2_widget_set_label_relations (GTK_LABEL (label), rt->pattern_entry);
+	e2_widget_set_label_relations (label, rt->pattern_entry);
 #endif
 	//handle <Return> key-presses when the entry is focused
 	g_signal_connect (G_OBJECT (rt->pattern_entry), "activate",
 		G_CALLBACK (_e2_name_filter_dialog_activated_cb), rt);
 
-	e2_widget_add_mid_label (vbox, _("example: *.c,*.h"), 0.0, FALSE, 0);
+	e2_widget_add_mid_label (dialog_vbox, _("example: *.c,*.h"), 0.0, FALSE, 0);
 
-	GtkWidget *table = e2_widget_add_table (vbox, 1, 2, FALSE, FALSE, 0);
+	GtkWidget *table = e2_widget_add_table (dialog_vbox, 1, 2, FALSE, FALSE, 0);
 	rt->invert_check = e2_button_add_toggle_to_table (table, _("Invert"),
 		view->name_filter.invert_mask, NULL, NULL, 0, 1, 0, 1);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		rt->invert_check,
+	e2_widget_set_safetip (rt->invert_check,
 		_("Show files that DO NOT match the given mask"));
 	rt->case_sensitive_check = e2_button_add_toggle_to_table (table, _("Case sensitive"),
 		view->name_filter.case_sensitive, NULL, NULL, 1, 2, 0, 1);
 
+	//cleanup if the dialog rt data becomes invalid
+	g_signal_connect (GTK_OBJECT(view->treeview), "destroy",
+		G_CALLBACK (_e2_name_filter_dialog_destroy_cb), rt);
+
 	//now the buttons
-//	if (!rt->itemstate)
-	if (!gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (item)))
-	  //user has just toggled off
-	  e2_dialog_add_defined_button (rt->dialog, &E2_BUTTON_REMOVE);
-	//non-modal dialog
+	if (!gtk_check_menu_item_get_active (item))
+		//user has just toggled off
+		e2_dialog_add_defined_button (rt->dialog, &E2_BUTTON_REMOVE);
+	e2_button_derive (&no_btn, &E2_BUTTON_NO, BTN_NO_CANCEL);
+
 	e2_dialog_show (rt->dialog, app.main_window, 0,
-		&E2_BUTTON_CANCEL, &E2_BUTTON_OK, NULL);
-//	gtk_main ();
+		&no_btn, &E2_BUTTON_APPLY, NULL);
 }
