@@ -1,6 +1,6 @@
-/* $Id: e2_ownership_dialog.c 868 2008-05-06 04:42:09Z tpgww $
+/* $Id: e2_ownership_dialog.c 2027 2009-12-20 01:04:06Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -25,6 +25,7 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include <grp.h>
 #include "e2_dialog.h"
 #include "e2_ownership_dialog.h"
+#include "e2_filelist.h"
 
 typedef struct _E2_OwnersDldRuntime
 {
@@ -35,7 +36,6 @@ typedef struct _E2_OwnersDldRuntime
 	gid_t group;
 	gboolean recurse;
 	gboolean permission;
-	DialogButtons result;
 } E2_OwnersDldRuntime;
 
 /**
@@ -51,7 +51,13 @@ static uid_t _e2_ownership_dialog_get_user_id (E2_OwnersDldRuntime *rt)
 	struct passwd *pw_buf;
 	uid_t user_id;
 
-	user = (gchar *) gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->user_combo)->child));
+	user = (gchar *) gtk_entry_get_text (
+#ifdef USE_GTK2_14
+		GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->user_combo)))
+#else
+		GTK_ENTRY (GTK_BIN (rt->user_combo)->child)
+#endif
+		);
 	user = e2_utf8_to_locale (user);
 	if (user != NULL)
 	{
@@ -85,7 +91,13 @@ static gid_t _e2_ownership_dialog_get_group_id (E2_OwnersDldRuntime *rt)
 	struct group *grp_buf;
 	gid_t group_id;
 
-	group = (gchar *) gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->group_combo)->child));
+	group = (gchar *) gtk_entry_get_text (
+#ifdef USE_GTK2_14
+		GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->user_combo)))
+#else
+		GTK_ENTRY (GTK_BIN (rt->group_combo)->child)
+#endif
+	);
 	group = e2_utf8_to_locale (group);
 	if (group != NULL)
 	{
@@ -107,32 +119,9 @@ static gid_t _e2_ownership_dialog_get_group_id (E2_OwnersDldRuntime *rt)
 	return group_id;
 }
 /**
-@brief dialog response callback
+@brief update recurse flag
 
-@param dialog the ownership-dialog
-@param response the response for the clicked button
-@param rt pointer to dialog data struct
-
-@return
-*/
-static void _e2_ownership_dialog_response_cb (GtkDialog *dialog,
-	gint response, E2_OwnersDldRuntime *rt)
-{
-	switch (response)
-	{
-		case GTK_RESPONSE_OK:
-		case E2_RESPONSE_APPLYTOALL:
-			rt->owner = _e2_ownership_dialog_get_user_id (rt);
-			rt->group = _e2_ownership_dialog_get_group_id (rt);
-			break;
-		default:
-		break;
-	}
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-	e2_dialog_response_decode_cb (dialog, response, &rt->result);
-}
-/**
-@brief update recurse flag if permission is valid
+This only applies if change is permitted.
 
 @param togglebutton the clicked button
 @param rt pointer to dialog data struct
@@ -142,52 +131,51 @@ static void _e2_ownership_dialog_response_cb (GtkDialog *dialog,
 static void _e2_ownership_dialog_toggle_recurse_button_cb
 	(GtkToggleButton *togglebutton, E2_OwnersDldRuntime *rt)
 {
-	if (rt->permission)
-		rt->recurse = GTK_TOGGLE_BUTTON (rt->recurse_dirs_button)->active;
-	return;
+	rt->recurse =
+#ifdef USE_GTK2_14
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->recurse_dirs_button));
+#else
+		GTK_TOGGLE_BUTTON (rt->recurse_dirs_button)->active;
+#endif
 }
 /**
-@brief show custom message about inadequate permission
+@brief change widget focus after a combobox entry is activated
 
-@param box the box to hold the data
+@param entry the activated widget
+@param focus the widget to be focused next
 
 @return
 */
-void e2_ownership_dialog_warn (GtkWidget *box)
+static void _e2_ownership_dialog_activate_cb (GtkEntry *entry, GtkWidget *focus)
 {
-	GtkWidget *frame = gtk_frame_new ("");
-	gtk_box_pack_start (GTK_BOX (box), frame, TRUE, TRUE, E2_PADDING);
-	gtk_widget_show (frame);
+	//prevent activation of any newly-focused widget too
+	g_signal_stop_emission_by_name ((gpointer)entry, "activate");
+	gtk_widget_grab_focus (focus);
+}
 
-	GtkWidget *sub_vbox = gtk_vbox_new (FALSE, 0);
-    gtk_container_add (GTK_CONTAINER (frame), sub_vbox);
-  	gtk_widget_show (sub_vbox);
-	GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
-    gtk_container_add (GTK_CONTAINER (sub_vbox), hbox);
-	e2_widget_add_mid_label (hbox, _("You are not authorised to make any change."), 0.5,
-		TRUE, E2_PADDING*2);
-	gtk_widget_show (hbox);
-	//padding line
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (sub_vbox), hbox);
-    e2_widget_add_mid_label (hbox, (" "), 0, FALSE, 0);
-  	gtk_widget_show (hbox);
+static void _e2_ownership_dialog_activate_cb2 (GtkEntry *entry, GtkWidget *dialog)
+{
+	g_signal_stop_emission_by_name ((gpointer)entry, "activate");
+	g_signal_emit_by_name (G_OBJECT (dialog), "response", E2_RESPONSE_APPLY, NULL);
 }
 /**
 @brief create and run a dialog for changing item permissions
 This is run from inside a queued thread-function
 @param localpath path of item to be processed
+@param multi TRUE if this dialog is part of a series for multiple items
+@param permitted TRUE if change is authorised
 @param owner_ret store for returning the new owner
 @param group_ret store for returning the new group
 @param recurse_ret store for returning whether to recurse the changes
-@param permission_ret store for returning whether a change is authorised
-@param multi TRUE if this dialog is part of a series for multiple items
+@param winx_ret store for popup window horiz boundary (may hold -1)
+@param winy_ret store for popup window vert boundary (may hold -1)
 
 @return enumerator corresponing to the clicked dialog button
 */
 DialogButtons e2_ownership_dialog_run (VPATH *localpath,
+	gboolean multi, gboolean permitted,
 	uid_t *owner_ret, gid_t *group_ret, gboolean *recurse_ret,
-	gboolean *permission_ret, gboolean multi)
+	gint *winx_ret, gint *winy_ret)
 {
 	GtkWidget *ownership_dialog;
 	GtkWidget *dialog_vbox;
@@ -197,82 +185,30 @@ DialogButtons e2_ownership_dialog_run (VPATH *localpath,
 //	E2_OwnersDldRuntime *rt = ALLOCATE (E2_OwnersDldRuntime);
 //	CHECKALLOCATEDWARN (rt, return GTK_RESPONSE_CANCEL);
 	E2_OwnersDldRuntime rt;
-
+	DialogButtons result;
+    struct passwd *pw_buf;
+   	struct group *grp_buf;
+	gchar *utf, *tmp;
 	struct stat statbuf;
 
 	if (e2_fs_lstat (localpath, &statbuf E2_ERR_NONE()))
 		return CANCEL;
 
-	GString *label_text = g_string_sized_new (NAME_MAX+20);
-	gboolean thisis_dir = e2_fs_is_dir3 (localpath E2_ERR_NONE());
-
-	ownership_dialog = e2_dialog_create (NULL, NULL, _("ownership"),
-		_e2_ownership_dialog_response_cb, &rt);	//rt);
-	dialog_vbox = GTK_DIALOG (ownership_dialog)->vbox;
-	action_area = GTK_DIALOG (ownership_dialog)->action_area; //this is for the buttons
-	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), E2_PADDING);
-
-//	table = e2_widget_add_table (dialog_vbox, 2, 3, TRUE, TRUE, 0); //2 rows, 3 cols, homogen, fill, no pad
-//	gtk_table_set_row_spacing (GTK_TABLE(table), 0, E2_PADDING);
-	gchar *label = (thisis_dir) ? _("Directory name") : _("Filename") ;
-	gchar *name = g_filename_display_basename (VPSTR (localpath));
-	g_string_printf (label_text, "%s: <b>%s</b>", label, name);
-	g_free (name);
-	//e2_widget_add_mid_label_to_table (table, label_text->str, 0, 1, 2, 0, 1);
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, E2_PADDING); //top, bottom padding
-	e2_widget_add_mid_label (hbox, label_text->str, 0, TRUE, E2_PADDING); //L, R padding
-	gtk_widget_show (hbox);
-
-	gchar *s = e2_fs_get_perm_string (statbuf.st_mode);
-	g_string_assign (label_text, s);
-	g_free (s);
-	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 7), ' ');
-	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 4), ' ');
-	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 1), ' ');
-	g_string_prepend (label_text, _("Permissions: "));
-	//  e2_widget_add_mid_label_to_table (table, label_text->str, 0, 1, 2, 1, 2);
-	hbox = gtk_hbox_new (FALSE, 0);
-	e2_widget_add_mid_label (hbox, label_text->str, 0, TRUE, E2_PADDING);
-	gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, 0);
-	gtk_widget_show (hbox);
-
-	g_string_free (label_text, TRUE);
-
-	table = e2_widget_add_table (dialog_vbox, 2, 3, TRUE, TRUE, 0);  //2 rows, 3 cols, homogen, fill, no pad
-	gtk_container_set_border_width (GTK_CONTAINER (table), E2_PADDING);
-	gtk_table_set_row_spacings (GTK_TABLE (table), E2_PADDING);
-	e2_widget_add_mid_label_to_table (table, _("User: "), 0, 0, 1, 0, 1);
-
-	rt.user_combo = e2_combobox_get (NULL, NULL, NULL,
-	E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_NO_AUTO_HISTORY | E2_COMBOBOX_CYCLE_HISTORY);
-	gtk_table_attach_defaults (GTK_TABLE (table), rt.user_combo, 1, 3, 0, 1);  // gint left col, gint right col, gint top row, gint bottom row)
-
-	e2_widget_add_mid_label_to_table (table, _("Group: "), 0, 0, 1, 1, 2);
-
-	rt.group_combo = e2_combobox_get (NULL, NULL, NULL,
-	E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_NO_AUTO_HISTORY | E2_COMBOBOX_CYCLE_HISTORY);
-	gtk_table_attach_defaults (GTK_TABLE (table), rt.group_combo, 1, 3, 1, 2);
-
-  {  //setup the combobox data
-    struct passwd *pw_buf;
-    struct group *grp_buf;
-	gchar *utf;
-    gchar id_buf[32];
-	gint ctr = 0;	//combo box entry counter, set to 1st value in list, for now
+	//setup the combobox history data
 	gint user_index = 0, grp_index = 0;  //default values for combobox display (= 1st entry)
+	GList *userlist = NULL, *grouplist = NULL;
 
 	uid_t src_uid = statbuf.st_uid;
 	gint myuid = getuid ();
-	setpwent ();
 
-	//path is to current dir
-	gboolean writeok = e2_fs_check_write_permission (localpath E2_ERR_NONE());
-	if (writeok)
+	if (permitted)
     {
+		gint ctr;	//combo box entry counter
 		rt.permission = TRUE;	//confirm ok to proceed
+		ctr = 0;
+		setpwent ();
 		while ((pw_buf = getpwent ()) != NULL)
-	   {	//screen out superfluous names
+		{	//screen out superfluous names
 			if (
 				(myuid > 0) //user is not root
 				&& ((guint) pw_buf->pw_uid > 0)  //buffer entry is not root's
@@ -282,38 +218,40 @@ DialogButtons e2_ownership_dialog_run (VPATH *localpath,
 			utf = e2_utf8_from_locale (pw_buf->pw_name);
 			if (utf == NULL)
 				utf = g_strdup_printf ("%d", (guint) pw_buf->pw_uid);
-			gtk_combo_box_append_text (GTK_COMBO_BOX (rt.user_combo), utf);
-			g_free (utf);
+			userlist = g_list_prepend (userlist, utf);
 			//remember the combo index for the owner
 			if ((uid_t) pw_buf->pw_uid == src_uid)
 				user_index = ctr;
 			else
 				ctr++;
 		}
+	    endpwent();
 		if ((pw_buf = getpwuid (statbuf.st_uid)) == NULL)
 		{
-			g_snprintf (id_buf, sizeof (id_buf), "%d", (guint) statbuf.st_uid);
-			gtk_combo_box_append_text (GTK_COMBO_BOX (rt.user_combo), id_buf);
+			userlist = g_list_prepend (userlist,
+				g_strdup_printf ("%d", (guint) statbuf.st_uid));
 			user_index = ctr;
 		}
+		userlist = g_list_reverse (userlist);
 		printd (DEBUG, "owner combo index = %d", user_index);
 
 		ctr = 0;
 		gid_t src_gid = statbuf.st_gid;
 		if (myuid == 0)
 		{
+		    setgrent ();
 			while ((grp_buf = getgrent ()) != NULL)
 			{
 				utf = e2_utf8_from_locale (grp_buf->gr_name);
 				if (utf == NULL)
 					utf = g_strdup_printf ("%d", (guint) grp_buf->gr_gid);
-				gtk_combo_box_append_text (GTK_COMBO_BOX (rt.group_combo), utf);
-				g_free (utf);
+				grouplist = g_list_prepend (grouplist, utf);
 				if ((gid_t) grp_buf->gr_gid == src_gid)
 					grp_index = ctr;
 				else
 					ctr++;
 			}
+			endgrent();
 		}
 		else //not root
 		{
@@ -324,10 +262,9 @@ DialogButtons e2_ownership_dialog_run (VPATH *localpath,
 				if ((grp_buf = getgrgid (grp_ids[ctr])) != NULL)
 				{
 					utf = e2_utf8_from_locale (grp_buf->gr_name);
-						if (utf == NULL)
-					utf = g_strdup_printf ("%d", (guint) grp_buf->gr_gid);
-					gtk_combo_box_append_text (GTK_COMBO_BOX (rt.group_combo), utf);
-					g_free (utf);
+					if (utf == NULL)
+						utf = g_strdup_printf ("%d", (guint) grp_buf->gr_gid);
+					grouplist = g_list_prepend (grouplist, utf);
 					if ((gid_t) grp_buf->gr_gid == src_gid)
 						grp_index = ctr;
 				}
@@ -335,93 +272,223 @@ DialogButtons e2_ownership_dialog_run (VPATH *localpath,
 		}
 		if ((grp_buf = getgrgid (statbuf.st_gid)) == NULL)
 		{
-			g_snprintf (id_buf, sizeof (id_buf), "%d", (guint) statbuf.st_gid);
-			gtk_combo_box_append_text (GTK_COMBO_BOX (rt.group_combo), id_buf);
+			grouplist = g_list_prepend (grouplist,
+				g_strdup_printf ("%d", (guint) statbuf.st_gid));
 			grp_index = ctr;
 		}
+		grouplist = g_list_reverse (grouplist);
 		printd (DEBUG, "group combo index = %d", grp_index);
 	}
-	else  //no write permission, no changes allowed, just show current values
+	else  //no changes allowed, just show current values
     {
 		rt.permission = FALSE;	//signal not ok to proceed, even if ok is clicked
     	if ((pw_buf = getpwuid (statbuf.st_uid)) != NULL)
-	    	gtk_combo_box_append_text (GTK_COMBO_BOX (rt.user_combo), g_strdup (pw_buf->pw_name));
-		else
-	 	{
-			g_snprintf (id_buf, sizeof (id_buf), "%d", (guint) statbuf.st_uid);
-			gtk_combo_box_append_text (GTK_COMBO_BOX (rt.user_combo), id_buf);
-		}
-    	if ((grp_buf = getgrgid (statbuf.st_gid)) != NULL)
-			gtk_combo_box_append_text (GTK_COMBO_BOX (rt.group_combo), g_strdup (grp_buf->gr_name));
-		else
 		{
-			g_snprintf (id_buf, sizeof (id_buf), "%d", (guint) statbuf.st_gid);
-			gtk_combo_box_append_text (GTK_COMBO_BOX (rt.group_combo), id_buf);
+			utf = e2_utf8_from_locale (pw_buf->pw_name);
+			if (utf == NULL)
+				utf = g_strdup_printf ("%d", (guint) pw_buf->pw_uid);
+			userlist = g_list_prepend (userlist, utf);
 		}
+		else
+			userlist = g_list_prepend (userlist,
+				g_strdup_printf ("%d", (guint) statbuf.st_uid));
+
+    	if ((grp_buf = getgrgid (statbuf.st_gid)) != NULL)
+		{
+			utf = e2_utf8_from_locale (grp_buf->gr_name);
+			if (utf == NULL)
+				utf = g_strdup_printf ("%d", (guint) statbuf.st_gid);
+			grouplist = g_list_prepend (grouplist, utf);
+		}
+		else
+			grouplist = g_list_prepend (grouplist,
+				g_strdup_printf ("%d", (guint) statbuf.st_gid));
+	}
+
+//tag DEBUGfreeze;
+	//maybe this helps gtk 2.16 crashes ?
+//	gdk_threads_enter ();
+
+	GString *label_text = g_string_sized_new (NAME_MAX+20);
+	gboolean thisis_dir = e2_fs_is_dir3 (localpath E2_ERR_NONE());
+
+	ownership_dialog = e2_dialog_create (NULL, NULL, _("ownership"),
+		DUMMY_RESPONSE_CB, NULL);
+
+	dialog_vbox =
+#ifdef USE_GTK2_14
+		gtk_dialog_get_content_area (GTK_DIALOG (ownership_dialog));
+#else
+		GTK_DIALOG (ownership_dialog)->vbox;
+#endif
+	action_area =
+#ifdef USE_GTK2_14
+		gtk_dialog_get_action_area (GTK_DIALOG (ownership_dialog));
+#else
+		GTK_DIALOG (ownership_dialog)->action_area; //this is for the buttons
+#endif
+	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), E2_PADDING);
+
+	gchar *label = (thisis_dir) ? _("Directory name") : _("Filename") ;
+	gchar *name = g_filename_display_basename (VPSTR (localpath));
+	g_string_printf (label_text, "%s: <b>%s</b>", label, name);
+	g_free (name);
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, E2_PADDING); //top, bottom padding
+	e2_widget_add_mid_label (hbox, label_text->str, 0, TRUE, E2_PADDING); //L, R padding
+	gtk_widget_show (hbox);
+
+	gchar *s = e2_fs_get_perm_string (statbuf.st_mode);
+	label_text = g_string_assign (label_text, s);
+	g_free (s);
+	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 7), ' ');
+	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 4), ' ');
+	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 1), ' ');
+	g_string_prepend (label_text, ": ");
+	g_string_prepend (label_text, _("Permissions"));
+	hbox = gtk_hbox_new (FALSE, 0);
+	e2_widget_add_mid_label (hbox, label_text->str, 0, TRUE, E2_PADDING);
+	gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, 0);
+	gtk_widget_show (hbox);
+
+	g_string_free (label_text, TRUE);
+
+	table = e2_widget_add_table (dialog_vbox, 2, 3, TRUE, TRUE, 0);  //2 rows, 3 cols, homogen, fill, no pad
+	gtk_container_set_border_width (GTK_CONTAINER (table), E2_PADDING);
+	gtk_table_set_row_spacings (GTK_TABLE (table), E2_PADDING);
+
+//	gdk_threads_enter (); //block entry-cursor blinking for now, to avoid sporadic bad keypress-latency
+
+	//comboboxes for user & group
+	tmp = e2_utils_strcat (_("User"), ": ");
+	e2_widget_add_mid_label_to_table (table, tmp, 0, 0, 1, 0, 1);
+	g_free (tmp);
+	rt.user_combo = e2_combobox_get ((ActivateFunc)NULL, NULL,
+		(userlist == NULL) ? NULL : &userlist,
+		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_NO_AUTO_HISTORY | E2_COMBOBOX_CYCLE_HISTORY);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (rt.user_combo), user_index);
+	gtk_table_attach_defaults (GTK_TABLE (table), rt.user_combo, 1, 3, 0, 1);  // gint left col, gint right col, gint top row, gint bottom row)
+	e2_list_free_with_data (&userlist);
+
+	tmp = e2_utils_strcat (_("Group"), ": ");
+	e2_widget_add_mid_label_to_table (table, tmp, 0, 0, 1, 1, 2);
+	g_free (tmp);
+	rt.group_combo = e2_combobox_get ((ActivateFunc)NULL, NULL,
+		(grouplist == NULL) ? NULL : &grouplist,
+		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_NO_AUTO_HISTORY | E2_COMBOBOX_CYCLE_HISTORY);
+    gtk_combo_box_set_active (GTK_COMBO_BOX (rt.group_combo), grp_index);
+	gtk_table_attach_defaults (GTK_TABLE (table), rt.group_combo, 1, 3, 1, 2);
+	e2_list_free_with_data (&grouplist);
+
+//	gdk_threads_leave ();	//seem to need some distance between getting combo's and this unlock
+
+	if (permitted)
+	{
+		rt.recurse = *recurse_ret;
+		if (thisis_dir)  //note: no recurse button now if multiple-selection
+		{
+			table = e2_widget_add_table (dialog_vbox, 1, 3, TRUE, TRUE, 0); //1 row, 3 cols, homogen, fill, no pad
+			gtk_table_set_row_spacing (GTK_TABLE(table), 0, E2_PADDING);
+			rt.recurse_dirs_button = e2_button_add_toggle_to_table (table, _("Recurse"),
+				rt.recurse, _e2_ownership_dialog_toggle_recurse_button_cb, &rt, //rt,
+				1, 2, 0, 1 ); // gint left, gint right, gint top, gint bottom
+		}
+	}
+	else
+	{
 		gtk_widget_set_sensitive (rt.user_combo, FALSE);
 		gtk_widget_set_sensitive (rt.group_combo, FALSE);
-	}
-    gtk_combo_box_set_active (GTK_COMBO_BOX (rt.user_combo), user_index);
-    gtk_combo_box_set_active (GTK_COMBO_BOX (rt.group_combo), grp_index);
-
-    setpwent ();
-    setgrent ();
-  }
-
-	if (rt.permission && thisis_dir)  //note: no recurse button now if multiple-selection
-	{
-		table = e2_widget_add_table (dialog_vbox, 1, 3, TRUE, TRUE, 0); //1 row, 3 cols, homogen, fill, no pad
-		gtk_table_set_row_spacing (GTK_TABLE(table), 0, E2_PADDING);
-		rt.recurse_dirs_button = e2_button_add_toggle_to_table (table, _("Recurse"),
-			FALSE, _e2_ownership_dialog_toggle_recurse_button_cb, &rt, //rt,
-			1, 2, 0, 1 ); // gint left, gint right, gint top, gint bottom
+		e2_dialog_setup_auth (dialog_vbox);	//frame, makes separator redundant
+		gtk_dialog_set_has_separator (GTK_DIALOG (ownership_dialog), FALSE);
 	}
 
-	if (!rt.permission)
-		e2_ownership_dialog_warn (dialog_vbox);
+	result = NO_TO_ALL;	//default in case of abort
+
+	E2_DialogFlags flags = 0;
 
 	//add buttons in the order that they will appear
 	GtkWidget *btn;
+	E2_Button no_btn;
 	if (multi)
 	{
 		e2_dialog_set_negative_response (ownership_dialog, E2_RESPONSE_NOTOALL);
-		e2_dialog_add_defined_button (ownership_dialog, &E2_BUTTON_NOTOALL);
-		btn = e2_dialog_add_button_custom (ownership_dialog, FALSE,
-			&E2_BUTTON_APPLYTOALL, NULL, NULL, NULL);
+		e2_dialog_add_defined_button (ownership_dialog, &E2_BUTTON_CANCEL);
+		btn = e2_dialog_add_defined_button (ownership_dialog, &E2_BUTTON_APPLYTOALL);
 		if (!rt.permission)
 			gtk_widget_set_sensitive (btn, FALSE);
+		e2_button_derive (&no_btn, &E2_BUTTON_NO, BTN_NO_SKIP);
+		flags |= E2_DIALOG_MULTI;
 	}
-
-	if (rt.permission || multi)
+	else
 	{
-		btn = e2_dialog_add_defined_button (ownership_dialog, &E2_BUTTON_CANCEL);
-		if (!rt.permission)
-			gtk_widget_set_sensitive (btn, FALSE);
+		e2_dialog_set_negative_response (ownership_dialog, GTK_RESPONSE_NO);
+		e2_button_derive (&no_btn, &E2_BUTTON_NO, BTN_NO_CANCEL);
 	}
 
-	e2_dialog_add_button_custom (ownership_dialog, TRUE, &E2_BUTTON_OK,
-		NULL, NULL, NULL);
+	e2_dialog_add_defined_button (ownership_dialog, &no_btn);
+	E2_BUTTON_APPLY.showflags |= E2_BTN_DEFAULT;	//CHECKME local copy ?
+	btn = e2_dialog_add_defined_button (ownership_dialog, &E2_BUTTON_APPLY);
+	if (!rt.permission)
+		gtk_widget_set_sensitive (btn, FALSE);
+
+	if (permitted)
+	{
+		g_signal_connect (
+#ifdef USE_GTK2_14
+			G_OBJECT (gtk_bin_get_child (GTK_BIN(rt.user_combo))),
+#else
+			G_OBJECT (GTK_BIN (rt.user_combo)->child),
+#endif
+			"activate",
+			G_CALLBACK(_e2_ownership_dialog_activate_cb), rt.group_combo);
+		if (thisis_dir)
+			//don't automatically apply the current values, in case user wants to check the recurse button
+			g_signal_connect (
+#ifdef USE_GTK2_14
+				G_OBJECT (gtk_bin_get_child (GTK_BIN(rt.group_combo))),
+#else
+				G_OBJECT (GTK_BIN (rt.group_combo)->child),
+#endif
+				"activate",
+				G_CALLBACK(_e2_ownership_dialog_activate_cb), btn);
+		else
+			g_signal_connect (
+#ifdef USE_GTK2_14
+				G_OBJECT (gtk_bin_get_child (GTK_BIN(rt.group_combo))),
+#else
+				G_OBJECT (GTK_BIN (rt.group_combo)->child),
+#endif
+				"activate",
+				G_CALLBACK(_e2_ownership_dialog_activate_cb2), ownership_dialog);
+	}
 
-	e2_dialog_setup (ownership_dialog, app.main_window);
-/*  non-modal not supported
-	e2_dialog_run (ownership_dialog, NULL,
-	E2_DIALOG_NONMODAL | E2_DIALOG_THREADED | E2_DIALOG_FREE);
-*/
 	gdk_threads_enter ();
-	e2_dialog_run (ownership_dialog, NULL, 0);
-	gtk_main ();
+	e2_dialog_setup (ownership_dialog, app.main_window);
+	e2_dialog_run (ownership_dialog, NULL, flags);
+	if (*winx_ret >= 0 && *winy_ret >= 0)
+		gtk_window_move (GTK_WINDOW (ownership_dialog), *winx_ret, *winy_ret);
+	//refreshing is enabled downstream, while waiting
+	//block until the user selects
+	result = e2_dialog_wait (ownership_dialog, TRUE, TRUE, multi, TRUE);//CHECKME more reliable if TRUE maincontext ?
 	gdk_threads_leave ();
 
-//	DialogButtons result = rt.result;
-//	if (result == OK || result == YES_TO_ALL)
-	if (rt.result == OK || rt.result == YES_TO_ALL)
+	if (result == OK || result == YES_TO_ALL)
 	{
-		*owner_ret = rt.owner;
-		*group_ret = rt.group;
-		*recurse_ret = rt.recurse;
-		*permission_ret = rt.permission;
+		if (permitted)
+		{
+			*owner_ret = _e2_ownership_dialog_get_user_id (&rt);
+			*group_ret = _e2_ownership_dialog_get_group_id (&rt);
+			*recurse_ret = rt.recurse;
+		}
 	}
-//	DEALLOCATE (E2_OwnersDldRuntime, rt);
-//	return result;
-	return rt.result;
+	if (GTK_IS_DIALOG(ownership_dialog)) //not explicitly closed by the user
+	{
+		gdk_threads_enter ();
+		gtk_window_get_position (GTK_WINDOW (ownership_dialog), winx_ret, winy_ret); //irrelevant if closed
+		//now it's safe to destroy widgets
+		gtk_widget_destroy (ownership_dialog);
+		gdk_threads_leave ();
+	}
+
+	return result;
 }
