@@ -1,6 +1,6 @@
-/* $Id: e2_ownership_dialog.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_ownership_dialog.h 1663 2009-05-09 09:59:37Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -28,9 +28,9 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 // the lowest alternate UID available to non-root users
 #define UID_LOWLIMIT 500
 
-void e2_ownership_dialog_warn (GtkWidget *box);
 DialogButtons e2_ownership_dialog_run (VPATH *localpath,
-	uid_t *owner_ret, gid_t *group_ret,
-	gboolean *recurse_ret, gboolean *permission_ret, gboolean multisrc);
+	gboolean multi, gboolean permitted,
+	uid_t *owner_ret, gid_t *group_ret, gboolean *recurse_ret,
+	gint *winx_ret, gint *winy_ret);
 
 #endif //ndef __E2_OWNERSHIP_DIALOG_H__
