@@ -1,6 +1,6 @@
-/* $Id: e2_permissions_dialog.c 868 2008-05-06 04:42:09Z tpgww $
+/* $Id: e2_permissions_dialog.c 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -25,10 +25,15 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "e2_dialog.h"
 #include "e2_permissions_dialog.h"
 #include "e2_ownership_dialog.h"
+#include "e2_filelist.h"
+
+//order of these determines order of chmod_buttons[] in dialog data
+enum { USR_READ, USR_WRITE, USR_EXEC, GRP_READ, GRP_WRITE, GRP_EXEC,
+	   OTH_READ, OTH_WRITE, OTH_EXEC, SETUID, SETGID, STICKY, PERM_COUNT };
 
 typedef struct _E2_PermsDldRuntime
 {
-	GtkWidget *chmod_buttons[12];
+	GtkWidget *chmod_buttons[PERM_COUNT];
 	GtkWidget *set_perms_button;
 	GtkWidget *add_perms_button;
 	GtkWidget *remove_perms_button;
@@ -36,80 +41,69 @@ typedef struct _E2_PermsDldRuntime
 	GtkWidget *recurse_dirs_button;
 	GtkWidget *recurse_other_button;
 	gboolean permission;
-	E2_RecurseType recurse;
-	mode_t mode;
-	gchar *stringmode;
-	DialogButtons result;
+	mode_t mode;	//current-permissions of item, logged during dialog creation
 } E2_PermsDldRuntime;
 
-enum { SETUID, SETGID, STICKY, USR_READ, USR_WRITE, USR_EXEC, GRP_READ,
-       GRP_WRITE, GRP_EXEC, OTH_READ, OTH_WRITE, OTH_EXEC };
-
-static mode_t mask[12] = {S_ISUID, S_ISGID, S_ISVTX, S_IRUSR, S_IWUSR, S_IXUSR,
-                          S_IRGRP, S_IWGRP, S_IXGRP, S_IROTH, S_IWOTH, S_IXOTH};
-//strings used for command string, not translated
-static gchar *add_strings[12] =
-	{"u+s", "g+s", "o+t", "u+r", "u+w", "u+x", "g+r", "g+w", "g+x", "o+r", "o+w", "o+x"};
-static gchar *remove_strings[12] =
-	{"u-s", "g-s", "o-t", "u-r", "u-w", "u-x", "g-r", "g-w", "g-x", "o-r", "o-w", "o-x"};
+//these flags must be in the same order as the enum above
+static mode_t mask[PERM_COUNT] =
+	{ S_IRUSR, S_IWUSR, S_IXUSR, S_IRGRP, S_IWGRP, S_IXGRP,
+      S_IROTH, S_IWOTH, S_IXOTH, S_ISUID, S_ISGID, S_ISVTX };
 
 /**
-@brief construct and save string-form mode string from current dialog settings
-The result is stored as a newly-allocated string at rt->mode. Maybe NULL on error.
+@brief construct mode-string corresponding to widget settings
+The returned string will begin with mode-indicator char 's', 'a' or 'r',
+followed by digits representing octal mode (e.g. "4755")
 @param rt pointer to dialog data struct
 
-@return
+@return newly-allocated string, or NULL in case of error
 */
-static void _e2_permissions_dialog_get_mode_string (E2_PermsDldRuntime *rt)
+static void _e2_permissions_dialog_get_mode_string (E2_ChmodType *op_ret,
+	mode_t *mode_ret, E2_PermsDldRuntime *rt)
 {
+	if (
+#ifdef USE_GTK2_14
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->set_perms_button))
+#else
+		GTK_TOGGLE_BUTTON (rt->set_perms_button)->active
+#endif
+	)
+		*op_ret = E2_CHMOD_SET;
+	else if (
+#ifdef USE_GTK2_14
+			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->add_perms_button))
+#else
+			GTK_TOGGLE_BUTTON (rt->add_perms_button)->active
+#endif
+	)
+		*op_ret = E2_CHMOD_ADD;
+	else if (
+#ifdef USE_GTK2_14
+			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->remove_perms_button))
+#else
+			GTK_TOGGLE_BUTTON (rt->remove_perms_button)->active
+#endif
+	)
+		*op_ret = E2_CHMOD_REM;
+	else //should never happen
+	{
+		printd (WARN, "Strange error in permissions dialog task");
+		return;	//default parameters set upsteam
+	}
+
+	mode_t mode = 0;
 	gint i;
-	GString *mode_string = g_string_sized_new (16);
-	if (GTK_TOGGLE_BUTTON (rt->set_perms_button)->active)
+	for (i = 0; i < PERM_COUNT; i++)
 	{
-		mode_t mode = 0;
-		for (i = 0; i < 12; i++)
-		{
-			if (GTK_TOGGLE_BUTTON (rt->chmod_buttons[i])->active)
-				mode |= mask[i];
-		}
-		g_string_printf (mode_string, "%o", (guint) mode);
+		if (
+#ifdef USE_GTK2_14
+			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->chmod_buttons[i]))
+#else
+			GTK_TOGGLE_BUTTON (rt->chmod_buttons[i])->active
+#endif
+			)
+			mode |= mask[i];
 	}
-	else if (GTK_TOGGLE_BUTTON (rt->add_perms_button)->active)
-	{
-		for (i = 0; i < 12; i++)
-		{
-			if (GTK_TOGGLE_BUTTON (rt->chmod_buttons[i])->active)
-			{
-				g_string_append (mode_string, add_strings[i]);
-				g_string_append_c (mode_string, ',');
-			}
-		}
-		i = mode_string->len;
-		if (i > 0)
-		  g_string_truncate (mode_string, i-1); // get rid of trailing comma
-	}
-	else if (GTK_TOGGLE_BUTTON (rt->remove_perms_button)->active)
-	{
-		for (i = 0; i < 12; i++)
-		{
-			if (GTK_TOGGLE_BUTTON (rt->chmod_buttons[i])->active)
-			{
-				g_string_append (mode_string, remove_strings[i]);
-				g_string_append_c (mode_string, ',');
-			}
-		}
-		i = mode_string->len;
-		if (i > 0)
-			g_string_truncate (mode_string, i-1); // get rid of trailing comma
-	}
-	else
-	{
-		printd (DEBUG, "Strange error in permissions dialog.");
-		g_string_free (mode_string, TRUE);
-		rt->stringmode = NULL;
-	}
-	rt->stringmode = mode_string->str;
-	g_string_free (mode_string, FALSE);
+	*mode_ret = mode; //redundant & ALLPERMS);
 }
 /**
 @brief add a check button for a current permission
@@ -136,7 +130,7 @@ static void _e2_permissions_dialog_add_chmod_button (
 {
 	if (label == NULL)
 	{    //align the button reasonably
-		GtkWidget *aligner =  gtk_alignment_new (0.1 , 0.5, 0, 0 );
+		GtkWidget *aligner =  gtk_alignment_new (0.1, 0.5, 0, 0 );
 		GtkWidget *check_button = gtk_check_button_new ();
 		gtk_container_add (GTK_CONTAINER (aligner), check_button);
 		gtk_table_attach (GTK_TABLE (table), aligner,
@@ -147,7 +141,6 @@ static void _e2_permissions_dialog_add_chmod_button (
 			0 //ypadding
 			);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_button), state);
-		gtk_widget_show (check_button);
 		rt->chmod_buttons[i] = check_button;
 	}
 	else  //there is a label
@@ -160,138 +153,121 @@ static void _e2_permissions_dialog_add_chmod_button (
 /**
 @brief "toggled" signal callback for add_perms_button and remove_perms_button
 This sets all permission-button states to FALSE
-@param widget the activated button widget
+@param button the toggled button
 @param rt pointer to dialog data struct
 
 @return
 */
-static void _e2_permissions_dialog_clear_chmod_buttons_cb (GtkWidget *widget,
+static void _e2_permissions_dialog_clear_chmod_buttons_cb (GtkToggleButton *button,
 	E2_PermsDldRuntime *rt)
 {
 	gint i;
-	for (i = 0; i < 12; i++)
+	for (i = 0; i < PERM_COUNT; i++)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rt->chmod_buttons[i]), FALSE);
 }
 /**
 @brief "toggled" signal callback for set_perms_button
-This sets all permission-button states to conform to mode FIXME
-@param widget the activated button widget
+This sets all permission-button states to conform to mode stored in @a rt
+@param button UNUSED the toggled button
 @param rt pointer to dialog data struct
 
 @return
 */
-static void _e2_permissions_dialog_reset_chmod_buttons_cb (GtkWidget *widget,
+static void _e2_permissions_dialog_reset_chmod_buttons_cb (GtkToggleButton *button,
 	E2_PermsDldRuntime *rt)
 {
-	mode_t st_mode = rt->mode;
-	gboolean flag;
+	mode_t mode = rt->mode;
 	gint i;
-	for (i = 0; i < 12; i++)
+	for (i = 0; i < PERM_COUNT; i++)
 	{
-		flag = st_mode & mask[i];
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rt->chmod_buttons[i]), flag);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rt->chmod_buttons[i]),
+			(mode & mask[i]));
 	}
 }
 /**
 @brief "toggled" signal callback for recurse_button
 
-@param widget the activated button widget
+@param button the toggled button
 @param rt pointer to dialog data struct
 
 @return
 */
-static void _e2_permissions_dialog_toggle_recurse_button_cb (GtkWidget *widget,
+static void _e2_permissions_dialog_toggle_recurse_button_cb (GtkToggleButton *button,
 	E2_PermsDldRuntime *rt)
 {
-	if (!rt->permission)
-		return;
-	gboolean flag = (GTK_TOGGLE_BUTTON (widget)->active);
-	gtk_widget_set_sensitive (rt->recurse_dirs_button, flag);
-	gtk_widget_set_sensitive (rt->recurse_other_button, flag);
+	if (rt->permission)
+	{
+		gboolean flag =
+#ifdef USE_GTK2_14
+			gtk_toggle_button_get_active (button);
+#else
+			button->active;
+#endif
+		gtk_widget_set_sensitive (rt->recurse_dirs_button, flag);
+		gtk_widget_set_sensitive (rt->recurse_other_button, flag);
+	}
 }
 /**
 @brief "toggled" signal callback for recurse_dirs and recurs_others buttons
 
-@param widget the activated button widget
+@param button the toggled button
 @param rt pointer to dialog data struct
 
 @return
 */
-static void _e2_permissions_dialog_toggle_recurse_type_cb (GtkWidget *widget,
+static void _e2_permissions_dialog_toggle_recurse_type_cb (GtkToggleButton *button,
 	E2_PermsDldRuntime *rt)
 {
-	if (!rt->permission)
-		return;
-	if (GTK_TOGGLE_BUTTON (widget)->active)
-		return;	//don't care about choice to turn recursion on
-	if (widget == rt->recurse_dirs_button)
+	if (rt->permission &&
+			//don't care about choice to turn recursion on
+#ifdef USE_GTK2_14
+			!gtk_toggle_button_get_active (button)
+#else
+			!button->active
+#endif
+	)
 	{
-		if (!(GTK_TOGGLE_BUTTON (rt->recurse_other_button)->active))
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rt->recurse_other_button),TRUE);
+		if (button == GTK_TOGGLE_BUTTON (rt->recurse_dirs_button))
+		{
+			if (!
+#ifdef USE_GTK2_14
+				gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->recurse_other_button))
+#else
+				GTK_TOGGLE_BUTTON (rt->recurse_other_button)->active
+#endif
+				)
+				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rt->recurse_other_button),TRUE);
+		}
+		else //button == rt->recurse_other_button
+		{
+			if (!
+#ifdef USE_GTK2_14
+				gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->recurse_dirs_button))
+#else
+				GTK_TOGGLE_BUTTON (rt->recurse_dirs_button)->active
+#endif
+			)
+				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rt->recurse_dirs_button),TRUE);
+		}
 	}
-	else //widget == rt->recurse_other_button
-	{
-		if (!(GTK_TOGGLE_BUTTON (rt->recurse_dirs_button)->active))
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rt->recurse_dirs_button),TRUE);
-	}
-}
-
-/**
-@brief dialog response callback
-
-@param dialog the permissions-dialog
-@param response the response for the clicked button
-@param result pointer to store for the dialog return value
-
-@return
-*/
-static void _e2_permissions_dialog_response_cb (GtkDialog *dialog,
-	gint response, E2_PermsDldRuntime *rt)
-{
-	switch (response)
-	{
-		case GTK_RESPONSE_OK:
-		case E2_RESPONSE_APPLYTOALL:
-			rt->recurse = E2_RECURSE_NONE;	//probably redundant
-			if (rt->permission)	//some widgets only exist if permission is TRUE;
-			{
-				_e2_permissions_dialog_get_mode_string (rt);	//set rt->stringmode
-				if (rt->recurse_button != NULL
-				  //the recurse button has been added to the dialog
-					&& GTK_TOGGLE_BUTTON (rt->recurse_button)->active)
-				{
-					if (GTK_TOGGLE_BUTTON (rt->recurse_dirs_button)->active)
-						rt->recurse = E2_RECURSE_DIRS;
-					if (GTK_TOGGLE_BUTTON (rt->recurse_other_button)->active)
-						rt->recurse |= E2_RECURSE_OTHER;
-					if (rt->recurse == (E2_RECURSE_DIRS | E2_RECURSE_OTHER))
-						rt->recurse = E2_RECURSE_ALL;
-				}
-			}
-			break;
-		default:
-		break;
-	}
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-	e2_dialog_response_decode_cb (dialog, response, &rt->result);
 }
 /**
 @brief create and run a dialog for changing item permissions
 
-@param localpath path of item to be processed
+@param localpath virtual path of item to be processed, with localised string
+@param multi TRUE if this dialog is part of a series for multiple items
+@param permitted TRUE if change is authorised
 @param mode_ret store for mode string to be used in the backend chmod function
 @param recurse_ret store for returning whether to recurse the changes
-@param permission_ret store for returning whether a change is authorised
-@param multi TRUE if this dialog is part of a series for multiple items
+@param winx_ret store for popup window horiz boundary (may hold -1)
+@param winy_ret store for popup window vert boundary (may hold -1)
 
 @return enumerator corresponing to the clicked dialog button
 */
-DialogButtons e2_permissions_dialog_run (
-				VPATH *localpath,
-				gchar **mode_ret,
-				E2_RecurseType *recurse_ret,
-				gboolean *permission_ret,
-				gboolean multi )
+DialogButtons e2_permissions_dialog_run (VPATH *localpath,
+	gboolean multi, gboolean permitted,
+	E2_ChmodType *op_ret, mode_t *mode_ret,
+	E2_RecurseType *recurse_ret, gint *winx_ret, gint *winy_ret)
 {
 	GtkWidget *permissions_dialog;
 	GtkWidget *dialog_vbox, *sub_vbox;
@@ -300,30 +276,41 @@ DialogButtons e2_permissions_dialog_run (
 	GtkWidget *action_area;
 	GtkWidget *table;
 
-//	gchar perm_string[32];
 	struct stat statbuf;
 	struct passwd *pw_buf;
 	struct group *grp_buf;
 	E2_PermsDldRuntime rt;
+	DialogButtons result;
 
 	if (e2_fs_lstat (localpath, &statbuf E2_ERR_NONE()))
 		return CANCEL;
 
-	rt.permission = e2_fs_check_write_permission (localpath E2_ERR_NONE());
-	rt.recurse = E2_RECURSE_NONE;
-	rt.stringmode = NULL;
+//tag DEBUGfreeze;
+	//maybe this helps gtk 2.16 crashes ?
+//	gdk_threads_enter ();
+
+	rt.permission = permitted;
 
 	GString *label_text = g_string_sized_new (NAME_MAX+20);
 	gboolean thisis_dir = e2_fs_is_dir3 (localpath E2_ERR_NONE());
 	permissions_dialog = e2_dialog_create (NULL, NULL, _("permissions"),
-		_e2_permissions_dialog_response_cb, &rt);
-	dialog_vbox = GTK_DIALOG (permissions_dialog)->vbox;
-	action_area = GTK_DIALOG (permissions_dialog)->action_area;
+		DUMMY_RESPONSE_CB, NULL);
+
+	dialog_vbox =
+#ifdef USE_GTK2_14
+		gtk_dialog_get_content_area (GTK_DIALOG (permissions_dialog));
+#else
+		GTK_DIALOG (permissions_dialog)->vbox;
+#endif
+	action_area =
+#ifdef USE_GTK2_14
+		gtk_dialog_get_action_area (GTK_DIALOG (permissions_dialog));
+#else
+		GTK_DIALOG (permissions_dialog)->action_area;
+#endif
 	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), E2_PADDING);
 	gtk_dialog_set_has_separator (GTK_DIALOG (permissions_dialog), FALSE);  //all info in frames, no need
 
-	//  table = e2_widget_add_table (dialog_vbox, 3, 3, TRUE, TRUE, 0);  //3 rows, 3 cols, homogen, fill, no pad
-	//  gtk_table_set_row_spacing (GTK_TABLE(table), 0, E2_PADDING);
 	gchar *label = (thisis_dir) ? _("Directory name") : _("Filename") ;
 	gchar *name = g_filename_display_basename (VPSTR (localpath));
 	g_string_printf (label_text, "%s: <b>%s</b>", label, name);
@@ -332,7 +319,6 @@ DialogButtons e2_permissions_dialog_run (
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, E2_PADDING); //top, bottom padding
 	e2_widget_add_mid_label (hbox, label_text->str, 0, TRUE, E2_PADDING); //L, R padding
-	gtk_widget_show (hbox);
 
 	label = _("User");
 	if ((pw_buf = getpwuid (statbuf.st_uid)) != NULL)
@@ -350,7 +336,6 @@ DialogButtons e2_permissions_dialog_run (
 	hbox = gtk_hbox_new (FALSE, 0);
 	e2_widget_add_mid_label (hbox, label_text->str, 0, TRUE, E2_PADDING);
 	gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, 0);
-	gtk_widget_show (hbox);
 
 	label = _("Group");
 	if ((grp_buf = getgrgid (statbuf.st_gid)) != NULL)
@@ -368,10 +353,9 @@ DialogButtons e2_permissions_dialog_run (
 	hbox = gtk_hbox_new (FALSE, 0);
 	e2_widget_add_mid_label (hbox, label_text->str, 0, TRUE, E2_PADDING);
 	gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, 0);
-	gtk_widget_show (hbox);
 
 	gchar *s = e2_fs_get_perm_string (statbuf.st_mode);
-	g_string_assign (label_text, g_utf8_next_char (s));	//skip the "type code" in 1st byte of string
+	label_text = g_string_assign (label_text, g_utf8_next_char (s));	//skip the "type code" in 1st byte of string
 	g_free (s);
 	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 6), ' ');
 	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 3), ' ');
@@ -381,15 +365,12 @@ DialogButtons e2_permissions_dialog_run (
 	hbox = gtk_hbox_new (FALSE, 0);
 	e2_widget_add_mid_label (hbox, label_text->str, 0, TRUE, E2_PADDING);
 	gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, 0);
-	gtk_widget_show (hbox);
 
 	frame = gtk_frame_new (_("Permissions"));
 	gtk_box_pack_start (GTK_BOX (dialog_vbox), frame, TRUE, TRUE, E2_PADDING);
-	gtk_widget_show (frame);
 
 	sub_vbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (frame), sub_vbox);
-	gtk_widget_show (sub_vbox);
 
 	table = e2_widget_add_table (sub_vbox, 4, 5, TRUE, TRUE, 0);  //4 rows, 5 cols, homogen, fill, no pad
 	e2_widget_add_mid_label_to_table (table, _("Read"), 0,		1,2,0,1);  //gint left, gint right, gint top, gint bottom)
@@ -427,7 +408,7 @@ DialogButtons e2_permissions_dialog_run (
 	_e2_permissions_dialog_add_chmod_button (table, 4,5,3,4, STICKY, _("Sticky"),
 		statbuf.st_mode & S_ISVTX, &rt);
 
-	if (!rt.permission || !thisis_dir)
+	if (!permitted || !thisis_dir)
 	{
 		//prevent testing of missing widgets in the response cb
 		rt.recurse_button = NULL;
@@ -435,15 +416,13 @@ DialogButtons e2_permissions_dialog_run (
 		rt.recurse_other_button = NULL;
 	}
 
-	if (rt.permission)
+	if (permitted)
 	{
-		frame = gtk_frame_new (_("Action:"));
+		frame = gtk_frame_new (_("Action"));
 		gtk_box_pack_start (GTK_BOX (dialog_vbox), frame, TRUE, TRUE, E2_PADDING);
-		gtk_widget_show (frame);
 
 		sub_vbox = gtk_vbox_new (FALSE, 0);
 		gtk_container_add (GTK_CONTAINER (frame), sub_vbox);
-		gtk_widget_show (sub_vbox);
 
 		table = e2_widget_add_table (sub_vbox, 1, 5, TRUE, TRUE, 0);  //1 rows, 5 cols, homogen, fill, no pad
 		rt.mode = statbuf.st_mode;
@@ -466,12 +445,7 @@ DialogButtons e2_permissions_dialog_run (
 			rt.recurse_button = e2_button_add_toggle_to_table (table, _("R_ecurse:"),
 				FALSE, _e2_permissions_dialog_toggle_recurse_button_cb, &rt,
 				1,2,0,1 );
-#ifdef USE_GTK2_12TIPS
-			gtk_widget_set_tooltip_text (
-#else
-			e2_widget_set_tooltip (NULL,
-#endif
-				rt.recurse_button,
+			e2_widget_set_safetip (rt.recurse_button,
 				_("If activated, changes will be applied to selected items and"
 					" also their descendents, if such items match your choice of"
 					" \"directories\" and/or \"others\" (anything not a directory)"));
@@ -485,49 +459,96 @@ DialogButtons e2_permissions_dialog_run (
 	}
 	else //no right to change object permissions
 		//show message
-		e2_ownership_dialog_warn (dialog_vbox);
+		e2_dialog_setup_auth (dialog_vbox);
 
 	g_string_free (label_text, TRUE);
+	result = NO_TO_ALL;	//default in case of abort
 
-	// add buttons in the order that they will appear
+	E2_DialogFlags flags = 0;
+
+	//add buttons in the order that they will appear
 	GtkWidget *btn;
+	E2_Button no_btn;
 	if (multi)
 	{
 		e2_dialog_set_negative_response (permissions_dialog, E2_RESPONSE_NOTOALL);
-		e2_dialog_add_defined_button (permissions_dialog, &E2_BUTTON_NOTOALL);
-		btn = e2_dialog_add_button_custom (permissions_dialog, FALSE,
-			&E2_BUTTON_APPLYTOALL, NULL, NULL, NULL);
-		if (!rt.permission)
+		e2_dialog_add_defined_button (permissions_dialog, &E2_BUTTON_CANCEL);
+		btn = e2_dialog_add_defined_button (permissions_dialog, &E2_BUTTON_APPLYTOALL);
+		if (!permitted)
 			gtk_widget_set_sensitive (btn, FALSE);
+		e2_button_derive (&no_btn, &E2_BUTTON_NO, BTN_NO_SKIP);
+		flags |= E2_DIALOG_MULTI;
 	}
-	if (rt.permission || multi)
+	else
 	{
-		btn = e2_dialog_add_defined_button (permissions_dialog, &E2_BUTTON_CANCEL);
-		if (!rt.permission)
-			gtk_widget_set_sensitive (btn, FALSE);
+		e2_dialog_set_negative_response (permissions_dialog, GTK_RESPONSE_NO);
+		e2_button_derive (&no_btn, &E2_BUTTON_NO, BTN_NO_CANCEL);
 	}
 
-	e2_dialog_add_button_custom (permissions_dialog, TRUE, &E2_BUTTON_OK,
-		NULL, NULL, NULL);
+	e2_dialog_add_defined_button (permissions_dialog, &no_btn);
+
+	E2_BUTTON_APPLY.showflags |= E2_BTN_DEFAULT;	//CHECKME local copy ?
+	btn = e2_dialog_add_defined_button (permissions_dialog, &E2_BUTTON_APPLY);
+	if (!permitted)
+		gtk_widget_set_sensitive (btn, FALSE);
 
-	e2_dialog_setup (permissions_dialog, app.main_window);
-/*  non-modal not supported
-	e2_dialog_run (permissions_dialog, NULL,
-	E2_DIALOG_NONMODAL | E2_DIALOG_THREADED | E2_DIALOG_FREE);
-*/
 	gdk_threads_enter ();
-	e2_dialog_run (permissions_dialog, NULL, 0);
-	gtk_main ();
+	e2_dialog_setup (permissions_dialog, app.main_window);
+	e2_dialog_run (permissions_dialog, NULL, flags);
+	if (*winx_ret >= 0 && *winy_ret >= 0)
+		gtk_window_move (GTK_WINDOW (permissions_dialog), *winx_ret, *winy_ret);
+	//block until the user selects
+	//refreshing is enabled downstream, while waiting
+	result = e2_dialog_wait (permissions_dialog, TRUE, TRUE, multi, TRUE);//CHECKME more reliable if TRUE maincontext ?
 	gdk_threads_leave ();
 
-	*permission_ret = rt.permission;
-	*recurse_ret = rt.recurse;
-	if (rt.result == OK || rt.result == YES_TO_ALL)
-		*mode_ret = rt.stringmode;	//possibly NULL
-	else if (rt.stringmode != NULL)
+	//defaults
+	*op_ret = E2_CHMOD_NONE;
+	*mode_ret = 0;
+	*recurse_ret = E2_RECURSE_NONE;
+	if (result == OK || result == YES_TO_ALL)
 	{
-		g_free (rt.stringmode);
-		*mode_ret = NULL;
+		if (permitted)
+		{
+			_e2_permissions_dialog_get_mode_string (op_ret, mode_ret, &rt);
+			//some widgets only exist if permission is TRUE;
+			if (rt.recurse_button != NULL
+			  //the recurse button has been added to the dialog
+#ifdef USE_GTK2_14
+				&& gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt.recurse_button))
+#else
+				&& GTK_TOGGLE_BUTTON (rt.recurse_button)->active
+#endif
+			)
+			{
+				if (
+#ifdef USE_GTK2_14
+					gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt.recurse_dirs_button))
+#else
+					GTK_TOGGLE_BUTTON (rt.recurse_dirs_button)->active
+#endif
+				)
+					*recurse_ret = E2_RECURSE_DIRS;
+				if (
+#ifdef USE_GTK2_14
+					gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt.recurse_other_button))
+#else
+					GTK_TOGGLE_BUTTON (rt.recurse_other_button)->active
+#endif
+				)
+					*recurse_ret |= E2_RECURSE_OTHER;
+			}
+		}
 	}
-	return rt.result;
+
+	if (GTK_IS_DIALOG (permissions_dialog)) //not explicitly closed by the user
+	{
+		gdk_threads_enter ();
+		gtk_window_get_position (GTK_WINDOW (permissions_dialog), winx_ret, winy_ret);
+		//now it's safe to destroy the window
+		gtk_widget_destroy (permissions_dialog);
+		gdk_threads_leave ();
+	}
+
+	return result;
 }
