@@ -1,6 +1,6 @@
-/* $Id: e2_permissions_dialog.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_permissions_dialog.h 1792 2009-07-14 12:29:50Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -26,13 +26,23 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 typedef enum
 {
-	E2_RECURSE_NONE = 0,
-	E2_RECURSE_DIRS = 1,
-	E2_RECURSE_OTHER = 1 << 2,
-	E2_RECURSE_ALL = 1 << 3
+	E2_RECURSE_NONE  = 0,
+	E2_RECURSE_DIRS  = 1,
+	E2_RECURSE_OTHER = 1 << 1
 } E2_RecurseType;
+#define E2_RECURSE_ALL E2_RECURSE_DIRS | E2_RECURSE_OTHER
 
-DialogButtons e2_permissions_dialog_run (VPATH *localpath, gchar **mode_ret,
-	E2_RecurseType *recurse_ret, gboolean *permission_ret, gboolean multi);
+typedef enum
+{
+	E2_CHMOD_NONE,
+	E2_CHMOD_SET,
+	E2_CHMOD_ADD,
+	E2_CHMOD_REM
+} E2_ChmodType;
+
+DialogButtons e2_permissions_dialog_run (VPATH *localpath,
+	gboolean multi, gboolean permitted,
+	E2_ChmodType *op_ret, mode_t *mode_ret,
+	E2_RecurseType *recurse_ret, gint *winx_ret, gint *winy_ret);
 
 #endif //ndef __E2_PERMISSIONS_DIALOG_H__
