@@ -1,6 +1,6 @@
-/* $Id: e2_size_filter_dialog.c 748 2007-11-22 22:04:33Z tpgww $
+/* $Id: e2_size_filter_dialog.c 1987 2009-10-14 00:41:33Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -32,9 +32,12 @@ typedef struct _E2_SzFltDlgData
 //	GtkWidget *menu_item;	//copy of pointer to clicked filters menu item
 //	gboolean itemstate;	//initial state of menu_item
 //	gboolean blocked = FALSE;	//flag to prevent toggles when not wanted
-	ViewInfo *view;	//data for filelist being processed
+	ViewInfo *view;		//data for filtered file pane
 } E2_SzFltDlgData;
 
+static void _e2_size_filter_dialog_response_cb (GtkDialog *dialog,
+	gint response, E2_SzFltDlgData *rt);
+
 /*static void _e2_size_filter_dialog_item_toggle_cb (void)
 {
   gulong handler = g_signal_handler_find (menu_item, G_SIGNAL_MATCH_FUNC,
@@ -65,6 +68,19 @@ static void _e2_size_filter_dialog_ok (E2_SzFltDlgData *rt)
 #endif
 }
 /**
+@brief cleanup during the destruction of the view related to a dialog
+@param object UNUSED the view-related object being destroyed
+@param rt pointer to data struct for the dialog
+@return
+*/
+static void _e2_size_filter_dialog_destroy_cb (GtkObject *object,
+	E2_SzFltDlgData *rt)
+{
+	g_signal_handlers_disconnect_by_func ((gpointer)rt->dialog,
+		_e2_size_filter_dialog_response_cb, rt); //no double-handling
+	_e2_size_filter_dialog_response_cb (GTK_DIALOG (rt->dialog), 0, rt);
+}
+/**
 @brief dialog response callback
 
 @param dialog the permissions-dialog
@@ -78,34 +94,30 @@ static void _e2_size_filter_dialog_response_cb (GtkDialog *dialog,
 {
 	switch (response)
 	{
-		case GTK_RESPONSE_OK:
+		case E2_RESPONSE_APPLY:
 			gtk_widget_hide (rt->dialog);
 			_e2_size_filter_dialog_ok (rt);
-		  	//prevent toggle when dialog is destroyed
-//			blocked = TRUE;
-		  break;
+			break;
 		case E2_RESPONSE_REMOVE:
-		  //update the cached copy of the state
-		  rt->view->size_filter.active = FALSE;
-		  gtk_widget_hide (rt->dialog);
-		  //show the results
-		  e2_toolbar_toggle_filter_button (rt->view);
-		  e2_fileview_refilter_list (rt->view);
+			//update the cached copy of the state
+			rt->view->size_filter.active = FALSE;
+			gtk_widget_hide (rt->dialog);
+			//show the results
+			e2_toolbar_toggle_filter_button (rt->view);
+			e2_fileview_refilter_list (rt->view);
 #ifdef E2_STATUS_DEMAND
-		  if (rt->view == curr_view)
-			e2_window_update_status_bar (NULL);
+			if (rt->view == curr_view)
+				e2_window_update_status_bar (NULL);
 #endif
-		  //prevent toggle when dialog is destroyed
-//		  blocked = TRUE;
-		  break;
-//		case GTK_RESPONSE_CANCEL:
+			break;
 		default:
 		  //the menu item will be re-toggled when dialog is destroyed
-		break;
+			break;
 	}
+	g_signal_handlers_disconnect_by_func ((gpointer)rt->view->treeview,
+		_e2_size_filter_dialog_destroy_cb, rt);
 	gtk_widget_destroy (rt->dialog);
 	DEALLOCATE (E2_SzFltDlgData, rt);
-//	gtk_main_quit ();
 }
 /**
 @brief handle Return keypresses in the size entry
@@ -117,7 +129,7 @@ static void _e2_size_filter_dialog_activated_cb (GtkEntry *entry,
 	E2_SzFltDlgData *rt)
 {
 	_e2_size_filter_dialog_response_cb (GTK_DIALOG (rt->dialog),
-		GTK_RESPONSE_OK, rt);
+		E2_RESPONSE_APPLY, rt);
 }
 /**
 @brief create file size filter dialog
@@ -125,15 +137,16 @@ static void _e2_size_filter_dialog_activated_cb (GtkEntry *entry,
 The state of @a item when it arrives here is opposite to that
 shown in the menu, when clicked
 
-@param item the clicked checkbox widget in the fillters menu
+@param item the activated item in the filters menu
 @param view data structure for the view to which the file list belongs
 
 @return
 */
-void e2_size_filter_dialog_create_cb (GtkWidget *item, ViewInfo *view)
+void e2_size_filter_dialog_create_cb (GtkCheckMenuItem *item, ViewInfo *view)
 {
 	GtkWidget *hbox;
 	gchar size_string[32];
+	E2_Button no_btn;
 
 	//save local copies, for later use in other functions
 //  menu_item = item;
@@ -143,9 +156,15 @@ void e2_size_filter_dialog_create_cb (GtkWidget *item, ViewInfo *view)
 	rt->view = view;
 
 	rt->dialog = e2_dialog_create (NULL, _("Display only the items which are:"),
-		_("size filter"), _e2_size_filter_dialog_response_cb, rt);
-	hbox = e2_widget_add_box (GTK_DIALOG (rt->dialog)->vbox, TRUE, E2_PADDING,
-		FALSE, FALSE, E2_PADDING);
+		_("size filter"),
+		(ResponseFunc)_e2_size_filter_dialog_response_cb, rt);
+	hbox = e2_widget_add_box (
+#ifdef USE_GTK2_14
+		gtk_dialog_get_content_area (GTK_DIALOG (rt->dialog)),
+#else
+		GTK_DIALOG (rt->dialog)->vbox,
+#endif
+		TRUE, E2_PADDING, FALSE, FALSE, E2_PADDING);
 
 	rt->operation_combo = e2_combobox_add (hbox, FALSE, 0, NULL, NULL, NULL,
 		E2_COMBOBOX_MENU_STYLE);
@@ -181,7 +200,7 @@ void e2_size_filter_dialog_create_cb (GtkWidget *item, ViewInfo *view)
 #ifdef E2_ASSISTED
 	GtkWidget *label = (GtkWidget *) g_object_get_data (G_OBJECT (rt->dialog),
 		"e2-dialog-label");
-	e2_widget_set_label_relations (GTK_LABEL (label), rt->size_entry);
+	e2_widget_set_label_relations (label, rt->size_entry);
 #endif
 	//handle <Return> key-presses when the entry is focused
 	g_signal_connect (G_OBJECT (rt->size_entry), "activate",
@@ -193,12 +212,16 @@ void e2_size_filter_dialog_create_cb (GtkWidget *item, ViewInfo *view)
 	e2_combobox_append_history_counted (rt->units_combo, 3, units_names);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (rt->units_combo), index) ;
 
+	//cleanup if the dialog rt data becomes invalid
+	g_signal_connect (GTK_OBJECT(view->treeview), "destroy",
+		G_CALLBACK (_e2_size_filter_dialog_destroy_cb), rt);
+
 	//now the buttons
-//  if (!itemstate)
-	if (!gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (item)))
+	if (!gtk_check_menu_item_get_active (item))
 		//user has just toggled off
 		e2_dialog_add_defined_button (rt->dialog, &E2_BUTTON_REMOVE);
+	e2_button_derive (&no_btn, &E2_BUTTON_NO, BTN_NO_CANCEL);
+
 	e2_dialog_show (rt->dialog, app.main_window, 0,
-		&E2_BUTTON_CANCEL, &E2_BUTTON_OK, NULL);
-//  gtk_main ();
+		&no_btn, &E2_BUTTON_APPLY, NULL);
 }
