@@ -1,6 +1,6 @@
-/* $Id: e2_bookmark.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_bookmark.h 1849 2009-09-20 10:34:04Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -23,10 +23,9 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 #include "emelfm2.h"
 
-gboolean e2_bookmark_open (gpointer from, E2_ActionRuntime *art);
 void e2_bookmark_add_cb (GtkWidget *widget, gchar *arg);
-gboolean e2_bookmark_click_cb (GtkWidget *widget,
-	GdkEventButton *event, GtkTreePath* path);
+gboolean e2_bookmark_button_press_cb (GtkWidget *widget, GdkEventButton *event,
+	GtkTreePath* path);
 
 //void e2_bookmark_clean ();
 void e2_bookmark_actions_register ();
