@@ -1,6 +1,6 @@
-/* $Id: e2_context_menu.c 853 2008-04-07 10:38:17Z tpgww $
+/* $Id: e2_context_menu.c 2053 2010-02-21 04:29:24Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2010 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -50,51 +50,80 @@ ToDo
 /*******************/
 
 /**
-@brief execute action corresponding to item selected from filetype tasks menu
-This is the callback for handling a selection of a filetype action from
-the context menu
-@param widget the selected menu item widget
+@brief execute action corresponding to item selected from a menu
+We do this outside the manu-item "activated" callback to avoid disrupting gtk's
+menu-activation process
+@param cmd pointer to allocated command-string, free'd here
+
+@return FALSE to remove the source
+*/
+static gboolean _e2_context_menu_run_filetype_action (gchar *cmd)
+{
+	printd (DEBUG, "timer callback: context menu run filetype action");
+	gdk_threads_enter ();
+	e2_filetype_exec_action (cmd);
+	gdk_threads_leave ();
+	g_free (cmd);
+	e2_utils_fake_event (); //CHECKME does this actually help?
+	return FALSE;
+}
+/**
+@brief setup to execute action corresponding to item selected from filetype tasks menu
+This is the callback for handling a selection of a filetype action from the
+context menu. To avoid disrupting gtk's menu-activation process, we do not run
+the action here.
+@param item the activated menu item widget
+@param cmd pointer to allocated command-string to be executed
 
 @return
 */
-static void _e2_context_menu_choose_filetype_action_cb (GtkWidget *widget)
+static void _e2_context_menu_choose_filetype_action_cb (GtkMenuItem *item,
+	const gchar *cmd)
 {
 	printd (DEBUG, "context menu choose filetype action cb");
-	e2_filetype_exec_action ((gchar *) gtk_widget_get_name (widget));
+	//an idle-callback is no good !?
+	g_timeout_add (100, (GSourceFunc) _e2_context_menu_run_filetype_action,
+		g_strdup(cmd));
 }
 /*
-static gboolean _e2_context_menu_item_click_cb (GtkWidget *widget,
-	GdkEventButton *event, E2_ActionRuntime *rt)
+static gboolean _e2_context_menu_item_click_cb (GtkWidget *menuitem,
+	GdkEventButton *event, E2_ActionRuntime *art)
 {
 	printd (DEBUG, "item_click_cb (widget:_,event:_,rt:_)");
-	if (event->button == 1 && event->type == GDK_BUTTON_PRESS)
+	if (event->button == 1 && event->type == GDK_BUTTON_PRESS
+#ifdef E2_MOUSECUSTOM
+		&& (event->state & E2_MODIFIER_MASK) == 0
+#endif
+		)
 	{
-		e2_action_run (widget, rt);
+		gtk_widget_hide (app.context_menu);
+		e2_action_run (menuitem, art);
+		e2_action_free_runtime (art);
 		return TRUE;
 	}
+	e2_action_free_runtime (art);
 	return FALSE;
 }
 */
 /**
-@brief context-menu destroy callback
-
+@brief context-menu "selection-done" signal callback
+This will be called _after_ running the action initiated by the selected item !
 @param menu the menu widget (= app.context_menu)
 @param data data specified when the callback was connected
 
 @return
 */
-static void _e2_context_menu_destroy_menu_cb (GtkWidget *menu, gpointer data)
+static void _e2_context_menu_selected_cb (GtkWidget *menu, gpointer data)
 {
-	printd (DEBUG, "destroy_menu_cb");
-//	printd (DEBUG, "destroy_menu_cb, refresh ref = %d", refresh_refcount);
-	//CHECKME is this specific destruction redundant ?
-	gtk_container_foreach (GTK_CONTAINER (menu), (GtkCallback) gtk_widget_destroy, NULL);
-	gtk_widget_destroy (menu);
-	//no enable refresh, here as it is done somewhere else!!
-//#ifdef E2_REFRESH_DEBUG
-//	printd (DEBUG, "enable refresh, destroy_menu_cb");
-//#endif
-//	e2_filelist_enable_refresh();
+	printd (DEBUG, "_e2_context_menu_selected_cb, menu: %x", menu);
+	//must not destroy the menu immediately, gtk upstream has not necessarily finished with it
+	//FIXME delay until action has finished running + 1 sec
+#ifdef USE_GLIB2_14
+	g_timeout_add_seconds (10,
+#else
+	g_timeout_add (10000,
+#endif
+		(GSourceFunc)e2_menu_destroy, menu);
 	app.context_menu = NULL;
 }
 
@@ -122,25 +151,29 @@ void e2_context_menu_create_desktop_actions_menu (GtkWidget *menu,
 			"Desktop Entry", "Actions", &count, NULL);
 		if (count > 0)
 		{
-			gchar *group, *label, *command;
 			gchar **iterator;
 			for (iterator = actions; *iterator != NULL; iterator++)
 			{
+				gchar *group;
 				group = g_strconcat ("Desktop Action ", *iterator, NULL);
 				if (g_key_file_has_group (key_file, group))
 				{
+					gchar *command;
 					command = g_key_file_get_string (key_file, group, "Exec", NULL);
 					if (command != NULL)
 					{
+						gchar *label;
+						GtkWidget *item;
 						label = g_key_file_get_string (key_file, group, "Name", NULL);
 						if (label == NULL)
 							label = command;
-						GtkWidget *menu_item = e2_menu_add (menu, label, NULL, NULL,
-							_e2_context_menu_choose_filetype_action_cb, NULL);
-						gtk_widget_set_name (GTK_WIDGET(menu_item), command);
+						item = e2_menu_add (menu, label, NULL, NULL,
+							_e2_context_menu_choose_filetype_action_cb, command);
+						//cleanup during menu destruction
+						g_object_set_data_full (G_OBJECT(item), "action-cmd-key",
+								command, g_free);
 						if (label != command)
 							g_free (label);
-						g_free (command);
 					}
 				}
 				g_free (group);
@@ -161,25 +194,27 @@ Each member of @a actions is like "command" or "label@command"
 void e2_context_menu_create_filetype_actions_menu (GtkWidget *menu,
 	const gchar **actions)
 {
-	gchar *sep;
+	gchar *s;
 	GtkWidget *menu_item;
 
 	while (*actions != NULL)
 	{
-		if ((sep = strchr (*actions, '@')) != NULL)  //if always ascii @, don't need g_utf8_strchr()
+		if ((s = strchr (*actions, '@')) != NULL)  //if always ascii @, don't need g_utf8_strchr()
 		{
-			*sep = '\0';
+			*s = '\0';
 			menu_item = e2_menu_add (menu, (gchar *)*actions, NULL, NULL,
-				_e2_context_menu_choose_filetype_action_cb, NULL);
-			*sep = '@';	//revert to original form (this is the 'source' data)
-			gtk_widget_set_name (menu_item, sep+1);
+				_e2_context_menu_choose_filetype_action_cb, s+1);
+			*s = '@';	//revert to original form (this is the 'source' data)
+			s++;	//point to command
 		}
 		else
 		{
-			menu_item = e2_menu_add (menu, (gchar *)*actions, NULL, NULL,
-				_e2_context_menu_choose_filetype_action_cb, NULL);
-			gtk_widget_set_name (menu_item, *actions);
+			s = (gchar *)*actions;
+			menu_item = e2_menu_add (menu, s, NULL, NULL,
+				_e2_context_menu_choose_filetype_action_cb, s);
 		}
+		//some open-with action code needs the command, from the menu item
+		g_object_set_data (G_OBJECT(menu_item), "action-cmd-key", s);
 		actions++;
 	}
 }
@@ -190,7 +225,7 @@ This adds menu-items for a filetype to @a menu
 
 @return
 */
-static void _e2_context_menu_add_filetype_item (GtkWidget *menu)
+void e2_context_menu_add_filetype_item (GtkWidget *menu)
 {
 	gboolean exec, freeext = TRUE;
 	gchar *ext, *ext2;
@@ -221,24 +256,41 @@ static void _e2_context_menu_add_filetype_item (GtkWidget *menu)
 		if (curr_view->spacedata != NULL)
 			exec = FALSE;//CHECKME commands for non-local items ?
 		else
-			exec = !e2_fs_access (&ddata, X_OK E2_ERR_NONE());
+			exec = e2_fs_is_exec2 (&ddata E2_ERR_NONE());
 #else
-		exec = !e2_fs_access (localpath, X_OK E2_ERR_NONE());
+		exec = e2_fs_is_exec2 (localpath E2_ERR_NONE());
 #endif
 		g_free (localpath);
 		freeext = FALSE;
+/*		//FIXME generalise detection of hidden items
+		if (!e2_fs_is_hidden (&ddata, E2_ERR_NONE()))
+			ext = info->filename; //look for extension from 1st byte
+//USELESS else if (*info->filename != '.')
+//			ext = info->filename;
+//UNREAL else if (*info->filename == '\0')
+//			ext = info->filename;
+		else // *info->filename == '.', a *NIX-kind of hidden file
+			ext = info->filename + sizeof (gchar); //look for extension from 2nd byte
+*/
+		ext = info->filename;
+		if (*ext == '.')
+			ext++;
 		//assumes extension is that part of the name after the leftmost '.'
-		ext = strchr (info->filename, '.');	//localised text, but assumes . is ascii
-		if (ext == info->filename //it's a dot file
-			|| (ext == NULL //no extension
+		ext = strchr (ext, '.');
+		if (ext == NULL || *(ext + sizeof (gchar)) == '\0') //no extension
+		{
+			if (/*!exec &&*/ e2_fs_is_text (
 #ifdef E2_VFS
-				&& e2_fs_is_text (&ddata E2_ERR_NONE()))) //runs 'file' again!!
+				&ddata
 #else
-				&& e2_fs_is_text (info->filename E2_ERR_NONE()))) //runs 'file' again!!
+				info->filename
 #endif
-			//fake text extension
-			ext = ".txt";	//too bad if this is not a recognised text extension in filetypes
-		else if (ext != NULL)
+				E2_ERR_NONE())) //runs 'file' again!!
+				//fake text extension
+				//too bad if this is not a recognised text extension in filetypes
+				ext = ".txt";
+		}
+		else //(ext != NULL && *(ext + sizeof (gchar)) != '\0'
 		{
 			ext2 = ext;
 			ext = F_DISPLAYNAME_FROM_LOCALE (ext);	//get utf
@@ -254,7 +306,7 @@ static void _e2_context_menu_add_filetype_item (GtkWidget *menu)
 		do
 		{
 			//skip leading dot "."
-			ext++;	//NCHR(ext); ascii '.'. always single char
+			ext += sizeof (gchar);	//ascii '.' always single char
 			actions = e2_filetype_get_actions (ext);
 			if (actions != NULL)
 			{
@@ -294,18 +346,18 @@ static void _e2_context_menu_add_filetype_item (GtkWidget *menu)
 			{
 				e2_context_menu_create_desktop_actions_menu (menu, localpath);
 				g_free (path);
-				F_FREE (localpath);
+				F_FREE (localpath, path);
 				break;
 			}
 			g_free (path);
-			F_FREE (localpath);
+			F_FREE (localpath, path);
 			sysdir++;
 		}
 		g_free (localname);
 	}
 }
 /**
-@brief populate context-menu @a menu
+@brief populate destroyable context-menu @a menu
 @param menu the context-menu widget
 @param model model for context-menu data treestore
 @param iter pointer to iter used to interrogate @a model
@@ -321,10 +373,10 @@ static void _e2_context_menu_add_items (GtkWidget *menu, GtkTreeModel *model,
 		(gtk_tree_selection_count_selected_rows (curr_view->selection) > 0);
 	if (mtype == 3)
 	{	// shift+ctrl menu
-		e2_menu_create_plugins_menu (menu, selected);
+		e2_menu_create_plugins_menu (menu, TRUE, selected);
 		return;
 	}
-	gchar *prefix = g_strconcat (_A(5), ".", NULL);
+	gchar *prefix = g_strconcat (_A(6), ".", NULL);
 	do
 	{
 		gchar *label, *icon, *command, *arg;
@@ -360,60 +412,71 @@ static void _e2_context_menu_add_items (GtkWidget *menu, GtkTreeModel *model,
 				{
 					case E2_ACTION_TYPE_SUBMENU:
 					{
-						GtkWidget *item = e2_menu_add (menu, label, icon, NULL, NULL, NULL);
-						GtkWidget *menu2;
+						GtkWidget *submenu;
 						if (arg == NULL || *arg == '\0') //no data == not a custom sub-menu
 						{
 							GtkTreeIter iter2;
 							if (gtk_tree_model_iter_children (model, &iter2, iter))
 							{
-								menu2 = gtk_menu_new ();
+								submenu = gtk_menu_new ();
 								//recurse
-								_e2_context_menu_add_items (menu2, model, &iter2, mtype);
+								_e2_context_menu_add_items (submenu, model, &iter2, mtype);
 							}
 							else
-								menu2 = NULL;
+								submenu = NULL;
 						}
 						else	//custom submenu
-							menu2 = e2_menu_create_custom_menu (arg);	//may be NULL
+							submenu = e2_menu_create_custom_menu (arg);	//may be NULL
 
-						if (menu2 != NULL)
+//						GtkWidget *item = e2_menu_add (menu, label, icon, NULL, NULL, NULL);
+						if (submenu != NULL)
 						{
-							GList *children = gtk_container_get_children (GTK_CONTAINER (menu2));
+							GList *children = gtk_container_get_children (GTK_CONTAINER (submenu));
 							if (g_list_length (children) == 0)
-							{
-								gtk_widget_destroy (item);
-								gtk_widget_destroy (menu2);
-							}
+								gtk_widget_destroy (submenu);
 							else
-								gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu2);
+							{
+								GtkWidget *item = e2_menu_add (menu, label, icon, NULL, NULL, NULL);
+								gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
+							}
 							g_list_free (children);
 						}
-						else
-							gtk_widget_destroy (item);
 					}
 						break;
 					case E2_ACTION_TYPE_BOOKMARKS:
 					{
-						E2_OptionSet *set = e2_option_get ("bookmarks");
 						GtkTreeIter iter;
+						gint pane = E2PANECUR;	//CHECKME always ok for context menu ? action may be specific
+						E2_OptionSet *set = e2_option_get ("bookmarks");
 						if (arg != NULL && *arg != '\0')
 						{
 							if (e2_tree_find_iter_from_str (set->ex.tree.model, 0, arg, &iter, TRUE))
 							{
 								GtkTreeIter iter2;
 								if (gtk_tree_model_iter_children (set->ex.tree.model, &iter2, &iter))
-									e2_menu_create_bookmarks_menu (action, set, menu, &iter2);
+									e2_menu_add_bookmark_items (menu, TRUE, action,
+										pane, set->ex.tree.model, &iter2);
 							}
-						} else if (gtk_tree_model_get_iter_first (set->ex.tree.model, &iter))
-								e2_menu_create_bookmarks_menu (action, set, menu, &iter);
+						}
+						else if (gtk_tree_model_get_iter_first (set->ex.tree.model, &iter))
+								e2_menu_add_bookmark_items (menu, TRUE, action,
+									pane, set->ex.tree.model, &iter);
 					}
 						break;
+					case E2_ACTION_TYPE_HISTORY:
+					{
+						GtkWidget *item = e2_menu_add (menu, label, icon, NULL, NULL, NULL);
+						E2_PaneRuntime *rt = e2_pane_get_runtime (curr_view->treeview,
+							arg, NULL);
+						GtkWidget *submenu = e2_pane_visited_menu (rt);
+						gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
+					}
+					break;
 					case E2_ACTION_TYPE_FILE_ACTIONS:
-						_e2_context_menu_add_filetype_item (menu);
+						e2_context_menu_add_filetype_item (menu);
 						break;
 					case E2_ACTION_TYPE_PLUGINS:
-						e2_menu_create_plugins_menu (menu, selected);
+						e2_menu_create_plugins_menu (menu, TRUE, selected);
 						break;
 					case E2_ACTION_TYPE_SEPARATOR:
 					{
@@ -427,17 +490,21 @@ static void _e2_context_menu_add_items (GtkWidget *menu, GtkTreeModel *model,
 						e2_menu_add_tear_off (menu);
 						break;
 					case E2_ACTION_TYPE_TOGGLE:
-						e2_menu_add_toggle (menu, label, icon, "", command, arg);
+						e2_menu_add_toggle (menu, TRUE, label, icon, NULL, command, arg);
 						break;
 					case E2_ACTION_TYPE_ITEM:
 					{
-						E2_ActionRuntime *actionrt = e2_action_pack_runtime (action, realarg, g_free);
-						GtkWidget *item;
-						//FIXME add tips
-						item = e2_menu_add (menu, label, icon, NULL, e2_action_run,
-								actionrt);
-						g_object_set_data_full (G_OBJECT (item), "free-callback-data", actionrt,
-							(GDestroyNotify) e2_action_free_runtime);
+						E2_ActionRuntime *art;
+						art = e2_action_pack_runtime (action, realarg, g_free);
+						if (art != NULL)
+						{
+							//note: callback relates to destroyable menu
+							GtkWidget *item = e2_menu_add (menu, label, icon, NULL,
+								e2_menu_action_activated_cb, art);
+							g_object_set_data_full (G_OBJECT(item), "e2-free-actruntime",
+								art, (GDestroyNotify)e2_action_free_runtime);
+							//FIXME add tip
+						}
 					}
 						break;
 					default:
@@ -461,7 +528,7 @@ static void _e2_context_menu_add_items (GtkWidget *menu, GtkTreeModel *model,
 /******************/
 
 /**
-@brief create and show main (filepane) context menu
+@brief create and pop up destroyable context-menu for a filelist pane
 
 @param button initiator enumerator, 0-menu-key or action, 1=L etc
 @param time time at which event was triggered
@@ -501,7 +568,8 @@ void e2_context_menu_show (guint button, guint32 time, gint type)
 		_e2_context_menu_add_items (app.context_menu, set->ex.tree.model, &iter,
 			type);
 		g_signal_connect (G_OBJECT (app.context_menu), "selection-done",
-			G_CALLBACK (_e2_context_menu_destroy_menu_cb), NULL);
+			G_CALLBACK (_e2_context_menu_selected_cb), NULL);
+
 		if (button == 0)
 			//this was a menu key press or specific action
 			gtk_menu_popup (GTK_MENU (app.context_menu), NULL, NULL,
@@ -523,16 +591,40 @@ void e2_context_menu_show (guint button, guint32 time, gint type)
 */
 gboolean e2_context_menu_show_menu_action (gpointer from, E2_ActionRuntime *art)
 {
-	gint type = 0;
-	gchar *arg = (gchar *)art->data;
-	if (arg != NULL)
+	const gchar *arg = NULL;
+	E2_PaneRuntime *rt = e2_pane_get_runtime (from, art->data, &arg);
+	if (rt == NULL)
+		return FALSE;
+	//swap active pane if needed
+	if (rt != curr_pane)
 	{
-		g_strstrip (arg);
-		if (g_str_equal (arg, _A(112)))	//_(shift
+		if (GTK_IS_WIDGET (app.context_menu))
+		{
+			e2_menu_selection_done_cb (app.context_menu, NULL);
+			app.context_menu = NULL;
+		}
+		e2_pane_activate_other ();
+	}
+	gint type = 0;
+	if (ACTION_BUTTON (art,1) || ACTION_BUTTON (art,3))
+	{
+		if (ACTION_MASK (art,GDK_SHIFT_MASK))
 			type = 1;
-		else if (g_str_equal (arg, _A(105)))	//_(ctrl
+		else if (ACTION_MASK (art,GDK_CONTROL_MASK))
+			type = 2;
+//		else if (ACTION_MASK (art,GDK_MOD1_MASK))
+//			type = ?;
+	}
+	if (type == 0 && arg != NULL)
+	{
+		gchar *tmp = g_strdup (arg);
+		g_strstrip (tmp);
+		if (!strcmp (tmp, _A(117)))	//_(shift
+			type = 1;
+		else if (!strcmp (tmp, _A(110)))	//_(ctrl
 			type = 2;
 		//CHECKME alt option here ?
+		g_free (tmp);
 	}
 	e2_context_menu_show (0, 0, type);
 
@@ -548,53 +640,54 @@ static void _e2_context_menu_tree_defaults (E2_OptionSet *set)
 {
 	e2_option_tree_setup_defaults (set,
 	g_strdup("context-menu=<"),  //internal name
-	g_strconcat("||false|false|",_A(5),".",_A(20),"|",NULL),
-	g_strconcat(_("Open _with.."),"|open_with_"E2IP".png|false|false|",
-		_A(5),".",_A(62),"|",NULL),
-	g_strconcat(_("_View"),"|view_"E2IP".png|false|false|",
-		_A(5),".",_A(101),"|",NULL),
-	g_strconcat(_("_Info"),"|gtk-dialog-info|false|false|",_A(5),".",_A(52),"|",NULL),
-	g_strconcat("||false|false|",_A(18),"|",NULL),
-	g_strconcat(_("_Actions"),"|gtk-execute|false|false|",_A(19),"|",NULL),
-	g_strconcat("\t",_("_Copy"),"|gtk-copy|true|false|",_A(5),".",_A(33),"|",NULL),
-	g_strconcat("\t",_("_Move"),"|move_"E2IP".png|true|false|",_A(5),".",_A(58),"|",NULL),
-	g_strconcat("\t",_("_Link"),"|symlink_"E2IP".png|true|false|",_A(5),".",_A(91),"|",NULL),
-	g_strconcat("\t",_("_Trash"),"|trash_"E2IP".png|true|false|",_A(5),".",_A(97),"|",NULL),
-	g_strconcat("\t","<span foreground='red'>",_("_Delete"),"</span>|delete_"E2IP".png|true|false|",
-		_A(5),".",_A(38),"|",NULL),
-	g_strconcat("\t",_("_Rename.."),"|gtk-convert|true|false|",_A(5),".",_A(73),"|",NULL),
-	g_strconcat("\t",_("Change _owners.."),"|owners_"E2IP".png|true|false|",
-		_A(5),".",_A(63),"|",NULL),
-	g_strconcat("\t",_("Change _permissions.."),"|permissions_"E2IP".png|true|false|",
-		_A(5),".",_A(67),"|",NULL),
-	g_strconcat("\t",_("Copy as.."),"|copy_as_"E2IP".png|true|false|",_A(5),".",_A(34),"|",NULL),
-	g_strconcat("\t",_("Move as.."),"|move_as_"E2IP".png|true|false|",_A(5),".",_A(59),"|",NULL),
-	g_strconcat("\t",_("Link as.."),"|symlink_as_"E2IP".png|true|false|",_A(5),".",_A(92),"|",NULL),
-	g_strconcat(_("_Plugins"),"|gtk-index|false|false|",_A(19),"|",NULL),
-	g_strconcat("\t||false|false|",_A(14),".",_A(24),"|",NULL),
-	g_strconcat("\t||false|false|",_A(18),"|",NULL),
-	g_strconcat("\t",_("_Edit plugins.."),"|gtk-preferences|false|false|",_A(2),".",_C(33),"|",NULL),
-	g_strconcat(_("_User commands"),"|user_commands_"E2IP".png|false|false|",_A(19),"|",NULL),
-	g_strconcat("\t",_("_Make new file.."),"|gtk-new|false|true|touch|'%{",_("Enter file name:"),"}'",NULL),	//_A(17)
-	g_strconcat("\t",_("_Compare files"),"||false|true|>cmp|-s %f %F && echo \"",_("The files are identical"),"\"\\n \\|\\| echo \"",_("The files are different"),"\"\\n",NULL),  	//_A(17)
-	g_strconcat("\t",_("Compare _directories"),"||false|true|diff|%d %D",NULL), 	//_A(17)
-//	g_strconcat("\t",_("_Easytag"),"||false|true|easytag|%d &",NULL),  //_A(17) not common enough to be a default
-	g_strconcat("\t",_("_Remove spaces"),"||false|true|>mv|%f `echo %f \\| sed -e 's/ //g'` 2>/dev/null &",NULL), 	//_A(17)
-	g_strconcat("\t",_("_Split file.."),"||false|true|split|-b %{",_("Enter the piece-size (in kB):"),"}k %f %f_",NULL), //_A(17)
-	g_strconcat("\t",_("Co_ncatenate files.."),"||false|true|cat|%f > '%{",_("Enter the name of the combined file:"),"}'",NULL), //_A(17)
-	g_strconcat("\t",_("_Free space"),"||false|true|>stat|-f %d \\| awk '/Blocks/ {printf \"%2.1f ", _("percent free"),"\",$5/$3*100}'",NULL),  //_A(17)
-	g_strconcat("\t||false|true|",_A(18),"|",NULL),
-	g_strconcat("\t",_("_Edit user commands.."),"|gtk-preferences|false|true|",_A(2),".",_A(32),"|",_C(8),NULL),
-	g_strconcat(_("Ma_ke dir.."),"|gtk-open|false|false|",_A(1),".",_A(56),"|",NULL),  //same _ as main toolbar
-	g_strconcat("||false|false|",_A(18),"|",NULL),
-	g_strconcat(_("_Bookmarks"),"|bookmark_"E2IP".png|false|false|",_A(19),"|",NULL),
-	g_strconcat("\t",_("Add _top"),"|add_mark_top_"E2IP".png|false|false|",_A(0),".",_A(26),"|",_A(114),NULL),
-	g_strconcat("\t||false|false|",_A(0),".",_A(24),"|",NULL),
-	g_strconcat("\t",_("Add _bottom"),"|add_mark_bottom_"E2IP".png|false|false|",_A(0),".",_A(26),"|",NULL),
-	g_strconcat("\t||false|false|",_A(18),"|",NULL),
-	g_strconcat("\t",_("_Edit bookmarks.."),"|gtk-preferences|false|false|",_A(2),".",_C(1),"|",NULL),
-	g_strconcat("||false|false|",_A(18),"|",NULL),
-	g_strconcat(_("_Edit filetype.."),"|gtk-preferences|false|false|",_A(2),".",_C(15),"|",NULL),
+	g_strconcat("||false|false|",_A(6),".",_A(23),"|",NULL),
+	g_strconcat(_("Open _with.."),"|open_with"E2ICONTB"|false|false|",
+		_A(6),".",_A(66),"|",NULL),
+	g_strconcat(_("_View"),"|view"E2ICONTB"|false|false|",
+		_A(6),".",_A(106),"|",NULL),
+	g_strconcat(_("_Info"),"|gtk-dialog-info|false|false|",_A(6),".",_A(55),"|",NULL),
+	g_strconcat("||false|false|",_A(21),"|",NULL),
+	g_strconcat(_("_Actions"),"|gtk-execute|false|false|",_A(22),"|",NULL),
+	g_strconcat("\t",_("_Copy"),"|gtk-copy|true|false|",_A(6),".",_A(36),"|",NULL),
+	g_strconcat("\t",_("_Move"),"|move"E2ICONTB"|true|false|",_A(6),".",_A(62),"|",NULL),
+	g_strconcat("\t",_("_Link"),"|symlink"E2ICONTB"|true|false|",_A(6),".",_A(96),"|",NULL),
+	g_strconcat("\t",_("_Trash"),"|trash"E2ICONTB"|true|false|",_A(6),".",_A(18),"|",NULL),
+	g_strconcat("\t","<span foreground='red'>",_("_Delete"),"</span>|delete"E2ICONTB"|true|false|",
+		_A(6),".",_A(42),"|",NULL),
+	g_strconcat("\t",_("_Rename.."),"|gtk-convert|true|false|",_A(6),".",_A(76),"|",NULL),
+	g_strconcat("\t",_("Change _owners.."),"|owners"E2ICONTB"|true|false|",
+		_A(6),".",_A(67),"|",NULL),
+	g_strconcat("\t",_("Change _permissions.."),"|permissions"E2ICONTB"|true|false|",
+		_A(6),".",_A(70),"|",NULL),
+	g_strconcat("\t",_("Copy as.."),"|copy_as"E2ICONTB"|true|false|",_A(6),".",_A(37),"|",NULL),
+	g_strconcat("\t",_("Move as.."),"|move_as"E2ICONTB"|true|false|",_A(6),".",_A(63),"|",NULL),
+	g_strconcat("\t",_("Link as.."),"|symlink_as"E2ICONTB"|true|false|",_A(6),".",_A(97),"|",NULL),
+	g_strconcat(_("_Plugins"),"|gtk-index|false|false|",_A(22),"|",NULL),
+	g_strconcat("\t||false|false|",_A(16),".",_A(27),"|",NULL),
+	g_strconcat("\t||false|false|",_A(21),"|",NULL),
+	g_strconcat("\t",_("_Edit plugins.."),"|gtk-preferences|false|false|",_A(3),".",_C(34),"|",NULL),
+	g_strconcat(_("_User commands"),"|user_commands"E2ICONTB"|false|false|",_A(22),"|",NULL),
+	g_strconcat("\t",_("_Make new file.."),"|gtk-new|false|true|touch|'%{",_("Enter file name:"),"}'",NULL),	//_A(20)
+	g_strconcat("\t",_("_Compare files"),"||false|true|>cmp|-s %f %F && echo \"",_("The files are identical"),"\"\\n \\|\\| echo \"",_("The files are different"),"\"\\n",NULL),  	//_A(20)
+	g_strconcat("\t",_("Compare _directories"),"||false|true|diff|%d %D",NULL), 	//_A(20)
+//	g_strconcat("\t",_("_Easytag"),"||false|true|easytag|%d &",NULL),  //_A(20) not common enough to be a default
+	g_strconcat("\t",_("_Remove spaces"),"||false|true|>mv|%f `echo %f \\| sed -e 's/ //g'` 2>/dev/null &",NULL), 	//_A(20)
+	g_strconcat("\t",_("_Split file.."),"||false|true|split|-b %{",_("Enter the piece-size (in kB):"),"}k %f %f_",NULL), //_A(20)
+	g_strconcat("\t",_("Co_ncatenate files.."),"||false|true|cat|%f > '%{",_("Enter the name of the combined file:"),"}'",NULL), //_A(20)
+	g_strconcat("\t",_("_Free space"),"||false|true|>stat|-f %d \\| awk '/Blocks/ {printf \"%2.1f ", _("percent free"),"\",$5/$3*100}'",NULL),  //_A(20)
+	g_strconcat("\t||false|true|",_A(21),"|",NULL),
+	g_strconcat("\t",_("_Edit user commands.."),"|gtk-preferences|false|true|",_A(3),".",_A(32),"|",_C(8),NULL),
+	g_strconcat(_("Ma_ke dir.."),"|gtk-open|false|false|",_A(1),".",_A(60),"|",NULL),  //same _ as main toolbar
+	g_strconcat("||false|false|",_A(21),"|",NULL),
+	g_strconcat(_("_Bookmarks"),"|bookmark"E2ICONTB"|false|false|",_A(22),"|",NULL),
+	g_strconcat("\t",_("Add _top"),"|add_mark_top"E2ICONTB"|false|false|",_A(0),".",_A(30),"|",_A(119),NULL),
+	g_strconcat("\t||false|false|",_A(0),".",_A(27),"|",NULL),
+	g_strconcat("\t",_("Add _bottom"),"|add_mark_bottom"E2ICONTB"|false|false|",_A(0),".",_A(30),"|",NULL),
+	g_strconcat("\t||false|false|",_A(21),"|",NULL),
+	g_strconcat("\t",_("_Edit bookmarks.."),"|gtk-preferences|false|false|",_A(3),".",_C(1),"|",NULL),
+	g_strconcat(_("_History"),"|gtk-jump-to|false|false|",_A(8),".",_A(27),"|",NULL),
+	g_strconcat("||false|false|",_A(21),"|",NULL),
+	g_strconcat(_("_Edit filetype.."),"|gtk-preferences|false|false|",_A(3),".",_A(45),"|",NULL),
 	g_strdup(">"),	//allow this to be freed
 	NULL);
 }
@@ -606,7 +699,7 @@ static void _e2_context_menu_tree_defaults (E2_OptionSet *set)
 void e2_context_menu_options_register (void)
 {
 	//this option does not require a rebuild after config change
-	gchar *group_name = g_strconcat (_C(32), ".", _C(8), NULL);
+	gchar *group_name = g_strconcat (_C(33), ".", _C(8), NULL);
 	E2_OptionSet *set = e2_option_tree_register ("context-menu", group_name, _C(8),  //_("context menu"
 		NULL, NULL, NULL, E2_OPTION_TREE_UP_DOWN | E2_OPTION_TREE_ADD_DEL,
 		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_FREEGROUP);
@@ -619,7 +712,7 @@ void e2_context_menu_options_register (void)
 		0, NULL, NULL);
 	e2_option_tree_add_column (set, _("Ctrl"), E2_OPTION_TREE_TYPE_BOOL, FALSE,"false",
 		0, NULL, NULL);
-	e2_option_tree_add_column (set, _("Action"), E2_OPTION_TREE_TYPE_SEL, 0, _A(18),
+	e2_option_tree_add_column (set, _("Action"), E2_OPTION_TREE_TYPE_SEL, 0, _A(21),
 		0, NULL,
 		GINT_TO_POINTER (E2_ACTION_EXCLUDE_GENERAL | E2_ACTION_EXCLUDE_MENU));
 	e2_option_tree_add_column (set, _("Argument"), E2_OPTION_TREE_TYPE_SEL, 0, "",
