@@ -1,6 +1,6 @@
-/* $Id: e2_dnd.c 739 2007-11-21 03:27:49Z tpgww $
+/* $Id: e2_dnd.c 1927 2009-09-29 00:26:39Z tpgww $
 
-Copyright (C) 2006-2007 tooar <tooar@gmx.net>.
+Copyright (C) 2006-2009 tooar <tooar@emelfm2.net>.
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -141,9 +141,25 @@ static gboolean _e2_dnd_auto_scroll2 (ViewInfo *view)
 	//often enough when auto-scrolling a large treeview
 	//my and my are for bottom-left of pointer FIXME make it work for top-left
 	gint my;
-	gdk_window_get_pointer (view->treeview->window, NULL, &my, NULL);
+	gdk_window_get_pointer (
+#ifdef USE_GTK2_14
+		gtk_widget_get_window (view->treeview),
+#else
+		view->treeview->window,
+#endif
+		NULL, &my, NULL);
 
-	if (my >= AUTO_SCROLL_MARGIN && my <= (view->treeview->allocation.height - AUTO_SCROLL_MARGIN))
+#ifdef USE_GTK2_18
+	GtkAllocation alloc;
+	gtk_widget_get_allocation (view->treeview, &alloc);
+#endif
+	if (my >= AUTO_SCROLL_MARGIN && my <= (
+#ifdef USE_GTK2_18
+		alloc.height
+#else
+		view->treeview->allocation.height
+#endif
+		- AUTO_SCROLL_MARGIN))
 	{
 		//mouse out of scroll zones now
 		app.timers[ASCROLL_T] = 0;
@@ -152,15 +168,31 @@ static gboolean _e2_dnd_auto_scroll2 (ViewInfo *view)
 	else if (my < AUTO_SCROLL_MARGIN)
 		y_scr = my - AUTO_SCROLL_MARGIN;	//-AUTO_SCROLL_MARGIN to -1
 	else
-		y_scr = my + AUTO_SCROLL_MARGIN - view->treeview->allocation.height; //1 to AUTO_SCROLL_MARGIN
-
+		y_scr = my + AUTO_SCROLL_MARGIN -
+#ifdef USE_GTK2_18
+		alloc.height;
+#else
+		view->treeview->allocation.height; //1 to AUTO_SCROLL_MARGIN
+#endif
 	GtkAdjustment *vadj = gtk_tree_view_get_vadjustment
 		(GTK_TREE_VIEW (view->treeview));
 	gdouble scrollnow = gtk_adjustment_get_value (vadj);
+	gdouble uppernow =
+#ifdef USE_GTK2_14
+		gtk_adjustment_get_upper (vadj);
+#else
+		vadj->upper;
+#endif
+	gdouble sizenow =
+#ifdef USE_GTK2_14
+		gtk_adjustment_get_page_size (vadj);
+#else
+		vadj->page_size;
+#endif
 	gdouble scroll_factor;
 	//accelerate the scroll by increasing the stepsize when we are further into
 	//the scroll margin. TrialnError fudge factor
-	scroll_factor = vadj->upper / 500.0
+	scroll_factor = uppernow / 500.0
 		* (exp (ABS (y_scr) * 2.0 / (gdouble) AUTO_SCROLL_MARGIN) - 0.8);
 	if (y_scr < 0)
 		scrollnow -= scroll_factor;
@@ -169,10 +201,10 @@ static gboolean _e2_dnd_auto_scroll2 (ViewInfo *view)
 	//clamp the new setting within relevant range
 	if (scrollnow < 0)
 		scrollnow = 0;
-	else if (scrollnow > vadj->upper - vadj->page_size)
-		scrollnow = vadj->upper - vadj->page_size - 0.1;
+	else if (scrollnow > uppernow - sizenow)
+		scrollnow = uppernow - sizenow - 0.1;
 	//no movement, turn off the scrolling CHECKME more iterations before turnoff ?
-	if (ABS (scrollnow-prior_scroll) < 0.1)	//0.001*vadj->upper)
+	if (ABS (scrollnow - prior_scroll) < 0.1)	//0.001*vadj->upper)
 	{
 		app.timers[ASCROLL_T] = 0;
 		return FALSE;
@@ -186,6 +218,11 @@ static gboolean _e2_dnd_auto_scroll2 (ViewInfo *view)
 	}
 	return TRUE;
 }
+/**
+@brief first (longer-interval) timer callback for scrolling treeview
+@param view rt data for the view where the cursor is ?
+@return
+*/
 static gboolean _e2_dnd_auto_scroll (ViewInfo *view)
 {
 	//substitute shorter loop for the initial one
@@ -305,7 +342,18 @@ gboolean e2_dnd_drag_motion_cb (GtkWidget *treeview, GdkDragContext *context,
 		}
 	}
 	//arrange auto-scrolling when mouse pointer is near top or bottom of treeview
-	if (y < (pointer_height + AUTO_SCROLL_MARGIN) || y > (view->treeview->allocation.height - AUTO_SCROLL_MARGIN))
+#ifdef USE_GTK2_18
+	GtkAllocation alloc;
+	gtk_widget_get_allocation (view->treeview, &alloc);
+#endif
+	if (y < (pointer_height + AUTO_SCROLL_MARGIN)
+	 || y > (
+#ifdef USE_GTK2_18
+			 alloc.height
+#else
+	         view->treeview->allocation.height
+#endif
+			 - AUTO_SCROLL_MARGIN))
 	{
 		if (app.timers[ASCROLL_T] == 0)
 		{
@@ -443,7 +491,7 @@ NOTE some apps omit the leading "//" from the first example, resulting in file:/
 #ifdef E2_VFSTMP
 	//FIXME dir when not mounted local
 #else
-		local_dir = F_FILENAME_TO_LOCALE (view->dir);
+		local_dir = D_FILENAME_TO_LOCALE (view->dir); //always dup, to avoid dirchange race
 #endif
 		GPtrArray *uri_array = g_ptr_array_sized_new (g_list_length (base) + 1);
 		//FIXME vfs
@@ -473,9 +521,12 @@ NOTE some apps omit the leading "//" from the first example, resulting in file:/
 		}
 		g_ptr_array_add (uri_array, NULL);	//NULL-terminate the array
 		gtk_selection_data_set_uris (data, (gchar **)uri_array->pdata);
-		F_FREE (local_dir);
+//#ifndef USE_GLIB2_22
+		g_ptr_array_foreach (uri_array, (GFunc)g_free, NULL);
+//#endif
 		g_ptr_array_free (uri_array, TRUE);
 		g_list_free (base);
+		g_free (local_dir);
 	}
 #ifdef E2_REFRESH_DEBUG
 	printd (DEBUG, "enable refresh, drag data get");
@@ -596,18 +647,25 @@ gboolean e2_dnd_drag_drop_cb (GtkWidget *treeview, GdkDragContext *context,
 	return FALSE;
 }
 */
+
+typedef struct _E2_MenuWait
+{
+	GtkWidget *chosen;	//pointer to chosen menu item
+	E2_MainLoop *loopdata;
+} E2_MenuWait;
+
 /**
 @brief get drag op menu item
 
-@param menuitem the clicked menu item widget
-@param chosen store for the chosen item
+@param menuitem the activated menu item widget
+@param w wait-data for this callback
 
 @return
 */
-static void _e2_dnd_drag_op_selected_cb (GtkWidget *menuitem, GtkWidget **chosen)
+static void _e2_dnd_drag_op_selected_cb (GtkMenuItem *menuitem, E2_MenuWait *w)
 {
-	*chosen = menuitem;
-	gtk_main_quit ();
+	w->chosen = GTK_WIDGET(menuitem);
+	e2_main_loop_quit (w->loopdata);
 }
 /**
 @brief execute drop
@@ -719,54 +777,64 @@ void e2_dnd_drag_data_received_cb (GtkWidget *treeview, GdkDragContext *context,
 	GdkDragAction action = context->actions;
 	if (menu_wanted && (action & GDK_ACTION_ASK))
 	{
-		GtkWidget *drag_op_menu = gtk_menu_new();
-		GtkWidget *item1, *item2, *item3, *item4, *chosen;
+		E2_MenuWait wdata;
+		//this loop for popup menu must be in default context and the menu will
+		//not survive any UI event
+		wdata.loopdata = e2_main_loop_new (TRUE);
+		if (wdata.loopdata != NULL)
+		{
+			GtkWidget *drag_op_menu = gtk_menu_new();
+			GtkWidget *item1, *item2, *item3, *item4;
 
-		if (action & GDK_ACTION_COPY)
-			item1 = e2_menu_add (drag_op_menu, _("_Copy"), NULL, NULL,
-				_e2_dnd_drag_op_selected_cb, &chosen);
-		else
-			item1 = NULL;
+			if (action & GDK_ACTION_COPY)
+				item1 = e2_menu_add (drag_op_menu, _("_Copy"), NULL, NULL,
+					_e2_dnd_drag_op_selected_cb, &wdata);
+			else
+				item1 = NULL;
 
-		if (action & GDK_ACTION_MOVE)
-			item2 = e2_menu_add (drag_op_menu, _("_Move"), NULL, NULL,
-				_e2_dnd_drag_op_selected_cb, &chosen);
-		else
-			item2 = NULL;
+			if (action & GDK_ACTION_MOVE)
+				item2 = e2_menu_add (drag_op_menu, _("_Move"), NULL, NULL,
+					_e2_dnd_drag_op_selected_cb, &wdata);
+			else
+				item2 = NULL;
 
-		if (action & GDK_ACTION_LINK)
-			item3 = e2_menu_add (drag_op_menu, _("_Link"), NULL, NULL,
-				_e2_dnd_drag_op_selected_cb, &chosen);
-		else
-			item3 = NULL;
+			if (action & GDK_ACTION_LINK)
+				item3 = e2_menu_add (drag_op_menu, _("_Link"), NULL, NULL,
+					_e2_dnd_drag_op_selected_cb, &wdata);
+			else
+				item3 = NULL;
 
-/*		if (?)	//drop target is an executable item
-			item4 = e2_menu_add (drag_op_menu, _("_SendTo"), NULL, NULL,
-				_e2_dnd_drag_op_selected_cb, &chosen);
-		else
-			item4 = NULL;
+/*			if (?)	//drop target is an executable item
+				item4 = e2_menu_add (drag_op_menu, _("_SendTo"), NULL, NULL,
+					_e2_dnd_drag_op_selected_cb, &wdata);
+			else
+				item4 = NULL;
 */
-		item4 = e2_menu_add (drag_op_menu, _("C_ancel"), NULL, NULL,
-			_e2_dnd_drag_op_selected_cb, &chosen);
+			item4 = e2_menu_add (drag_op_menu, _("C_ancel"), NULL, NULL,
+				_e2_dnd_drag_op_selected_cb, &wdata);
+			wdata.chosen = item4; //default in case of abort == cancel
 
-		chosen = item4;
+			gtk_menu_popup (GTK_MENU (drag_op_menu), NULL, NULL, NULL, NULL, 0,
+				(guint32) time);
 
-		gtk_menu_popup (GTK_MENU (drag_op_menu), NULL, NULL, NULL, NULL, 0,
-			(guint32) time);
-		gtk_main ();
-		gtk_widget_destroy (drag_op_menu);
+			e2_main_loop_run (wdata.loopdata);
 
-		if (chosen == item1)
-			action = GDK_ACTION_COPY;
-		else if (chosen == item2)
-			action = GDK_ACTION_MOVE;
-		else if (chosen == item3)
-			action = GDK_ACTION_LINK;
-/*		else if (chosen == item4)
-			setup to handle the sendto request
+			e2_menu_selection_done_cb (drag_op_menu, NULL);
+
+			if (wdata.chosen == item1)
+				action = GDK_ACTION_COPY;
+			else if (wdata.chosen == item2)
+				action = GDK_ACTION_MOVE;
+			else if (wdata.chosen == item3)
+				action = GDK_ACTION_LINK;
+/*			else if (wdata.chosen == item4)
+				setup to handle the sendto request
 */
-		else //the user cancelled
-			goto cleanbadexit;
+			else //the user cancelled
+				goto cleanbadexit;
+		}
+		else
+			goto cleanbadexit;	//do nothing
 	}
 	else	//don't ask
 		action = context->suggested_action;
@@ -858,7 +926,7 @@ default handler from running. Look at the source code for the default handler in
 gtktreeview.c to get an idea what your handler should do.
 */
 	//CHECKME nothing else needs doing here ?
-	g_signal_stop_emission_by_name (G_OBJECT (view->treeview),
+	g_signal_stop_emission_by_name ((gpointer)view->treeview,
 		"drag-data-received");
 	return;
 }
