@@ -1,6 +1,6 @@
-/* $Id: e2_filelist.c 856 2008-05-05 02:00:15Z tpgww $
+/* $Id: e2_filelist.c 2056 2010-02-21 04:52:19Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>.
+Copyright (C) 2004-2010 tooar <tooar@emelfm2.net>.
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -48,10 +48,10 @@ extern sigset_t dnotify_signal_set;
 #endif
 
 //disable-refresh counter for both filelists
-gint refresh_refcount;
+volatile gint refresh_refcount;
 //disable-refresh counter for single filelists
-static gint refresh_active_refcount;
-static gint refresh_inactive_refcount;
+static volatile gint refresh_active_refcount;
+static volatile gint refresh_inactive_refcount;
 //refresh deferral counters
 static gint pane1repeats = 0;
 static gint pane2repeats = 0;
@@ -69,10 +69,10 @@ static gboolean _e2_filelist_check_active_pane (E2_ListChoice pane)
 	switch (pane)
 	{
 		case PANE1:
-			active = (curr_view == &app.pane1_view);
+			active = (curr_view == &app.pane1.view);
 			break;
 		case PANE2:
-			active = (curr_view == &app.pane2_view);
+			active = (curr_view == &app.pane2.view);
 			break;
 		case PANEINACTIVE:
 			active = FALSE;
@@ -94,8 +94,7 @@ void e2_filelist_disable_one_refresh (E2_ListChoice pane)
 {
 	if (_e2_filelist_check_active_pane (pane))
 	{
-		refresh_active_refcount++;
-		if (refresh_active_refcount == 1)
+		if (g_atomic_int_exchange_and_add (&refresh_active_refcount, 1) == 0)
 		{
 			LISTS_LOCK
 			curr_view->listcontrols.norefresh = TRUE;
@@ -104,8 +103,7 @@ void e2_filelist_disable_one_refresh (E2_ListChoice pane)
 	}
 	else
 	{
-		refresh_inactive_refcount++;
-		if (refresh_inactive_refcount == 1)
+		if (g_atomic_int_exchange_and_add (&refresh_inactive_refcount, 1) == 0)
 		{
 			LISTS_LOCK
 			other_view->listcontrols.norefresh = TRUE;
@@ -124,27 +122,27 @@ void e2_filelist_enable_one_refresh (E2_ListChoice pane)
 {
 	if (_e2_filelist_check_active_pane (pane))
 	{
-		refresh_active_refcount--;
-		if (refresh_active_refcount == 0 && refresh_refcount == 0)
+		if (g_atomic_int_exchange_and_add (&refresh_active_refcount, -1) == 1
+			&& g_atomic_int_get (&refresh_refcount) == 0)
 		{
 			LISTS_LOCK
 			curr_view->listcontrols.norefresh = FALSE;
 			LISTS_UNLOCK
 		}
 		if (refresh_active_refcount < 0)
-			refresh_active_refcount = 0;
+			g_atomic_int_set (&refresh_active_refcount, 0);
 	}
 	else
 	{
-		refresh_inactive_refcount--;
-		if (refresh_inactive_refcount == 0 && refresh_refcount == 0)
+		if (g_atomic_int_exchange_and_add (&refresh_inactive_refcount, -1) == 1
+			&& g_atomic_int_get (&refresh_refcount) == 0)
 		{
 			LISTS_LOCK
 			other_view->listcontrols.norefresh = FALSE;
 			LISTS_UNLOCK
 		}
 		if (refresh_inactive_refcount < 0)
-			refresh_inactive_refcount = 0;
+			g_atomic_int_set (&refresh_inactive_refcount, 0);
 	}
 }
 /**
@@ -180,9 +178,9 @@ gboolean e2_filelist_enable_refresh_action (gpointer from, E2_ActionRuntime *art
 */
 void e2_filelist_reset_refresh (void)
 {
-	if (refresh_refcount != 0)
+	if (g_atomic_int_get (&refresh_refcount) != 0)
 	{
-		refresh_refcount = 1;
+		g_atomic_int_set (&refresh_refcount, 1);
 		e2_filelist_enable_refresh ();
 	}
 }
@@ -193,18 +191,19 @@ middle of such an operation would free FileInfo structs as a side-effect.
 /**
 @brief  disable refreshing of pane filelists and config data
 
-Increase ref count, if == 1, stop the timer process which calls the refresh fns
+Increases the ref-count, then if 1, stop the timer process which calls the refresh fns
 Why is config data check bundled here ?
+Expects BGL open?
 
-@return
+return
 */
 void e2_filelist_disable_refresh (void)
 {
-	refresh_refcount++;
+	gint preinc = g_atomic_int_exchange_and_add (&refresh_refcount, 1);
 #ifdef E2_REFRESH_DEBUG
-	printd (DEBUG, "disable refresh, refresh ref now = %d", refresh_refcount);
+	printd (DEBUG, "disable refresh, refresh ref now = %d", preinc + 1);
 #endif
-	if (refresh_refcount == 1)
+	if (preinc == 0)
 	{
 		if (app.timers[REFRESHBEGIN_T] > 0)
 		{
@@ -240,29 +239,20 @@ void e2_filelist_disable_refresh (void)
 /**
 @brief re-enable polling of pane filelists and config data
 
-Decrease ref count, if == 0, re-enable/restart the processes which detect
+Decreases the ref-count, then if 0, re-enable/restart the processes which detect
 changes of filelist content and/or config data
 Config timer is bundled here because the same func refreshes the panes
+Expects BGL open
 
 @return
 */
 void e2_filelist_enable_refresh (void)
 {
-	refresh_refcount--;
+	gint predec = g_atomic_int_exchange_and_add (&refresh_refcount, -1);
 #ifdef E2_REFRESH_DEBUG
-	printd (DEBUG, "enable refresh, refresh ref now = %d", refresh_refcount);
+	printd (DEBUG, "enable refresh, refresh ref now = %d", predec - 1);
 #endif
-	if (refresh_refcount < 0)
-	{
-		printd (WARN, "The auto-refresh refresh count is < 0");
-		gdk_threads_enter ();
-		e2_window_output_show (NULL, NULL);
-		e2_output_print_error (
-		_("Something is wrong with the auto-refresh mechanism"), FALSE);
-		gdk_threads_leave ();
-	}
-
-	if (refresh_refcount == 0)
+	if (predec == 1)
 	{
 		if (e2_option_bool_get ("auto-refresh"))
 		{
@@ -285,6 +275,25 @@ void e2_filelist_enable_refresh (void)
 		e2_window_enable_status_update (-1);
 #endif
 	}
+	else if (predec < 1)
+	{
+		printd (WARN, "The auto-refresh refresh count is < 0");
+		gdk_threads_enter ();
+		e2_window_output_show (NULL, NULL);
+		e2_output_print_error (
+		_("Something is wrong with the auto-refresh mechanism"), FALSE);
+		gdk_threads_leave ();
+	}
+}
+/**
+@brief idle function to run refresh hooklist
+@param view pointer to view data struct
+@return FALSE to abort the source
+*/
+static gboolean _e2_fileview_more_refresh (ViewInfo *view)
+{
+	e2_hook_list_run (&view->hook_refresh, view);
+	return FALSE;
 }
 /**
 @brief timer destroy function after cancelling filelists checking
@@ -304,10 +313,14 @@ static gboolean _e2_filelist_refresh_manage (gpointer data)
 {
 	pthread_t thisID;
 	gpointer result;
-	gboolean cvdone, ovdone;
+	gboolean cvdone, ovdone, cvhook, ovhook;
 	LISTS_LOCK
 	ViewInfo *cv = curr_view;	//log these, in case active pane is changed during this process
 	ViewInfo *ov = other_view;
+	cvhook = ovhook = FALSE;
+
+	e2_utf8_set_name_conversion_if_requested ();
+
 retest:
 	if (cv->listcontrols.refresh_requested)
 	{
@@ -329,8 +342,11 @@ retest:
 #endif
 			//refresh function expects BGL open
 			if (pthread_create (&thisID, NULL,
-				(gpointer) e2_fileview_refresh_list, cv) == 0)
-					pthread_join (thisID, &result);
+				(gpointer(*)(gpointer))e2_fileview_refresh_list, cv) == 0)
+			{
+				pthread_join (thisID, &result); //only 1 refresh at a time
+				cvhook = TRUE;	//send it downstream
+			}
 			else
 				printd (WARN, "Failed to create list refresh thread");
 			LISTS_LOCK
@@ -361,8 +377,11 @@ retest2:
 			printd (DEBUG, "accepting request to refresh %s", ov->dir);
 #endif
 			if (pthread_create (&thisID, NULL,
-				(gpointer) e2_fileview_refresh_list, ov) == 0)
-					pthread_join (thisID, &result);
+				(gpointer(*)(gpointer))e2_fileview_refresh_list, ov) == 0)
+			{
+				pthread_join (thisID, &result); //only 1 refresh at a time
+				ovhook = TRUE;
+			}
 			else
 				printd (WARN, "Failed to create list refresh thread");
 			LISTS_LOCK
@@ -395,6 +414,13 @@ retest2:
 	}
 
 	LISTS_UNLOCK
+	//CHECKME do this separately e.g. in idle, to avoid any delay ?
+	if (cvhook)
+		g_idle_add ((GSourceFunc)_e2_fileview_more_refresh, curr_view);
+//		e2_hook_list_run (&curr_view->hook_refresh, curr_view);
+	if (ovhook)
+//		e2_hook_list_run (&other_view->hook_refresh, other_view);
+		g_idle_add ((GSourceFunc)_e2_fileview_more_refresh, other_view);
 
 	return FALSE;
 }
@@ -413,7 +439,7 @@ gboolean e2_filelist_request_refresh (gchar *dir, gboolean immediate)
 #ifdef E2_VFSTMP
 	//CHECKME v-dir ok
 #endif
-	if (g_str_equal (curr_view->dir, dir))
+	if (!strcmp (curr_view->dir, dir))
 	{
 		matched = TRUE;
 		LISTS_LOCK
@@ -423,7 +449,7 @@ gboolean e2_filelist_request_refresh (gchar *dir, gboolean immediate)
 #ifdef E2_VFSTMP
 	//CHECKME v-dir ok
 #endif
-	if (g_str_equal (other_view->dir, dir))
+	if (!strcmp (other_view->dir, dir))
 	{
 		matched = TRUE;
 		LISTS_LOCK
@@ -453,8 +479,8 @@ static gboolean _e2_filelist_refresh_ok (E2_ListChoice pane)
 	   Instead we assume small dirs can be processed without much adverse effect */
 	gint times, itemcount = (pane == PANE1) ?
 		//these are counts of total items, not filtered items
-		gtk_tree_model_iter_n_children (GTK_TREE_MODEL (app.pane1_view.store), NULL):
-		gtk_tree_model_iter_n_children (GTK_TREE_MODEL (app.pane2_view.store), NULL);
+		gtk_tree_model_iter_n_children (GTK_TREE_MODEL (app.pane1.view.store), NULL):
+		gtk_tree_model_iter_n_children (GTK_TREE_MODEL (app.pane2.view.store), NULL);
 	if (itemcount < 51)
 		times = 1;
 	else if (itemcount < 251)
@@ -542,11 +568,11 @@ gboolean e2_filelist_check_dirty (gpointer userdata)
 	{
 		LISTS_LOCK
 		if (p1dirty && (userdata != NULL || _e2_filelist_refresh_ok (PANE1)))
-			app.pane1_view.listcontrols.refresh_requested = TRUE;
+			app.pane1.view.listcontrols.refresh_requested = TRUE;
 		else
 			p1dirty = FALSE;
 		if (p2dirty && (userdata != NULL || _e2_filelist_refresh_ok (PANE2)))
-			app.pane2_view.listcontrols.refresh_requested = TRUE;
+			app.pane2.view.listcontrols.refresh_requested = TRUE;
 		else
 			p2dirty = FALSE;
 		LISTS_UNLOCK
@@ -667,6 +693,7 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 	if (store == NULL)
 		return NULL;
 
+	guint i;
 	GList *tmp;
 	FileInfo *infoptr;
 	GtkTreeIter iter;
@@ -727,6 +754,7 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 			break;
 	}
 
+	gboolean anyconvert = FALSE;
 	gboolean caseignore = ! e2_option_bool_get ("namesort-case-sensitive");
 	GtkStyle *style = gtk_rc_get_style (curr_view->treeview);
 	GdkColor *default_color = &style->text[GTK_STATE_NORMAL];
@@ -744,21 +772,84 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 	//iterate through the listed FileInfo's
 	for (tmp = entries; tmp != NULL; tmp = tmp->next)
 	{
-		infoptr = (FileInfo *) tmp->data;
+		gboolean convert; //also a cleanup flag
+		gchar *s;
+		register char c;
 
-		//convert to utf if appropriate
-		buf[FILENAME] = F_DISPLAYNAME_FROM_LOCALE (infoptr->filename);
-		//directories (and links to dirs) get a trailing separator
-		//FIXME speed this by using a more efficient process to check for dir
-		if (e2_fs_is_dir (infoptr, view))
+		infoptr = (FileInfo *) tmp->data;
+		//with bad unmount behaviour, there can be no actual data
+		if (infoptr == NULL)
+			continue; //CHECKME remove the item from list ?
+		//scan the namestring for non-ASCII char(s)
+		convert = FALSE;
+		for (s = infoptr->filename; (c = *s) != '\0'; s++)
 		{
-			freeme = buf[FILENAME];
-			buf[FILENAME] = e2_utils_strcat (freeme, "/");
-			if (freeme != infoptr->filename)
-				g_free (freeme);
+			if (c < 0) //non-ASCII
+			{
+				convert = TRUE;
+				anyconvert = TRUE;
+				break;
+			}
+		}
+		//convert to UTF-8 if appropriate
+		if (convert)
+		{
+			buf[FILENAME] = g_filename_display_name (infoptr->filename);
+/*			if (g_utf8_validate (infoptr->filename, -1, NULL))
+			{
+				convert = FALSE;
+				buf[FILENAME] = infoptr->filename;
+			}
+			else
+			{	//_not display_ so that it can be casefolded and/or collated properly ??
+				buf[FILENAME] = g_locale_to_utf8 (infoptr->filename, -1, NULL, NULL, NULL);
+				if (buf[FILENAME] == NULL)
+					buf[FILENAME] = e2_utf8_from_locale_fallback (infoptr->filename);
+			}
+*/
+		}
+		else
+			buf[FILENAME] = infoptr->filename;
+
+		if (caseignore)
+		{
+			freeme = g_utf8_casefold (buf[FILENAME], -1);
+#ifdef USE_GTK2_8
+			buf[NAMEKEY] = g_utf8_collate_key_for_filename (freeme, -1);
+#else
+			buf[NAMEKEY] = g_utf8_collate_key (freeme, -1);
+#endif
+			g_free (freeme);
+		}
+		else
+#ifdef USE_GTK2_8
+			buf[NAMEKEY] = g_utf8_collate_key_for_filename (buf[FILENAME], -1);
+#else
+			buf[NAMEKEY] = g_utf8_collate_key (buf[FILENAME], -1);
+#endif
+
+		//dirlink check done when info was created
+		if (infoptr->statbuf.st_mode & (S_IFDIR | E2_DIRLNK))
+		{
+			//directories (and links to dirs) get a trailing separator
+			s = buf[FILENAME];
+			if (convert)
+			{
+				guint l = strlen (s);
+				if ((s = (gchar *)realloc (s, l+2)) != NULL)
+				{
+					buf[FILENAME] = s;
+					s += l;
+					*s++ = G_DIR_SEPARATOR;
+					*s = '\0';
+				}
+			}
+			else
+			{
+				convert = TRUE;	//cleanup this one too
+				buf[FILENAME] = e2_utils_strcat (s, G_DIR_SEPARATOR_S);
+			}
 		}
-		else if (buf[FILENAME] == infoptr->filename)	//no actual conversion, above
-			buf[FILENAME] = g_strdup (buf[FILENAME]);
 
 		if (comma == NULL)
 		{	//use condensed size format
@@ -790,7 +881,6 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 
 			guint len = strlen (size_buf);
 			guint ths = len-1;  //0-based index
-			guint i;
 			while (ths > 2 && len < sizeof(size_buf))
 			{
 				for (i = len-1; i > ths-3; i--)
@@ -801,7 +891,7 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 				ths = i;
 			}
 		}
-		buf[SIZE] = g_strdup (size_buf);	//content is ascii, no need to convert to utf
+		buf[SIZE] = size_buf;	//content is ascii/utf-8, no need to convert or free
 
 		buf[PERM] = e2_fs_get_perm_string (infoptr->statbuf.st_mode); //string is already utf
 
@@ -838,22 +928,6 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 		strftime (changed_buf, sizeof(changed_buf), strf_string, tm_ptr);
 		buf[CHANGED] = e2_utf8_from_locale_fast (changed_buf);
 
-		if (caseignore)
-		{
-			freeme = g_utf8_casefold (buf[FILENAME], -1);
-#ifdef USE_GTK2_8
-			buf[NAMEKEY] = g_utf8_collate_key_for_filename (freeme, -1);
-#else
-			buf[NAMEKEY] = g_utf8_collate_key (freeme, -1);
-#endif
-			g_free (freeme);
-		}
-		else
-#ifdef USE_GTK2_8
-			buf[NAMEKEY] = g_utf8_collate_key_for_filename (buf[FILENAME], -1);
-#else
-			buf[NAMEKEY] = g_utf8_collate_key (buf[FILENAME], -1);
-#endif
 		GdkColor *foreground;
 		switch (infoptr->statbuf.st_mode & S_IFMT)
 		{
@@ -923,34 +997,42 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 			-1);
 
 		//cleanup
-		gint i;
-		for (i = 0; i <= NAMEKEY; i++)
+		if (convert)
+			g_free (buf[FILENAME]);
+		for (i = PERM; i <= NAMEKEY; i++)
 			g_free (buf[i]);
 
 	}
+
+	if (anyconvert != view->convert)
+	{
+		view->convert = anyconvert;
+		//flag need for change of conversion funcs at a suitable time
+		app.reconvert_requested = TRUE;
+	}
 /*
-//		gettimeofday (&strt, NULL); //FOR DEBUGGING, check time this load takes
-//		gettimeofday (&nd, NULL);
-		if (nd.tv_usec < strt.tv_usec)
-		{
-		    int nsec = (strt.tv_usec - nd.tv_usec) / 1000000 + 1;
-    		strt.tv_usec -= 1000000 * nsec;
-    		strt.tv_sec += nsec;
-		}
-		if (nd.tv_usec - strt.tv_usec > 1000000)
-		{
-			int nsec = (strt.tv_usec - nd.tv_usec) / 1000000;
-			strt.tv_usec += 1000000 * nsec;
-			strt.tv_sec -= nsec;
-		}
-		result.tv_sec = result.tv_sec + nd.tv_sec - strt.tv_sec;
-		result.tv_usec = result.tv_usec + nd.tv_usec - strt.tv_usec;
-		if (result.tv_usec > 1000000)
-		{
-			int nsec = result.tv_usec / 1000000 + 1;
-    		result.tv_usec -= 1000000 * nsec;
-    		result.tv_sec += nsec;
-		}
+//	gettimeofday (&strt, NULL); //FOR DEBUGGING, check time this load takes
+//	gettimeofday (&nd, NULL);
+	if (nd.tv_usec < strt.tv_usec)
+	{
+		int nsec = (strt.tv_usec - nd.tv_usec) / 1000000 + 1;
+		strt.tv_usec -= 1000000 * nsec;
+		strt.tv_sec += nsec;
+	}
+	if (nd.tv_usec - strt.tv_usec > 1000000)
+	{
+		int nsec = (strt.tv_usec - nd.tv_usec) / 1000000;
+		strt.tv_usec += 1000000 * nsec;
+		strt.tv_sec -= nsec;
+	}
+	result.tv_sec = result.tv_sec + nd.tv_sec - strt.tv_sec;
+	result.tv_usec = result.tv_usec + nd.tv_usec - strt.tv_usec;
+	if (result.tv_usec > 1000000)
+	{
+		int nsec = result.tv_usec / 1000000 + 1;
+		result.tv_usec -= 1000000 * nsec;
+		result.tv_sec += nsec;
+	}
 
 //	printd (DEBUG, "populating rows took %f seconds", result.tv_sec + result.tv_usec / 1000000.0 );
 */
