@@ -1,6 +1,6 @@
-/* $Id: e2_filetype.c 853 2008-04-07 10:38:17Z tpgww $
+/* $Id: e2_filetype.c 1840 2009-09-17 01:15:11Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -28,7 +28,7 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "e2_dialog.h"
 #endif
 
-/**
+/* *
 @brief get extensions array for filetype category @a category
 
 Not much use - in the config data, the categories are user-editable !!
@@ -60,7 +60,7 @@ Not much use - in the config data, the categories are user-editable !!
 					if (gtk_tree_model_iter_children (set->ex.tree.model, &iter3, &iter2))
 					{
 						gtk_tree_model_get (set->ex.tree.model, &iter2, 1, &extension, -1);
-						if (g_str_equal (extension, _C(13)))  //extensions node found
+						if (!strcmp (extension, _C(13)))  //extensions node found
 						{
 							g_free (extension);
 							do
@@ -116,9 +116,9 @@ const gchar **e2_filetype_get_actions (gchar *ext)
 {
 	const gchar **actions;
 
-	if ( g_str_equal (ext, _("<directory>"))
-	  || g_str_equal (ext, _("<executable>"))
-	  || g_str_equal (ext, _("<none>"))
+	if ( !strcmp (ext, _("<directory>"))
+	  || !strcmp (ext, _("<executable>"))
+	  || !strcmp (ext, _("<none>"))
 	  || (!e2_option_bool_get ("anycase-filetypes") && e2_fs_mount_is_cased (curr_view)))
 		actions = (const gchar **) g_hash_table_lookup (app.filetypes, ext);
 	else
@@ -198,7 +198,7 @@ void e2_filetype_add_all (void)
 					gtk_tree_model_get (set->ex.tree.model, &iter2, 1, &extension, -1);
 					if (gtk_tree_model_iter_children (set->ex.tree.model, &iter3, &iter2))
 					{
-						if (g_str_equal (extension, _C(13)))  //extensions node found
+						if (!strcmp (extension, _C(13)))  //extensions node found
 						{
 							g_free (extension);
 							do
@@ -208,7 +208,7 @@ void e2_filetype_add_all (void)
 								g_ptr_array_add (e_ray, extension);
 							} while (gtk_tree_model_iter_next (set->ex.tree.model, &iter3));
 						}
-						else if (g_str_equal (extension, _C(6)))  //commands node found
+						else if (!strcmp (extension, _C(6)))  //commands node found
 						{
 							g_free (extension);
 							do
@@ -272,11 +272,11 @@ to remember to append the %f
 */
 void e2_filetype_exec_action (gchar *command)
 {
+	e2_command_run (command, E2_COMMAND_RANGE_FILE_ACTION, curr_view->treeview
 #ifdef E2_COMMANDQ
-	e2_command_run (command, E2_COMMAND_RANGE_FILE_ACTION, FALSE);
-#else
-	e2_command_run (command, E2_COMMAND_RANGE_FILE_ACTION);
+	, FALSE
 #endif
+	);
 }
 /**
 @brief change filetype config data for a selected item
@@ -311,7 +311,7 @@ static gboolean _e2_filetype_config (gpointer from, E2_ActionRuntime *art)
 	//open the config dialog at the appropriate place
 	gchar *utf = F_FILENAME_FROM_LOCALE (ext);	//DISPLAY ??
 	e2_filetype_config_show (utf);
-	F_FREE (utf);
+	F_FREE (utf, ext);
 	return TRUE;
 }
 /**
@@ -474,9 +474,9 @@ void e2_filetype_apply_allnew (void)
 */
 void e2_filetype_actions_register (void)
 {
-	gchar *action_name = g_strconcat(_A(2),".",_C(15),NULL);
-	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
-		_e2_filetype_config, NULL, FALSE);
+	E2_Action action =
+	{g_strconcat(_A(3),".",_A(45),NULL),_e2_filetype_config, FALSE, E2_ACTION_TYPE_ITEM, 0, NULL, NULL};
+	e2_action_register (&action);
 }
 /**
 @brief install default tree options for filetypes
@@ -493,7 +493,7 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strconcat ("\t\t|",_("<directory>"),"|",NULL), //_I(
 		g_strconcat ("\t|",_C(6),"|",NULL),  // _("commands")
 			g_strconcat ("\t\t|",_("_Open"),"|cd %%f",NULL),	//must be no quotes in path!
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(61),NULL), //<
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(65),NULL), //<
 			g_strconcat ("\t\t|",_("_Mount"),"|mount %p",NULL),	//CHECKME quotes in path ok?
 			g_strconcat ("\t\t|",_("_Unmount"),"|umount %p",NULL),
 	g_strconcat (_("executables"),"||",NULL),
@@ -506,7 +506,7 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 //			g_strconcat ("\t\t|",_("Edit with _vi"),"|x vi",NULL),
 //			g_strconcat ("\t\t|",_("Edit with _emacs"),"|x emacs",NULL),
 			g_strconcat ("\t\t|",_("Edit _with.."),"|%{(editors)@",_("Editor command:"),"} %p",NULL),
-			g_strconcat ("\t\t|",_("_Edit"),"|",_A(5),".",_A(39),NULL), //_("file.edit") //<
+			g_strconcat ("\t\t|",_("_Edit"),"|",_A(6),".",_A(43),NULL), //_("file.edit") //<
 	g_strconcat (_("office documents"),"||",NULL),
 		g_strconcat ("\t|",_C(13),"|",NULL),
 			g_strdup ("\t\t|doc|"),
@@ -527,6 +527,7 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 		g_strconcat ("\t|",_C(13),"|",NULL),
 			g_strdup ("\t\t|pdf|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
+			g_strdup ("\t\t|evince|evince"),
 			g_strdup ("\t\t|xpdf|xpdf"),
 			g_strdup ("\t\t|acroread|acroread"),
 	g_strconcat (_("postscript documents"),"||",NULL),
@@ -538,9 +539,9 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 		g_strconcat ("\t|",_C(13),"|",NULL),
 			g_strdup ("\t\t|txt|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
-//not loaded	g_strconcat ("\t\t|",PLUGIN,":",_("View"),"|",_A(5),".",_("view"),NULL),  //conformed name
-			g_strconcat ("\t\t|",_("View"),"|",_A(5),".",_A(101),NULL), //_("file.view") //<
-			g_strconcat ("\t\t|",_("_Edit"),"|",_A(5),".",_A(39),NULL), //_("file.edit") //<
+//not loaded	g_strconcat ("\t\t|",PLUGIN,":",_("View"),"|",_A(6),".",_("view"),NULL),  //conformed name
+			g_strconcat ("\t\t|",_("View"),"|",_A(6),".",_A(106),NULL), //_("file.view") //<
+			g_strconcat ("\t\t|",_("_Edit"),"|",_A(6),".",_A(43),NULL), //_("file.edit") //<
 //this removed in favour of the default context menu item open-with
 //			g_strconcat ("\t\t|",_("Edit _with.."),"|%{(editors)@",_("Editor command:"),"} %p",NULL),
 //			g_strconcat ("\t\t|",_("Edit with vi"),"|x vi",NULL),
@@ -597,12 +598,12 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strdup ("\t\t|tar|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 			g_strconcat ("\t\t|",_("Unpack"),"|tar -xf %f",NULL),
-			g_strconcat ("\t\t|",_("Unpack in other pane"),"|",_A(13),".",_A(72),";x tar -C %D -xf %f;",_A(13),".",_A(71),NULL), //<*2
+			g_strconcat ("\t\t|",_("Unpack in other pane"),"|",_A(14),".",_A(75),";x tar -C %D -xf %f;",_A(14),".",_A(74),NULL), //<*2
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(5),".",_A(100),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(118),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(100),NULL),  //conformed name
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(104),NULL),  //conformed name
 #endif
 			g_strconcat ("\t\t|",_("_List contents"),"|>tar -tf %f \\| less",NULL),
 	g_strconcat (_("gzip tarballs"),"||",NULL),
@@ -611,12 +612,12 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strdup ("\t\t|tgz|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 			g_strconcat ("\t\t|",_("_Unpack"),"|tar -zxf  %f",NULL),
-			g_strconcat ("\t\t|",_("Unpack in _other pane"),"|",_A(13),".",_A(72),";tar -C %D -zxf %f;",_A(13),".",_A(71),NULL), //<*2
+			g_strconcat ("\t\t|",_("Unpack in _other pane"),"|",_A(14),".",_A(75),";tar -C %D -zxf %f;",_A(14),".",_A(74),NULL), //<*2
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(5),".",_A(100),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(118),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(100),NULL),   //conformed name
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(104),NULL),   //conformed name
 #endif
 			g_strconcat ("\t\t|",_("_List contents"),"|>tar -ztf %f \\| less",NULL),
 	g_strconcat (_("bzip2 tarballs"),"||",NULL),
@@ -626,26 +627,39 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strdup ("\t\t|tbz|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 			g_strconcat ("\t\t|",_("_Unpack"),"|>bzip2 -d -c %f \\| tar -xf -",NULL),
-			g_strconcat ("\t\t|",_("Unpack in _other pane"),"|",_A(13),".",_A(72),";>bzip2 -d -c %f \\| tar -C %D -xf -;",_A(13),".",_A(71),NULL), //<*2
+			g_strconcat ("\t\t|",_("Unpack in _other pane"),"|",_A(14),".",_A(75),";>bzip2 -d -c %f \\| tar -C %D -xf -;",_A(14),".",_A(74),NULL), //<*2
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(5),".",_A(100),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(118),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(100),NULL),    //conformed name
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(104),NULL),    //conformed name
 #endif
 			g_strconcat ("\t\t|",_("_List contents"),"|>bzip2 -d -c %f \\| tar -tf - \\| less",NULL),
+	g_strconcat (_("lzma tarballs"),"||",NULL),
+		g_strconcat ("\t|",_C(13),"|",NULL),
+			g_strdup ("\t\t|tar.lzma|"),
+			g_strdup ("\t\t|tlz|"),
+		g_strconcat ("\t|",_C(6),"|",NULL),
+			g_strconcat ("\t\t|",_("_Unpack"),"|>lzma -d -c %f \\| tar -xf -",NULL),
+			g_strconcat ("\t\t|",_("Unpack in _other pane"),"|",_A(14),".",_A(75),";>lzma -d -c %f \\| tar -C %D -xf -;",_A(14),".",_A(74),NULL), //<*2
+#ifdef E2_VFS
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
+#else
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(104),NULL),    //conformed name
+#endif
+			g_strconcat ("\t\t|",_("_List contents"),"|>lzma -d -c %f \\| tar -tf - \\| less",NULL),
 	g_strconcat (_("zip archives"),"||",NULL),
 		g_strconcat ("\t|",_C(13),"|",NULL),
 			g_strdup ("\t\t|zip|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 			g_strconcat ("\t\t|",_("_Unzip"),"|unzip %f",NULL),
-			//space after %F in next line is needed ?
-			g_strconcat ("\t\t|",_("Unzip in _other pane"),"|",_A(10),".",_A(90),";unzip %F;",_A(10),".",_A(90),NULL), //<*2
+			g_strconcat ("\t\t|",_("Unzip in _other pane"),"|",_A(14),".",_A(95),";unzip %P;",_A(14),".",_A(95),NULL), //<*2
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(5),".",_A(100),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(118),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(100),NULL),    //conformed name
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(104),NULL),    //conformed name
 #endif
 			g_strconcat ("\t\t|",_("_List contents"),"|>unzip -l %f \\| less",NULL),
 /* plugin functionality not tested yet, for these
@@ -654,20 +668,20 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strdup ("\t\t|arj|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open")"|",_A(5),".",_A(100),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(118),NULL),
+			g_strconcat ("\t\t|",_("_Open")"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(100),NULL),
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(104),NULL),
 #endif
 	g_strconcat (_("rar archives"),"||",NULL),
 		g_strconcat ("\t|",_C(13),"|",NULL),
 			g_strdup ("\t\t|rar|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open")"|",_A(5),".",_A(100),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(118),NULL),
+			g_strconcat ("\t\t|",_("_Open")"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(100),NULL),
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(104),NULL),
 #endif
 */
 	g_strconcat (_("RPM packages"),"||",NULL),
@@ -677,8 +691,8 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strconcat ("\t\t|",_("In_formation"),"|rpm -qlip %f",NULL),
 			g_strconcat ("\t\t|",_("_Install"),"|su rpm -Uvh %f",NULL),
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(5),".",_A(100),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(118),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
 #endif
 	g_strconcat (_("source code files"),"||",NULL),
 		g_strconcat ("\t|",_C(13),"|",NULL),
@@ -691,7 +705,7 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strconcat ("\t\t|",_("Edit _with.."),"|%{(editors)@",_("Editor command:"),"} %p",NULL),
 //			g_strconcat ("\t\t|",_("Edit with _vi"),"|x vi",NULL),
 //			g_strconcat ("\t\t|",_("Edit with _emacs"),"|x emacs",NULL),
-			g_strconcat ("\t\t|",_("_Edit"),"|",_A(5),".",_A(39),NULL), //_("file.edit") //<
+			g_strconcat ("\t\t|",_("_Edit"),"|",_A(6),".",_A(43),NULL), //_("file.edit") //<
 	g_strconcat (_("object files"),"||",NULL),
 		g_strconcat ("\t|",_C(13),"|",NULL),
 			g_strdup ("\t\t|o|"),
@@ -735,7 +749,7 @@ void e2_filetype_options_register (void)
 
 	e2_option_tree_prepare_defaults (set, _e2_filetype_tree_defaults);
 
-	group_name = g_strconcat(_C(15) ,".",_C(26),":",_C(25),NULL);  //_("filetypes.options:miscellaneous"   //FREEME group_name not here !
+	group_name = g_strconcat(_C(15) ,".",_C(27),":",_C(26),NULL);  //_("filetypes.options:miscellaneous"   //FREEME group_name not here !
 	e2_option_bool_register ("anycase-filetypes", group_name, _("case-insensitive filetypes"),
 		_("This causes text-case to always be ignored when matching a file-extension"), NULL, FALSE,
 		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_FREEGROUP ); //no rebuild
