@@ -1,6 +1,6 @@
-/* $Id: e2_fileview.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_fileview.h 1962 2009-10-07 03:06:10Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -29,6 +29,8 @@ typedef enum { E2_REFRESH, E2_RECREATE, E2_CHANGE } E2_RefreshType;
 typedef enum { CD_NOTFINISHED, CD_TIMEOUT, CD_SUCCESS } E2_CDType;
 //filelist treestore column-number enumerator
 enum { FILENAME = 0, SIZE, PERM, OWNER, GROUP, MODIFIED, ACCESSED, CHANGED, MAX_COLUMNS };
+#define EXTENSION -1 //enum for sort action
+
 //directory-read failure codes
 enum
 {
@@ -82,7 +84,7 @@ typedef struct _E2_ViewInfo
 	VOLATILE GtkWidget *treeview;
 	VOLATILE GtkTreeSelection *selection;	//pointer to selection for this view's treeview
 	VOLATILE GHashTable *selected_names;  //names of selected items (for reselection), or NULL
-	gint sort_column;  //copy of current sort column, can be <0
+	gint sort_column;	//copy of current sort column, can be <0
 	gboolean extsort;	//TRUE when name-sorting is by extension
 	GtkSortType sort_order; //copy of current sort order
 	GtkWidget *sort_arrows[MAX_COLUMNS];
@@ -90,11 +92,14 @@ typedef struct _E2_ViewInfo
 //	E2_FSType dirtype;	//flags for type of namespace this view belongs to
 	E2_FSSensitive case_sensitive_names; //cache, FIXME needs to be cleared when removable device changes
 #ifdef E2_VFS
-	VOLATILE PlaceInfo *spacedata;	//pointer to extra data for virual namespace, NULL for mounted dir
+	VOLATILE PlaceInfo *spacedata;	//pointer to data in relevant member of vpaths cache, NULL for mounted dir
+//	vpath vdir;		//pointer to data in relevant member of vpaths cache
 #endif
-	gchar dir[PATH_MAX];	//path shown in this view (utf-8) with vpreface if approprate
-	GList *dir_history;  //list of structs with focused line etc for each opened dir in the current namespace
-						//FIXME clean list data at end-session
+	gchar dir[PATH_MAX];	//path shown in this view (UTF-8, with trailing "/")
+							//typically same as corresponding (freeable) rt->path,
+							// ? but with easier access for regular usage
+//	GList *dir_history; //list of structs with focused line etc for each opened dir in the current namespace
+						//data are shared with app.dir_history
 //#ifndef E2_FAM
 	time_t dir_mtime;
 	time_t dir_ctime;
@@ -127,26 +132,49 @@ typedef struct _E2_ViewInfo
 
 	gboolean filtered_before;	//state of prior call to filter function
 	gboolean filter_directories;
+	guint filtercount;	//bumped at each re-filter, for e.g. thumbs dialog, to poll for filter-changes
 	gboolean show_hidden;  //whether to display hidden items in this view
 	guint total_items;	//total displayed and undisplayed items in the dir - 2 (to ignore . and ..)
-	gint row;	//index of most-recently clicked row in the treeview
-	gint drop_row;	//index of most-recently moused row during a drag operation
-	gboolean lit;	//TRUE if drop-row is highlighted
+	gint row;			//0-based index of most-recently clicked row in the treeview
+	gint drop_row;		//0-based index of most-recently moused row during a drag operation
+	gboolean lit;		//TRUE if drop-row is highlighted
+	gboolean convert;	//TRUE when any item's name has non-ASCII char(s), so conversion is needed for all
 //	E2_RefreshType refreshtype; //whether a view update is a refresh, recreate or cd
 //	gboolean refresh_requested;	//TRUE when something has proposed that the view be refreshed
 //	gboolean norefresh;	//flag signalling refresh is disabled (though backend checks are not)
 //	gboolean listing ; //flag to block recursive filelist updates
 	E2_Listman listcontrols;
 	gchar last_find[NAME_MAX]; //buffer for storing names used in find task
-//	GHookList hook_refresh;	//data for functions to run during post-refresh idle-time function
+	GHookList hook_refresh;	//data for functions to run during post-refresh idle-time function
 } ViewInfo;
 
 typedef struct _E2_FileInfo
 {
-	gchar filename[NAME_MAX+1];  //localised string
-	struct stat statbuf;
+	gchar filename[NAME_MAX+1];  //item basename, localised string
+#ifdef E2_VFS
+	//in a collection, each item can have any path
+	vpath *vdir; //pointer to hashed dir data for this item, or NULL if view->dir applies
+				 //path string is localised, with no trailing separator
+#endif
+	struct stat statbuf;	//sometimes-customised form of fs statbuf
 } FileInfo;
 
+//enable automatic logging of selected items, and re-selection
+//#define TAG_ALL
+
+typedef struct _E2_DirHistoryEntry
+{
+	gchar path[PATH_MAX];	//must be at start of stuct (struct ptr sometimes used for lists of strings)
+#ifdef E2_VFS
+	PlaceInfo *spacedata;	//pointer to data in relevant member of vpaths cache
+#endif
+	gchar firstname [NAME_MAX+1]; //copy of info->filename for 1st selected item
+	gint selrow;	//liststore index of 1st selected row, -1 if no selection
+	gint toprow;	//liststore index of top visible row
+	E2_FSSensitive case_sensitive_names;
+	GHashTable *selitems; //for use by tag-plugin or with TAG_ALL
+} E2_DirHistoryEntry;
+
 typedef struct _E2_SelectedItemInfo
 {
 	gchar filename[NAME_MAX+1];  //localised string no trailing /
@@ -184,12 +212,25 @@ typedef struct _E2_SelectPattern
 	gboolean negated;
 } E2_SelectPattern;
 
+#define WAIT_FOR_REFRESH(view) \
+	while (TRUE) \
+	{ \
+		gboolean __busy; \
+		LISTS_LOCK \
+		__busy = view->listcontrols.refresh_working \
+			 || view->listcontrols.cd_working; \
+		LISTS_UNLOCK \
+		if (!__busy) \
+			break; \
+		usleep (100000); \
+	}
+
 gint e2_fileview_ext_sort (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
 	GtkSortType *direction);
 gboolean e2_fileview_sort_column (gint colnum, ViewInfo *view);
 void e2_fileview_refilter_list (ViewInfo *view);
-void e2_fileview_filter_dirs_cb (GtkWidget *widget, ViewInfo *view);
-void e2_fileview_remove_filters_cb (GtkWidget *widget, ViewInfo *view);
+void e2_fileview_filter_dirs_cb (GtkCheckMenuItem *widget, ViewInfo *view);
+void e2_fileview_remove_filters_cb (GtkMenuItem *widget, ViewInfo *view);
 void e2_fileview_initialize_filters (ViewInfo *view);
 
 void e2_fileview_set_menu_position (GtkMenu *menu,
@@ -211,7 +252,7 @@ void e2_fileview_clear_row_background (ViewInfo *view,
 	GtkTreeIter *iter);
 void e2_fileview_update_col_cachedata (void);
 void e2_fileview_get_scroll_data (ViewInfo *view, gint *leftcol, gint *toprow);
-void e2_fileview_scroll_to_position (ViewInfo *view, gint leftcol, gint toprow);
+gboolean e2_fileview_scroll_to_position (ViewInfo *view, gint leftcol, gint toprow);
 //gint e2_fileview_find_name_col (ViewInfo *view); unused
 gboolean e2_fileview_prepare_list (ViewInfo *view);
 void e2_fileview_scroll_list (GThread *viewthread);
@@ -222,10 +263,13 @@ gboolean e2_fileview_cd_manage (E2_Listman *data);
 GtkWidget *e2_fileview_create_list (ViewInfo *view);
 void e2_fileview_select_all (GtkWidget *widget, ViewInfo *view);
 void e2_fileview_clean_selected (GPtrArray *selected);
-void e2_fileview_clean_history (GList **history);
+void e2_fileview_clean1_history (gpointer data);
 GPtrArray *e2_fileview_get_selected (ViewInfo *view);
 GList *e2_fileview_get_selected_local (ViewInfo *view);
 FileInfo *e2_fileview_get_selected_first_local (ViewInfo *view, gboolean updir);
 gchar *e2_fileview_get_row_name (ViewInfo *view, gint row);
-
+void e2_fileview_register_keybindings (GtkWidget *treeview, ViewInfo *view);
+#ifdef E2_MOUSECUSTOM
+void e2_fileview_register_pointerbindings (GtkWidget *treeview, ViewInfo *view);
+#endif
 #endif //ndef __E2_FILEVIEW_H__
