@@ -1,6 +1,6 @@
-/* $Id: e2_main.c 893 2008-05-20 09:42:51Z tpgww $
+/* $Id: e2_main.c 2056 2010-02-21 04:52:19Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2010 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -120,17 +120,27 @@ The following items are covered:
 #include "e2_plugins.h"
 #include "e2_complete.h"
 #include "e2_task.h"
+#ifdef E2_MOUSECUSTOM
+# include "e2_mousebinding.h"
+#endif
 
 #define UPGRADE_PNAME "e2p_upgrade.so"
 
-extern gint refresh_refcount;
+extern volatile gint refresh_refcount;
 extern GList *open_history;
 #ifdef USE_GLIB2_10
 extern GHashTable *actions_hash;
 #endif
 
+#ifdef WAIT_POLL
+pthread_mutex_t wait_mutex; // = PTHREAD_MUTEX_INITIALIZER;	if no recursion
+#define WAIT_LOCK pthread_mutex_lock (&wait_mutex);
+#define WAIT_UNLOCK pthread_mutex_unlock (&wait_mutex);
+#endif
+
 static void _e2_main_system_shutdown (gint num); //__attribute__ ((noreturn));
 
+#ifndef NATIVE_BGL
 /**
 @brief local management of gdk threads mutex
 These allow a conservative approach to locking without risk of deadlock
@@ -138,58 +148,94 @@ These allow a conservative approach to locking without risk of deadlock
 */
 void e2_main_close_gdklock (void)
 {
-#ifdef NATIVE_BGL
-	gdk_threads_enter ();
-#else
 	//a single thread can't write to screen more than once at a time,
 	//so deadlocks just reflect "unnecessary" re-locks, and can be ignored
-# ifdef DEBUG_MESSAGES
+#ifdef DEBUG_MESSAGES
 	if (pthread_mutex_lock (&gdklock) == EDEADLK)
 		printd (DEBUG, "attempted BGL re-lock");
-# else
+#else
 	pthread_mutex_lock (&gdklock);
-# endif
 #endif
 }
 void e2_main_open_gdklock (void)
 {
-#ifdef NATIVE_BGL
-	gdk_threads_leave ();
-#else
-# ifdef DEBUG_MESSAGES
+#ifdef DEBUG_MESSAGES
 	if (pthread_mutex_unlock (&gdklock) == EPERM)
 		printd (DEBUG, "bad attempt to open BGL");
-# else
+#else
 	pthread_mutex_unlock (&gdklock);
-# endif
 #endif
 }
-
+#endif
+/* *
+@brief aborted-thread cleanup function
+@param data pointerised boolean, non-NULL to re-close BGL
+*/
+/*void e2_main_cleanup_gdklock (gpointer data)
+{
+	printd (DEBUG, "e2_main_cleanup_gdklock");
+	gboolean FIXME;
+	if (data != NULL)
+	{
+		//e2_main_close_gdklock ();
+		CLOSEBGL_IF_OPEN
+	}
+	printd (DEBUG, "e2_main_cleanup_gdklock FINISH");
+}
+*/
+#ifndef NATIVE_BGL
 /**
-@brief idle and then timer callback to start refreshing filelists after both filelists are initialised
-This tests that both view->dir strings are not ""
+@brief re-initialize BGL
+@param free TRUE to kill the current mutex first
+
+@return
+*/
+void e2_main_replace_gdklock (gboolean free)
+{
+//PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP isn't always available to init mutex
+	pthread_mutexattr_t attr;
+	if (free)
+		pthread_mutex_destroy (&gdklock);
+	pthread_mutexattr_init (&attr);
+#ifdef DEBUG_MESSAGES
+	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_ERRORCHECK);
+#else
+	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_RECURSIVE);
+#endif
+	pthread_mutex_init (&gdklock, &attr);
+	pthread_mutexattr_destroy (&attr);
+}
+#endif
+/**
+@brief idle and then timer callback to do non-time-critical things at session start
+This waits until after both filelists are initialised, as determined by both
+view->dir strings being not empty (""). Then does various things: start refreshing
+filelists, etc
 @param data NULL specified when idle was established, non-NULL for timer
 
 @return TRUE until the filelists have been established
 */
-static gboolean _e2_main_refresh_initialise (gpointer data)
+static gboolean _e2_main_defer_init (gpointer data)
 {
-	if (app.pane1_view.dir == NULL || *app.pane1_view.dir == '\0'
-	 || app.pane2_view.dir == NULL || *app.pane2_view.dir == '\0'
-	 || app.pane1_view.listcontrols.cd_working
-	 || app.pane2_view.listcontrols.cd_working)
+	if (app.pane1.view.dir == NULL || *app.pane1.view.dir == '\0'
+	 || app.pane2.view.dir == NULL || *app.pane2.view.dir == '\0'
+	 || app.pane1.view.listcontrols.cd_working
+	 || app.pane2.view.listcontrols.cd_working)
 	{
 		if (data != NULL)	//not first first callback
 			return TRUE;
 		//start a timer to check completion
 		//app.timers[?] = CHECKME no need to force shutdown ?
-		g_timeout_add (100, (GSourceFunc) (_e2_main_refresh_initialise),
+		g_timeout_add (100, (GSourceFunc) (_e2_main_defer_init),
 			GINT_TO_POINTER (100));
 			return FALSE;
 	}
 	//this func may be called at some racy time when these need to be reset
-	app.pane1_view.listcontrols.norefresh = FALSE;
-	app.pane2_view.listcontrols.norefresh = FALSE;
+	app.pane1.view.listcontrols.norefresh = FALSE;
+	app.pane2.view.listcontrols.norefresh = FALSE;
+	//set the 'real' initial conversion funcs
+	e2_utf8_set_name_conversion (app.pane1.view.convert || app.pane2.view.convert);
+
 #ifdef E2_REFRESH_DEBUG
 	printd (DEBUG, "enable refresh, session start after filelists filled");
 #endif
@@ -197,6 +243,41 @@ static gboolean _e2_main_refresh_initialise (gpointer data)
 #ifndef E2_REFRESH_DEBUG
 	printd (DEBUG, "all change-polling initialised after filelists filled");
 #endif
+
+#ifdef E2_HAL
+	e2_hal_init ();
+#endif
+#ifdef E2_DEVKIT
+	e2_devkit_init ();
+#endif
+
+#ifndef USE_GTK2_99
+	//add a discard if gtk is old or does not provide a stock discard icon as it should
+//	GtkStyle *style = gtk_widget_get_style (app.main_window);
+	GtkIconSet *iset = gtk_style_lookup_icon_set (
+#ifdef USE_GTK2_14
+		gtk_widget_get_style (app.main_window),
+#else
+		app.main_window->style,
+#endif
+		GTK_STOCK_DISCARD);
+	if (iset == NULL)
+	{
+		GtkIconSource *isrc = gtk_icon_source_new ();
+		gtk_icon_source_set_filename (isrc, ICON_DIR G_DIR_SEPARATOR_S GTK_STOCK_DISCARD "_20.png"); //or ".svg");
+		gtk_icon_source_set_direction_wildcarded (isrc, TRUE);
+		gtk_icon_source_set_size_wildcarded (isrc, TRUE);
+		iset = gtk_icon_set_new ();
+		gtk_icon_set_add_source (iset, isrc);
+		GtkIconFactory *extra_gtk_icons = gtk_icon_factory_new ();
+		gtk_icon_factory_add (extra_gtk_icons, GTK_STOCK_DISCARD, iset);
+
+		gtk_icon_factory_add_default (extra_gtk_icons);
+		g_object_unref (G_OBJECT (extra_gtk_icons));	//ensure cleanup on gtk close
+	}
+	//else ??
+#endif
+
 	return FALSE;
 }
 /**
@@ -209,7 +290,7 @@ static gboolean _e2_main_refresh_initialise (gpointer data)
 static void _e2_main_check_locale_broken (const gchar *locale)
  {
 	gboolean broken = FALSE;
-	if (g_str_equal (locale, "C"))
+	if (!strcmp (locale, "C"))
 		broken = TRUE;
 	else
 	{
@@ -256,22 +337,16 @@ static void _e2_main_check_locale_broken (const gchar *locale)
 */
 static void _e2_main_check_locale (void)
 {
-	printd (DEBUG, "check_locale ()");
-	const gchar *locale = gtk_set_locale ();
-	printd (NOTICE, "current locale is '%s'", locale);
-	const gchar *broken = g_getenv ("G_BROKEN_FILENAMES");
-	if (broken != NULL)
+//	printd (DEBUG, "check_locale ()");
+	if (g_getenv ("G_BROKEN_FILENAMES") != NULL)
+	{
+		const gchar *locale = gtk_set_locale (); //repeat: already done by gtk_init()
+		printd (NOTICE, "current locale is '%s'", locale);
 		_e2_main_check_locale_broken (locale);
-
-#ifdef ENABLE_NLS
-	printd (DEBUG, "setting locale base dir to '%s'", LOCALE_DIR);
-	bindtextdomain (BINNAME, LOCALE_DIR);
-	textdomain (BINNAME);
-	bind_textdomain_codeset (BINNAME, "UTF-8");
-#endif
+	}
 }
 
-/**
+/* *
 @brief blind log handler used when gtk messages-logging is suppressed
 
 @return
@@ -301,6 +376,257 @@ static void _e2_main_message_handler (const gchar *log_domain,
 	}
 }
 
+#ifndef WAIT_POLL
+//GMainContext *localctx = NULL; //CHECKME cleared ?
+#endif
+
+/**
+@brief create new local wait loop
+@param mainctx TRUE if wait is initiated from main-context
+@return the loop data struct or NULL in case of error
+*/
+E2_MainLoop *e2_main_loop_new (gboolean mainctx)
+{
+#ifdef WAIT_POLL
+	E2_MainLoop *loopdata = ALLOCATE0 (E2_MainLoop);
+	CHECKALLOCATEDWARN (loopdata, return NULL;)
+	//without a threadID yet, an (unlikely) immediate abort will not find this
+	//loopdata in the list, hence leak
+	loopdata->maincontext = mainctx;
+#else
+/*	GMainContext *ctx;
+	if (mainctx)
+		ctx = NULL;
+	else
+	{
+		if (localctx == NULL)
+			localctx = g_main_context_new ();
+		ctx = localctx;
+	}
+	GMainLoop *loop = g_main_loop_new (ctx, FALSE);
+*/
+	GMainLoop *loop = g_main_loop_new (NULL, FALSE);
+	if (loop == NULL)
+		return NULL;
+
+	E2_MainLoop *loopdata = MALLOCATE (E2_MainLoop);
+	CHECKALLOCATEDWARN (loopdata, return NULL;)
+	loopdata->threadID = 0;
+	loopdata->loop = loop;
+#endif
+	app.mainloops = g_slist_prepend (app.mainloops, loopdata);
+	return loopdata;
+}
+#ifdef WAIT_POLL
+/**
+@brief cleanup after finishing or aborting a wait 'loop'
+@param loopdata pointer to loop data struct
+
+@return
+*/
+static void _e2_main_loop_clean_loop (E2_MainLoop *loopdata)
+{
+	printd (DEBUG, "In %s, loopdata: %x thread %u", __PRETTY_FUNCTION__, loopdata, pthread_self());
+	DEALLOCATE (E2_MainLoop, loopdata);
+	app.mainloops = g_slist_remove (app.mainloops, loopdata);
+//	printd (DEBUG, "End %s, loopdata: %x", __PRETTY_FUNCTION__, loopdata);
+}
+#endif
+/**
+@brief restore BGL after finishing or aborting a wait 'loop'
+Expects BGL open/off.
+@param loopdata pointer to loop data struct
+
+@return
+*/
+static void _e2_main_loop_clean_BGL (E2_MainLoop *loopdata)
+{
+	printd (DEBUG, "Start %s, loopdata: %x thread %u", __PRETTY_FUNCTION__, loopdata, pthread_self());
+	while (g_main_context_iteration (NULL, FALSE)); //CHECKME useful ?
+	gdk_threads_enter ();
+//	printd (DEBUG, "_e2_main_loop_clean_BGL, thread %u, BGL back on", pthread_self());
+	gdk_flush (); //stops UI freezes ? (gtk does this)
+//	printd (DEBUG, "End %s, loopdata: %x", __PRETTY_FUNCTION__, loopdata);
+}
+
+#ifdef WAIT_POLL
+# define LOOP_WAIT_USEC 50000 //20Hz polling should be responsive enough
+#endif
+/**
+@brief block until wait is ended (as indicated by flag in @a loopdata) then cleanup
+Function is abort-safe. BGL is expected to be on/closed
+@param loopdata pointer to loop data struct
+
+@return
+*/
+void e2_main_loop_run (E2_MainLoop *loopdata)
+{
+	//a full-blown mainloop here is overkill?, and may?? compromise BGL mananagment
+#ifdef WAIT_POLL
+	gboolean wait;
+#endif
+	loopdata->threadID = pthread_self ();	//for aborting if that happens
+	gint saverefcount;
+
+	gdk_flush ();
+	pthread_cleanup_push ((gpointer)e2_filelist_reset_refresh, NULL);
+		//enable refresh
+		saverefcount = g_atomic_int_get (&refresh_refcount);
+		if (saverefcount > 0)
+		{
+#ifdef E2_REFRESH_DEBUG
+			printd (DEBUG, "enable refresh, e2_main_loop_run");
+#endif
+			e2_filelist_enable_refresh ();
+		}
+#ifdef WAIT_POLL
+		pthread_cleanup_push ((gpointer)_e2_main_loop_clean_loop, loopdata);
+#endif
+			pthread_cleanup_push ((gpointer)_e2_main_loop_clean_BGL, loopdata);
+				gdk_threads_leave ();
+//				printd (DEBUG, "e2_main_loop_run, BGL off");
+#ifdef WAIT_POLL
+//				pthread_cleanup_push ((gpointer)_e2_main_loop_clean, loopdata);
+				printd (DEBUG, "starting wait-flag poll loop");
+				/* We can't block the parent thread, it's needed to update the UI
+				   including processing the user's input to end this loop. But we
+				   don't want to hog too much resources while sitting here waiting */
+				do
+				{
+//					FIXME the maincontext flag is not set correctly for dialogs that do local
+//					locking e.g. ownership and permissions
+					usleep (LOOP_WAIT_USEC); //may be aborted during this
+
+					//"un-related" events may need to be processed e.g. if an abort-confirmation dialog is popped up
+//					WAIT_FOR_EVENTS_UNLOCKED
+#ifdef DEBUG_MESSAGES
+					if (g_main_context_iteration (NULL, FALSE))
+					{
+						while (g_main_context_iteration (NULL, FALSE));
+						printd (DEBUG, "mainloop source(s) were dispatched");
+					}
+#else
+					while (g_main_context_iteration (NULL, FALSE));
+#endif
+
+					WAIT_LOCK
+					wait = !loopdata->finished;
+					WAIT_UNLOCK
+				} while (wait);
+				printd (DEBUG, "finished wait-flag poll loop");
+#else
+				pthread_cleanup_push ((gpointer)e2_main_loop_quit, loopdata);
+				g_main_loop_run (loopdata->loop);
+				pthread_cleanup_pop (0);	//don't cleanup loop data this way (that re-enters the quitter!)
+#endif
+//				_e2_main_loop_clean_BGL (loopdata);	//pop(1) BAD when aborting ?
+				gdk_threads_enter ();
+				printd (DEBUG, "BGL back on");
+			pthread_cleanup_pop (0);	//BGL back on
+
+#ifdef WAIT_POLL
+		_e2_main_loop_clean_loop (loopdata); //pop(1) BAD when aborting ?
+//		printd (DEBUG, "goto thread cleanup of cleanup loop");
+		pthread_cleanup_pop (0);	//cleanup loop WAS 1, BAD when aborting
+#endif
+//		printd (DEBUG, "after thread cleanup of cleanup loop");
+
+	if (saverefcount > 0)
+	{
+#ifdef E2_REFRESH_DEBUG
+		printd (DEBUG, "disable refresh, e2_main_loop_run");
+#endif
+		e2_filelist_disable_refresh ();
+	}
+	pthread_cleanup_pop (0);
+//	usleep (25000);
+//	WAIT_FOR_EVENTS
+}
+
+/**
+@brief quit a wait loop
+BGL is expected to be on/closed
+@param loopdata pointer to loop data struct
+
+@return
+*/
+void e2_main_loop_quit (E2_MainLoop *loopdata)
+{
+#ifdef WAIT_POLL
+//	printd (DEBUG, "e2_main_loop_quit");
+	WAIT_LOCK
+	loopdata->finished = TRUE;
+	e2_utils_fake_event ();
+	WAIT_UNLOCK
+	//allow the downstream cleanup to happen
+	usleep (LOOP_WAIT_USEC);
+//	printd (DEBUG, "e2_main_loop_quit ENDS");
+#else
+	if (loopdata->loop != NULL)
+	{
+		//ASAP prevent other attempts to stop this loop
+		GMainLoop *tmp = loopdata->loop;
+		loopdata->loop = NULL;
+		GMainContext *ctx = g_main_loop_get_context (tmp);
+		gboolean mainctx = (ctx == g_main_context_default ());
+		if (g_main_loop_is_running (tmp));
+		{
+			if (mainctx)
+			{
+				gdk_threads_leave ();
+				while (g_main_context_pending (ctx)) //INTERFERES with other mainloops
+					g_main_context_iteration (ctx, FALSE);
+				gdk_threads_enter ();
+			}
+			g_main_loop_quit (tmp);
+			usleep (5000);	//allow some post-block activity
+		}
+		g_main_loop_unref (tmp);
+//		if (!mainctx)
+//			g_main_context_unref (ctx);
+	}
+	DEMALLOCATE (E2_MainLoop, loopdata);
+	app.mainloops = g_slist_remove (app.mainloops, loopdata);
+#endif
+}
+/**
+@brief abort any wait whose watcher is @a ID, and cleanup
+@param ID thread ID to match
+
+@return TRUE if something was found
+*/
+gboolean e2_main_loop_abort (pthread_t ID)
+{
+	printd (DEBUG, "e2_main_loop_abort, ID: %u", ID);
+	gboolean killed = FALSE;
+#ifdef WAIT_POLL
+	gboolean wait = FALSE;
+#endif
+	GSList *member = app.mainloops;
+	while (member != NULL)
+	{
+		E2_MainLoop *loopdata = (E2_MainLoop *)member->data;
+		member = member->next;	//in case the list is modified by loop quit
+		if (loopdata->threadID == ID)
+		{
+			killed = TRUE;
+#ifdef WAIT_POLL
+			wait = wait || !loopdata->maincontext;
+#endif
+			//to avoid racing with e2_main_loop_run() we don't kill loopdata here
+			e2_main_loop_quit (loopdata);
+			//may be > 1 wait per thread, don't finish yet
+		}
+	}
+#ifdef WAIT_POLL
+	//for non-main threads (which may be killed by upstream) allow the downstream
+	//cleanup to happen
+	if (wait)
+		usleep (LOOP_WAIT_USEC);
+#endif
+	return killed;
+}
+
   /****************/
  /***** main *****/
 /****************/
@@ -322,31 +648,44 @@ gint main (gint argc, gchar *argv[])
 	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_RECURSIVE);
 	pthread_mutex_init (&list_mutex, &attr);
 	pthread_mutex_init (&history_mutex, &attr);
-#ifndef NATIVE_BGL
-//PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP isn't always available to init mutex
-//	pthread_mutexattr_t attr;
-	pthread_mutexattr_init (&attr);
-	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_ERRORCHECK);
-	pthread_mutex_init (&gdklock, &attr);
-//	pthread_mutexattr_destroy (&attr);
+#ifdef WAIT_POLL
+	pthread_mutex_init (&wait_mutex, &attr);
 #endif
 	pthread_mutexattr_destroy (&attr);
+#ifndef NATIVE_BGL
+	e2_main_replace_gdklock (FALSE);
+#endif
 
 	g_thread_init (NULL);
 #ifndef NATIVE_BGL
 	gdk_threads_set_lock_functions (e2_main_close_gdklock, e2_main_open_gdklock);
 #endif
 	gdk_threads_init ();	//setup gdk mutex
+#ifdef ENABLE_NLS
+	//before gtk_init(), ensure that _() in recent gtk's works cleanly
+	textdomain (BINNAME);
+	printd (DEBUG, "setting locale base dir to '%s'", LOCALE_DIR);
+	bindtextdomain (BINNAME, LOCALE_DIR);
+	bind_textdomain_codeset (BINNAME, "UTF-8");
+#endif
 	//gtk initialization
 	printd (DEBUG, "gtk init");
 	gtk_init (&argc, &argv);
+#ifndef USE_GTK2_12TIPS
+	//log runtime version
+	app.gtkversion = gtk_major_version * 10000 + gtk_minor_version * 100 + gtk_micro_version;
+#endif
 	g_set_application_name (_("emelFM2"));
 
 	e2_cl_option_process (argc, argv);
 	_e2_main_check_locale ();
-#ifndef E2_FILES_UTF8ONLY
-	e2_fs_check_coding ();	//setup for path/filename conversion if needed
-#endif
+//#ifndef E2_FILES_UTF8ONLY
+//	e2_fs_check_coding ();	//setup for path/filename conversion if needed
+//#else
+	//set worst-case for commands etc, pending filelists loading which sets real state
+	e2_utf8_set_name_conversion (TRUE);
+	app.pane1.view.convert = app.pane2.view.convert = TRUE;
+//#endif
 	//send all messages to the handler
 	GLogLevelFlags levels = G_LOG_LEVEL_MASK | G_LOG_FLAG_RECURSION | G_LOG_FLAG_FATAL;
 	g_log_set_handler ("Glib", levels, _e2_main_message_handler, NULL);
@@ -387,6 +726,8 @@ gint main (gint argc, gchar *argv[])
 		e2_option_read_array (values);
 		g_strfreev (values);
 	}
+//	else
+//		app.keytrans = TRUE; //translate the newly-created keybindings, later on
 
 	//after config & updates, install default tree options where needed
 	e2_option_tree_install_defaults ();
@@ -406,14 +747,23 @@ gint main (gint argc, gchar *argv[])
 	//if this is a fresh start,
 	//do this before bars/menus created
 	e2_plugins_load_all (); // load plugins specified in the config data
+	//some of these are used during pane creation
+#ifdef E2_VFS
+	app.vdir_cache = g_hash_table_new_full (e2_fileview_vdir_hash,
+		e2_fileview_vdir_equal, e2_fieview_vdir_free, NULL); //uses same data for key and value
+#endif
+	app.dir_history = g_hash_table_new_full (
+#ifdef E2_VFS
+		g_direct_hash, g_direc_equal, NULL,
+#else
+		g_str_hash, g_str_equal, g_free,
+#endif
+		e2_fileview_clean1_history
+	);
 
-	//build GUI
+	//create and show the main window before the filelist creation (in case there's a big delay)
 	printd (DEBUG, "create main window");
 	e2_window_create (&app.window);
-	//show the window before the filelist creation (in case there's a big delay)
-	gdk_threads_enter ();
-	gtk_widget_show (app.main_window);
-	gdk_threads_leave ();
 
 #ifdef E2_FAM
 	//before change dir, signal unknown state
@@ -434,7 +784,7 @@ gint main (gint argc, gchar *argv[])
 	e2_window_enable_status_update (-1);
 #endif
 	//set refresh counter to its real initial value
-	refresh_refcount = 0;
+	g_atomic_int_set (&refresh_refcount, 0);
 #ifdef E2_REFRESH_DEBUG
 	printd (DEBUG, "disable refresh, session start before filelists built");
 #endif
@@ -442,7 +792,12 @@ gint main (gint argc, gchar *argv[])
 	e2_filelist_disable_refresh ();
 
 	e2_button_setup_labels ();	//maybe needed by dialog when opening dir
-
+	e2_utils_translate_keys (TRUE, FALSE);	//setup for specific keycode management
+	//all widgets are in place, register bindings in case needed in dialog
+	e2_keybinding_register_all ();
+#ifdef E2_MOUSECUSTOM
+	e2_mousebinding_register_all (); //includes gestures if relevant
+#endif
 	//session-starts with BGL closed
 //	gdk_threads_leave ();
 	//threaded cd's may start at any time, including after main loop is running
@@ -456,8 +811,8 @@ gint main (gint argc, gchar *argv[])
 	gint pos = gtk_paned_get_position (GTK_PANED (app.window.output_paned));
 	if (pos == 0)
 	{
-		gchar *command = g_strconcat(_A(1),".",_A(42),NULL);  //_("command.focus")
-		e2_action_run_simple (command, NULL);
+		gchar *command = g_strconcat(_A(1),".",_A(47),NULL);  //_("command.focus")
+		e2_action_run_simple_from (command, NULL, app.main_window);
 		g_free (command);
 	}
 
@@ -466,24 +821,34 @@ gint main (gint argc, gchar *argv[])
 	e2_filetype_add_all ();	//process filetypes into data structure
 
 	e2_cache_list_register ("open-history", &open_history); //history for 'open with' command
-	e2_cache_int_register ("config-width", &app.cfgdlg_width, -1);	//default to 'normal' window size
-	e2_cache_int_register ("config-height", &app.cfgdlg_height, -1);
+	//if no cache data from last usage, we don't want an unpleasant config dialog size
+	e2_cache_int_register ("config-width", &app.cfg_alloc.width, 300);
+	e2_cache_int_register ("config-height", &app.cfg_alloc.height, 400);
 	e2_option_set_trash_dir ();
 
 	GList *tmp;
 	for (tmp = e2_cl_options.startup_commands; tmp != NULL; tmp = g_list_next (tmp))
+		e2_command_run (tmp->data, E2_COMMAND_RANGE_DEFAULT, app.main_window
 #ifdef E2_COMMANDQ
-		e2_command_run (tmp->data, E2_COMMAND_RANGE_DEFAULT, FALSE);
-#else
-		e2_command_run (tmp->data, E2_COMMAND_RANGE_DEFAULT);
+		, FALSE
 #endif
+	);
 	//if instructed, detach from controlling terminal and run in background
 	if (e2_cl_options.detached
 #ifdef DEBUG_MESSAGES
 		&& (e2_cl_options.debug_level < 5)
 #endif
 	)
-		daemon (1, 1);
+	{
+#if 0
+		//FIXME manual daemon setup, as per <URL>linux-daemon-howto.html
+#else
+		if (daemon (1, 1) != 0)	//this func not available for System V-based OS
+		{
+			printd (WARN, "daemon creation failed, running normally instead");
+		}
+#endif
+	}
 
 	//trap signals that might be (but probably aren't) emitted when a
 	//session-manager requests a shutdown/"save-yourself"
@@ -515,20 +880,17 @@ gint main (gint argc, gchar *argv[])
 //now done in plugin init	g_idle_add ((GSourceFunc) e2_vfs_initialise, NULL);
 #endif
 
-	//start change polling after both initial filelists are completed
-	g_idle_add ((GSourceFunc) _e2_main_refresh_initialise , NULL);
+	//start change polling etc after both initial filelists are completed
+	g_idle_add ((GSourceFunc) _e2_main_defer_init, NULL);
 
 	//get all actions, for help document
 //	e2_action_list_all ();
 
-#ifdef E2_HAL
-	e2_hal_init ();
-#endif
-
-	printd (DEBUG, "enter main loop");
+	printd (DEBUG, "enter top-level main loop");
 	gdk_threads_enter ();
 	gtk_main ();
-	gdk_threads_leave ();
+	printd (DEBUG, "leave top-level main loop");
+	gdk_threads_leave ();	//should never get to here
 	exit (0);
 }
 
@@ -556,14 +918,28 @@ gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean d
 				(!e2_option_bool_get ("command-persist"), TRUE);
 			if (running > 0)
 			{
+				//can't use standard warning dialog, both its labels are unsuitable
 				gchar *prompt = g_strdup_printf (_("%u process(es) are running"), running);
-				DialogButtons choice = e2_dialog_warning (prompt);
+				GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_WARNING,
+					prompt, _("confirm"), DUMMY_RESPONSE_CB, NULL);
 				g_free (prompt);
-				if (choice != OK)
+
+				E2_Button yes_btn = { _("_Continue"), GTK_STOCK_EXECUTE,
+					_("Continue working"), E2_BTN_DEFAULT | E2_BTN_TIPPED, 0,
+					E2_BUTTON_YES.response };
+				E2_Button no_btn = { _("_Quit"), GTK_STOCK_QUIT,
+					NULL, 0, 0, E2_BUTTON_NO.response };
+
+				DialogButtons choice = e2_dialog_show (dialog, app.main_window,
+					E2_DIALOG_BLOCKED | E2_DIALOG_FREE,
+					&yes_btn, &no_btn, NULL);
+
+				if (choice != CANCEL)
 					return FALSE;
 			}
 		}
 	}
+
 	guint i;
 	for (i = 0; i < MAX_TIMERS; i++)
 	{
@@ -571,16 +947,34 @@ gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean d
 			g_source_remove (app.timers[i]);
 	}
 	//CHECKME other non-static timers e.g. keybinding timers, edit blink timers etc
-	e2_task_abort (TRUE);
+
+	e2_task_cleanup (FALSE, pthread_self());	//cleanup action/command processing
+
 #ifdef E2_FAM
 	e2_fs_FAM_disconnect ();
 #endif
 #ifdef E2_HAL
 	e2_hal_disconnect ();
 #endif
+#ifdef E2_DEVKIT
+	e2_devkit_disconnect ();
+#endif
 #ifdef E2_IMAGECACHE
 	e2_cache_image_clearall ();
 #endif
+	//before unloading plugins, in case they have config options
+	if (saveconfig)
+	{
+		e2_option_file_write (NULL);
+#ifdef USE_GTK2_18
+		//main window dimensions for cache
+		//FIXME handle maximised window
+		gtk_widget_get_allocation (app.main_window, &app.main_alloc);
+#endif
+		e2_cache_file_write ();
+		printd (DEBUG, "touch config dir");
+		e2_fs_touch_config_dir (); //signal to the world that changes saved
+	}
 	e2_plugins_unload_all (TRUE);	//backup caches, cleanup etc
 /*#ifdef E2_VFS
 	if (vfs.loaded)
@@ -589,24 +983,32 @@ gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean d
 		e2_plugins_unload1 (p, TRUE);
 	}
 #endif */
-	if (saveconfig)
-	{
-		e2_option_file_write (NULL);
-		e2_cache_file_write ();
-		printd (DEBUG, "touch config dir");
-		e2_fs_touch_config_dir (); //signal to the world that changes saved
-	}
+	if (app.pane1.hook_change_dir.is_setup)
+		g_hook_list_clear (&app.pane1.hook_change_dir);
+	if (app.pane2.hook_change_dir.is_setup)
+		g_hook_list_clear (&app.pane2.hook_change_dir);
+	if (app.pane1.view.hook_refresh.is_setup)
+		g_hook_list_clear (&app.pane1.view.hook_refresh);
+	if (app.pane2.view.hook_refresh.is_setup)
+		g_hook_list_clear (&app.pane2.view.hook_refresh);
 #ifdef USE_GLIB2_10
 //	GList *member;
 	//need to deallocate the glib slices, at least
 	//NOTE this must be after all cleanups involving actions
+	//printd (DEBUG, "clean actions");
+	//e2_actions_clean ()
 	g_hash_table_destroy (actions_hash);
 	//NOTE this must be after all cleanups involving options
+/*	printd (DEBUG, "clean options");
+	e2_option_destroy_config ();//why bother, if the kernel does it all anyway?
+								//cache(s) @ cache or named (lists ??)
+								//FIXME?? = all those non-constant config strings
+*/
 	g_hash_table_destroy (options_hash);
 /*	e2_cache_clean ();
 	e2_complete_clear ();
 	e2_alias_clean ();
-	e2_command_line_clean ();
+	e2_command_line_clean_all ();
 	for (member = app.taskhistory; member != NULL; member = member->next)
 	{
 		//also free the data in the rt ??
@@ -616,11 +1018,9 @@ gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean d
 	//output tabs
 	for (member = app.tabslist; member != NULL; member = member->next)
 		DEALLOCATE (E2_OutputTabRuntime, (E2_OutputTabRuntime *) member->data);
-	//toolbar data
-	E2_ToolbarData **thisbar;
-	for (thisbar = app.bars; *thisbar != NULL; thisbar++)
-		DEALLOCATE (E2_ToolbarData, (E2_ToolbarData *)*thisbar);
-	e2_keybinding_clean (); //keybinding runtimes
+*/
+	e2_toolbar_data_clean ();	//toolbar data
+/*	e2_keybinding_clean (); //keybinding runtimes
 	//FIXME deallocate other slices:
 	//EncArray
 	//E2_ToggleBox's ?
@@ -638,31 +1038,34 @@ gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean d
 	//these might be in some backup place, for VFS
 	//dirline histories ?
 	//directory history list items
-	e2_fileview_clean_history (&app.pane1_view.dir_history);
-	e2_fileview_clean_history (&app.pane2_view.dir_history);
-	g_list_free (app.pane1.opendirs);	//this for non-cached list without heaped data
+	g_hash_table_destroy (app.keysnative);
+	if (app.keyslocal != NULL)
+		g_hash_table_destroy (app.keyslocal);
+#ifdef E2_VFS
+	g_hash_table_destroy (app.vdir_cache);
+#endif
+//	printd (DEBUG, "clear dirs hash");
+	g_hash_table_destroy (app.dir_history);
+	g_list_free (app.pane1.opendirs);	//does not have heaped data
 	g_list_free (app.pane2.opendirs);	//ditto
-/*	printd (DEBUG, "clean options");
-	destroy_config ();  //why bother, if the kernel does it all anyway?
-								//cache(s) @ cache or named (lists ??)
-								//FIXME?? = all those non-constant config strings
-	printd (DEBUG, "clean bookmarks");
+/*	printd (DEBUG, "clean bookmarks");
 	e2_bookmark_clean ();
-	printd (DEBUG, "clean actions");
-	e2_actions_clean ();
 */
-	i = gtk_main_level ();	//mainloop level isn't decremented by quit func
-	printd (DEBUG, "gtk_main_quit () from level %d", i);
-	while (i > 0)
-	{
-		gtk_main_quit ();
-		i--;
-	}
-
 	if (doexit)
 	{
+		gtk_main_quit ();
+
+		pthread_mutex_destroy (&list_mutex);
+		pthread_mutex_destroy (&history_mutex);
+#ifdef WAIT_POLL
+		pthread_mutex_destroy (&wait_mutex);
+#endif
+#ifdef NATIVE_BGL
 		gdk_threads_leave ();
-//		printd (DEBUG, "exit (0)");
+#else
+		pthread_mutex_destroy (&gdklock);
+#endif
+		printd (DEBUG, "exit(0) from lounge");
 		exit (0);
 	}
 	return FALSE;
