@@ -1,6 +1,6 @@
-/* $Id: e2_output.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_output.h 1896 2009-09-24 08:00:32Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -102,6 +102,10 @@ void e2_output_print2 (E2_OutputTabRuntime *tab, const gchar *msg,
 void e2_output_get_bottom_iter (E2_OutputTabRuntime *rt);
 void e2_output_scroll_to_bottom (E2_OutputTabRuntime *rt);
 
+void e2_output_register_keybindings (GtkWidget *textview);
+#ifdef E2_MOUSECUSTOM
+void e2_output_register_pointerbindings (GtkWidget *textview);
+#endif
 void e2_output_update_style (void);
 GtkWidget *e2_output_initialise (void);
 void e2_output_set_menu_position (GtkWidget *menu, gint *x, gint *y,
