@@ -1,6 +1,6 @@
-/* $Id: e2_plugins.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_plugins.h 2005 2009-11-11 22:11:02Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -82,14 +82,13 @@ Plugin *e2_plugins_open1 (gchar *filepath);
 gboolean e2_plugins_unload1 (Plugin *p, gboolean force);
 void e2_plugins_load_all (void);
 void e2_plugins_unload_all (gboolean force);
-void e2_plugins_store_child_data (GtkTreeModel *model, GtkTreeIter *iter, Plugin *p);
+void e2_plugins_store_data (GtkTreeModel *model, GtkTreeIter *iter, Plugin *p,
+	gboolean inmenu, gchar *localpath);
+void e2_plugins_freshen_data (void);
 void e2_plugins_update (void);
 gboolean e2_plugins_configure (gpointer from, E2_ActionRuntime *art);
-E2_Action *e2_plugins_action_register (gchar *name, E2_ACTION_TYPE type,
-	void *func, gpointer data, gboolean has_arg, E2_ACTION_EXCLUDE exclude,
-	gpointer data2);
+E2_Action *e2_plugins_action_register (const E2_Action *newaction);
 gboolean e2_plugins_action_unregister (gchar *name);
-void e2_plugins_do_action (GtkWidget *widget, E2_Action *action);
 //Plugin *get_plugin_by_name (gchar *name);
 Plugin *e2_plugins_check_installed (const gchar *signature);
 Plugin *e2_plugins_load_plugin (const gchar *signature);
@@ -98,7 +97,8 @@ gboolean e2_plugins_find_function (const gchar *signature, const gchar *func_nam
 Plugin *e2_plugins_create_child (Plugin *parent);
 GList *e2_plugins_get_list (void);
 E2_OptionSet *e2_plugins_option_register (E2_OptionType type, gchar *name,
-	gchar *group, gchar *desc, gchar *tip, gchar *depends, E2_OptionFlags flags);
+	gchar *group, gchar *desc, gchar *tip, gchar *depends,
+	E2_OptionSetupExtra *ex, E2_OptionFlags flags);
 gboolean e2_plugins_option_unregister (gchar *name);
 void e2_plugins_options_register (void);
 
