@@ -1,6 +1,6 @@
-/* $Id: e2_task.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_task.h 1840 2009-09-17 01:15:11Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -97,7 +97,9 @@ typedef enum
 	E2_TASKTYPE_ACTION
 } E2_TaskDataType;
 
-typedef struct _E2_ActionTaskData
+typedef struct _E2_ActionTaskData E2_ActionTaskData;
+
+struct _E2_ActionTaskData
 {
 	VOLATILE E2_TaskType tasktype;	//what type of task
 	VOLATILE gboolean result;	//result from the function performing the task
@@ -111,14 +113,14 @@ typedef struct _E2_ActionTaskData
 	VOLATILE E2_Action *action;	//action (hence, name and setup data)
 	VOLATILE gpointer rt_data;	//action runtime data (usually string or NULL, maybe flags)
 	VOLATILE GtkWidget *initiator;	//the widget which was activated to initiate the task
-	VOLATILE gpointer taskfunc;	//the function to perform the task
-	VOLATILE gpointer callback;	//the function to call upon completion (maybe NULL)
+	VOLATILE gboolean (*taskfunc) (E2_ActionTaskData*);	//the function to perform the task
+	VOLATILE void (*callback) ();	//the function to call upon completion (maybe NULL)
 	//to prevent races, only 1 actioner and 1 monitor at any time - hence static ID's
 //	VOLATILE pthread_t athreadID;	//id of the action-thread assigned to run the task
 //	VOLATILE pthread_t mthreadID;	//id of the monitor-thread assigned to run the task
 	VOLATILE time_t timelimit;	//if not finished by this, abort the task
 	VOLATILE E2_TaskStatus *status;	//pointer to status in E2_TaskRuntime (for changes)
-} E2_ActionTaskData;
+};
 
 //task-queue data for commands and actions
 typedef struct _E2_TaskRuntime
@@ -187,11 +189,12 @@ typedef struct _E2_ChownData
 	GList *dirdata;
 } E2_ChownData;
 
+const gchar *e2_task_find_prior_command (gint backcount, gchar *begins);
 E2_TaskRuntime *e2_task_find_running_task (glong pid);
 E2_TaskRuntime *e2_task_find_last_running_child (gboolean anytab);
 E2_TaskRuntime *e2_task_set_data (glong pid, E2_TaskDataType mode, gchar *command);
 
-void e2_task_abort (gboolean all);
+void e2_task_cleanup (gboolean stay, pthread_t mainID);
 
 #define e2_task_do_task(type,art,from,taskfunc,callback) \
 	e2_task_run_task (type, art, from, taskfunc, callback, TRUE, TRUE)
@@ -199,31 +202,21 @@ void e2_task_abort (gboolean all);
 	e2_task_run_task (type, art, from, taskfunc, callback, FALSE, TRUE)
 
 gboolean e2_task_run_task (E2_TaskType type, E2_ActionRuntime *art,
-	gpointer from, gpointer taskfunc, gpointer callback, gboolean immediate,
-	gboolean pane_data);
+	gpointer from, gboolean (*taskfunc) (E2_ActionTaskData *),
+	void (*callback) (), gboolean immediate, gboolean pane_data);
 void e2_task_refresh_lists (E2_ActionTaskData *qed);
 void e2_task_advise (void);
 
 gchar *e2_task_tempname (const gchar *path);
-gboolean e2_task_sync_dirs (gpointer from, E2_ActionRuntime *art);
-gboolean e2_task_view (gpointer from, E2_ActionRuntime *art);
 gboolean e2_task_drop (E2_TaskType type, gchar *srcdir_local,
 	gchar *destdir_local, GPtrArray *names_array);
-gboolean e2_task_refresh (gpointer from, E2_ActionRuntime *art);
-//gboolean e2_task_open (gpointer from, E2_ActionRuntime *art);
-gboolean e2_task_open_with (gpointer from, E2_ActionRuntime *art);
-//gboolean e2_task_open_in_other_pane (gpointer from, E2_ActionRuntime *art);
-
-gboolean e2_task_configure (gpointer from, E2_ActionRuntime *art);
-gboolean e2_task_configure_default (gpointer from, E2_ActionRuntime *art);
-//gboolean e2_task_find (gpointer from, E2_ActionRuntime *art);
-
 gboolean e2_task_backend_copy (VPATH *src, VPATH *dest, E2_FileTaskMode mode);
 gboolean e2_task_backend_move (VPATH *src, VPATH *dest);
 gboolean e2_task_backend_link (VPATH *target, VPATH *name);
 gboolean e2_task_backend_rename (VPATH *oldsrc, VPATH*newsrc);
 gboolean e2_task_backend_delete (VPATH *localpath);
-gboolean e2_task_backend_chmod (VPATH *localpath, gchar *mode, E2_RecurseType recurse);
+gboolean e2_task_backend_chmod (VPATH *localpath, E2_ChmodType optype,
+	mode_t newmode, E2_RecurseType recurse);
 gboolean e2_task_backend_chown (VPATH *localpath, uid_t owner_id, gid_t group_id,
 	gboolean recurse);
 gboolean e2_task_backend_open (VPATH *localpath, gboolean ask);
