@@ -1,6 +1,6 @@
-/* $Id: e2_task_backend.c 799 2008-02-03 23:11:42Z tpgww $
+/* $Id: e2_task_backend.c 1802 2009-07-16 02:59:11Z tpgww $
 
-Copyright (C) 2005-2008 tooar <tooar@gmx.net>
+Copyright (C) 2005-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -43,92 +43,35 @@ These functions are thread-safe, when E2_TREEWALK is defind
 gpointer chaclfunc;
 
 /**
-@brief parse a symbolic permissions string
-The string @a tmp1 would be like 'o+x' or 'g-r' etc
-@param tmp1 permissions string to parse
-@param setmask ptr to store for bits to be set in the mode flags
-@param setmask ptr to store for bits to be cleared from the mode flags
+@brief store operation-specific flags in @a data
+@param optype enum for the type of operation being performed
+@param opmode mode flags to be used in the operation
+@param data pointer to data struct
 @return
 */
-static void _e2_task_backend_parse_flags (gchar *tmp1,
-	mode_t *setmask, mode_t *clearmask)
+static void _e2_task_backend_parse_mode (E2_ChmodType optype, mode_t opmode,
+	E2_ChmodData *data)
 {
-	gint shifter;
-	switch (tmp1[0])
-	{	//get the scope
-		case 'g':
-			shifter=3;
+	switch (optype)
+	{
+		case E2_CHMOD_SET:
+			data->clearmask = ~ALLPERMS; //clear everything
+			data->setmask = opmode;
 			break;
-		case 'u':
-			shifter=6;
+		case E2_CHMOD_ADD:
+			data->clearmask = ALLPERMS; //clear nothing
+			data->setmask = opmode;
 			break;
-		case 'o':
-		default:
-			shifter=0;
+		case E2_CHMOD_REM:
+			data->clearmask = ~opmode;
+			data->setmask = 0;
+			break;
+		default:	//should never happen
+			printd (DEBUG, "Ignoring bad mode string for chmod");
+			data->clearmask = ALLPERMS;
+			data->setmask = 0;
 			break;
 	}
-	gint mask=0;	//default
-	switch (tmp1[2])
-	{  //get the mode
-		case  'r':
-			mask=S_IROTH;
-			break;
-		case  'w':
-			mask=S_IWOTH;
-			break;
-		case  'x':
-			mask=S_IXOTH;
-			break;
-		case  's':
-			if (tmp1[0] == 'u')
-				mask=S_IROTH;
-			else if (tmp1[0] == 'g')
-				mask=S_IWOTH;
-			break;
-		case  't':
-			mask=S_IXOTH;
-			shifter = 9;
-			break;
-		default:
-			break;
-	}
-	if (tmp1[1] == '+')
-		*setmask |= mask << shifter;
-	else if (tmp1[1] == '-')
-		*clearmask	|= mask << shifter;
-}
-/**
-@brief parse octal or symbolic permissions string @a mode
-@param mode mode string
-@param data pointer to data struct
-@return nothing, the results are stored in global static variables
-*/
-static void _e2_task_backend_parse_mode (gchar *mode, E2_ChmodData *data)
-{
-	data->setmask = 0;
-	data->clearmask = 0;
-	if (strchr (mode, ',') != NULL)	//always ascii
-	{	//multiple adds or removes
-		gchar **tmp1, **tmp2;
-		tmp1 = tmp2 = g_strsplit (mode, ",", -1);
-		while (*tmp1 != NULL)
-		{
-			_e2_task_backend_parse_flags (*tmp1, &(data->setmask), &(data->clearmask));
-			tmp1++;
-		}
-		g_strfreev (tmp2);
-	}
-	else
-		if ((strchr (mode, '+') != NULL) || (strchr (mode, '-') != NULL))	//always ascii
-			//single add or remove
-			_e2_task_backend_parse_flags (mode, &(data->setmask), &(data->clearmask));
-	else
-	{	//octal string, 3 or 4 digits, no checking
-		glong tmp = strtol (mode, NULL, 8);
-		data->setmask = (mode_t) tmp;
-		data->clearmask = ALLPERMS;
-	}
-	data->clearmask = ~data->clearmask;
 }
 /**
 @brief set permissions of item @a path to @a mode
@@ -146,7 +89,7 @@ or a privileged user can change them.
  *EFTYPE
 mode has the S_ISVTX ("sticky") bit set, and the named file is not a directory.
 
-@param localpath localised string, absolute path of item to be processed
+@param localpath data including localised string, absolute path of item to be processed
 @param mode mode_t to be set
 @return TRUE if succeeds
 */
@@ -198,11 +141,11 @@ static E2_TwResult _e2_task_twcb_chmod (VPATH *localpath,
 				dirfix = member->data;
 				if (dirfix != NULL)
 				{
-					if (g_str_equal (dirfix->path, localpath))
+					if (!strcmp (dirfix->path, localpath))
 					{
-						if (!_e2_task_backend_chmod1 (localpath,
-							dirfix->mode E2_ERR_PTR()))
-								retval = E2TW_FIXME;
+						//try to set new or original mode
+						if (!_e2_task_backend_chmod1 (localpath, dirfix->mode E2_ERR_PTR()))
+							retval = E2TW_FIXME;
 						g_free (dirfix->path);
 						DEALLOCATE (E2_DirEnt, dirfix);
 						user_data->dirdata = g_list_delete_link (user_data->dirdata, member);
@@ -214,17 +157,21 @@ static E2_TwResult _e2_task_twcb_chmod (VPATH *localpath,
 			}
 			break;
 		case E2TW_DRR:	//directory now readable
-			retval |= E2TW_DRKEEP;	//no permission reversion in walker
+			if (user_data->scope & E2_RECURSE_DIRS)
+				retval |= E2TW_DRKEEP;	//no permission reversion in walker
 		case E2TW_D:
 			//ensure dir is writable, if we can
 			if (e2_fs_tw_adjust_dirmode (localpath, statptr, (W_OK | X_OK)) == 0)
 			{
-				//failed to set missing W and/or X perm
-				//take a shot at doing the change, anyhow, probably fails
-				mode = ((statptr->st_mode & user_data->clearmask) | user_data->setmask) & ALLPERMS;
-				e2_fs_chmod (localpath, mode E2_ERR_NONE());
-				//FIXME warn user about failure
-				retval |= E2TW_SKIPSUB;	//don't try to do any descendant
+				if (user_data->scope & E2_RECURSE_DIRS)
+				{
+					//failed to set missing W and/or X perm
+					//take a shot at doing the change, anyhow, probably fails
+					mode = ((statptr->st_mode & user_data->clearmask) | user_data->setmask) & ALLPERMS;
+					e2_fs_chmod (localpath, mode E2_ERR_NONE());
+					//FIXME warn user about failure
+					retval |= E2TW_SKIPSUB;	//don't try to do any descendant
+				}
 			}
 			else	//dir can be processed
 			{
@@ -232,35 +179,42 @@ static E2_TwResult _e2_task_twcb_chmod (VPATH *localpath,
 				dirfix = ALLOCATE (E2_DirEnt);
 				CHECKALLOCATEDWARNT (dirfix, retval=E2TW_STOP;break;)
 				dirfix->path = g_strdup (VPSTR(localpath));
-				dirfix->mode = ((statptr->st_mode & user_data->clearmask) | user_data->setmask) & ALLPERMS;
+				if (user_data->scope & E2_RECURSE_DIRS)
+					dirfix->mode = ((statptr->st_mode & user_data->clearmask)
+					| user_data->setmask) & ALLPERMS;
+				else
+					dirfix->mode = statptr->st_mode & ALLPERMS;	//want to restore the original value
 				user_data->dirdata = g_list_append (user_data->dirdata, dirfix);
 			}
 			break;
 		case E2TW_DM:	//dir not opened (reported upstream)
 		case E2TW_DL:	//ditto
 		case E2TW_DNR:	//unreadable directory (for which, error is reported upstream)
-						//chmod for this will probably fail, but try anyhow
-//			mode = statptr->st_mode;
-			//ensure dir is writable, if we can
-			//we don't need X permission to handle these
-			newmode = e2_fs_tw_adjust_dirmode (localpath, statptr, W_OK);
-			mode = ((statptr->st_mode & user_data->clearmask) | user_data->setmask) & ALLPERMS;
-			if (newmode == 0)
+			if (user_data->scope & E2_RECURSE_DIRS)
 			{
-				//take a shot at doing the change, anyhow, probably fails
-				e2_fs_chmod (localpath, mode E2_ERR_NONE());
-				//FIXME warn user about failure
-				retval = E2TW_FIXME;
-			}
-			else	//dir can be processed
-			{
-				if (!_e2_task_backend_chmod1 (localpath, mode E2_ERR_PTR()))
+				//chmod for this will probably fail, but try anyhow
+//				mode = statptr->st_mode;
+				//ensure dir is writable, if we can
+				//we don't need X permission to handle these
+				newmode = e2_fs_tw_adjust_dirmode (localpath, statptr, W_OK);
+				mode = ((statptr->st_mode & user_data->clearmask) | user_data->setmask) & ALLPERMS;
+				if (newmode == 0)
+				{
+					//take a shot at doing the change, anyhow, probably fails
+					e2_fs_chmod (localpath, mode E2_ERR_NONE());
+					//FIXME warn user about failure
 					retval = E2TW_FIXME;
-				//FIXME warn user about failure
+				}
+				else	//dir can be processed
+				{
+					if (!_e2_task_backend_chmod1 (localpath, mode E2_ERR_PTR()))
+						retval = E2TW_FIXME;
+					//FIXME warn user about failure
+				}
 			}
 			break;
 		case E2TW_F:
-			if (user_data->scope != E2_RECURSE_DIRS)
+			if (user_data->scope & E2_RECURSE_OTHER)
 			{  //change mode now, if we can
 //				mode = (statptr->st_mode & user_data->clearmask) | user_data->setmask;
 //				if (S_ISREG (statptr->st_mode))
@@ -367,7 +321,7 @@ static E2_TwResult _e2_task_twcb_chown (VPATH *localpath,
 				dirfix = (E2_DirEnt *)member->data;
 				if (dirfix != NULL)
 				{
-					if (g_str_equal (dirfix->path, localpath))
+					if (!strcmp (dirfix->path, localpath))
 					{
 						if (!_e2_task_backend_chown1 (localpath, (struct stat *)statptr,
 							user_data->new_uid, user_data->new_gid E2_ERR_PTR()))
@@ -579,18 +533,14 @@ static gboolean _e2_task_backend_delete (VPATH *localpath E2_ERR_ARG())
 		retval = E2_ERR_PIS (ENOENT);	//ok if nothing to delete, otherwise error
 		if (retval)
 		{
-			if (E2_ERR_NAME != NULL && *E2_ERR_NAME != NULL)
-			{
-				g_error_free (*E2_ERR_NAME);
-				*E2_ERR_NAME = NULL;
-			}
+			E2_ERR_PCLEAR
 		}
 		return retval;
 #else
 		return (E2_ERR_PIS (ENOENT));	//ok if nothing to delete, otherwise error
 #endif
 	}
-	if (S_ISDIR (statbuf.st_mode) && !S_ISLNK (statbuf.st_mode))
+	if (S_ISDIR (statbuf.st_mode))
 	{	//dir
 /*
 		//subject to permission, make sure we can traverse the dir
@@ -601,13 +551,7 @@ static gboolean _e2_task_backend_delete (VPATH *localpath E2_ERR_ARG())
 			if (!_e2_task_backend_chmod1 (localpath, thismode E2_ERR_SAMEARG()))
 				return FALSE;
 		}
-#ifdef E2_VFS
-		if (E2_ERR_NAME != NULL && *E2_ERR_NAME != NULL)
-		{
-			g_error_free (*E2_ERR_NAME);
-			*E2_ERR_NAME = NULL;
-		}
-#endif
+		E2_ERR_CLEAR
 		//try for simple deletion first
 		if (! e2_fs_rmdir (localpath E2_ERR_SAMEARG()))
 		{
@@ -617,13 +561,7 @@ static gboolean _e2_task_backend_delete (VPATH *localpath E2_ERR_ARG())
 		{
 			return FALSE;	//error if failed for some reason other than not-empty
 		}
-#ifdef E2_VFS
-		if (E2_ERR_NAME != NULL && *E2_ERR_NAME != NULL)
-		{
-			g_error_free (*E2_ERR_NAME);
-			*E2_ERR_NAME = NULL;
-		}
-#endif
+		E2_ERR_CLEAR
 */
 		//dir not empty, recursively delete its contents
 		//FIXME which errors to report and/or ignore
@@ -805,13 +743,16 @@ Any error messages expect BGL to be open
 static gboolean _e2_task_backend_linkcopy1 (VPATH *src, VPATH *dest)
 {
 	gchar linkto[PATH_MAX];	//+NAME_MAX];
+#ifdef E2_VFS
+	VPATH sdata;
+	VPATH tdata;
+#endif
 	E2_ERR_DECLARE
 
 	//get target of link to copy
 	gint len = e2_fs_readlink (src, linkto, sizeof (linkto) E2_ERR_PTR());
 	if (len > 0)
 	{
-		linkto[len] = '\0';
 		if (e2_option_bool_get ("relative-symlinks"))
 		{
 			gchar *freeme;
@@ -820,11 +761,16 @@ static gboolean _e2_task_backend_linkcopy1 (VPATH *src, VPATH *dest)
 				  || g_str_has_prefix (linkto, "."G_DIR_SEPARATOR_S)))
 			{	//the target is not a relative or absolute path
 				freeme = g_path_get_dirname (VPSTR(src));
-				if (!g_str_equal (freeme, "."))
+				if (strcmp (freeme, "."))
 				{
 					gchar *temp = g_build_filename (freeme, linkto, NULL);
 					g_free (freeme);
-					freeme = e2_utils_create_relative_path (temp, VPCSTR(dest));
+#ifdef E2_VFS
+					sdata.path = temp; sdata.spacedata = src->spacedata;
+					freeme = e2_utils_create_relative_path (&sdata, dest);
+#else
+					freeme = e2_utils_create_relative_path (temp, dest);
+#endif
 					g_free (temp);
 					g_strlcpy (linkto, freeme, sizeof(linkto));
 				}
@@ -832,7 +778,12 @@ static gboolean _e2_task_backend_linkcopy1 (VPATH *src, VPATH *dest)
 			}
 			else
 			{
-				freeme = e2_utils_create_relative_path (linkto, VPCSTR(dest));
+#ifdef E2_VFS
+				sdata.path = linkto; sdata.spacedata = src->spacedata;
+				freeme = e2_utils_create_relative_path (&sdata, dest);
+#else
+				freeme = e2_utils_create_relative_path (linkto, dest);
+#endif
 				g_strlcpy (linkto, freeme, sizeof(linkto));
 				g_free (freeme);
 			}
@@ -840,8 +791,8 @@ static gboolean _e2_task_backend_linkcopy1 (VPATH *src, VPATH *dest)
 
 		gchar *temp = e2_task_tempname (VPSTR(dest));
 #ifdef E2_VFS
-		VPATH sdata = { linkto, src->spacedata };
-		VPATH tdata = { temp, dest->spacedata };
+		sdata.path = linkto; sdata.spacedata = src->spacedata;
+		tdata.path = temp; tdata.spacedata = dest->spacedata;
 		if (!e2_fs_symlink (&sdata, &tdata E2_ERR_NONE()))
 #else
 		if (!e2_fs_symlink (linkto, temp E2_ERR_NONE()))
@@ -930,21 +881,17 @@ static gboolean _e2_task_backend_mkdir1 (VPATH *dest, mode_t mode)
 }
 /**
 @brief check whether is ok to overwrite @a dest
-@param src absolute path of item to copy, localised string
-@param dest absolute path of existing item to be overwritten, localised string
+@param src virtual path with localised absolute path of item to copy/move
+@param dest virtual path with localised absolute path existing item to be overwritten
 @param mode pointer to store for flag to update
 @param retval pointer to store for treewalk exit code to update
 @return code corresponding to user's choice
 */
-static DialogButtons _e2_task_backend_confirm (gchar *src, gchar *dest,
+static DialogButtons _e2_task_backend_confirm (VPATH *src, VPATH *dest,
 	E2_FileTaskMode *mode, E2_TwResult *retval)
 {
-	e2_filelist_enable_refresh ();  //allow updates while we wait
-	gdk_threads_enter ();
-	//no change to task status while waiting
+	//note: no change to task status while waiting
 	DialogButtons choice = e2_dialog_ow_check (src, dest, BOTHALL);
-	gdk_threads_leave ();
-	e2_filelist_disable_refresh ();
 	switch (choice)
 	{
 		case YES_TO_ALL:
@@ -995,7 +942,7 @@ static E2_TwResult _e2_task_twcb_copy (VPATH *localpath,
 				dirfix = member->data;
 				if (dirfix != NULL)
 				{
-					if (g_str_equal (dirfix->path, localpath))
+					if (!strcmp (dirfix->path, localpath))
 					{
 						gpointer quickfunc;
 						quickfunc = chaclfunc;	//should be atomic, no need for mutex
@@ -1064,7 +1011,12 @@ static E2_TwResult _e2_task_twcb_copy (VPATH *localpath,
 					break;
 				if (user_data->taskmode & E2_FTM_CHECK)
 				{
-					choice = _e2_task_backend_confirm ((gchar *)localpath, dest,
+					choice = _e2_task_backend_confirm (localpath,
+#ifdef E2_VFS
+						&ddata,
+#else
+						dest,
+#endif
 						&user_data->taskmode, &retval);
 					if (choice == CANCEL || choice == NO_TO_ALL)
 					{
@@ -1118,7 +1070,12 @@ static E2_TwResult _e2_task_twcb_copy (VPATH *localpath,
 				}
 				if (user_data->taskmode & E2_FTM_CHECK)
 				{
-					choice = _e2_task_backend_confirm ((gchar *)localpath, dest,
+					choice = _e2_task_backend_confirm (localpath,
+#ifdef E2_VFS
+						&ddata,
+#else
+						dest,
+#endif
 						&user_data->taskmode, &retval);
 					if (choice == CANCEL || choice == NO_TO_ALL)
 					{
@@ -1165,8 +1122,13 @@ static E2_TwResult _e2_task_twcb_copy (VPATH *localpath,
 				&& e2_fs_access2 (dest E2_ERR_NONE()) == 0)	//destination item exists already
 #endif
 			{
-				choice = _e2_task_backend_confirm ((gchar *)localpath, dest,
-						&user_data->taskmode, &retval);
+				choice = _e2_task_backend_confirm (localpath,
+#ifdef E2_VFS
+					&ddata,
+#else
+					dest,
+#endif
+					&user_data->taskmode, &retval);
 				if (choice == CANCEL || choice == NO_TO_ALL)
 					break;
 			}
@@ -1304,7 +1266,7 @@ gboolean e2_task_backend_copy (VPATH *src, VPATH *dest, E2_FileTaskMode mode)
 				gchar *dstr = e2_utils_strcat (data.newroot,
 					dirfix->path + data.oldroot_len);
 #ifdef E2_VFS
-				fixdata.localpath = dstr;
+				fixdata.path = dstr;
 #endif
 				if (mode & E2_FTM_SAMETIME)
 				{
@@ -1463,8 +1425,8 @@ gboolean e2_task_backend_move (VPATH *src, VPATH *dest)
 			gchar *msg = g_strdup_printf (_("Cannot move %s to %s"), utf, utf2);
 			e2_fs_error (msg E2_ERR_MSGL());
 			E2_ERR_CLEAR
-			F_FREE (utf);
-			F_FREE (utf2);
+			F_FREE (utf, VPSTR(src));
+			F_FREE (utf2, VPSTR(dest));
 			g_free (msg);
 			return FALSE;
 		}
@@ -1499,9 +1461,9 @@ gboolean e2_task_backend_link (VPATH *target, VPATH *name)
 #endif
 	if (e2_option_bool_get ("relative-symlinks"))
 	{
-		freeme = e2_utils_create_relative_path (VPCSTR(target), VPCSTR(name));
+		freeme = e2_utils_create_relative_path (target, name);
 #ifdef E2_VFS
-		fdata.localpath = freeme;
+		fdata.path = freeme;
 		fdata.spacedata = name->spacedata;
 #endif
 	}
@@ -1566,8 +1528,8 @@ gboolean e2_task_backend_rename (VPATH *oldsrc, VPATH *newsrc)
 		gchar *msg = g_strdup_printf (_("Cannot rename %s to %s"), utf, utf2);
 		e2_fs_error (msg E2_ERR_MSGL());
 		E2_ERR_CLEAR
-		F_FREE (utf);
-		F_FREE (utf2);
+		F_FREE (utf, VPSTR(oldsrc));
+		F_FREE (utf2, VPSTR(newsrc));
 		g_free (msg);
 		if (temp != VPSTR(newsrc))
 			g_free (temp);
@@ -1595,8 +1557,8 @@ gboolean e2_task_backend_rename (VPATH *oldsrc, VPATH *newsrc)
 	return TRUE;
 }
 /**
-@brief delete item @a path
-@param path localised string, absolute path of item to delete
+@brief delete item @a localpath
+@param localpath data ibncluding localised string, absolute path of item to delete
 @return TRUE if delete succeeds
 */
 gboolean e2_task_backend_delete (VPATH *localpath)
@@ -1617,23 +1579,24 @@ gboolean e2_task_backend_delete (VPATH *localpath)
 /**
 @brief change permissions of item @a path to @a mode
 
-Only an item's owner (as judged by the effective uid of the process)
-or a privileged user, can change item permissions.
-If invoked on a non-dir, or a dir without recursion, it is processed
-here. If recursion is invoked on a dir, a ntfw funtion is invoked.
-By that, all nested dir access permissons will be set to include x,
-non-dir items will be have their new permissions set when
-they are 'reported'. Finally, here, dirs will be set to their proper
-permissions (bottom-up order) at the end of the process
+Only an item's owner (as judged by the effective uid of the process) or a
+privileged user, can change item permissions. Symlinks are never changed, nor
+(unless explicitly included in the process) are link targets.
+If invoked on a non-dir, or a dir without recursion, it is processed here.
+If recursion is invoked on a dir, a treewalk function is invoked. By that, all
+nested dir access permissons will be set to include x, non-dir items will be
+have their new permissions set when they are 'reported'. Finally, here, dirs
+will be set to their proper permissions (bottom-up order) at the end of the process.
 Recursive chmod works on the host filesystem only.
-Links are not affected, their target is processed. (OK ??)
 Error messages expect BGL open
-@param path localised string, absolute path of item to process
-@param mode permissions string, symbolic or octal
+@param localpath data including localised string, absolute path of item to process
+@param optype enumerator for the type of operation - set add or remove
+@param opmode permissions flags to be set, added or removed
 @param recurse code for recurse type (ignored if @a path is not a dir)
 @return TRUE if operation succeeds
 */
-gboolean e2_task_backend_chmod (VPATH *localpath, gchar *mode, E2_RecurseType recurse)
+gboolean e2_task_backend_chmod (VPATH *localpath, E2_ChmodType optype,
+	mode_t opmode, E2_RecurseType recurse)
 {
 	E2_ChmodData data;
 	struct stat statbuf;
@@ -1657,7 +1620,7 @@ gboolean e2_task_backend_chmod (VPATH *localpath, gchar *mode, E2_RecurseType re
 #endif
 			data.continued_after_problem = FALSE;
 			data.scope = recurse;
-			_e2_task_backend_parse_mode (mode, &data);
+			_e2_task_backend_parse_mode (optype, opmode, &data);
 			//no processed dirs yet
 			data.dirdata = NULL;
 
@@ -1682,7 +1645,7 @@ gboolean e2_task_backend_chmod (VPATH *localpath, gchar *mode, E2_RecurseType re
 					E2_ERR_CLEAR	//should really not do this, but use E2_ERR_NONE arg after any error happens
 					E2_DirEnt *dirfix = member->data;
 #ifdef E2_VFS
-					fixdata.localpath = dirfix->path;
+					fixdata.path = dirfix->path;
 					if (!_e2_task_backend_chmod1 (&fixdata, dirfix->mode E2_ERR_PTR()))
 #else
 					if (!_e2_task_backend_chmod1 (dirfix->path, dirfix->mode E2_ERR_PTR()))
@@ -1723,8 +1686,8 @@ gboolean e2_task_backend_chmod (VPATH *localpath, gchar *mode, E2_RecurseType re
 		{
 			return FALSE;	//link permissions can't be changed
 		}
-		//decode the mode string
-		_e2_task_backend_parse_mode (mode, &data);
+		//decode the mode masks
+		_e2_task_backend_parse_mode (optype, opmode, &data);
 		//get new mode
 		mode_t new_mode = ((statbuf.st_mode & data.clearmask) | data.setmask) & ALLPERMS;
 		if (_e2_task_backend_chmod1 (localpath, new_mode E2_ERR_PTR()))
@@ -1734,7 +1697,7 @@ gboolean e2_task_backend_chmod (VPATH *localpath, gchar *mode, E2_RecurseType re
 	return FALSE; //warning prevention only
 }
 /**
-@brief change ownership of item @a path to @a owner_id and @a group_id
+@brief change ownership of item @a localpath to @a owner_id and @a group_id
 
 If invoked on a non-dir, or a dir without recursion, it is processed
 here. If recursion is invoked on a dir, a ntfw funtion is invoked.
@@ -1747,7 +1710,7 @@ Links are not affected, their target is processed. (CHECKME ok??)
 Only an item's owner (as judged by the effective uid of the process)
 or a privileged user, can change item permissions.
 Assumes BGL is open
-@param path localised string, absolute path of item to process
+@param localpath data including localised string, absolute path of item to process
 @param owner_id uid to be applied
 @param group_id gid to be applied
 @param recurse_dirs TRUE to recurse (ignored if @a path is not a dir)
@@ -1800,7 +1763,7 @@ gboolean e2_task_backend_chown (VPATH *localpath, uid_t owner_id,
 				{
 					E2_DirEnt *dirfix = member->data;
 #ifdef E2_VFS
-					fixdata.localpath = dirfix->path;
+					fixdata.path = dirfix->path;
 					if (!_e2_task_backend_chown1 (&fixdata, &statbuf, owner_id, group_id E2_ERR_PTR()))
 #else
 					if (!_e2_task_backend_chown1 (dirfix->path, &statbuf, owner_id, group_id E2_ERR_PTR()))
@@ -1858,7 +1821,7 @@ If no default action exists, the user is invited to create one
 No thread-protection is done here or downstream, the caller must handle that
 if needed.
 
-@param path localised string, absolute or relative path of the item to be 'opened'
+@param localpath data including localised string, absolute or relative path of the item to be 'opened'
 @param ask TRUE to invite user to choose some actions
 
 @return TRUE if the operation succeeded
@@ -1877,7 +1840,7 @@ gboolean e2_task_backend_open (VPATH *localpath, gboolean ask)
 #endif
 
 	usepath = VPSTR (localpath);
-	if (g_path_is_absolute (usepath) || g_str_equal (usepath, ".."))
+	if (g_path_is_absolute (usepath) || !strcmp (usepath, ".."))
 		addpath = FALSE;
 	else
 	{
@@ -1912,7 +1875,7 @@ don't need real path yet
 */
 
 #ifdef E2_VFS
-	tdata.localpath = usepath;
+	tdata.path = usepath;
 	tdata.spacedata = localpath->spacedata;
 	//checking a virtual dir in the process of being mounted may cause bad delay
 	//so we fake a rough guess at what is intended ...
@@ -1926,81 +1889,96 @@ don't need real path yet
 #endif
 	{
 		exec = FALSE;	//no special treatment for dirs
-		freeme = ext = g_strconcat (".", _("<directory>"), NULL);//exec stays FALSE
+		freeme = ext = g_strconcat (".", _("<directory>"), NULL);
 	}
 	else
 	{
-//	printd (DEBUG, "check X permission of %s", path);
+//	printd (DEBUG, "check '%s' for executable", path);
+		exec = e2_fs_is_exec2 (
 #ifdef E2_VFS
-		exec = !e2_fs_access (&tdata, X_OK E2_ERR_NONE());
+			&tdata E2_ERR_NONE());
 #else
-		exec = !e2_fs_access (usepath, X_OK);
+			usepath E2_ERR_NONE());
 #endif
 		freeme = NULL; //not a constructed extension, no free needed
 		base = g_path_get_basename (utf);
-		ext = strchr (base, '.');	//assumes '.' is ascii
+/*		//FIXME generalise detection of hidden items
+		if (!e2_fs_is_hidden (&tdata, E2_ERR_NONE()))
+			ext = base; //look for extension from 1st byte
+//USELESS else if (*base != '.')
+//			ext = base;
+//UNREAL else if (*base == '\0')
+//			ext = base;
+		else // *base == '.', a *NIX-kind of hidden file
+			ext = base + sizeof (gchar); //look for extension from 2nd byte
+*/
+		ext = (*base == '.') ? base + sizeof (gchar) : base;
+		ext = strchr (ext, '.');
 	}
 
 	if (ext == NULL //no extension (".<none>" can apply to one type only, probably executables)
-		|| ext == base) //it's a hidden file
+		|| *(ext + sizeof (gchar)) == '\0')
 	{
-#ifdef E2_VFS
-		if (!exec && e2_fs_is_text (&tdata E2_ERR_NONE()))
-#else
-		if (!exec && e2_fs_is_text (usepath))
-#endif
+		if (exec)	//executable non-dir
 		{
-#ifdef E2_VFS
-			retval = e2_task_backend_view (&tdata);
-#else
-			retval = e2_task_backend_view (usepath);
-#endif
-		}
-		else if (!exec)	//&& non-text is implied
-		{
-			if (ask)
-			{
-#ifdef E2_VFS
-				res = e2_filetype_dialog_create (&tdata, FALSE, FALSE, FALSE); //enter command or view
-#else
-				res = e2_filetype_dialog_create (usepath, FALSE, FALSE, FALSE); //enter command or view
-#endif
-				retval = (res == E2_TYPE_HANDLED);
-			}
-			else
-				retval = FALSE;
-		}
-		else //executable non-dir
-		{
-/*
-			if (g_str_equal (dir, ".")
+/*			if (!strcmp (dir, ".")
 				&& ! g_str_has_prefix (path, E2_COMMAND_PREFIX))
 					command = g_strdup_printf ("%s%s", E2_COMMAND_PREFIX, utf);
 			else
 				command = path;
+			res = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT, ?
 #ifdef E2_COMMANDQ
-			res = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT, TRUE);
-#else
-			res = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT);
+			, TRUE
 #endif
+			);
 			if (command != path)
 				g_free (command);
 */
 			//FIXME better aesthetic, shorten the command string where appropriate
-			res = e2_command_run (utf, E2_COMMAND_RANGE_DEFAULT
+			//CHECKME quote the path ?
+			res = e2_command_run (utf, E2_COMMAND_RANGE_DEFAULT, app.main_window
 #ifdef E2_COMMANDQ
 			, TRUE
 #endif
 			);
 			retval = (res == 0);
 		}
+		else if (e2_fs_is_text (
+#ifdef E2_VFS
+			&tdata E2_ERR_NONE()))
+#else
+			usepath))
+#endif
+		{
+			retval = e2_task_backend_view (
+#ifdef E2_VFS
+			&tdata);
+#else
+			usepath);
+#endif
+		}
+		else //non-exec && non-text
+		{
+			if (ask)
+			{
+				res = e2_filetype_dialog_create (
+#ifdef E2_VFS
+				 &tdata, FALSE, FALSE, FALSE); //enter command or view
+#else
+				 usepath, FALSE, FALSE, FALSE); //enter command or view
+#endif
+				retval = (res == E2_TYPE_HANDLED);
+			}
+			else
+				retval = FALSE;
+		}
 	}
 	else // the file has an extension of some sort
 	{
 		gchar *action, *real_action;
 		do
 		{
-			ext++;	//NCHR(ext); //skip the . prefix
+			ext += sizeof (gchar);	//skip the . prefix
 			action = e2_filetype_get_default_action (ext);
 			if (action != NULL)
 			{
@@ -2037,11 +2015,15 @@ don't need real path yet
 					}
 				}
 				//not exec, or not ask, or exec+ask but user chose default action
-				//always ascii @, don't need g_utf8_strchr()
-				command = e2_utils_replace_name (real_action, utf);
-				if (command == NULL)	//no replaced macro in real_action
-					command = g_strdup_printf ("%s \"%s\"", real_action, utf);
-				res = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT
+				command = e2_utils_replace_name_macros (real_action, utf);
+				if (command == real_action)	//no replaced macro in real_action
+				{
+//tag E2_BADQUOTES
+					gchar *qp = e2_utils_quote_string (utf);
+					command = g_strconcat (real_action, " ", qp, NULL);
+					g_free (qp);
+				}
+				res = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT, app.main_window
 #ifdef E2_COMMANDQ
 				, TRUE
 #endif
@@ -2058,12 +2040,12 @@ don't need real path yet
 			if (exec)	//just execute it
 			{
 /*				dir = g_path_get_dirname (path);
-				if (g_str_equal (dir, ".")
+				if (!strcmp (dir, ".")
 					&& ! g_str_has_prefix (path, E2_COMMAND_PREFIX))
 						command = g_strdup_printf ("%s%s", E2_COMMAND_PREFIX, utf);
 				else
 					command = path;
-				res = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT);
+				res = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT, ?
 #ifdef E2_COMMANDQ
 				, TRUE
 #endif
@@ -2072,7 +2054,7 @@ don't need real path yet
 				if (command != path)
 					g_free (command);
 */
-				res = e2_command_run (utf, E2_COMMAND_RANGE_DEFAULT
+				res = e2_command_run (utf, E2_COMMAND_RANGE_DEFAULT, app.main_window
 #ifdef E2_COMMANDQ
 				, TRUE
 #endif
@@ -2095,7 +2077,7 @@ don't need real path yet
 
 	if (addpath)
 		g_free (usepath);
-	F_FREE (utf);
+	F_FREE (utf, usepath);
 	if (freeme != NULL)
 		g_free (freeme);
 	if (base != NULL)
@@ -2117,15 +2099,21 @@ gboolean e2_task_backend_view (VPATH *localpath)
 		if (*viewer != '\0')
 		{
 			gchar *utf = F_FILENAME_FROM_LOCALE (VPSTR (localpath));
-			gchar *command = e2_utils_replace_name (viewer, utf);
-			if (command == NULL)
-				command = g_strdup_printf ("%s \"%s\"", viewer, utf);
-			gint res = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT
+			gchar *command = e2_utils_replace_name_macros (viewer, utf);
+			if (command == viewer)	//no replacement of macro in viewer
+			{
+//tag E2_BADQUOTES
+				//CHECKME may be bad if a file.view action is supplied
+				gchar *qp = e2_utils_quote_string (utf);
+				command = g_strconcat (viewer, " ", qp, NULL);
+				g_free (qp);
+			}
+			gint res = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT, curr_view->treeview
 #ifdef E2_COMMANDQ
 			, TRUE
 #endif
 			);
-			F_FREE (utf);
+			F_FREE (utf, VPSTR (localpath));
 			g_free (command);
 			return (res == 0);
 		}
