@@ -1,6 +1,6 @@
-/* $Id: e2_toolbar.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_toolbar.h 2053 2010-02-21 04:29:24Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2010 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -29,15 +29,14 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #define FOLDBARS
 
 typedef enum
-{	//these are ordered for decreasing visual impact when changes
-	//are made
-	//if order is changed, correspondingly change order of array rows
-	//in e2_toolbar_options_register()
+{	//these are ordered for decreasing visual impact when changes are made
+	//if order is changed, correspondingly change order of array rows in
+	//e2_toolbar_options_register()
 	E2_BAR_TASK,
 	E2_BAR_PANE1,
 	E2_BAR_PANE2,
 	E2_BAR_COMMAND,
-	E2_BAR_COUNT	//this is the no. of bars, not a 'type'
+	E2_BAR_COUNT	//the no. of bars
 } E2_BarType;
 
 typedef struct _E2_ToolbarRuntime
@@ -48,9 +47,10 @@ typedef struct _E2_ToolbarRuntime
 	GtkWidget *toolbar_container;
 	GtkWidget *toolbar_container_box;
 #ifdef FOLDBARS
-	GtkWidget *toolbarvbox;	//hbox holding just a dir-line
+	GtkWidget *toolbar_foldbox;	//vbox holding toolbar_container
+								// & sometimes a separate commandline tool
 #endif
-	gboolean blocked;  //TRUE when bar's context menu is being created
+	volatile gint blocked;  //non-0 when bar's context menu is being created
 	gboolean hidden;
 	E2_OptionSet *set;
 	E2_OptionSet *show;
@@ -139,6 +139,7 @@ typedef enum
 
 gchar *toggles_array [E2_TOGGLE_COUNT];
 
+void e2_toolbar_rebadge (E2_ToolbarRuntime *rt);
 void e2_toolbar_initialise_space_handler (E2_ToolbarRuntime *rt);
 void e2_toolbar_set_menu_position (GtkMenu *menu, gint *x, gint *y,
 	gboolean *push_in, GtkWidget *button);
@@ -148,12 +149,13 @@ gboolean e2_toolbar_toggle_button_get_state (gchar *action_name);
 void e2_toolbar_toggle_button_set_state (gchar *action_name, gboolean state);
 void e2_toolbar_toggle_buttons_set_destroyed (E2_ToolbarRuntime *rt);
 void e2_toolbar_toggle_filter_button (ViewInfo *view);
-void e2_toolbar_initialise (E2_BarType barnum);
+//void e2_toolbar_initialise (E2_BarType barnum);
 void e2_toolbar_create (E2_ToolbarRuntime *rt);
 void e2_toolbar_recreate (E2_ToolbarRuntime *rt);
 void e2_toolbar_recreate_all (void);
 void e2_toolbar_destroy (E2_ToolbarRuntime *rt);
 void e2_toolbar_data_create (void);
+void e2_toolbar_data_clean (void);
 void e2_toolbar_actions_register (void);
 void e2_toolbar_options_register (E2_BarType barnum);
 void e2_toolbar_toolbaroptions_register (void);
