@@ -1,6 +1,6 @@
-/* $Id: emelfm2.h 893 2008-05-20 09:42:51Z tpgww $
+/* $Id: emelfm2.h 1978 2009-10-12 07:32:01Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -28,12 +28,14 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "build.h"
 
 #include <sys/types.h>
+#include <inttypes.h>
 //needed for various statbuf declarations
 #include <sys/stat.h>
 #include <limits.h>
 //some general things ...
 #include <stdio.h>
 #include <stdlib.h>
+#include <pthread.h>
 
 //#define GTK_DISABLE_DEPRECATED see Makefile
 #include <gtk/gtk.h>
@@ -53,49 +55,113 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 # if defined(__WORDSIZE) && __WORDSIZE == 64
 #  define PRIu64 "lu"
 # else
-//guess
+//guess it's a 32-bit platform
 #  define PRIu64 "llu"
 # endif
 #endif
 
 //include code valid only for gtk versions > 2.6, according to the
 //version in use when compiling
-//#define E2_CURRENTGTK_VERSION see makefile and build.h
+//#define E2_CURRENTLIBS see Makefile and build.h
 //mechanism for using glib/gtk facilities > min standard (2.6.0)
-#ifdef E2_CURRENTGTK_VERSION
-#if GLIB_CHECK_VERSION (2,8,0)
-#define USE_GLIB2_8
-#endif
-#if GLIB_CHECK_VERSION (2,10,0)
-#define USE_GLIB2_10
-#endif
-#if GLIB_CHECK_VERSION (2,12,0)
-#define USE_GLIB2_12
-#endif
-#if GLIB_CHECK_VERSION (2,14,0)
-#define USE_GLIB2_14
-#endif
-#if GTK_CHECK_VERSION (2,8,0)
-#define USE_GTK2_8
-#endif
-#if GTK_CHECK_VERSION (2,10,0)
-#define USE_GTK2_10
-#endif
-#if GTK_CHECK_VERSION (2,12,0)
-#define USE_GTK2_12
+#ifdef E2_CURRENTLIBS
+# if GLIB_CHECK_VERSION (2,8,0)
+#  define USE_GLIB2_8
+# endif
+# if GLIB_CHECK_VERSION (2,10,0)
+#  define USE_GLIB2_10
+# endif
+# if GLIB_CHECK_VERSION (2,12,0)
+#  define USE_GLIB2_12
+# endif
+# if GLIB_CHECK_VERSION (2,14,0)
+#  define USE_GLIB2_14
+# endif
+# if GLIB_CHECK_VERSION (2,16,0)
+#  define USE_GLIB2_16
+# endif
+# if GLIB_CHECK_VERSION (2,18,0)
+#  define USE_GLIB2_18
+# endif
+# if GLIB_CHECK_VERSION (2,20,0)
+#  define USE_GLIB2_20
+# endif
+# if GLIB_CHECK_VERSION (2,22,0)
+#  define USE_GLIB2_22
+# endif
+
+# if GTK_CHECK_VERSION (2,8,0)
+#  define USE_GTK2_8
+# endif
+# if GTK_CHECK_VERSION (2,10,0)
+#  define USE_GTK2_10
+# endif
+# if GTK_CHECK_VERSION (2,12,0)
+#  define USE_GTK2_12
+# endif
+# if GTK_CHECK_VERSION (2,14,0)
+#  define USE_GTK2_14
+# endif
+# if GTK_CHECK_VERSION (2,16,0)
+#  define USE_GTK2_16
+# endif
+# if GTK_CHECK_VERSION (2,18,0)
+#  define USE_GTK2_18
+# endif
+# if GTK_CHECK_VERSION (2,20,0)
+#  define USE_GTK2_20
 # endif
-#if GTK_CHECK_VERSION (2,14,0)
-#define USE_GTK2_14
-#endif
 #endif
+//some things which didn't make it into 2.14 have been re-badged as USE_GTK2_99 CHECK LATER
 
 #ifdef USE_GTK2_12
-//this token governs use of gtk 2.12 tooltips
+//this tag governs use of gtk 2.12 tooltips
 #define USE_GTK2_12TIPS
-//this token governs use of gtk 2.12 notebook-tab DnD
+//this tag governs use of gtk 2.12 notebook-tab DnD
 #define USE_GTK2_12DND
 #endif
 
+//longest item name (bytes, NOT chars) is a *NIX define
+//#define NAME_MAX 255
+//length of various buffers
+#define E2_MAX_LEN 1024
+//msec interval between pane-directory polls
+//NOTE that the actual refresh interval may be a multiple (up to 3) times this
+//to avoid too many refreshes during e.g. archive creation
+#define E2_FILESCHECK_INTERVAL 1000
+#define E2_FILESCHECK_INTERVAL_S 1
+//msec interval between config-directory polls
+//this is only used if E2_FAM not defined
+#define E2_CONFIGCHECK_INTERVAL 5000
+#define E2_CONFIGCHECK_INTERVAL_S 5
+//msec interval between status line updates
+#define E2_STATUSREPORT_INTERVAL 1500
+
+//allowed msec interval between clicks to be treated as a double
+//if gtk's value is smaller, then it is set to this instead
+#define E2_CLICKINTERVAL 330
+
+//padding for various widgets
+#define E2_PADDING_LARGE 8
+#define E2_PADDING 5
+#define E2_PADDING_SMALL 3
+#define E2_PADDING_XSMALL 2
+
+#define E2_COMMAND_PREFIX "."G_DIR_SEPARATOR_S
+#define PLUGIN action_labels[16]
+// in these, "home" is a pointer determined at runtime
+#define E2_CONFIG_DIR home,"."BINNAME
+//#define E2_TRASH_DIR E2_CONFIG_DIR, "Trash"
+//#define SYSTEM_TRASH_DIR home, ".Trash"
+//#define E2_CONFIG_FILE "config"  no good for gettext ?? do not translate
+
+//default build with horizontal panes - see Makefile
+//#define E2_PANES_HORIZONTAL
+//default pointsize and file-extension for custom toolbar-icon files
+#define E2ICONTB "_48.png"
+//default pointsize and file-extension for custom dialog-button-icon files
+#define E2ICOND "_32.png"
+
 //tag for development of on-demand status-bar updates
 //NOTE need to check all BGL effects
 //#define E2_STATUS_DEMAND
@@ -149,44 +215,27 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #define E2_ALTLEFTMOUSE
 //#endif
 
-//bytes, NOT characters this is a linux define
-//#define NAME_MAX 255
-//length of various buffers
-#define E2_MAX_LEN 1024
-//msec interval between pane-directory polls
-//NOTE that the actual refresh interval may be a multiple (up to 3) times this
-//to avoid too many refreshes during e.g. archive creation
-#define E2_FILESCHECK_INTERVAL 1000
-#define E2_FILESCHECK_INTERVAL_S 1
-//msec interval between config-directory polls
-//this is only used if E2_FAM not defined
-#define E2_CONFIGCHECK_INTERVAL 5000
-#define E2_CONFIGCHECK_INTERVAL_S 5
-//msec interval between status line updates
-#define E2_STATUSREPORT_INTERVAL 1500
+//include code for user-defined pointer buttons (and in future, gestures)
+//#define E2_MOUSECUSTOM see Makefile and build.h
+#ifdef E2_MOUSECUSTOM
+//enable button.fake action NOT TESTED
+//# define WITH_BUTTONFAKE
+//suport pointer gestures
+#define E2_PTRGESTURES
+#endif
 
-//allowed msec interval between clicks to be treated as a double
-//if gtk's value is smaller, then it is set to this instead
-#define E2_CLICKINTERVAL 330
+//enable key.fake action LITTLE TESTED
+//#define WITH_KEYFAKE
 
-//padding for various widgets
-#define E2_PADDING_LARGE 8
-#define E2_PADDING 5
-#define E2_PADDING_SMALL 3
-#define E2_PADDING_XSMALL 2
+//support addition of categories to bindings tree option data
+//#define E2_TRANSIENTBINDINGS see Makefile and build.h for this
+//support addition of treestore iters from strings like 1.2.3, esp for transient bindings
+//#define E2_TREEINCREMENT
 
-#define E2_COMMAND_PREFIX "."G_DIR_SEPARATOR_S
-#define PLUGIN action_labels[14]
-// in these, "home" is a pointer determined at runtime
-#define E2_CONFIG_DIR home,"."BINNAME
-//#define E2_TRASH_DIR E2_CONFIG_DIR, "Trash"
-//#define SYSTEM_TRASH_DIR home, ".Trash"
-//#define E2_CONFIG_FILE "config"  no good for gettext ?? do not translate
-
-//default build with horizontal panes - see Makefile
-//#define E2_PANES_HORIZONTAL
-//default icon pointsize - see Makefile
-//#define E2IP "24"
+//enable action to brute-force-terminate a selected running command or action
+//#define WITH_TASKABORT
+//enable file.untrash action LITTLE TESTED
+//#define WITH_UNTRASH
 
 //enable config/button/menu image-cacheing
 #define E2_IMAGECACHE
@@ -197,28 +246,21 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #ifdef E2_RAINBOW
 //no. of color values in each allocated block of memory
 #define ATOMSPERCHUNK 10
+# ifndef USE_GLIB2_10
 typedef struct _E2_ColorData
 {
 	GMemChunk *chunk;	//data structure for a block of ATOMSPERCHUNK GdkColor structs
 	GdkColor **pointers;	//allocated space for ATOMSPERCHUNK pointers
 } E2_ColorData;
+# endif
 #endif
 
 //maybe useful for multi-processor optimisation ?
 //#define VOLATILE volatile
 #define VOLATILE
 
-//support key.alias action
-//#define E2_KEYALIAS
-//support addition of items to tree options (eg keybindings)
-//#define E2_TRANSIENTKEYS see Makefile for this
-#ifdef E2_TRANSIENTKEYS
-# ifndef E2_KEYALIAS
-#  define E2_KEYALIAS
-# endif
-#endif
-//support addition of treestore iters from strings like 1.2.3
-//#define E2_TREEINCREMENT
+//include code for handling [%]%e [%]%E macros
+#define E2_BADQUOTES
 
 //support mountpoint-related capabilities
 #define E2_FS_MOUNTABLE
@@ -228,6 +270,18 @@ typedef struct _E2_ColorData
 #define E2_TABS_DETACH
 #endif
 
+#define E2_OVERRIDE_MASK (GDK_SHIFT_MASK|GDK_CONTROL_MASK|GDK_MOD1_MASK)
+#ifdef USE_GTK2_10
+# define E2_MODIFIER_MASK \
+	(GDK_SHIFT_MASK|GDK_CONTROL_MASK|GDK_MOD1_MASK\
+	|GDK_MOD3_MASK|GDK_MOD4_MASK|GDK_MOD5_MASK\
+	|GDK_SUPER_MASK|GDK_HYPER_MASK|GDK_META_MASK)
+#else
+# define E2_MODIFIER_MASK \
+	(GDK_SHIFT_MASK|GDK_CONTROL_MASK|GDK_MOD1_MASK\
+	|GDK_MOD3_MASK|GDK_MOD4_MASK|GDK_MOD5_MASK)
+#endif
+
 //use replacement command-running (see Makefile)
 //NOT FINISHED
 //#ifndef E2_NEW_COMMAND
@@ -249,6 +303,9 @@ typedef struct _E2_ColorData
 # endif
 #endif
 
+//tailor UI things for very small screen
+//#define E2_SMALLSCREEN
+
 //support vfs for file/dir processing
 //NOT FINISHED
 //needs cacheing of view data, backend data structs, history, marks
@@ -262,6 +319,13 @@ typedef struct _E2_ColorData
 # endif
 #endif
 
+//use libmagic instead of file(1)
+#define E2_MAGIC
+#ifdef E2_MAGIC
+//for use with dlopen()
+# define MAGIC_LIB_NAME "libmagic.so.1"
+#endif
+
 //enable filelist liststore copying instead of getting fresh data from source
 #ifdef E2_VFS
 #define STORECOPY
@@ -283,11 +347,12 @@ typedef struct _E2_ColorData
 # define E2_ERR_MSGL(a) a, __sys_err->message
 # define E2_ERR_IS(num) (__sys_err != NULL && __sys_err->code == num)
 # define E2_ERR_ISNOT(num) (__sys_err == NULL || __sys_err->code != num)
+# define E2_ERR_CLEAR if (__sys_err != NULL) { g_error_free (__sys_err); __sys_err = NULL; }
 //these are for use in a func with GError ** argument
 # define E2_ERR_PIS(num) (__sys_err != NULL && (*__sys_err)->code == num)
 # define E2_ERR_PISNOT(num) (__sys_err == NULL || (*__sys_err)->code != num)
 # define E2_ERR_BACKUP(a) GError *a = NULL;if (__sys_err == NULL) __sys_err = &a;
-# define E2_ERR_CLEAR if (__sys_err != NULL) { g_error_free (__sys_err); __sys_err = NULL; }
+# define E2_ERR_PCLEAR if (__sys_err != NULL && *__sys_err != NULL) { g_error_free (*__sys_err); *__sys_err = NULL; }
 #else
 # define E2_ERR_NAME
 # define E2_ERR_INIT
@@ -301,9 +366,10 @@ typedef struct _E2_ColorData
 # define E2_ERR_MSGL(a)
 # define E2_ERR_IS(num) (errno==num)
 # define E2_ERR_ISNOT(num) (errno!=num)
+# define E2_ERR_CLEAR
 # define E2_ERR_PIS(num) (errno==num)
 # define E2_ERR_PISNOT(num) (errno!=num)
-# define E2_ERR_CLEAR
+# define E2_ERR_PCLEAR
 # define E2_ERR_BACKUP(a)
 #endif
 
@@ -318,6 +384,20 @@ typedef struct _E2_ColorData
 # define VPSTR(p) (gchar*)p
 #endif
 
+#ifdef E2_VFS
+# define CURRDIR curr_view->dir.path
+# define OTHRDIR other_view->dir.path
+# define VIEWDIR view->dir.path
+#else
+# define CURRDIR curr_view->dir
+# define OTHRDIR other_view->dir
+# define VIEWDIR view->dir
+#endif
+
+#define ActivateFunc void(*)(GtkEntry*,gpointer)
+#define ResponseFunc void(*)(GtkDialog*,gint,gpointer)
+#define HookFunc gboolean(*)(gpointer,gpointer)
+
 //type of "dir" in a pane
 typedef enum
 {
@@ -336,6 +416,7 @@ typedef enum
 typedef enum { E2PLACE_UNMOUNTED, E2PLACE_MOUNTING, E2PLACE_MOUNTED } PlaceReady;
 typedef struct _E2_PlaceInfo
 {
+	struct _E2_PlaceInfo* parentplace;	//for cd .. past root, checking availability of full path etc
 	gchar *priv_name; /* for mounted dir NULL CHECKME synth dir
 	   for remote site escaped utf8, no trailer, login-ready URI before the filepath proper with p/w etc
 	   for archive localised absolute path, with trailer, of temp dir used for unpacking whole or part
@@ -348,7 +429,8 @@ typedef struct _E2_PlaceInfo
 	gchar *plain_pw;	//edit-ready password, utf8, or NULL if no P/W
 	gchar *alias;	//user-friendly short-form utf8 for menu label etc, NULL or "" if not in use
 	gchar *workplace;	//path of local fs dir, absolute localised no trailer
-	E2_FSType dirtype;	//flags for type of namespace this is
+	E2_FSType dirtype;	//flags for broad type of namespace this is
+	//CHECKME data for specific type e.g. scheme
 //UNUSED	E2_FSHandler handler;	//enumerator of lib/plugin which handles fs-operations for this space
 									//CHECK could be op-specific
 //UNUSED	E2_VFSOpFlags cando;	//flags for the sorts of operations that can be performed
@@ -362,18 +444,15 @@ typedef struct _E2_PlaceInfo
 	GList *pane2_history; //pane2 dir_history, but specific to this namespace
 	GList *pane1_visited; //pane1 visited, opendirs, but specific to this namespace
 	GList *pane2_visited; //pane2 visited, opendirs, but specific to this namespace
-
-	struct _E2_PlaceInfo* parentplace;	//for cd .. past root
 	//etc
 } PlaceInfo;
 
 //data for vfs operations uning path-argument(s)
 typedef struct _E2_Vpath
 {
-	const gchar *localpath; /*first, so that vpath* == gchar **
+	const gchar *path; /*first in the struct, so that vpath* == gchar **
 						typically an absolute path string
-						encoding may actually be utf-8 if a handler lib
-						requires that */
+						encoding may be localised or UTF-8 */
 	PlaceInfo *spacedata;
 //	GError *err; //replacement for error args in fs functions
 } vpath;
@@ -441,36 +520,41 @@ typedef struct _E2_Vpath
        _res; })
 #endif
 
-//do not implement slices everywhere until session-end cleanups are coded
-//#ifdef USE_GLIB2_10
-#if 0
-# define ALLOCATE0(type) (type *) g_slice_alloc0 (sizeof (type));
-# define ALLOCATE(type) (type *) g_slice_alloc (sizeof (type));
-# define DEALLOCATE(type,ptr) g_slice_free1 (sizeof (type), ptr);
+//do not implement slices everywhere until comprehensive session-end cleanups are done
+#if 0 //def USE_GLIB2_10
+# define ALLOCATE(type) (type*)g_slice_alloc(sizeof(type));
+# define ALLOCATE0(type) (type*) g_slice_alloc0(sizeof(type));
+# define DEALLOCATE(type,ptr) g_slice_free1(sizeof(type),ptr);
 #else
 # ifdef USE_GLIB2_8
-#  define ALLOCATE0(type) (type *) g_try_malloc0 (sizeof (type));
+#  define ALLOCATE(type) (type*)g_try_malloc(sizeof(type));
+#  define ALLOCATE0(type) (type*)g_try_malloc0(sizeof(type));
+#  define DEALLOCATE(type,ptr) g_free(ptr);
 # else
-#  define ALLOCATE0(type) (type *) calloc (1, sizeof (type));
+#  define ALLOCATE(type) (type*)malloc(sizeof(type));
+#  define ALLOCATE0(type) (type*)calloc(1,sizeof(type));
+#  define DEALLOCATE(type,ptr) free(ptr);
 # endif
-# define ALLOCATE(type) (type *) g_try_malloc (sizeof (type));
-# define DEALLOCATE(type,ptr) g_free (ptr);
 #endif
 //for ALLOCATE's that are too small for a slice
 #ifdef USE_GLIB2_8
-# define MALLOCATE0(type) (type *) g_try_malloc0 (sizeof (type));
+# define MALLOCATE(type) (type*)g_try_malloc(sizeof(type));
+# define MALLOCATE0(type) (type*)g_try_malloc0(sizeof(type));
+# define DEMALLOCATE(type,ptr) g_free(ptr);
 #else
-# define MALLOCATE0(type) (type *) calloc (1, sizeof (type));
+# define MALLOCATE(type) (type*)malloc(sizeof(type));
+# define MALLOCATE0(type) (type*)calloc(1,sizeof(type));
+# define DEMALLOCATE(type,ptr) free(ptr);
 #endif
-#define MALLOCATE(type) (type *) g_try_malloc (sizeof (type));
-#define DEMALLOCATE(type,ptr) g_free (ptr);
 
 #ifdef USE_GLIB2_8
-# define NEW(type,n) g_try_new (type, n)
-# define NEW0(type,n) g_try_new0 (type, n)
+# define NEW(type,n) g_try_new(type,n)
+# define NEW0(type,n) g_try_new0(type,n)
+# define FREENEW g_free
 #else
-# define NEW(type,n) (type*)malloc (sizeof(type)*n)
-# define NEW0(type,n) (type*)calloc (n, sizeof(type))
+# define NEW(type,n) (type*)malloc(sizeof(type)*n)
+# define NEW0(type,n) (type*)calloc(n,sizeof(type))
+# define FREENEW free
 #endif
 
 //survive or exit gracefully when allocation fails
@@ -486,6 +570,7 @@ typedef struct _E2_Vpath
 
 //index enumerator for cancellable timers
 //CHECKME some other repeating, non-trivial-delay, timers are non-static
+//NOTE also need to kill any [un]mount timers for devices(s) in hal/devkit
 enum
 {
 	FAMPOLL_T,	//polling for reports from file alteration monitor
@@ -508,6 +593,20 @@ typedef enum
 } E2_FAMonitor;
 #endif
 
+//define to use local polling instead of full-blown main loop for blocking dialogs
+#define WAIT_POLL
+typedef struct _E2_MainLoop
+{
+	pthread_t threadID;	//thread which started this loop
+#ifdef WAIT_POLL
+	gboolean maincontext; //TRUE when wait was initiated from main thread, in
+						// which case the watching is done by separate, dedicated, thread
+	gboolean finished;	//TRUE when the wait is ready to be ended
+#else
+	GMainLoop *loop;
+#endif
+} E2_MainLoop;
+
 typedef struct _E2_MainData
 {
 	GtkWidget *main_window;
@@ -515,23 +614,24 @@ typedef struct _E2_MainData
 	GtkWidget *hbox_main;
 	GtkWidget *vbox_main;
 	GtkWidget *outbook; //notebook container for tabbed output-panes
+//#ifdef E2_POLKIT
+//	GtkWidget *status_bar_label1;
+//#endif
 	GtkWidget *status_bar_label2;
 	//this is a hbox, at the right of the status bar, for general usage
 	// needs to be shown when used
 	GtkWidget *status_bar_box3;
 	E2_WindowRuntime window;
-	E2_PaneRuntime pane1;
-	E2_PaneRuntime pane2;
-	ViewInfo pane1_view;
-	ViewInfo pane2_view;
+	E2_PaneRuntime pane1;	//includes ViewInfo
+	E2_PaneRuntime pane2;	//ditto
 	E2_ToolbarRuntime toolbar;
 	E2_ToolbarRuntime commandbar;
 	E2_AliasRuntime aliases;
-	E2_OutputRuntime output;	//non-tab-specific data for output pane
-	E2_OutputTabRuntime tab;	//tab-specific data for output pane
-	GList *tabslist;	//list of E2_OutputTabRuntime's for each tab
-	gint tabcount;	//no. of tabs (cached)
-	E2_ToolbarData **bars;	//null-terminated array of pointers to toolbar data
+	E2_OutputRuntime output;//non-tab-specific data for output pane
+	E2_OutputTabRuntime tab;//tab-specific data for output pane
+	GList *tabslist;		//list of E2_OutputTabRuntime's for each tab
+	gint tabcount;			//no. of tabs (cached)
+	E2_ToolbarData **bars;	//NULL-terminated array of pointers to toolbar data
 	GList *command_lines;	//list of rt data structs of command-lines
 
 #if defined(E2_FAM) && !defined(E2_FAM_KERNEL)
@@ -540,38 +640,60 @@ typedef struct _E2_MainData
 #endif
 #ifdef E2_FAM
 	E2_FAMonitor monitor_type;
-	gint FAMreq;	//request no used for monitoring config file changes
+	gint FAMreq;			//request no used for monitoring config file changes
 #endif
-	guint timers[MAX_TIMERS];	//glib timer id's
+	guint timers[MAX_TIMERS];//glib timer id's
 	time_t config_mtime;	//last-logged (seconds) timestamp of config file
 #ifndef USE_GTK2_12TIPS
 	GtkTooltips *tooltips;
+	gint gtkversion;		//runtime version, = major * 10000 + minor * 100 + micro
 #endif
+#ifdef E2_VFS
+	GHashTable *vdir_cache; //table of vath's for places used in the session, each with 0x1 data
+#endif
+	GHashTable *dir_history;//table of E2_DirHistoryEntry's for dirs opened in session
+	GHashTable *keysnative;	//table of keycodes for asciifying localised alphabetic
+							//bindings, each index is a pointerised locale-specific
+							//keycode, each corresponding value is a pointerized
+							//lowercase code GDK_a .. GDK_z
+	GHashTable *keyslocal;	//table of strings for localising alphabetic key-bindings,
+							//each index is a lowercase letter string "a" .. "z",
+							//each corresponding value is a UTF-8 string
+							//representing the corresponding localised letter
+							//This table is not often used
 	GList *plugins;	//list of data structs for all loaded plugins, "non-children"
 					//in menu-order (children-submenu-order depends on a list in
 					//corresponding parent)
 	GHashTable *filetypes;
-	GSList *typelist;	//list of string arrays, needed tor cleanup
+	GSList *typelist;		//list of string arrays, needed tor cleanup
 #ifdef E2_IMAGECACHE
-	GHashTable *icons;	//table of E2_Images, for cached icons
+	GHashTable *icons;		//table of E2_Images, for cached icons
 #endif
 #ifdef E2_RAINBOW
 	GHashTable *colors;
 	GList *colorchunks;
 #endif
-	GSList *used_stores;	//for deferred liststore clearing
+	GSList *mainloops;			//local mainloop information in E2_MainLoop's
+	GSList *used_stores;		//for deferred liststore clearing
 	GList *taskhistory;
-#ifndef E2_FILES_UTF8ONLY
-	gboolean utf8_filenames;	//TRUE when filesystem coding is utf-8 or ascii only
-#endif
+//#ifndef E2_FILES_UTF8ONLY
+//	gboolean utf8_filenames;	//TRUE when filesystem coding is utf-8 or ascii only
+//#endif
 	GdkWindowState mainwindow_state;	//flags indicating changes to main_window
 	GtkWidget *context_menu;	//main context-menu when it is popped up
-	gint cfgdlg_width;  //config dialog width (cached)
-	gint cfgdlg_height; //config dialog height (cached)
-	gchar cfgfile_version[20];  //store for config version string eg "0.1.9", used only for upgrades
-	gboolean rebuild_enabled;	//set FALSE to block window recreation
+#ifdef USE_GTK2_18
+	GtkAllocation main_alloc;	//main window dimensions for cache
+#endif
+	GtkAllocation cfg_alloc;	//config dialog dimensions for cache
+	gchar cfgfile_version[20];	//store for config version string eg "0.1.9", used only for upgrades
+	gboolean rebuild_enabled;	//FALSE to block window recreation
+//	gboolean keytrans;			//TRUE for automatic translation next time keybindings are sync'd
+	gboolean reconvert_requested; //TRUE when a change of encoding-status has been logged during a filelist refill
+//#ifdef E2_THUMBNAILS
+//	gboolean libgimp_connected; //flag to ensure one-time initiation of libgimpthumb
+//#endif
 #ifdef E2_STATUS_BLOCK
-	gboolean status_working;	//set TRUE to block reentrant status-checking etc
+	gboolean status_working;	//TRUE to block reentrant status-checking etc
 #endif
 	GHookList hook_pane_focus_changed;
 } E2_MainData;
@@ -593,40 +715,50 @@ pthread_mutex_t history_mutex;
 
 //for testing - define to use gtk's own mutex for BGL
 //#define NATIVE_BGL
-#ifndef NATIVE_BGL
+#ifdef NATIVE_BGL
+# warning "using native gdk mutex"
+	//WAIT_FOR_EVENTS expects BGL closed
+# define CLOSEBGL_IF_OPEN \
+	WAIT_FOR_EVENTS_UNLOCKED
+# define OPENBGL_IF_CLOSED
+#else //ndef NATIVE_BGL
 //local management of gdk mutex, to enable tolerant re-locking by the same thread
-#include <pthread.h>
 //direct mutex-manipulation
-#define gdk_threads_enter e2_main_close_gdklock
-#define gdk_threads_leave e2_main_open_gdklock
 pthread_mutex_t gdklock;	//BGL replacement
 void e2_main_close_gdklock (void);
 void e2_main_open_gdklock (void);
+# ifdef DEBUG_MESSAGES
+#  define gdk_threads_enter e2_main_close_gdklock
+#  define gdk_threads_leave e2_main_open_gdklock
+#else
+#  define gdk_threads_enter() pthread_mutex_lock (&gdklock)
+#  define gdk_threads_leave() pthread_mutex_unlock (&gdklock)
+#endif
+void e2_main_replace_gdklock (gboolean free);
 
 	//not locked now and not already locked - this should never happen
 	//prevent unlock
-#define CLOSEBGL_IF_OPEN \
+# define CLOSEBGL_IF_OPEN \
 	gint _lockres; \
 	_lockres = pthread_mutex_trylock (&gdklock); \
-	printd (DEBUG, "dialog title trylock result %d", _lockres); \
+	printd (DEBUG, "%s BGL trylock result %d", __PRETTY_FUNCTION__, _lockres); \
 	if (!(_lockres == 0 || _lockres == EBUSY)) \
 	{ \
-		printd (WARN, "dialog title close BGL failed"); \
+		printd (WARN, "close BGL failed"); \
 		WAIT_FOR_EVENTS_UNLOCKED \
 		_lockres = 1; \
 	}
 
-#define	OPENBGL_IF_CLOSED \
+# define	OPENBGL_IF_CLOSED \
 	if (_lockres == 0) \
 		pthread_mutex_unlock (&gdklock);
-
-#else //def NATIVE_BGL
-	//WAIT_FOR_EVENTS expects BGL closed
-#define CLOSEBGL_IF_OPEN \
-	WAIT_FOR_EVENTS_UNLOCKED
-#define	OPENBGL_IF_CLOSED
 #endif
+//void e2_main_cleanup_gdklock (gpointer data);
 
+E2_MainLoop *e2_main_loop_new (gboolean mainctx);
+void e2_main_loop_run (E2_MainLoop *loopdata);
+void e2_main_loop_quit (E2_MainLoop *loopdata);
+gboolean e2_main_loop_abort (pthread_t ID);
 gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean doexit);
 gboolean e2_main_user_shutdown (gpointer from, E2_ActionRuntime *art);
 //	__attribute__ ((noreturn));
