@@ -1,6 +1,6 @@
-/* $Id: e2_fs.c 864 2008-05-05 23:35:10Z tpgww $
+/* $Id: e2_fs.c 2026 2009-12-19 23:50:44Z tpgww $
 
-Copyright (C) 2005-2008 tooar <tooar@gmx.net>
+Copyright (C) 2005-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -49,6 +49,10 @@ Not implemented yet.
 #ifndef MNT_LOCAL
 # include <sys/statvfs.h>
 #endif
+#ifdef E2_MAGIC
+# include <magic.h>
+# include <dlfcn.h>
+#endif
 #include "e2_fs.h"
 #include "e2_dialog.h"
 #ifdef E2_VFS
@@ -82,10 +86,7 @@ static gboolean _e2_fs_S_ISEXE (mode_t mode)
 }
 #endif //0
 
-#ifdef E2_VFSTMP
-	//FIXME may need to check coding for each displayed dir, if that's possible
-#endif
-#ifndef E2_FILES_UTF8ONLY
+/* #ifndef E2_FILES_UTF8ONLY
 //check whether a character encoding is compatible with utf
 static gboolean _e2_fs_charset_is_utf (gchar *filecoding)
 {
@@ -104,12 +105,13 @@ static gboolean _e2_fs_language_is_english (void)
 		lang = g_getenv ("LANG");
 	return (lang != NULL && strstr (lang, "en_") != NULL);
 }
-/**
+*/
+/* *
 @brief check whether the native filesystem encoding is utf-8 (or ascii), and setup for conversion accordingly
 
 @return
 */
-void e2_fs_check_coding (void)
+/*void e2_fs_check_coding (void)
 {
 	gchar *filecoding;
 	app.utf8_filenames = FALSE;
@@ -133,7 +135,7 @@ void e2_fs_check_coding (void)
 			else
 				app.utf8_filenames = _e2_fs_language_is_english ();
 		}
-		else if (g_str_equal (filecoding, "@locale"))
+		else if (!strcmp (filecoding, "@locale"))
 		{
 			const gchar *charset;
 			app.utf8_filenames = g_get_charset (&charset)
@@ -163,8 +165,8 @@ void e2_fs_check_coding (void)
 		e2_fname_from_locale = e2_fname_dupfrom_locale = e2_utf8_filename_from_locale;
 		e2_fname_free = g_free;
 	}
-}
-#endif //ndef E2_FILES_UTF8ONLY
+} */
+//#endif //ndef E2_FILES_UTF8ONLY
 
 /**
 @brief check whether @a localpath exists
@@ -315,7 +317,7 @@ gboolean e2_fs_ingroup (gid_t gid)
 			gint i = 0;
 	        while (grp_buf->gr_mem [i] != NULL)
 			{
-				if (g_str_equal (grp_buf->gr_mem [i], myname))
+				if (!strcmp (grp_buf->gr_mem [i], myname))
 				{
 					ingroup = TRUE;
 					break;
@@ -329,64 +331,111 @@ gboolean e2_fs_ingroup (gid_t gid)
 }
 /**
 @brief execute the file(1) command on @a localpath, and search each line of its output for @a string
-
-Used in e2_fs_is_text and e2_fs_is_executable
+Used in e2_fs_is_text, e2_fs_is_exec2, and e2_fs_is_executable
+If @a localpath is a symlink, it will be traversed.
 
 @param localpath localised name of file to test
-@param string ascii string for which to search, in the output from 'find'
+@param string ascii string for which to search, in the output from file command
 
 @return TRUE if @a string is found
 */
 static gboolean _e2_fs_grep_file_output (VPATH *localpath, gchar *string E2_ERR_ARG())
 {
+	E2_ERR_PCLEAR
+
 #ifdef E2_VFS
 	if (e2_fs_item_is_mounted (localpath))
 	{
 #endif
-		gchar line[PATH_MAX + 20];
+#ifdef E2_MAGIC
+		MagicIface ifc;
+
+		if (e2_utils_fill_magic_iface (&ifc))
+		{
+			magic_t handle = ifc.open (
+				MAGIC_PRESERVE_ATIME | MAGIC_RAW | MAGIC_ERROR | MAGIC_SYMLINK);
+			if (handle != NULL)
+			{
+				ifc.load (handle, NULL); //load failure will result in NULL msg
+				const gchar *msg = ifc.file (handle, VPCSTR (localpath));
+				if (msg != NULL)
+				{
+					gboolean retval = (strstr (msg, string) != NULL);
+					ifc.close (handle);
+					dlclose (ifc.libhandle);
+					return retval;
+				}
+				else
+				{
+					//FIXME warning
+# ifdef E2_VFSTMP
+					if (E2_ERR_NAME != NULL)
+					{
+						populate *E2_ERR_NAME
+						X = ifc.error (handle);
+					}
+# endif
+					ifc.close (handle);
+				}
+			}
+			else
+			{
+				//FIXME warning
+# ifdef E2_VFSTMP
+				if (E2_ERR_NAME != NULL)
+				{
+					populate *E2_ERR_NAME
+				}
+# endif
+			}
+			dlclose (ifc.libhandle);
+		}
+#else
 		//CHECKME weirdness when > 1 opening of the same file,
 		//avoided by setting the cwd before opening the pipe
-		gchar *local = F_FILENAME_TO_LOCALE (curr_view->dir);
-#ifdef E2_VFS
+		gchar *local = D_FILENAME_TO_LOCALE (curr_view->dir); //always dup, to avoid dirchange race
+# ifdef E2_VFS
 		VPATH data = { local, NULL };	//always local
-		if (e2_fs_chdir_local (&data E2_ERR_NONE()))
-#else
+		if (e2_fs_chdir_local (&data E2_ERR_SAMEARG()))
+# else
 		if (e2_fs_chdir_local (local E2_ERR_NONE()))
-#endif
+# endif
 		{
 		// FIXME warn user
-			F_FREE (local);
+			g_free (local);
 			return FALSE;
 		}
-		F_FREE (local);
+		g_free (local);
 
 		size_t bsize = E2_MAX_LEN;
-#ifdef USE_GLIB2_10
+# ifdef USE_GLIB2_10
 		gchar *buf = (gchar *) g_slice_alloc ((gulong) bsize);
-#else
+# else
 		gchar *buf = (gchar *) g_try_malloc ((gulong) bsize);
-#endif
+# endif
 		CHECKALLOCATEDWARN (buf, return FALSE;);
 
-		//not g_strdup_printf or g_snprintf (in case they don't like localised text??)
-		snprintf (line, sizeof (line), ("file \"%s\""), 	//do not translate
-			VPSTR(localpath));
-		E2_FILE *pipe = e2_fs_open_pipe (line);
+//tag E2_BADQUOTES
+		gchar *qp = e2_utils_quote_string (VPCSTR (localpath));
+		gchar *command = e2_utils_strcat ("file -Lpr ", qp);	//want localised
+		g_free (qp);
+		E2_FILE *pipe = e2_fs_open_pipe (command);
+		g_free (command);
 		if (pipe == NULL)
 		{
-#ifdef USE_GLIB2_10
+# ifdef USE_GLIB2_10
 			g_slice_free1 (bsize, buf);
-#else
+# else
 			g_free (buf);
-#endif
+# endif
 			return FALSE;
 		}
 		gboolean retval = FALSE;
-#ifdef __USE_GNU
+# ifdef __USE_GNU
 		while (getdelim (&buf, &bsize, '\n', pipe) > 1)	//not an empty line
-#else
+# else
 		while (fgets (buf, bsize, pipe) != NULL && *buf != '\n')
-#endif
+# endif
 		{
 			if (strstr (buf, string))
 			{
@@ -395,27 +444,23 @@ static gboolean _e2_fs_grep_file_output (VPATH *localpath, gchar *string E2_ERR_
 			}
 		}
 		e2_fs_pipe_close (pipe);
-#ifdef USE_GLIB2_10
+# ifdef USE_GLIB2_10
 		g_slice_free1 (bsize, buf);
-#else
+# else
 		g_free (buf);
-#endif
+# endif
 		return retval;
+#endif
 #ifdef E2_VFS
 	}
 	else	//item is virtual
 	{
-/*		if (E2_ERR_NAME != NULL && *E2_ERR_NAME != NULL)
-		{
-			g_error_free (*E2_ERR_NAME);	//just in case?
-			*E2_ERR_NAME = NULL;
-		} */
 # ifdef E2_VFSTMP
 		//get information about virtual item, how ??
 # endif
-		return FALSE;
 	}
 #endif
+	return FALSE;
 }
 /**
 @brief test whether @a localpath is a text file
@@ -441,12 +486,40 @@ gboolean e2_fs_is_text (VPATH *localpath E2_ERR_ARG())
 #endif
 }
 /**
+@brief test whether @a localpath is an executable item
+
+@param localpath localised name of item to test
+
+@return TRUE if it is an executable file (or at least, looks like one)
+*/
+gboolean e2_fs_is_exec2 (VPATH *localpath E2_ERR_ARG())
+{
+	if (e2_fs_access (localpath, X_OK E2_ERR_NONE()) == 0)
+	{
+#ifdef E2_VFS
+		if (e2_fs_item_is_mounted (localpath))
+			//FIXME "executable" doesn't grab all script files, but "application/" is no good
+			return _e2_fs_grep_file_output (localpath, "executable" E2_ERR_SAMEARG()); //do not translate
+		else	//item is virtual
+		{
+# ifdef E2_VFSTMP
+		//FIXME do this some other way for virtual items e.g. test mimetype
+# endif
+			return FALSE;
+		}
+#else
+		return _e2_fs_grep_file_output (localpath, "executable" E2_ERR_SAMEARG()); //do not translate
+#endif
+	}
+	return FALSE;
+}
+/**
 @brief test whether @a info belongs to an executable item
 If the item is a symlink, its target will be tested
 @param info pointer to a FileInfo data struct
 @param view pointer to view data struct from which to get path info for @a info
 
-@return TRUE if system reports x permission, and the file (1) command says it's executable
+@return TRUE if system reports x permission, and the file(1) command says it's an application file
 */
 gboolean e2_fs_is_executable (FileInfo *info, ViewInfo *view)
 {
@@ -469,16 +542,12 @@ gboolean e2_fs_is_executable (FileInfo *info, ViewInfo *view)
 			if (e2_fs_item_is_mounted (localpath))
 			{
 #endif
+				//see comment above about "executable" scan here
 				retval = _e2_fs_grep_file_output (localpath, "executable" E2_ERR_NONE ());  //string not to be translated
 #ifdef E2_VFS
 			}
 			else	//item is virtual
 			{
-/*				if (E2_ERR_NAME != NULL && *E2_ERR_NAME != NULL)
-				{
-					g_error_free (*E2_ERR_NAME);	//just in case?
-					*E2_ERR_NAME = NULL;
-				} */
 # ifdef E2_VFSTMP
 			//FIXME full path for vfs item
 			//FIXME do this some other way for virtual items e.g. test mimetype
@@ -605,13 +674,7 @@ gboolean e2_fs_recurse_mkdir (VPATH *localpath, gint mode E2_ERR_ARG())
 			if (E2_ERR_PISNOT (ENOENT) || e2_fs_mkdir (localpath, mode E2_ERR_NONE()))	//FIXME vfs
 			{
 				*s = c;
-#ifdef E2_VFS
-				if (E2_ERR_NAME != NULL && *E2_ERR_NAME != NULL)
-				{
-					g_error_free (*E2_ERR_NAME);
-					*E2_ERR_NAME = NULL;
-				}
-#endif
+				E2_ERR_PCLEAR
 				return TRUE;
 			}
 		}
@@ -627,13 +690,7 @@ gboolean e2_fs_recurse_mkdir (VPATH *localpath, gint mode E2_ERR_ARG())
 	{
 		if (E2_ERR_PISNOT (ENOENT) || e2_fs_mkdir (localpath, mode E2_ERR_NONE()))	//FIXME vfs
 		{
-#ifdef E2_VFS
-			if (E2_ERR_NAME != NULL && *E2_ERR_NAME != NULL)
-			{
-				g_error_free (*E2_ERR_NAME);
-				*E2_ERR_NAME = NULL;
-			}
-#endif
+			E2_ERR_PCLEAR
 			return TRUE;
 		}
 	}
@@ -674,7 +731,7 @@ gboolean e2_fs_chdir (gchar *utfpath E2_ERR_ARG())
 		retval = TRUE;
 
 	E2_ERR_CLEARBACKUP (localerr);
-	F_FREE (local);
+	F_FREE (local, utfpath);
 	return retval;
 }
 /**
@@ -721,25 +778,32 @@ gboolean e2_fs_cd_isok (gchar *utfpath E2_ERR_ARG())
 			success = FALSE;
 		}
 	}
-	else if (e2_fs_dir_is_native (utfpath))
-		message = g_strdup_printf (_("Directory '%s' does not exist"), utfpath);
 	else
 	{
-#ifdef E2_VFS
-		if (E2_ERR_NAME != NULL && *E2_ERR_NAME != NULL)
+		//dir doesn't exist, or is inaccessible due to lack of permission
+		if (e2_fs_dir_is_native (utfpath, TRUE))
 		{
-			g_error_free (*E2_ERR_NAME);
-			*E2_ERR_NAME = NULL;
-		}
+#ifdef E2_VFS
+			if (e2_fs_access (&data, R_OK | X_OK E2_ERR_SAMEARG()))
+#else
+			if (e2_fs_access (local, R_OK | X_OK E2_ERR_SAMEARG()))
 #endif
-		success = TRUE;	//missing path ok when it's non-mounted
+				message = g_strdup_printf (_("Cannot access directory '%s' - No permission"), utfpath);
+			else
+				message = g_strdup_printf (_("Directory '%s' does not exist"), utfpath);
+		}
+		else
+		{
+			E2_ERR_PCLEAR
+			success = TRUE;	//missing path ok when it's non-mounted
+		}
 	}
 
 	if (!success)
 	{
 		e2_output_print_error (message, TRUE);
 	}
-	F_FREE (local);
+	F_FREE (local, utfpath);
 	return success;
 }
 /**
@@ -776,22 +840,33 @@ gboolean e2_fs_get_valid_path (gchar **utfpath, gboolean accessible E2_ERR_ARG()
 		//CHECKME do a full interpretation here ?
 		if (local[0] == '~') //home dir check
 		{
-			freeme = local;
 			if (local[1] == '\0')
 			{
+				g_free (local);
 				local = g_strdup (g_get_home_dir ());
-				g_free (freeme);
 			}
 			else
 				if (local[1] == G_DIR_SEPARATOR)
 			{
-				local = g_strconcat (g_get_home_dir (), local + 1, NULL);
+				freeme = local;
+				local = g_strconcat (g_get_home_dir (), local + sizeof(gchar), NULL);
 				g_free (freeme);
 			}
+			else
+			{	//try to get some user's home
+				freeme = e2_utils_get_home_path (*utfpath); //FIXME local was chug'd
+				if (freeme != NULL)
+				{
+					g_free (local);
+					local = D_FILENAME_TO_LOCALE (freeme);
+					g_free (freeme);
+				}
+			}
+
 		}
 
 #ifdef E2_VFS
-		data.localpath = local;
+		data.path = local;
 		data.spacedata = NULL;
 		if (e2_fs_is_dir3 (&data E2_ERR_NONE())
 			&& (!accessible || !e2_fs_access (&data, R_OK | X_OK E2_ERR_NONE())))
@@ -811,7 +886,7 @@ gboolean e2_fs_get_valid_path (gchar **utfpath, gboolean accessible E2_ERR_ARG()
 	//FIXME new path may be virtual item eg archive ?
 #endif
 #ifdef E2_VFS
-//		data.localpath = local; e2_utils_get_parent_path() doesn't change path
+//		data.path = local; e2_utils_get_parent_path() doesn't change path
 		if (e2_fs_is_dir3 (&data E2_ERR_NONE())
 			&& (!accessible || !e2_fs_access (&data, R_OK | X_OK E2_ERR_NONE())))
 #else
@@ -842,7 +917,7 @@ gboolean e2_fs_get_valid_path (gchar **utfpath, gboolean accessible E2_ERR_ARG()
 @brief complete a directory string in @a entry after a keypress @a pressed_char
 This works only for native directories
 @param entry the entry widget to be updated
-@param pressed_char utf8-string form of pressed key which triggered the completion
+@param pressed_char UTF-8 string-form of pressed key which triggered the completion
 @param pane enumerator of pane to use for default dir, 1,2,0=current
 
 @return TRUE if completion was performed
@@ -855,6 +930,20 @@ static gboolean _e2_fs_complete_dir (GtkWidget *entry, gchar *pressed_char, guin
 	gint pos = start + 1;	//+1 to include the key just pressed
 	gchar *entrytext = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, start);
 	gchar *text = g_strconcat (entrytext, pressed_char, NULL);
+
+	//do macros/variables replacement CHECKME worth it ?
+	printd (DEBUG, "_e2_fs_complete_dir, macro/var interpretation");
+	gchar *tmp = text;
+	gchar *freeme = e2_utils_expand_macros (text, NULL); //note %% changes to %
+	if (freeme == NULL || freeme == GINT_TO_POINTER (0x1))
+		freeme = tmp;
+	text = e2_utils_replace_vars (freeme, TRUE);
+	if (freeme != tmp)
+		g_free (freeme);
+	if (strcmp (tmp, text))
+		pos = g_utf8_strlen (text, -1);
+	g_free (tmp);
+
 	gchar *utfdir;
 	if (g_path_is_absolute (text))
 		utfdir = g_path_get_dirname (text);
@@ -863,24 +952,24 @@ static gboolean _e2_fs_complete_dir (GtkWidget *entry, gchar *pressed_char, guin
 		switch (pane)
 		{
 			case E2PANE1:
-				utfdir = app.pane1_view.dir;
+				utfdir = app.pane1.view.dir;
 				break;
 			case E2PANE2:
-				utfdir = app.pane2_view.dir;
+				utfdir = app.pane2.view.dir;
 				break;
 //			case E2PANECUR:
 			default:
 				utfdir = curr_view->dir;
 				break;
 		}
-		gchar *freeme = g_strconcat (utfdir, text, NULL);
+		freeme = g_strconcat (utfdir, text, NULL);
 		utfdir = g_path_get_dirname (freeme);
 		g_free (freeme);
 	}
 #ifdef E2_VFSTMP
 	//CHECKME enable vfs completion ??
 #endif
-	if (e2_fs_dir_is_native (utfdir))
+	if (e2_fs_dir_is_native (utfdir, FALSE))	//CHECKME parent-checking too?
 	{
 		GList *found = NULL;
 		retval = (e2_complete_str (&text, &pos, &found, E2_COMPLETE_FLAG_DIRS, pane) > 0);
@@ -891,6 +980,7 @@ static gboolean _e2_fs_complete_dir (GtkWidget *entry, gchar *pressed_char, guin
 			gchar *newtext = g_strconcat (text, entrytext, NULL);
 			gtk_entry_set_text (GTK_ENTRY (entry), newtext);
 			gtk_editable_set_position (GTK_EDITABLE (entry), pos);
+			e2_command_line_update_highlight (entry, newtext);
 			g_free (newtext);
 			e2_list_free_with_data (&found);
 		}
@@ -919,6 +1009,7 @@ gboolean e2_fs_complete_dir (GtkWidget *entry, guint keyval, guint pane)
 	guint32 unikey = gdk_keyval_to_unicode (keyval);
 	if (unikey > 0)
 	{
+		gint start, end;
 		gchar keystring[8];
 		gint bytes = g_unichar_to_utf8 (unikey, keystring);
 		*(keystring+bytes) = '\0';
@@ -926,12 +1017,14 @@ gboolean e2_fs_complete_dir (GtkWidget *entry, guint keyval, guint pane)
 		switch (type)
 		{
 			case 1:	//insert completed text, and move after it
+				if (gtk_editable_get_selection_bounds (GTK_EDITABLE (entry), &start, &end))
+					gtk_editable_delete_text (GTK_EDITABLE (entry), start, end);
 				if (_e2_fs_complete_dir (entry, keystring, pane))
 					return TRUE;
 				break;
 			case 2:	//select completed text
 			{
-				gint start, end, newpos, oldpos;
+				gint newpos, oldpos;
 				if (gtk_editable_get_selection_bounds (GTK_EDITABLE (entry), &start, &end))
 				{
 					//when there's a selection, cursor position = end, fix that
@@ -956,14 +1049,15 @@ gboolean e2_fs_complete_dir (GtkWidget *entry, guint keyval, guint pane)
 	return FALSE;
 }
 /**
-@brief evaluate whether @a utfpath is a native dir
+@brief evaluate whether @a utfpath is a native dir or optionally, a descendant of one
 Native means local and mounted, not fuse or other vfs
 This is for tailoring pre-change-dir and pre-refresh-dir checking
 @param utfpath absolute path of dir to check, utf-8 string
+@param descend TRUE to check for a descendant
 
 @return TRUE if @a utfpath is or seems to be native
 */
-gboolean e2_fs_dir_is_native (gchar *utfpath)
+gboolean e2_fs_dir_is_native (const gchar *utfpath, gboolean descend)
 {
 #if defined(__linux__) || defined(__FreeBSD__)
 	GList *fusemounts, *member;
@@ -984,9 +1078,9 @@ gboolean e2_fs_dir_is_native (gchar *utfpath)
 	gchar *localpath;
 	struct statfs fstatus;
 
-	localpath = F_FILENAME_TO_LOCALE (utfpath);
+	localpath = D_FILENAME_TO_LOCALE (utfpath);
+retry:
 	result = statfs (localpath, &fstatus);
-	F_FREE (localpath);
 	if (result)
 	{
 		if (result == ESTALE)
@@ -994,31 +1088,42 @@ gboolean e2_fs_dir_is_native (gchar *utfpath)
 			printd (DEBUG, "%s resides in an unmounted VFS", utfpath);
 			//CHECKME mount it ?
 		}
+		g_free (localpath);
 		return TRUE;	//any operation on utfpath will fail ASAP
 	}
 	if (!(fstatus.f_flags & MNT_LOCAL))
-		return FALSE;
+	{
+//		if (errno == EINTR) CHECKME relevance ?
+//			goto retry;
+//		else
+			if (errno == ENOENT && descend && e2_utils_get_parent_path (localpath, TRUE))
+			goto retry;
+		g_free (localpath);
+		return (errno == EACCES);	//no permission implies a local dir
+	}
 #else
 	struct statvfs fstatus;
 	gchar *localpath;
-	localpath = F_FILENAME_TO_LOCALE (utfpath);
+	localpath = D_FILENAME_TO_LOCALE (utfpath);
 retry:
 	if (statvfs (localpath, &fstatus))
 	{
 		if (errno == EINTR)
 			goto retry;
-//		if (errno == ENOLINK)
+//		else if (errno == ENOLINK)
 //			re-mount it ??
-		F_FREE (localpath);
-		return FALSE;
+		else if (errno == ENOENT && descend && e2_utils_get_parent_path (localpath, TRUE))
+			goto retry;
+		g_free (localpath);
+		return (errno == EACCES);	//no permission implies a local dir
 	}
-	F_FREE (localpath);
 #endif
 	//FIXME MORE TESTS HERE e.g.
 	//from string contents e.g. "//:"
 	//from plugin data
 	//from vfs history data
 	//from other e2 data
+	g_free (localpath);
 	return TRUE;
 }
 #ifdef E2_VFS
@@ -1136,7 +1241,7 @@ static gpointer _e2_fs_read_mounted_dir (E2_DRFuncArgs *data)
 	if (threaded)
 	{
 		pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, &oldtype);
-//		pthread_cleanup_push ((gpointer)closedir, (gpointer)dp); can't be conditional or inside braces
+//		pthread_cleanup_push ((gpointer)closedir, (gpointer)dp); can't be inside braces without corresponding pop
 	}
 
 	struct dirent *entryptr;
@@ -1173,7 +1278,7 @@ static gpointer _e2_fs_read_mounted_dir (E2_DRFuncArgs *data)
 			if (processor != NULL)
 			{
 #ifdef E2_VFS
-				ddata.localpath = data->localpath;
+				ddata.path = data->localpath;
 				if (!processor (&ddata, u.entry.d_name, &entries, data->cb_data))
 #else
 				if (!processor (data->localpath, u.entry.d_name, &entries, data->cb_data))
@@ -1230,11 +1335,11 @@ static void _e2_fs_slowread_response_cb (GtkDialog *dialog, gint response,
 				//after this cancellation, the main thread will cleanup
 			}
 			break;
-		case E2_RESPONSE_YESTOALL:	//no more reminders
+		case E2_RESPONSE_USER1:	//no more reminders
 //CHECKME pthread_mutex_lock (&?_mutex);
 			wid = data->dialog; 	//race-management
 			data->dialog = NULL;
-			if (wid != NULL && GTK_IS_WIDGET (wid))
+			if (GTK_IS_WIDGET (wid))
 				gtk_widget_destroy (wid);	//== (GTK_WIDGET(dialog)
 //			pthread_mutex_unlock (&?_mutex);
 			//if the racy action-thread ends about now, signal to main
@@ -1288,12 +1393,27 @@ static gpointer _e2_fs_progress_monitor (E2_DRead *data)
 		pthread_mutex_unlock (&condition_mutex);
 		printd (DEBUG,"dir-monitor-thread (ID=%lu) %d-sec sleep ended", (gulong) data->mid, secs);
 
-		gboolean shown = (data->dialog != NULL && GTK_WIDGET_VISIBLE (data->dialog));
+		gboolean shown = (data->dialog != NULL &&
+#ifdef USE_GTK2_18
+			gtk_widget_get_visible (data->dialog));
+#else
+			GTK_WIDGET_VISIBLE (data->dialog));
+#endif
 		if (data->dialog == NULL) //once-only, create the dialog
+		{
+			gdk_threads_enter ();
 			data->dialog = e2_dialog_slow (_("Reading directory data"),
-				_("directory read"), _e2_fs_slowread_response_cb, data);
+				_("directory read"),
+			(ResponseFunc)_e2_fs_slowread_response_cb, data);
+			gdk_threads_leave ();
+		}
 
-		if (data->dialog != NULL && !GTK_WIDGET_VISIBLE (data->dialog))
+		if (data->dialog != NULL &&
+#ifdef USE_GTK2_18
+			!gtk_widget_get_visible (data->dialog));
+#else
+			!GTK_WIDGET_VISIBLE (data->dialog))
+#endif
 		{
 			gdk_threads_enter ();
 			gtk_widget_show (data->dialog);
@@ -1359,8 +1479,9 @@ gpointer e2_fs_dir_foreach (VPATH *localpath, E2_FsReadWatch monitor,
 		//create joinable read thread, not with glib thread funcs, as we need to
 		//kill thread(s)
 		pthread_t ID;
-		if (!pthread_create (&ID, NULL, (gpointer)_e2_fs_read_mounted_dir, &args))
-			printd (DEBUG,"read-dir-thread (ID=%lu) started", ID);
+		if (pthread_create (&ID, NULL,
+			(gpointer(*)(gpointer))_e2_fs_read_mounted_dir, &args) == 0)
+				printd (DEBUG,"read-dir-thread (ID=%lu) started", ID);
 		else
 		{
 			//FIXME message to user
@@ -1383,11 +1504,13 @@ gpointer e2_fs_dir_foreach (VPATH *localpath, E2_FsReadWatch monitor,
 		//redundant if read-thread has finished already, but then it gets killed
 		//immediately, anyhow
 		E2_DRead data = { ID, 0, NULL };
+
 		pthread_attr_t attr;
 		pthread_attr_init (&attr);
 		pthread_attr_setdetachstate (&attr, PTHREAD_CREATE_DETACHED);
+
 		if (pthread_create (&data.mid, &attr,
-			(gpointer) _e2_fs_progress_monitor, &data) == 0)
+			(gpointer(*)(gpointer))_e2_fs_progress_monitor, &data) == 0)
 				printd (DEBUG,"dir-monitor-thread (ID=%lu) started", data.mid);
 		else
 		{
@@ -1398,12 +1521,14 @@ gpointer e2_fs_dir_foreach (VPATH *localpath, E2_FsReadWatch monitor,
 		printd (DEBUG,"main thread blocking until read ends or times out");
 		pthread_join (ID, &entries);
 
+		pthread_attr_destroy (&attr);
+
 		e2_window_clear_status_message ();	//no BGL management neeeded
 		WAIT_FOR_EVENTS_UNLOCKED;
 		//some race-minimisation here
 		GtkWidget *wid = data.dialog;
 		data.dialog = NULL;
-		if (wid != NULL && GTK_IS_WIDGET (wid))
+		if (GTK_IS_WIDGET (wid))
 		{
 			gdk_threads_enter ();
 			gtk_widget_destroy (wid);
@@ -1440,6 +1565,70 @@ gpointer e2_fs_dir_foreach (VPATH *localpath, E2_FsReadWatch monitor,
 #endif
 	return entries;
 }
+
+/**
+@brief get the target of symbolic link represented by @a localpath
+The string in targetbuf will be 0-terminated and stripped of irrelevant leading
+relativities
+@param localpath descriptor for the symlink, with localised path
+@param targetbuf buffer into which the result can be copied
+@param targetsize the length of @a targetbuf
+
+@return the length of the string in @a targetbuf, or < 0 upon error
+*/
+gint e2_fs_readlink (VPATH *localpath, gchar *targetbuf, size_t targetsize E2_ERR_ARG())
+{
+	gint len;
+	const gchar *t;
+#ifdef E2_VFS
+	VPATH ddata;
+#endif
+
+#ifdef E2_VFSTMP
+	if (e2_fs_item_is_mounted (localpath))
+	{
+#endif
+		len = readlink (VPCSTR(localpath), targetbuf, targetsize);
+		if (len < 0)
+		{
+			*targetbuf = '\0';
+			return len;
+		}
+//		else if (len == targetsize)
+//			FIXME can't tell if target is longer
+#ifdef E2_VFSTMP
+	}
+	else	//need to do virtual
+	{
+		FIXME vfs version
+		if (len < 0)
+		{
+			e2_fs_set_error_from_errno (E2_ERR_NAME);
+			*targetbuf = '\0';
+			return len;
+		}
+	}
+#endif
+	if (len == targetsize)
+		len--;
+	targetbuf[len] = '\0';
+
+#ifdef E2_VFS
+	ddata.path = targetbuf;
+	ddata.spacedata = localpath->spacedata;
+	t = e2_utils_skip_relative_path (&ddata);
+#else
+	t = e2_utils_skip_relative_path (targetbuf);
+#endif
+	if (t > targetbuf)
+	{
+		memcpy (targetbuf, t, len + targetbuf - t + 1);
+		len -= (t-targetbuf);
+	}
+
+	return len;
+}
+
 /**
 @brief get real path for @a local_path if it's a link
 
@@ -1449,6 +1638,7 @@ gpointer e2_fs_dir_foreach (VPATH *localpath, E2_FsReadWatch monitor,
 */
 gboolean e2_fs_walk_link (gchar **local_path E2_ERR_ARG())
 {
+//CHECKME use e2_utils_skip_relative_path (VPATH *path); here ?
 	struct stat sb;
 #ifdef E2_VFSTMP
 	FIXME get proper spacedata
@@ -1487,14 +1677,10 @@ gboolean e2_fs_walk_link (gchar **local_path E2_ERR_ARG())
 		}
 		else	//item is virtual
 		{
-/*			if (E2_ERR_NAME != NULL && *E2_ERR_NAME != NULL)
-			{
-				g_error_free (*E2_ERR_NAME);	//just in case?
-				*E2_ERR_NAME = NULL;
-			} */
-	# ifdef E2_VFSTMP
+			E2_ERR_PCLEAR
+# ifdef E2_VFSTMP
 			//FIXME vfs if relevant, get information about virtual item, how ??
-	# endif
+# endif
 			return FALSE;
 		}
 #endif
@@ -1643,7 +1829,7 @@ gint e2_fs_symlink (VPATH *target, VPATH *name, GError **E2_ERR_NAME)
 	{
 		e2_fs_set_error_custom (E2_ERR_NAME, 126,
 			"%s and %s are not in the same namespace",
-			VPSTR(target), VPSTR(name));
+			VPSTR(target), VPSTR(name), NULL);
 		return -1;
 	}
 
@@ -1672,26 +1858,6 @@ gint e2_fs_symlink (VPATH *target, VPATH *name, GError **E2_ERR_NAME)
 	return result;
 }
 
-gint e2_fs_readlink (VPATH *localpath, gchar *targetbuf, size_t bufsize,
-	GError **E2_ERR_NAME)
-{
-	gint result;
-	if (e2_fs_item_is_mounted (localpath))
-	{
-		result = readlink (VPCSTR(localpath), targetbuf, bufsize);
-		if (result != 0)
-			e2_fs_set_error_from_errno (E2_ERR_NAME);
-		return result;
-	}
-	else	//need to do virtual
-	{
-#ifdef E2_VFSTMP
-		//FIXME do virtual change
-#endif
-		return -1;
-	}
-}
-
 gint e2_fs_unlink (VPATH *localpath, GError **E2_ERR_NAME)
 {
 	gint result;
@@ -1980,9 +2146,8 @@ gboolean e2_fs_touchnow (VPATH *localpath E2_ERR_ARG())
 /**
 @brief update m/a/c times of config dir
 
-This should be called after writing the config/cache files
-to let any other instance know that it should re-read those
-files (if that option is in force)
+This should be called after writing config/cache files, to let any other
+instance know that it should re-read those files (if that option is in force).
 Error message expects BGL to be closed
 
 @return
@@ -2031,7 +2196,7 @@ void e2_fs_touch_config_dir (void)
 #endif
 		E2_ERR_CLEAR
 	}
-	F_FREE (local);
+	F_FREE (local, e2_cl_options.config_dir);
 }
 /**
 @brief tolerantly open file @a localpath and return its descriptor
@@ -2082,7 +2247,7 @@ E2_FILE *e2_fs_open_writestream (const gchar *localpath E2_ERR_ARG())
 			gchar *msg = g_strdup_printf (_("Cannot open '%s' for writing - %s"),
 				utf, g_strerror (errno));
 			e2_output_print_error (msg, TRUE);
-			F_FREE (utf);
+			F_FREE (utf, localpath);
 		}
 #ifdef E2_VFS
 	}
@@ -2133,7 +2298,7 @@ gint e2_fs_put_stream (E2_FILE *stream, const gchar *string, gchar *localpath
 #endif
 				);
 			e2_output_print_error (msg, TRUE);
-			F_FREE (utf);
+			F_FREE (utf, localpath);
 			E2_ERR_CLEARBACKUP (localerr);
 		}
 #ifdef E2_VFSTMP
@@ -2149,11 +2314,11 @@ gint e2_fs_put_stream (E2_FILE *stream, const gchar *string, gchar *localpath
 /**
 @brief read from file into buffer
 
-@param descriptor file descriptor
+@param descriptor file descriptor for the file to read
 @param buffer start of memory to hold the read data
 @param bufsize size of @a buffer
 
-@return no. of bytes read, < 0 on error
+@return no. of bytes read, or -1 on error
 */
 ssize_t e2_fs_read (gint descriptor, gpointer buffer, /*size_t*/ gulong bufsize E2_ERR_ARG())
 {
@@ -2164,9 +2329,8 @@ ssize_t e2_fs_read (gint descriptor, gpointer buffer, /*size_t*/ gulong bufsize
 	do nread = read (descriptor, buffer, bufsize);
 		while (nread < 0 && E2_ERR_IS (EINTR));
 	return nread; */
-	if (bufsize == 0)
-		return 0;
-	ssize_t nread;
+
+	ssize_t nread = -1;	//error if buffer size is 0
 	size_t space = (size_t) bufsize;
 	size_t total = 0;
 	gpointer target = buffer;
@@ -2175,14 +2339,17 @@ ssize_t e2_fs_read (gint descriptor, gpointer buffer, /*size_t*/ gulong bufsize
 		nread = read (descriptor, target, space);
 		if (nread > 0)
 		{
-			space -= nread;
 			total += nread;
 			target += nread;	//irrelevant if we're finished now
+			space -= nread;
 		}
-		else if (nread == 0
-				|| errno != EINTR) //nread < 0
+		else if (nread == 0	//EOF, no error
+				||			//nread < 0
+				errno != EINTR) //un-recoverable error
 			break;
 	}
+	if (nread < 0)
+		return -1;
 	return (ssize_t)total;
 }
 /**
@@ -2195,28 +2362,27 @@ Error messages expect BGL open/off
 @param localpath path of file to be read, localised string
 @param contents store for ptr start of the allocated contents
 @param contlength store for no. of bytes written, or NULL if not interested
-@param string TRUE to add a trailing 0 if that's missing
+@param terminate TRUE to add a trailing 0 if that's not present in file as read
 
 @return TRUE if @a localpath was read successfully
 */
 gboolean e2_fs_get_file_contents (VPATH *localpath, gpointer *contents,
-	/*ssize_t*/ gulong *contlength, gboolean string E2_ERR_ARG())
+	/*ssize_t*/ gulong *contlength, gboolean terminate E2_ERR_ARG())
 {
 	gboolean retval;
+	const gchar *fmt;
 #ifdef E2_VFS
 	if (e2_fs_item_is_mounted (localpath))
 	{
 #endif
-		struct stat sb;
-		retval = TRUE;
-		const gchar *fmt = NULL;
 		gint fdesc = e2_fs_safeopen (VPCSTR(localpath), O_RDONLY, 0);
 		if (fdesc >= 0)
 		{
+			struct stat sb;
 			if (!e2_fs_fstat (fdesc, &sb))
 			{
-				size_t len = sb.st_size;
-				if (string)
+				size_t nread, len = sb.st_size;
+				if (terminate)
 					len += sizeof (gchar);	//in case we need to append a 0
 
 //				*contents = malloc (len);	//not size-limited, but imposes cleanup hassles
@@ -2232,37 +2398,47 @@ gboolean e2_fs_get_file_contents (VPATH *localpath, gpointer *contents,
 					return FALSE;
 				}
 
-				/*ssize_t*/gulong nread = e2_fs_read (fdesc, *contents, sb.st_size E2_ERR_SAMEARG());
-				if (nread < 0 //|| nread < (gulong)sb.st_size
-				   )
+				retval = TRUE;
+				fmt = NULL;
+				if (sb.st_size > 0)
 				{
-					g_free (*contents);	//or free() if needed
-					*contents = NULL;
-					nread = 0;
-					retval = FALSE;
-					fmt = _("Error reading file %s");
-				}
-				else if (string)
-				{
-					gchar *s;
-					if (nread >= sizeof (gchar))
+					nread = e2_fs_read (fdesc, *contents, sb.st_size E2_ERR_SAMEARG());
+					if (nread < 0 //|| nread < (gulong)sb.st_size
+					   )
 					{
-						s = *contents + nread - sizeof (gchar);
-						if (*s == '\0')
-							nread -= sizeof (gchar);	//OK ?
-						else
+						g_free (*contents);	//or free() if needed
+						*contents = NULL;
+						nread = 0;
+						retval = FALSE;
+						fmt = _("Error reading file %s");
+					}
+					else if (terminate)
+					{
+						gchar *s;
+						if (nread >= sizeof (gchar))
 						{
-							s += sizeof (gchar);
+							s = *contents + nread - sizeof (gchar);
+							if (*s == '\0')
+								nread -= sizeof (gchar);	//OK ?
+							else
+							{
+								s += sizeof (gchar);
+								*s = '\0';
+							}
+						}
+						else	//nothing read
+						{
+							s = *contents;
 							*s = '\0';
 						}
 					}
-					else	//nothing read
-					{
-						s = *contents;
-						*s = '\0';
-					}
 				}
-
+				else
+				{
+					nread = 0;
+					if (terminate)
+						*contents = '\0';
+				}
 				if (contlength != NULL)
 					*contlength = nread;
 			}
@@ -2303,10 +2479,10 @@ gboolean e2_fs_get_file_contents (VPATH *localpath, gpointer *contents,
 	return retval;
 }
 /**
-@brief read from file into buffer
+@brief write from buffer to file
 
-@param descriptor file descriptor
-@param buffer start of memory to hold the write data
+@param descriptor file descriptor of file to write
+@param buffer start of memory holding the write data
 @param bufsize size of @a buffer
 
 @return no. of bytes written, < 0 on error
@@ -2325,7 +2501,7 @@ ssize_t e2_fs_write (gint descriptor, gpointer buffer, /*size_t*/ gulong bufsize
 }
 /**
 @brief write whole file @a localpath, from memory starting at @a contents
-No overwrite checking is done
+No overwrite checking is done, no data syncing is done
 @param localpath path of file to be write, localised string
 @param contents ptr to the start of the data to save
 @param contlength byte-length of data to save
@@ -2364,7 +2540,10 @@ gboolean e2_fs_set_file_contents (VPATH *localpath, gpointer contents,
 				}
 			}
 			if (retval)
-				ftruncate (fdesc, written);
+			{
+				//CHECKME e2_fs_writeflush (fdesc);
+				TEMP_FAILURE_RETRY (ftruncate (fdesc, written));
+			}
 			e2_fs_safeclose (fdesc);
 		}
 
@@ -2407,6 +2586,27 @@ gint e2_fs_safeclose (gint file_desc)
 	return res;
 }
 /**
+@brief flush file contents to storage
+File descriptors are only relevant for local files, so no vfs treatment
+is needed
+@param file_desc file descriptor
+
+@return 0 on success, -1 on error
+*/
+gint e2_fs_writeflush (gint file_desc)
+{
+	gint res;
+	errno = 0;
+#ifdef __E2BSD__
+	res = fsync (file_desc);
+#else
+	res = fdatasync (file_desc);
+#endif
+	if (res == -1 && errno == EINVAL)
+		res = 0;
+	return res;
+}
+/**
 @brief perform a native, blockwise, file copy
 Error messages assume BGL is open
 @param vsrc localised string, absolute path of item to copy
@@ -2596,16 +2796,14 @@ E2_FILE *e2_fs_open_pipe (gchar *command)
 @brief read piped command output into memory
 A terminating 0 is always added
 Error message expects BGL to be closed
-@param command utf command string to be executed
-@param output store for ptr to command output string
+@param command localised command string to be executed
+@param output store for ptr to newly-allocated command output string
 
-@return TRUE if @a command result was read successfully
+@return TRUE if @a command result was read successfully and byte-length of output > 0
 */
 gboolean e2_fs_get_command_output (gchar *command, gpointer *output)
 {
-	gchar *local = F_FILENAME_TO_LOCALE (command);
-	E2_FILE *pipe = e2_fs_open_pipe (local);
-	F_FREE (local);
+	E2_FILE *pipe = e2_fs_open_pipe (command);
 	if (pipe == NULL)
 		return FALSE;
 	gchar *msg = g_strdup_printf (
@@ -2656,38 +2854,69 @@ gboolean e2_fs_get_command_output (gchar *command, gpointer *output)
 	if (total == 0)
 		g_free (buf);
 	else
+	{
+		store = g_try_realloc (buf, total);	//maybe we can make it smaller
+		if (store != NULL)
+			buf = store;
 		*output = buf;
+	}
 	return (total > 0);
 }
 /**
-@brief read up to @a count -1 bytes of piped command output into memory
-A terminating 0 is always added
-@param command utf command string to be executed
-@param contents ptr to where the contents are to be stored
+@brief read up to @a count -1 bytes of piped command output into newly-allocated memory
+A terminating 0 is appended after a successful read
+@param command localised command string to be executed
+@param contents store for address of read contents
 @param count size of block to read + 1 (for added 0)
 
 @return TRUE if @a command result was read successfully
 */
 gboolean e2_fs_sniff_command_output (gchar *command, gpointer *contents, gulong count)
 {
-	gchar *local = F_FILENAME_TO_LOCALE (command);
-	E2_FILE *pipe = e2_fs_open_pipe (local);
-	F_FREE (local);
-	if (pipe == NULL)
-		return FALSE;
-	*contents = g_try_malloc (count);
-	if (*contents == NULL)
+	gboolean retval = FALSE;
+	E2_FILE *pipe = e2_fs_open_pipe (command);
+	if (pipe != NULL)
 	{
-		//FIXME warn user
+#ifdef USE_GLIB2_10
+		*contents = g_slice_alloc (count);
+#else
+		*contents = g_try_malloc (count);
+#endif
+		if (*contents != NULL)
+		{
+			ssize_t bytes_read, total = 0;
+reread:
+			errno = 0;
+			bytes_read = fread ((*contents + total), 1, (size_t)(count - total - 1), pipe);
+			if (bytes_read >= 0)
+			{
+				total += bytes_read;
+				if (total == count - 1) //got as much as we want
+					retval = TRUE;
+				else
+					goto reread;
+			}
+			else if (feof (pipe))
+			{
+				if (total > 0)
+					retval = TRUE;
+			}
+			else if (!ferror (pipe) || errno == EAGAIN || errno == EINTR)
+				goto reread;
+
+			if (retval)
+				*(*((gchar**)contents) + total) = '\0';
+			else
+				//FIXME warn user upon error
+#ifdef USE_GLIB2_10
+				g_slice_free1 (count, *contents);
+#else
+				g_free (*contents);
+#endif
+		}
 		e2_fs_pipe_close (pipe);
-		return FALSE;
 	}
-	ssize_t bytes_read = fread (*contents, 1, (size_t)count-1, pipe);
-	if (bytes_read < 0)
-		return FALSE;
-	*(*((gchar**)contents) + bytes_read) = '\0';
-	e2_fs_pipe_close (pipe);
-	return TRUE;
+	return retval;
 }
 /* *
 @brief blockwize read from stdin into memory
@@ -2928,7 +3157,7 @@ gboolean e2_fs_error_local (const gchar *format, VPATH *local
 		, reason
 #endif
 		);
-	F_FREE (utf);
+	F_FREE (utf, VPSTR(local));
 	g_free (msg);
 	return FALSE;
 }
@@ -2948,7 +3177,7 @@ gboolean e2_fs_error_simple (const gchar *format, VPATH *local)
 	gdk_threads_enter ();
 	e2_output_print_error (msg, TRUE);
 	gdk_threads_leave ();
-	F_FREE (utf);
+	F_FREE (utf, VPSTR(local));
 	return FALSE;
 }
 
