@@ -1,6 +1,6 @@
-/* $Id: e2_fs.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_fs.h 1964 2009-10-07 11:18:15Z tpgww $
 
-Copyright (C) 2005-2008 tooar <tooar@gmx.net>
+Copyright (C) 2005-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -44,6 +44,8 @@ gint e2_fs_access2 (VPATH *localpath, GError **error);
 gint e2_fs_access3 (VPATH *localpath, gint how, GError **error);
 gboolean e2_fs_check_write_permission (VPATH *localpath, GError **error);
 gboolean e2_fs_is_text (VPATH *localpath, GError **error);
+gboolean e2_fs_is_exec2 (VPATH *localpath, GError **error);
+//gboolean e2_fs_is_hidden (VPATH *localpath, GError **error);
 //gboolean e2_fs_is_link (gchar *localpath, GError **error);
 gboolean e2_fs_walk_link (gchar **local_path, GError **error);
 gboolean e2_fs_is_dir3 (VPATH *localpath, GError **error);
@@ -87,14 +89,16 @@ gboolean e2_fs_vfsfunc_ready (gpointer *func);
 #define e2_fs_lchown lchown
 #define e2_fs_utime utime
 #define e2_fs_symlink symlink
-#define e2_fs_readlink readlink
 
 gint e2_fs_stat (VPATH *localpath, struct stat *buf);
 gint e2_fs_access2 (VPATH *localpath);
 gint e2_fs_access3 (VPATH *localpath, gint how);
 gboolean e2_fs_check_write_permission (VPATH *localpath);
 gboolean e2_fs_is_text (VPATH *localpath);
+gboolean e2_fs_is_exec2 (VPATH *localpath);
+//gboolean e2_fs_is_hidden (VPATH *localpath);
 //gboolean e2_fs_is_link (VPATH *localpath);
+gint e2_fs_readlink (VPATH *localpath, gchar *targetbuf, size_t targetsize);
 gboolean e2_fs_walk_link (gchar **local_path);
 gboolean e2_fs_is_dir3 (VPATH *localpath);
 gboolean e2_fs_recurse_mkdir (VPATH *localpath, gint mode);
@@ -162,11 +166,24 @@ gboolean e2_fs_ingroup (gid_t gid);
 E2_FILE *e2_fs_open_pipe (gchar *command);	//always native
 
 gchar *e2_fs_get_perm_string (mode_t mode);	//string utility only
-gboolean e2_fs_dir_is_native (gchar *utfpath);	//native comparisons only
+gboolean e2_fs_dir_is_native (const gchar *utfpath, gboolean descend);	//native comparisons only
 gboolean e2_fs_complete_dir (GtkWidget *entry, guint keyval, guint pane);
 
 gboolean e2_fs_is_executable (FileInfo *info, ViewInfo *view);
 gboolean e2_fs_is_dir (FileInfo *info, ViewInfo *view);
+
+//these flags are not standard, their position is > those used in S_IFMT (0170000)
+//FIXME generalise this
+//flag added to cached statbufs to turn off sorting dirs ahead of non-dirs
+#define E2_MINGLEDIRS 0400000
+#ifdef S_IFLNK
+//flag added to cached statbufs to sort dir-links like dirs
+# define E2_DIRLNK 0200000
+# define ISDIR(mode) ((S_ISDIR(mode)||(mode&E2_DIRLNK)==E2_DIRLNK) && !(mode & E2_MINGLEDIRS))
+#else
+# define ISDIR(mode) (S_ISDIR(mode) && !(mode & E2_MINGLEDIRS))
+#endif
+
 //inline gboolean e2_fs_is_dir_fast (FileInfo *info, struct stat *statbuf);
 //make this always inline ...
 //here we use stat() because we want to traverse the link
@@ -189,6 +206,7 @@ gboolean e2_fs_sniff_command_output (gchar *command, gpointer *contents,
 
 gint e2_fs_safeopen (const gchar *localpath, gint openflags, mode_t mode);
 gint e2_fs_safeclose (gint file_desc);
+gint e2_fs_writeflush (gint file_desc);
 void e2_fs_touch_config_dir (void);	//always native
 
 //away-from-normal declaration here, to avoid build problems !!
@@ -207,10 +225,16 @@ void e2_fs_FAM_check_dirty (gboolean *p1dirty, gboolean *p2dirty, gboolean *cfgd
 void e2_fs_FAM_check_dirty (gboolean *p1dirty, gboolean *p2dirty);
 void e2_fs_FAM_config_stamp (void);
 #endif
+
 #ifdef E2_FAM_KERNEL
 //void e2_fs_FAM_cancel_monitor_dir (gchar *path);
 //void e2_fs_FAM_monitor_dir (gchar *path);
 void e2_fs_FAM_clean_reports (gchar *path);
+void e2_fs_FAM_less_monitor_dir (gchar *path);
+void e2_fs_FAM_more_monitor_dir (gchar *path);
+#else
+# define e2_fs_FAM_less_monitor_dir(p)
+# define e2_fs_FAM_more_monitor_dir(p)
 #endif
 
 //flags for instructing e2_fs_tw how to perform its function
@@ -275,7 +299,10 @@ gboolean e2_fs_tw (VPATH *start_item, E2_TwResult (*callback) (),
 //a limit on level of nested dirs that will be processed in a treewalk
 #define E2_DIRNEST_LIMIT 32
 
-//gchar *e2_fs_mount_get_mountpoint_for_device (const gchar *device);
+gchar *e2_fs_mount_get_enclosing_point (const gchar *localpath);
+#ifdef E2_HAL
+gchar *e2_fs_mount_get_mountpoint_for_device (const gchar *device);
+#endif
 gboolean e2_fs_mount_is_mountpoint (VPATH *localpath);
 gboolean e2_fs_mount_is_removable (const gchar *utfpath);
 gboolean e2_fs_mount_is_ejectable (const gchar *utfpath);
@@ -288,11 +315,27 @@ gboolean e2_fs_mount_is_fusepoint (VPATH *localpath);
 void e2_fs_mount_actions_register (void);
 #endif
 
+#define E2_MOUNTCOMMAND "mount"
+#define E2_UNMOUNTCOMMAND "umount"
+//unless HAL code is patched, this is the only viable automount directory-path
+#define E2_MOUNTPLACE G_DIR_SEPARATOR_S "media"
+
+//shorthand for bsd-like os's supported here
+#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
+#define __E2BSD__
+#endif
+
 #ifdef E2_HAL
 gboolean e2_hal_device_is_removable (const gchar *devpath);
 gboolean e2_hal_device_is_ejectable (const gchar *devpath);
 void e2_hal_init (void);
 void e2_hal_disconnect (void);
 #endif
+#ifdef E2_DEVKIT
+gboolean e2_devkit_device_is_removable (const gchar *devpath);
+gboolean e2_devkit_device_is_ejectable (const gchar *devpath);
+void e2_devkit_init (void);
+void e2_devkit_disconnect (void);
+#endif
 
 #endif //ndef __E2_FS_H__
