@@ -1,6 +1,6 @@
-/* $Id: e2_fs_FAM_dnotify.c 574 2007-08-06 04:41:08Z tpgww $
+/* $Id: e2_fs_FAM_dnotify.c 1977 2009-10-12 01:11:54Z tpgww $
 
-Copyright (C) 2005-2007 tooar <tooar@gmx.net>
+Copyright (C) 2005-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -47,6 +47,9 @@ sigset_t dnotify_signal_set;
 #define DNOTIFY_FLAGS \
 	DN_ACCESS | DN_MODIFY | DN_CREATE | DN_DELETE | \
 	DN_RENAME | DN_ATTRIB | DN_MULTISHOT
+#define DNSHORT_FLAGS \
+	DN_MODIFY | DN_CREATE | DN_DELETE | \
+	DN_RENAME | DN_ATTRIB | DN_MULTISHOT
 
 //value to be stored in array to signal an unused column
 #define FD_UNUSED 0
@@ -199,7 +202,7 @@ static gint _e2_fs_FAM_dnotify_monitor_dir (gchar *path)
 	{
 		if (reports[i][REPORT_FD] != FD_UNUSED	//e.g. not cancelled
 			&& paths[i] != NULL
-			&& g_str_equal (paths[i], path))
+			&& !strcmp (paths[i], path))
 		{
 			//if so, just ref it
 			reports[i][REPORT_REF]++;
@@ -215,7 +218,7 @@ static gint _e2_fs_FAM_dnotify_monitor_dir (gchar *path)
 	//otherwise register it with dnotify
 	gchar *local = F_FILENAME_TO_LOCALE (path);
 	gint fd = e2_fs_safeopen (local, O_RDONLY | O_NONBLOCK, 0);
-	F_FREE (local);
+	F_FREE (local, path);
 
 	if (fd >= 0)
 	{
@@ -404,16 +407,15 @@ void e2_fs_FAM_disconnect (void)
 /**
 @brief change monitored directory
 
-if current-dir not also in the other pane, cancel current-dir
-monitoring
+if old-dir not also in the other pane, cancel old-dir monitoring
 if new-dir not also in other pane, start monitoring new-dir
-If new-dir is not monitored (FAM error or dir is already monitored
-in other pane) the FAM request for the pane is set to -1
+If new-dir is not monitored (FAM error or dir is already monitored in other pane)
+the FAM request for the pane is set to -1
 
-@a olddir needs trailing '/', for valid comparing with rt->path etc
+@a olddir needs trailing '/', for valid comparing with rt->view.dir etc
 
 @param olddir utf-8 string with absolute path of dir to stop monitoring
-@param rt data struct for the pane to which the cd applies, including the new dir in rt->path
+@param rt data struct for the pane to which the cd applies, including the new dir in rt->view.dir
 
 @return
 */
@@ -434,7 +436,7 @@ void e2_fs_FAM_change (gchar *olddir, E2_PaneRuntime *rt)
 #ifdef E2_VFSTMP
 		//dir when path is non-mounted
 #else
-	if (g_str_equal (olddir, ort->path))
+	if (!strcmp (olddir, ort->view.dir))
 #endif
 	{	//panes are showing same dir now
 		//connect to the other pane if need be, to get correct pane id
@@ -444,8 +446,8 @@ void e2_fs_FAM_change (gchar *olddir, E2_PaneRuntime *rt)
 #ifdef E2_VFSTMP
 		//dir when path is non-mounted
 #else
-			ort->FAMreq = _e2_fs_FAM_dnotify_monitor_dir (ort->path);
-//			printd (DEBUG, "started monitoring of other dir %s", ort->path);
+			ort->FAMreq = _e2_fs_FAM_dnotify_monitor_dir (ort->view.dir);
+//			printd (DEBUG, "started monitoring of other dir %s", ort->view.dir);
 #endif
 		}
 	}
@@ -453,16 +455,16 @@ void e2_fs_FAM_change (gchar *olddir, E2_PaneRuntime *rt)
 #ifdef E2_VFSTMP
 		//dirs when path is non-mounted
 #else
-	if (!g_str_equal (ort->path, rt->path))	//NB panes may be same at session start
+	if (strcmp (ort->view.dir, rt->view.dir))	//NB panes may be same at session start
 	{	//new dir not already monitored
-		rt->FAMreq = _e2_fs_FAM_dnotify_monitor_dir (rt->path);
-//		printd (DEBUG, "started monitoring of this dir %s", rt->path);
+		rt->FAMreq = _e2_fs_FAM_dnotify_monitor_dir (rt->view.dir);
+//		printd (DEBUG, "started monitoring of this dir %s", rt->view.dir);
 	}
 #endif
 	else
 	{
 		rt->FAMreq = -1;
-//		printd (DEBUG, "skipped monitoring of this dir %s", rt->path);
+//		printd (DEBUG, "skipped monitoring of this dir %s", rt->view.dir);
 	}
 }
 /**
@@ -481,7 +483,7 @@ void e2_fs_FAM_clean_reports (gchar *path)
 #ifdef E2_VFSTMP
 	//FIXME dir when not mounted local
 #else
-	gint fd = (g_str_equal (curr_view->dir, path)) ?
+	gint fd = (!strcmp (curr_view->dir, path)) ?
 #endif
 		curr_pane->FAMreq : other_pane->FAMreq;
 	if (fd != -1)
@@ -501,7 +503,7 @@ void e2_fs_FAM_clean_reports (gchar *path)
 		E2_UNBLOCK
 	}
 }
-/**
+/* *
 @brief Maybe get rid of last pending change-report for directory NOT @a path
 Adjusts the report-count for the other pane, if that's the parent of @a path
 Used as part of filelist refreshing
@@ -520,7 +522,7 @@ void e2_fs_FAM_clean_other_reports (gchar *path)
 #ifdef E2_VFSTMP
 	//FIXME dir when not mounted local
 #else
-	gchar *otherdir = (g_str_equal (curr_view->dir, path)) ?
+	gchar *otherdir = (!strcmp (curr_view->dir, path)) ?
 			other_view->dir : curr_view->dir;
 #endif
 	if (g_str_has_prefix (path, otherdir))
@@ -543,7 +545,7 @@ void e2_fs_FAM_clean_other_reports (gchar *path)
 #ifdef E2_VFSTMP
 	//FIXME dir when not mounted local
 #else
-		fd = (g_str_equal (curr_view->dir, otherdir)) ?
+		fd = (!strcmp (curr_view->dir, otherdir)) ?
 #endif
 			curr_pane->FAMreq : other_pane->FAMreq;
 		if (fd != -1)
@@ -564,7 +566,7 @@ void e2_fs_FAM_clean_other_reports (gchar *path)
 		}
 	}
 } */
-/**
+/* *
 @brief begin dnotify monitoring of directory @a path
 Used as part of filelist refreshing (possibly threaded),
 as part of a pair with e2_fs_FAM_cancel_monitor_dir ()
@@ -580,7 +582,7 @@ which means there was no monitoring before the cancellation
 #ifdef E2_VFSTMP
 		//dir when path is non-mounted
 #else
-		(g_str_equal (curr_pane->path, path)) ?
+		(!strcmp (curr_pane->path, path)) ?
 #endif
 		curr_pane : other_pane;
 	if (rt->FAMreq != -1)
@@ -590,7 +592,7 @@ which means there was no monitoring before the cancellation
 //		printd (DEBUG, "(as part of a resumption)");
 	}
 } */
-/**
+/* *
 @brief cancel dnotify monitoring of directory @a path
 Used as part of filelist refreshing (possibly threaded),
 as part of a pair with e2_fs_FAM_monitor_dir ()
@@ -605,7 +607,7 @@ which means there is no monitoring anyway
 #ifdef E2_VFSTMP
 		//dir when path is non-mounted
 #else
-		(g_str_equal (curr_pane->path, path)) ?
+		(!strcmp (curr_pane->path, path)) ?
 #endif
 		curr_pane : other_pane;
 	if (rt->FAMreq != -1)
@@ -615,6 +617,55 @@ which means there is no monitoring anyway
 	}
 } */
 /**
+@brief reduce the scope of dnotify monitoring of directory @a path
+There is no actual change if the fd for the relevant pane is -1 which means
+there is no monitoring anyway
+Any prior reports for the dir are not affected
+@param path utf-8 string with path of dir whose watch is to be altered
+@return
+*/
+void e2_fs_FAM_less_monitor_dir (gchar *path)
+{
+	E2_PaneRuntime *rt =
+#ifdef E2_VFSTMP
+		//dir when path is non-mounted
+#else
+		(!strcmp (curr_pane->path, path)) ?
+#endif
+		curr_pane : other_pane;
+	if (rt->FAMreq != -1)
+	{	//dir was being monitored before
+		fcntl (rt->FAMreq, F_SETSIG, DNOTIFY_SIGNAL);
+		if (fcntl (rt->FAMreq, F_NOTIFY, DNSHORT_FLAGS))
+			printd (ERROR, "Error in limiting dnotify monitoring");
+	}
+}
+/**
+@brief resume normal scope of dnotify monitoring of directory @a path after
+ a e2_fs_FAM_less_monitor_dir()
+There is no actual change if the fd for the relevant pane is -1 which means
+there is no monitoring anyway
+Any prior reports for the dir are not affected
+@param path utf-8 string with path of dir whose watch is to be altered
+@return
+*/
+void e2_fs_FAM_more_monitor_dir (gchar *path)
+{
+	E2_PaneRuntime *rt =
+#ifdef E2_VFSTMP
+		//dir when path is non-mounted
+#else
+		(!strcmp (curr_pane->path, path)) ?
+#endif
+		curr_pane : other_pane;
+	if (rt->FAMreq != -1)
+	{	//dir was being monitored before
+		fcntl (rt->FAMreq, F_SETSIG, DNOTIFY_SIGNAL);
+		if (fcntl (rt->FAMreq, F_NOTIFY, DNOTIFY_FLAGS))
+			printd (ERROR, "Error in reinstating full dnotify monitoring");
+	}
+}
+/**
 @brief setup monitoring of config file
 
 @return TRUE if the connection was successfully established
@@ -625,7 +676,7 @@ gboolean e2_fs_FAM_monitor_config (void)
 	gchar *config_parent =  g_path_get_dirname (config_file);
 	gboolean retval = FALSE;
 
-	if (!g_str_equal (config_parent, "."))
+	if (strcmp (config_parent, "."))
 	{
 		app.FAMreq = _e2_fs_FAM_dnotify_monitor_dir (config_parent);
 		if (app.FAMreq >= 0)
@@ -655,7 +706,7 @@ gboolean e2_fs_FAM_cancel_monitor_config (void)
 {
 	gchar *config_file = g_build_filename (e2_cl_options.config_dir, default_config_file, NULL);
 	gchar *config_parent =  g_path_get_dirname (config_file);
-	gboolean retval = (!g_str_equal (config_parent, ".")
+	gboolean retval = (strcmp (config_parent, ".")
 		 && _e2_fs_FAM_dnotify_demonitor_dir (app.FAMreq));
 
 	if (retval)
@@ -699,17 +750,17 @@ void e2_fs_FAM_poll (gboolean *p1altered, gboolean *p2altered, gboolean *cfgalte
 #ifdef E2_VFSTMP
 	//FIXME dir when not mounted local
 #else
-	gchar *local = F_FILENAME_TO_LOCALE (app.pane1.path);
+	gchar *local = D_FILENAME_TO_LOCALE (app.pane1.path); //always dup, to avoid dirchange race
 #endif
 	*p1altered = e2_fs_access (local, R_OK | X_OK E2_ERR_PTR());
-	F_FREE (local);
+	g_free (local);
 #ifdef E2_VFSTMP
 	//FIXME dir when not mounted local
 #else
-	local = F_FILENAME_TO_LOCALE (app.pane2.path);
+	local = D_FILENAME_TO_LOCALE (app.pane2.path); //always dup, to avoid dirchange race
 #endif
 	*p2altered = e2_fs_access (local, R_OK | X_OK E2_ERR_PTR());
-	F_FREE (local);
+	g_free (local);
 	//not interested whether config is still present
 	*cfgaltered = FALSE;
 
