@@ -1,5 +1,6 @@
-/* $Id: e2_fs_mount.c 853 2008-04-07 10:38:17Z tpgww $
+/* $Id: e2_fs_mount.c 1840 2009-09-17 01:15:11Z tpgww $
 
+Copyright (C) 2007-2009 <tooar@emelfm2.net>
 Copyright (C) 2004-2007 Florian Zaehringer <flo.zaehringer@web.de>
 Copyright (C) 1999-2004 Bill Wilson <bill@gkrellm.net>
 
@@ -34,11 +35,6 @@ Functions dealing with fstab / mtab have in part been sourced from gkrellm
 #include <string.h>
 #include <unistd.h>
 
-//shorthand for bsd-like os's supported here
-#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
-#define __E2BSD__
-#endif
-
 //some file-system identifiers (from <linux/magic.h>, which isn't generally available)
 #ifndef MSDOS_SUPER_MAGIC
 # define MSDOS_SUPER_MAGIC 0x4d44
@@ -209,16 +205,15 @@ gboolean e2_fs_mount_is_mountpoint (VPATH *localpath)
 	if (localpath->spacedata != NULL)
 		return FALSE;	//mountpoints are always local
 #endif
+	//root path is not detected by mountpoints lister
+	if (!strcmp (VPSTR (localpath), G_DIR_SEPARATOR_S))
+		return TRUE;
 
 	if (e2_fs_lstat (localpath, &sb E2_ERR_NONE()) || !S_ISDIR (sb.st_mode))
 		return FALSE;
 
 	gboolean matched = FALSE;
-#ifdef E2_VFS
 	gchar *utf = F_FILENAME_FROM_LOCALE (VPSTR(localpath));
-#else
-	gchar *utf = F_FILENAME_FROM_LOCALE (localpath);
-#endif
 	GList *points = e2_fs_mount_get_mounts_list ();
 	if (points != NULL)
 	{
@@ -236,19 +231,22 @@ gboolean e2_fs_mount_is_mountpoint (VPATH *localpath)
 		}
 	}
 #endif
-	F_FREE (utf);
+	F_FREE (utf, VPSTR(localpath));
 	return matched;
 }
-/* *
+#ifdef E2_HAL
+/**
 @brief find mountpoint dir whose device is @a device
 @param device localised string with device of mountpoint to be found
 
 @return newly allocated string with mountpoint, or NULL upon error or no match
 */
-/*gchar *e2_fs_mount_get_mountpoint_for_device (const gchar *device)
+gchar *e2_fs_mount_get_mountpoint_for_device (const gchar *device)
 {
+	//FIXME convert this func to check & get any related string(s):
+	//	device, mountpoint, mount options
 	gchar *mountpath = NULL;
-	//CHECKME lock data file while accessing it ??
+	//CHECKME lock data file while accessing it ?? re-entrant forms of functions ?
 #if defined(__E2BSD__) || defined(__linux__)
     struct fstab *fs;
 
@@ -336,7 +334,7 @@ gboolean e2_fs_mount_is_mountpoint (VPATH *localpath)
 #endif	//which OS
 	return mountpath;
 }
-*/
+#endif //def E2_HAL
 /**
 @brief add @a dir to list @a mounts
 
@@ -512,7 +510,7 @@ static gboolean _e2_fs_mount_permission (gchar* device, gchar *options)
 #endif
 } */
 
-#ifdef E2_HAL
+#if defined (E2_HAL) || defined (E2_DEVKIT)
 /**
 @brief get device corresponding to mounted directory @a utfpath
 @param absolute path, utf8, may have trailing separator
@@ -791,16 +789,16 @@ gchar *_e2_fs_get_mounted_device (const gchar *utfpath)
 #endif	//which OS
 //=========================
 	g_free (clean);
-	F_FREE (local);
+	F_FREE (local, clean);
 	return bestdev;
 }
-#endif	//def E2_HAL
+#endif	//def (E2_HAL) || def (E2_DEVKIT)
 /**
-@brief create list of mounted partitions in the native filesystem
+@brief create list of mounted partitions (other than "/") in the native filesystem
 
-The list is created by the appropriate os-specifc protocol
-
-@return list of mounted partitions (utf-8 strings), or NULL
+The list is created by the appropriate os-specifc protocol.
+Root dir is exlcluded because we don't need to ever unmount that.
+@return list of mounted partitions (UTF-8 strings), or NULL
 */
 GList *e2_fs_mount_get_mounts_list (void)	//gboolean permcheck)
 {
@@ -940,9 +938,10 @@ GList *e2_fs_mount_get_mounts_list (void)	//gboolean permcheck)
 	return mounts_list;
 }
 /**
-@brief create list of mountable partitions in the native filesystem
+@brief create list of mountable partitions (other than "/") in the native filesystem
 
-The list is created by the appropriate os-specifc protocol
+The list is created by the appropriate os-specifc protocol.
+Root dir is exlcluded because we don't need to ever unmount that.
 
 @return list of partitions (utf-8 strings), or NULL
 */
@@ -1058,13 +1057,53 @@ GList *e2_fs_mount_get_mountable_list (void)	//gboolean permcheck)
 #endif	//which OS
 	return fstab_list;
 }
-#ifdef E2_HAL
+/**
+@brief find the top dir of the device where @a localpath is
+This is intended only for native dirs, so no VPATH involved
+It analyses mounted-devices data, so is only meaningful when @a localpath is mounted
+@param localpath absolute localised path of an item in local filesystem, may have trailing separator
+@return newly allocated localised path, defaults to "/"
+*/
+gchar *e2_fs_mount_get_enclosing_point (const gchar *localpath)
+{
+	GList *mounts = e2_fs_mount_get_mounts_list ();	//list of UTF-8 paths other than /
+	if (mounts != NULL)
+	{
+		GList *member;
+		guint tlen = 0;
+		const gchar *thispoint = NULL;
+		gchar *converted = F_FILENAME_FROM_LOCALE (localpath);
+		//assume that the longest matching mountpoint path is the one we want
+		for (member = mounts; member != NULL; member = member->next)
+		{
+			gchar *s = (gchar *)member->data;
+			if (g_str_has_prefix (converted, s))
+			{
+				guint len = 0;	//warning prevention
+				if (thispoint == NULL || tlen <= (len = strlen (s)))
+				{
+					thispoint = s;
+					tlen = len;
+				}
+			}
+		}
+		F_FREE (converted, localpath);
+		if (thispoint != NULL)
+			converted = D_FILENAME_TO_LOCALE (thispoint);
+		else
+			converted = g_strdup (G_DIR_SEPARATOR_S);	//NULL;
+		e2_list_free_with_data (&mounts);
+		return converted;
+	}
+	return g_strdup (G_DIR_SEPARATOR_S);	//NULL;
+}
+#if defined (E2_HAL) || defined (E2_DEVKIT)
 /**
 @brief check whether device on which @a utfpath resides is removable
-@param utfpath absolute path string, utf8, may be quoted, and/or have spaces
-and/or trailing separator
 This is intended only for native dirs, so no VPATH involved
 It analyses mounted-devices data, so can only be used when @a utfpath is mounted
+@param utfpath absolute path string, utf8, may be quoted, and/or have spaces
+and/or trailing separator
 @return TRUE if the dir is on a removable device
 */
 gboolean e2_fs_mount_is_ejectable (const gchar *utfpath)
@@ -1073,7 +1112,12 @@ gboolean e2_fs_mount_is_ejectable (const gchar *utfpath)
 	gchar *device = _e2_fs_get_mounted_device (utfpath);
 	if (device != NULL)
 	{
-		retval = e2_hal_device_is_ejectable (device);
+		retval =
+# ifdef E2_HAL
+		e2_hal_device_is_ejectable (device);
+# else
+		e2_devkit_device_is_ejectable (device);
+# endif
 		g_free (device);
 	}
 	else
@@ -1092,11 +1136,16 @@ It analyses mounted-devices data, so can only be used when @a utfpath is mounted
 gboolean e2_fs_mount_is_removable (const gchar *utfpath)
 {
 	gboolean retval;
-#ifdef E2_HAL
+#if defined (E2_HAL) || defined (E2_DEVKIT)
 	gchar *device = _e2_fs_get_mounted_device (utfpath);
 	if (device != NULL)
 	{
-		retval = e2_hal_device_is_removable (device);
+		retval =
+# ifdef E2_HAL
+		e2_hal_device_is_removable (device);
+# else
+		e2_devkit_device_is_removable (device);
+# endif
 		g_free (device);
 	}
 	else
@@ -1145,7 +1194,7 @@ gboolean e2_fs_mount_is_cased (ViewInfo *view)
 		else
 		{
 #endif
-			gchar *local = F_FILENAME_TO_LOCALE (view->dir);
+			gchar *local = D_FILENAME_TO_LOCALE (view->dir); //always dup, to avoid dirchange race
 //=========== OS-specific stuff =============
 #if defined(__linux__)
 #include <sys/vfs.h>
@@ -1171,7 +1220,7 @@ retry:
 						break;
 				}
 			}
-#ifdef E2_HAL
+#if defined (E2_HAL) || defined (E2_DEVKIT)
 			if (e2_fs_mount_is_removable (view->dir)) //removable device
 #endif
 				view->case_sensitive_names |= E2FSCASE_NOCACHE;
@@ -1192,7 +1241,7 @@ retry:
 				else
 					view->case_sensitive_names = E2FSCASE_SENSITIVE;
 			}
-#ifdef E2_HAL
+#if defined (E2_HAL) || defined (E2_DEVKIT)
 			if (e2_fs_mount_is_removable (view->dir)) //on removable device
 #endif
 				view->case_sensitive_names |= E2FSCASE_NOCACHE;
@@ -1213,7 +1262,7 @@ retry:
 				else
 					view->case_sensitive_names = E2FSCASE_SENSITIVE;
 			}
-#ifdef E2_HAL
+#if defined (E2_HAL) || defined (E2_DEVKIT)
 			if (e2_fs_mount_is_removable (view->dir)) //on removable device
 #endif
 				view->case_sensitive_names |= E2FSCASE_NOCACHE;
@@ -1234,13 +1283,13 @@ retry:
 				else
 					view->case_sensitive_names = E2FSCASE_SENSITIVE;
 			}
-#ifdef E2_HAL
+#if defined (E2_HAL) || defined (E2_DEVKIT)
 			if (e2_fs_mount_is_removable (view->dir)) //on removable place
 #endif
 				view->case_sensitive_names |= E2FSCASE_NOCACHE;
 #endif	//which OS
 //=========================
-			F_FREE (local);
+			g_free (local);
 #ifdef E2_VFSTMP
 		}
 #endif
@@ -1255,10 +1304,8 @@ retry:
 */
 void e2_fs_mount_actions_register (void)
 {
-	gchar *action = g_strconcat (_A(57),".",_A(24), NULL);
-	e2_action_register (action, E2_ACTION_TYPE_ITEM,
-		e2_menu_create_mounts_menu, NULL, TRUE,
-		E2_ACTION_EXCLUDE_MENU,
-		NULL);
+	E2_Action action =
+	{g_strconcat(_A(61),".",_A(27),NULL),e2_menu_create_mounts_menu,TRUE,E2_ACTION_TYPE_ITEM,E2_ACTION_EXCLUDE_MENU,NULL,NULL};
+	e2_action_register (&action);
 }
 #endif //def E2_FS_MOUNTABLE
