@@ -1,6 +1,6 @@
-/* $Id: e2_fs_walk.c 744 2007-11-22 21:36:34Z tpgww $
+/* $Id: e2_fs_walk.c 1683 2009-05-13 00:00:51Z tpgww $
 
-Copyright (C) 2005-2007 tooar <tooar@gmx.net>
+Copyright (C) 2005-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -203,24 +203,21 @@ static E2_TwResult _e2_fs_tw_link (VPATH *localpath)
 	else
 	{
 		buf[read] = '\0';
-		gchar *converted1 = F_FILENAME_FROM_LOCALE (VPSTR(localpath));
-		gchar *converted2 = F_FILENAME_FROM_LOCALE (buf);
-		gchar *targetpath = e2_utils_translate_relative_path (converted1, converted2);
-		F_FREE (converted1);
-		F_FREE (converted2);
+		//CHECKME should not matter that localpath is not necessarily a dir
+		gchar *targetpath = e2_utils_translate_relative_path (VPSTR(localpath), buf);
 		//targetpath has a trailing /, with which access() can't cope
-		*(targetpath + strlen (targetpath) - sizeof (gchar)) = '\0';
+		gint len = strlen (targetpath);
+		if (len > 1)
+			*(targetpath + len - sizeof (gchar)) = '\0';
 
-		converted1 = F_FILENAME_TO_LOCALE (targetpath);
 		//access() looks through links
 #ifdef E2_VFS
-		VPATH data = { converted1, localpath->spacedata };
+		VPATH data = { targetpath, localpath->spacedata };
 		code = (e2_fs_access (&data, F_OK E2_ERR_NONE())) ? E2TW_SLN : E2TW_SL;
 #else
-		code = (e2_fs_access (converted1, F_OK E2_ERR_NONE())) ? E2TW_SLN : E2TW_SL;
+		code = (e2_fs_access (targetpath, F_OK E2_ERR_NONE())) ? E2TW_SLN : E2TW_SL;
 #endif
 		g_free (targetpath);
-		F_FREE (converted1);
 	}
 	return code;
 }
@@ -449,7 +446,7 @@ static E2_TwResult _e2_fs_tw_dir (VPATH *localpath, gint depth, E2_TwData *twdat
 	//to open them, after all non-dirs at this level have been processed
 	parked_dirs = NULL;
 #ifdef E2_VFS
-	ddata.localpath = itempath;
+	ddata.path = itempath;
 	ddata.spacedata = localpath->spacedata;
 #endif
 	//walk the list of entries in the dir (tree-depth now = newdepth)
