@@ -1,6 +1,6 @@
-/* $Id: e2_button.c 682 2007-10-28 05:33:18Z tpgww $
+/* $Id: e2_button.c 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -21,57 +21,190 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "emelfm2.h"
 #include "e2_button.h"
 
-E2_Button E2_BUTTON_OK =
-	{ "",N_("_OK"),GTK_STOCK_OK, NULL,
-		E2_BTN_DEFAULT,E2_BTN_DEFAULT,GTK_RESPONSE_OK};
-E2_Button E2_BUTTON_CANCEL =
-	{ "",N_("_Cancel"),GTK_STOCK_CANCEL,NULL,0,0,GTK_RESPONSE_CANCEL};
 E2_Button E2_BUTTON_YES =
-	{ "",N_("_Yes"),GTK_STOCK_YES, NULL,
+	{ NULL,GTK_STOCK_YES, NULL,	//label is always context-dependent
 		E2_BTN_DEFAULT,E2_BTN_DEFAULT,GTK_RESPONSE_YES};
 E2_Button E2_BUTTON_NO =
-	{ "",N_("_No"),GTK_STOCK_NO,NULL,0,0,GTK_RESPONSE_NO};
-E2_Button E2_BUTTON_YESTOALL =
-//	{ "",N_("Yes to _all"),GTK_STOCK_APPLY,NULL,0,0,E2_RESPONSE_YESTOALL};
-	{ "",N_("Yes to _all"),"apply_all_"E2IP".png",NULL,0,0,E2_RESPONSE_YESTOALL};
-E2_Button E2_BUTTON_NOTOALL =
-	{ "",N_("_Stop"),GTK_STOCK_STOP,NULL,0,0,E2_RESPONSE_NOTOALL};
+	{ NULL,GTK_STOCK_NO,NULL,0,0,GTK_RESPONSE_NO};	//label is always context-dependent
+E2_Button E2_BUTTON_CANCEL =
+	{ N_("_Cancel"),GTK_STOCK_CANCEL,NULL,0,0,E2_RESPONSE_NOTOALL};
 E2_Button E2_BUTTON_APPLY =
-	{ "",N_("_Apply"),GTK_STOCK_APPLY,NULL,0,0,GTK_RESPONSE_APPLY};
+	{ N_("Commi_t"),GTK_STOCK_OK,NULL,0,0,E2_RESPONSE_APPLY}; //often co-used with _Cancel, _All
 E2_Button E2_BUTTON_APPLYTOALL =
-//	{ "",N_("_Apply to all"),GTK_STOCK_APPLY,NULL,0,0,E2_RESPONSE_APPLYTOALL};
-	{ "",N_("_Apply to all"),"apply_all_"E2IP".png",NULL,0,0,E2_RESPONSE_APPLYTOALL};
+	{ N_("_All"),"apply_all"E2ICOND,N_("Change all unprocessed items"),
+		E2_BTN_TIPPED,E2_BTN_TIPPED,E2_RESPONSE_APPLYTOALL};
+E2_Button E2_BUTTON_DISCARD =
+	{ N_("_Discard"),GTK_STOCK_DISCARD,N_("Abandon any changes"),
+		E2_BTN_TIPPED,E2_BTN_TIPPED,E2_RESPONSE_DISCARD};
 E2_Button E2_BUTTON_REFRESH =
-	{ "",N_("_Refresh"),GTK_STOCK_REFRESH,NULL,0,0,E2_RESPONSE_REFRESH};
+	{ N_("_Refresh"),GTK_STOCK_REFRESH,NULL,0,0,E2_RESPONSE_REFRESH};
 E2_Button E2_BUTTON_CLOSE =
-	{ "",N_("_Close"),GTK_STOCK_CLOSE,NULL,0,0,GTK_RESPONSE_CLOSE};
+	{ N_("_Close"),GTK_STOCK_CLOSE,NULL,0,0,GTK_RESPONSE_CLOSE};
 E2_Button E2_BUTTON_CREATE =
-	{ "",N_("C_reate"),GTK_STOCK_YES,NULL,0,0,E2_RESPONSE_CREATE};  //co-use with close/cancel
+	{ N_("C_reate"),GTK_STOCK_YES,NULL,0,0,E2_RESPONSE_CREATE};  //co-use with close/cancel
 E2_Button E2_BUTTON_REMOVE =
-	{ "",N_("_Remove"),GTK_STOCK_REMOVE,NULL,0,0,E2_RESPONSE_REMOVE}; //check no co-use with refresh
+	{ N_("_Remove"),GTK_STOCK_REMOVE,NULL,0,0,E2_RESPONSE_REMOVE}; //check no co-use with refresh
+E2_Button E2_BUTTON_MORE =
+	{ N_("_Apply"),"proceed"E2ICOND,NULL,0,0,E2_RESPONSE_MORE};
 
+//array of labels which may be applied to pending or existing dialog-buttons
+//(usually yes or no) indexed by E2_ButtonText enum
+static const gchar *defined_button_labels[] =
+{
+	//+ve choices
+	N_("Commi_t"),	//might appear with _All or _Cancel
+	N_("_Proceed"),
+	N_("_Delete"),
+	//-ve choices
+	N_("_Cancel"),
+	N_("_Omit"),	//often appears with "_Stop"
+	N_("_Keep"),	//typically with "Delete"
+};
+//array of tips corresponding to above labels, NULL when no tip applies
+static const gchar *defined_button_tips[] =
+{
+	//+ve choices
+	NULL,
+	NULL,
+	NULL,
+	//-ve choices
+	N_("Do not do anything"),
+	N_("Do not change this item"),
+	N_("Do not change this"),
+};
+
+/**
+@brief initialize translated labels and tips for defined buttons
+
+@return
+*/
 void e2_button_setup_labels (void)
 {
-	E2_BUTTON_OK.label = gettext (E2_BUTTON_OK.default_label);
-	E2_BUTTON_CANCEL.label = gettext (E2_BUTTON_CANCEL.default_label);
-	E2_BUTTON_YES.label = gettext (E2_BUTTON_YES.default_label);
-	E2_BUTTON_NO.label = gettext (E2_BUTTON_NO.default_label);
-	E2_BUTTON_YESTOALL.label = gettext (E2_BUTTON_YESTOALL.default_label);
-	E2_BUTTON_NOTOALL.label = gettext (E2_BUTTON_NOTOALL.default_label);
-	E2_BUTTON_APPLY.label = gettext (E2_BUTTON_APPLY.default_label);
-	E2_BUTTON_APPLYTOALL.label = gettext (E2_BUTTON_APPLYTOALL.default_label);
-	E2_BUTTON_REFRESH.label = gettext (E2_BUTTON_REFRESH.default_label);
-	E2_BUTTON_CLOSE.label = gettext (E2_BUTTON_CLOSE.default_label);
-	E2_BUTTON_CREATE.label = gettext (E2_BUTTON_CREATE.default_label);
-	E2_BUTTON_REMOVE.label = gettext (E2_BUTTON_REMOVE.default_label);
-}
+	guint i, count;
+	count = sizeof(defined_button_labels)/sizeof(defined_button_labels[0]);
+	for (i = 0; i < count; i++)
+	{
+		defined_button_labels[i] = gettext (defined_button_labels[i]);
+		if (defined_button_tips[i] != NULL)
+			defined_button_tips[i] = gettext (defined_button_tips[i]);
+	}
 
+	E2_BUTTON_YES.label = defined_button_labels[BTN_YES_COMMIT];
+	E2_BUTTON_NO.label = defined_button_labels[BTN_NO_CANCEL];
+	E2_BUTTON_CANCEL.label = gettext (E2_BUTTON_CANCEL.label);
+	E2_BUTTON_APPLY.label = gettext (E2_BUTTON_APPLY.label);
+	E2_BUTTON_APPLYTOALL.label = gettext (E2_BUTTON_APPLYTOALL.label);
+	E2_BUTTON_DISCARD.label = gettext (E2_BUTTON_DISCARD.label);
+	E2_BUTTON_REFRESH.label = gettext (E2_BUTTON_REFRESH.label);
+	E2_BUTTON_CLOSE.label = gettext (E2_BUTTON_CLOSE.label);
+	E2_BUTTON_CREATE.label = gettext (E2_BUTTON_CREATE.label);
+	E2_BUTTON_REMOVE.label = gettext (E2_BUTTON_REMOVE.label);
+	E2_BUTTON_MORE.label = gettext (E2_BUTTON_MORE.label);
+
+	E2_BUTTON_APPLYTOALL.tip = gettext (E2_BUTTON_APPLYTOALL.tip);
+	E2_BUTTON_DISCARD.tip = gettext (E2_BUTTON_DISCARD.tip);
+	E2_BUTTON_MORE.tip = gettext (E2_BUTTON_MORE.tip);
+}
+/**
+@brief set label and tip for pending button @a button, and maybe the icon
+This does not affect the "default" label
+@param button pointer to button data struct
+@param base pointer to static defined-button data struct, or NULL
+@param type 0-based index into labels array
+
+@return
+*/
+void e2_button_derive (E2_Button *button, E2_Button *base, E2_ButtonText type)
+{
+	if (base != NULL)	//populate struct if wanted
+		*button = *base;
+	if (type >= 0 && type < (sizeof(defined_button_labels)/sizeof(defined_button_labels[0])))
+	{
+		button->label = defined_button_labels[type];
+		button->tip = (gchar *)defined_button_tips[type];
+		if (button->tip != NULL)
+			button->showflags |= E2_BTN_TIPPED;
+		else
+			button->showflags &= ~E2_BTN_TIPPED;
+		if (type == BTN_NO_CANCEL)
+			button->stock = GTK_STOCK_CANCEL;
+	}
+}
+/**
+@brief set label and tip for existing button @a button
+This does not affect the button's "default" label
+@param button the button widget to be updated
+@param type 0-based index into labels array
+
+@return
+*/
+void e2_button_set_indexed_text (GtkWidget *button, E2_ButtonText type)
+{
+	if (type >= 0 && type < (sizeof(defined_button_labels)/sizeof(defined_button_labels[0])))
+	{
+		e2_button_set_label (button, defined_button_labels[type]);
+		if (defined_button_tips[type] != NULL)
+		{
+			e2_widget_set_safetip (button, defined_button_tips[type]);
+		}
+		else
+		{
+#ifdef USE_GTK2_12TIPS
+			gtk_widget_set_has_tooltip (button, FALSE);
+#else
+			GtkTooltipsData *tdata = gtk_tooltips_data_get (button);
+			if (tdata != NULL)
+			{
+				g_free (tdata->tip_text);
+				tdata->tip_text = NULL;
+				g_free (tdata->tip_private);
+				tdata->tip_private = NULL;
+			}
+#endif
+		}
+	}
+}
+/**
+@brief set new label for existing widget @a button
+
+e2 uses non-standard dialog buttons, containing alignment
+containing h/vbox, the latter with label packed after icon (if any)
+
+@param button the button widget to be updated
+@param label string optionally with mnemonic
+
+@return
+*/
+void e2_button_set_label (GtkWidget *button, const gchar *label)
+{
+	if (label != NULL && *label != '\0')
+	{
+		GtkWidget *bbox =
+#ifdef USE_GTK2_14
+			gtk_bin_get_child (GTK_BIN (button));
+		bbox = gtk_bin_get_child (GTK_BIN (bbox));
+		GList *children = gtk_container_get_children (GTK_CONTAINER (bbox));
+		GtkWidget *child = (GtkWidget*) children->data;
+		if (!GTK_IS_LABEL (child))
+			child = (GtkWidget*) children->next->data;
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (child), label);
+		g_list_free (children);
+#else
+			GTK_BIN (GTK_BIN (button)->child)->child;
+		GtkBoxChild *child = (GtkBoxChild *) GTK_BOX (bbox)->children->data;
+		if (!GTK_IS_LABEL (child->widget))
+			child = (GtkBoxChild *) GTK_BOX (bbox)->children->next->data;
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (child->widget), label);
+#endif
+	}
+}
 /**
 @brief set new (or no) icon for existing button @a button
 
-e2 uses non-standard buttons, containing alignment
-containing h/vbox, the latter with image (if any) packed 1st
-@a stock may be a gtk stock name or a custom icon filename
+e2 uses non-standard dialog buttons, containing alignment containing h/vbox,
+the latter with image (if any) packed first. @a stock may be a gtk stock name
+or a custom icon filename. The new image is shown immediately, so this func is
+not intended for buttons which are ultimately included in a "show all".
 
 @param button the button widget to be updated
 @param stock string describing icon or NULL to clear the icon
@@ -80,16 +213,38 @@ containing h/vbox, the latter with image (if any) packed 1st
 */
 void e2_button_set_image (GtkWidget *button, gchar *stock)
 {
-	if (e2_option_bool_get ("dialog-button-icons"))
+	gint choice = e2_option_sel_get ("dialog-button-icons");
+	if (choice == 0)
+	{
+#ifdef USE_GTK2_18
+		gboolean show;
+		GtkSettings* defs = gtk_settings_get_default ();
+		g_object_get (G_OBJECT (defs), "gtk-button-images", &show, NULL);
+		if (show)
+#endif
+			choice = 1;
+	}
+	if (choice == 1)
 	{
 		GtkWidget *image = (stock != NULL) ?
 			e2_widget_get_icon (stock, GTK_ICON_SIZE_BUTTON) : NULL;
 		if (stock == NULL || image != NULL)
 		{
-			GtkWidget *bbox = GTK_BIN (GTK_BIN (button)->child)->child;
+			GtkWidget *bbox =
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (button));
+			bbox = gtk_bin_get_child (GTK_BIN (bbox));
+			GList *children = gtk_container_get_children (GTK_CONTAINER (bbox));
+			GtkWidget *child1 = (GtkWidget *) children->data;
+			if (GTK_IS_IMAGE (child1))
+				gtk_container_remove (GTK_CONTAINER (bbox), child1);
+			g_list_free (children);
+#else
+				GTK_BIN (GTK_BIN (button)->child)->child;
 			GtkBoxChild *child1 = (GtkBoxChild *) GTK_BOX (bbox)->children->data;
-			if (GTK_WIDGET_TYPE (child1->widget) == g_type_from_name ("GtkImage"))
+			if (GTK_IS_IMAGE (child1->widget))
 				gtk_container_remove (GTK_CONTAINER (bbox), child1->widget);
+#endif
 			if (image != NULL)
 			{
 				gtk_box_pack_start (GTK_BOX (bbox), image, FALSE, FALSE, 0);
@@ -100,41 +255,27 @@ void e2_button_set_image (GtkWidget *button, gchar *stock)
 	}
 }
 /**
-@brief get button-button which can be default and focused
-
-@param label button label which may contain markup and/or mnemonic
-@param stock string describing icon, or NULL
-@param tip tooltip for the button, or NULL
-@param callback callback function for "clicked" signal, or NULL
-@param data data specified for the callback
-
-@return the button widget
-*/
-GtkWidget *e2_button_get (gchar *label, gchar *stock, gchar *tip,
-	gpointer callback, gpointer data)
-{
-	GtkWidget *button = e2_button_get_full (label, stock, GTK_ICON_SIZE_BUTTON,
-		tip, callback, data, E2_BUTTON_CAN_DEFAULT | E2_BUTTON_CAN_FOCUS);
-	gtk_widget_show_all (button);
-	return button;
-}
-/**
 @brief create button
 This is used for toolbar buttons, and (maybe indirectly) for dialog buttons
 @param label button label which may contain markup and/or mnemonic, or NULL
 @param stock string describing icon, or NULL
 @param size code for size of button-icon
 @param tip tooltip for the button, or NULL
-@param callback callback function for "clicked" signal, or NULL
+@param callback callback function for button's "clicked" signal, or NULL
 @param data data specified for the callback
 @param flags flags indicating button properties
 
-@return the button widget
+@return the (shown) button widget
 */
-GtkWidget *e2_button_get_full (gchar *label, gchar *stock, GtkIconSize size,
-	gchar *tip, gpointer callback, gpointer data, E2_ButtonFlags flags)
+GtkWidget *e2_button_get_full (const gchar *label, const gchar *stock, GtkIconSize size,
+	const gchar *tip, void (*callback)(/*GtkButton*,gpointer*/), gpointer data,
+	E2_ButtonFlags flags)
 {
 	GtkWidget *button = gtk_button_new ();
+#ifdef USE_GTK2_18
+	gtk_widget_set_can_default (button, (flags & E2_BUTTON_CAN_DEFAULT) > 0);
+	gtk_widget_set_can_focus (button, (flags & E2_BUTTON_CAN_FOCUS) > 0);
+#else
 	if (flags & E2_BUTTON_CAN_DEFAULT)
 		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
 	else
@@ -143,15 +284,10 @@ GtkWidget *e2_button_get_full (gchar *label, gchar *stock, GtkIconSize size,
 		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_FOCUS);
 	else
 		GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+#endif
 
 	if (tip != NULL)
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		button, tip);
-
+		e2_widget_set_safetip (button, tip);
 	if (callback != NULL)
 		g_signal_connect_data (G_OBJECT (button), "clicked",
 			G_CALLBACK (callback), data,
@@ -159,7 +295,7 @@ GtkWidget *e2_button_get_full (gchar *label, gchar *stock, GtkIconSize size,
 
 	GtkWidget *bbox;
 	if (flags & E2_BUTTON_ICON_ABOVE_LABEL)
-		bbox = gtk_vbox_new (FALSE, 2);	//fixed gap, not E2_PADDING_XSMALL
+		bbox = gtk_vbox_new (FALSE, 2);	//constant gap, not E2_PADDING_XSMALL
 	else
 		bbox = gtk_hbox_new (FALSE, 2);
 
@@ -168,7 +304,7 @@ GtkWidget *e2_button_get_full (gchar *label, gchar *stock, GtkIconSize size,
 	if (stock != NULL)
 	{
 		GtkWidget *img = e2_widget_get_icon (stock, size);
-		if ((flags & E2_BUTTON_SHOW_MISSING_ICON) && (img == NULL))
+		if (img == NULL && (flags & E2_BUTTON_SHOW_MISSING_ICON))
 			img = e2_widget_get_icon (GTK_STOCK_MISSING_IMAGE, size);
 		if (img != NULL)
 		{
@@ -183,6 +319,8 @@ GtkWidget *e2_button_get_full (gchar *label, gchar *stock, GtkIconSize size,
 		GtkWidget *lab = gtk_label_new ("");
 		gtk_label_set_markup_with_mnemonic (GTK_LABEL (lab), label);
 		gtk_box_pack_start (GTK_BOX (bbox), lab, TRUE, TRUE, 0);
+		if (stock != NULL)
+			gtk_widget_add_mnemonic_label (button, lab); //needed to make it work dunno why
 		need_align = TRUE;
 	}
 
@@ -194,10 +332,26 @@ GtkWidget *e2_button_get_full (gchar *label, gchar *stock, GtkIconSize size,
 	}
 	else	//should never happen
 		gtk_container_add (GTK_CONTAINER (button), bbox);
-
 	return button;
 }
 /**
+@brief get button-widget which can be default and focused
+Downstream code requires BGL closed
+@param label button label which may contain markup and/or mnemonic
+@param stock string describing icon, or NULL
+@param tip tooltip for the button, or NULL
+@param callback callback function for button's "clicked" signal, or NULL
+@param data data to send to the callback
+
+@return the (shown) button widget
+*/
+GtkWidget *e2_button_get (const gchar *label, const gchar *stock, const gchar *tip,
+	void (*callback)(/*GtkButton*,gpointer*/), gpointer data)
+{
+	return (e2_button_get_full (label, stock, GTK_ICON_SIZE_BUTTON,
+		tip, callback, data, E2_BUTTON_CAN_DEFAULT | E2_BUTTON_CAN_FOCUS));
+}
+/**
 @brief add button-sized button to the start of @a box
 
 @param box widget to hold the button
@@ -212,11 +366,10 @@ GtkWidget *e2_button_get_full (gchar *label, gchar *stock, GtkIconSize size,
 @return the button widget
 */
 GtkWidget *e2_button_add (GtkWidget *box, gboolean exp, guint pad, gchar *label,
-	gchar *stock, gchar *tip, gpointer callback, gpointer data)
+	gchar *stock, gchar *tip, void (*callback)(/*GtkButton*,gpointer*/), gpointer data)
 {
 	GtkWidget *button = e2_button_get (label, stock, tip, callback, data);
 	gtk_box_pack_start (GTK_BOX (box), button, exp, TRUE, pad);
-	gtk_widget_show_all (button);
 	return button;
 }
 /**
@@ -233,12 +386,11 @@ GtkWidget *e2_button_add (GtkWidget *box, gboolean exp, guint pad, gchar *label,
 
 @return the button widget
 */
-GtkWidget *e2_button_add_end (GtkWidget *box, gboolean exp, guint pad,
-	gchar *label, gchar *stock, gchar *tip, gpointer callback, gpointer data)
+GtkWidget *e2_button_add_end (GtkWidget *box, gboolean exp, guint pad, gchar *label,
+	gchar *stock, gchar *tip, void (*callback)(/*GtkButton*,gpointer*/), gpointer data)
 {
 	GtkWidget *button = e2_button_get (label, stock, tip, callback, data);
 	gtk_box_pack_end (GTK_BOX (box), button, exp, TRUE, pad);
-	gtk_widget_show_all (button);
 	return button;
 }
 /**
@@ -249,33 +401,31 @@ GtkWidget *e2_button_add_end (GtkWidget *box, gboolean exp, guint pad,
 @param label button label which may contain markup and/or mnemonic
 @param tip tooltip for the button, or NULL
 @param callback callback function for "toggled" signal, or NULL
-@param data data specified for the callback
+@param data data specified for @a callback
 
 @return the button widget
 */
-GtkWidget *e2_button_get_toggle (gboolean check, gboolean state,
-	gchar *label, gchar *tip, gpointer callback, gpointer data)
+GtkWidget *e2_button_get_toggle (gboolean check, gboolean state, gchar *label,
+	gchar *tip, void (*callback)(GtkToggleButton*,gpointer), gpointer data)
 {
 	GtkWidget *button;
 	if (check)
 		button = gtk_check_button_new_with_mnemonic (label);
 	else
 		button = gtk_toggle_button_new_with_mnemonic (label);
+#ifdef USE_GTK2_18
+	gtk_widget_set_can_default (button, TRUE);
+	gtk_widget_set_can_focus (button, TRUE);
+#else
 	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
 	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_FOCUS);
+#endif
 	if (state)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 	if (tip != NULL)
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		button, tip);
+		e2_widget_set_safetip (button, tip);
 	if (callback != NULL)
 		g_signal_connect (G_OBJECT (button), "toggled", G_CALLBACK (callback), data);
-	gtk_widget_show (button);
-
 	return button;
 }
 /**
@@ -294,7 +444,8 @@ GtkWidget *e2_button_get_toggle (gboolean check, gboolean state,
 @return the button widget
 */
 GtkWidget *e2_button_add_toggle (GtkWidget *box, gboolean check, gboolean state,
-	gchar *label, gchar *tip, gboolean exp, guint pad, gpointer callback, gpointer data)
+	gchar *label, gchar *tip, gboolean exp, guint pad,
+	void (*callback)(GtkToggleButton*,gpointer), gpointer data)
 {
 	GtkWidget *button = e2_button_get_toggle (check, state, label, tip, callback, data);
 	gtk_box_pack_start (GTK_BOX (box), button, exp, TRUE, pad);
@@ -315,7 +466,8 @@ GtkWidget *e2_button_add_toggle (GtkWidget *box, gboolean check, gboolean state,
 @return the button widget
 */
 GtkWidget *e2_button_add_radio (GtkWidget *box, gchar *label, GSList *group,
-	gboolean state, gboolean exp, guint pad, gpointer callback, gpointer data)
+	gboolean state, gboolean exp, guint pad,
+	void (*callback)(GtkToggleButton*,gpointer), gpointer data)
 {
 	GtkWidget *radio_button = (label != NULL) ?
 		gtk_radio_button_new_with_mnemonic (group, label):
@@ -326,8 +478,6 @@ GtkWidget *e2_button_add_radio (GtkWidget *box, gchar *label, GSList *group,
 		g_signal_connect (G_OBJECT (radio_button), "toggled",
 			G_CALLBACK (callback), data);
 	gtk_box_pack_start (GTK_BOX (box), radio_button, exp, TRUE, pad);
-	gtk_widget_show (radio_button);
-
 	return radio_button;
 }
 /**
@@ -347,7 +497,7 @@ GtkWidget *e2_button_add_radio (GtkWidget *box, gchar *label, GSList *group,
 GtkWidget *e2_button_add_to_table (
 	GtkWidget *table,
 	gchar *label,
-	gpointer callback,
+	void (*callback)(GtkButton*,gpointer),
 	gpointer data,
 	gint left, gint right, gint top, gint bottom)
 {
@@ -357,8 +507,6 @@ GtkWidget *e2_button_add_to_table (
 		g_signal_connect (G_OBJECT (button), "clicked",
 			G_CALLBACK (callback), data);
 	gtk_table_attach_defaults (GTK_TABLE (table), button, left, right, top, bottom);
-	gtk_widget_show (button);
-
 	return button;
 }
 /**
@@ -380,7 +528,7 @@ GtkWidget *e2_button_add_toggle_to_table (
 	GtkWidget *table,
 	gchar *label,
 	gboolean state,
-	gpointer callback,
+	void (*callback)(/*GtkToggleButton*,gpointer*/),
 	gpointer data,
 	gint left, gint right, gint top, gint bottom)
 {
@@ -394,8 +542,6 @@ GtkWidget *e2_button_add_toggle_to_table (
 			G_CALLBACK (callback), data);
 	gtk_table_attach_defaults (GTK_TABLE (table), check_button,
 		                        left, right, top, bottom);
-	gtk_widget_show (check_button);
-
 	return check_button;
 }
 /**
@@ -405,7 +551,7 @@ GtkWidget *e2_button_add_toggle_to_table (
 @param label button label which may contain mnemonic
 @param group radio group, or NULL to start a group
 @param state the intitial state of the button
-@param callback callback function for "clicked" signal, or NULL
+@param callback callback function for "toggled" signal, or NULL
 @param data data specified for the callback
 @param left
 @param right
@@ -419,7 +565,7 @@ GtkWidget *e2_button_add_radio_to_table (
 	gchar *label,
 	GSList *group,
 	gboolean state,
-	gpointer callback,
+	void (*callback)(/*GtkToggleButton*,gpointer*/),
 	gpointer data,
 	gint left, gint right, gint top, gint bottom)
 {
@@ -431,7 +577,5 @@ GtkWidget *e2_button_add_radio_to_table (
 			G_CALLBACK (callback), data);
 	gtk_table_attach_defaults (GTK_TABLE (table), radio_button,
 		                        left, right, top, bottom);
-	gtk_widget_show (radio_button);
-
 	return radio_button;
 }
