@@ -1,6 +1,6 @@
-/* $Id: e2_combobox.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_combobox.h 2034 2009-12-21 02:37:51Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -56,7 +56,9 @@ void e2_combobox_activated_cb (GtkWidget *entry, gpointer data);
 void e2_combobox_block_changed (GtkWidget *combo);
 void e2_combobox_unblock_changed (GtkWidget *combo);
 void e2_combobox_set_active (GtkWidget *combo, gint num);
-gchar *e2_combobox_get_active (GtkWidget *combo);
+gchar *e2_combobox_get_active_text (GtkWidget *combo);
+void e2_combobox_clear_value (GtkWidget *combo, const gchar *value,
+	gboolean with_entry);
 //void e2_combobox_save_history (GtkWidget *combo, GList **list);
 void e2_combobox_append_history (GtkWidget *combo, GList *list);
 void e2_combobox_append_history_counted (GtkWidget *combo, guint num, gchar **array);
@@ -65,9 +67,10 @@ gboolean e2_combobox_has_history (GtkComboBox *combo);
 //void e2_combobox_select_last (GtkWidget *combo);
 //gchar *e2_combobox_last_text (GtkComboBox *combo);
 gchar *e2_combobox_first_text (GtkComboBox *combo);
-GtkWidget *e2_combobox_get (gpointer activate_cb, gpointer activate_data,
-	GList **history, E2_ComboBoxFlags flags);
+GtkWidget *e2_combobox_get (void (*activate_cb)(GtkEntry*,gpointer),
+	gpointer activate_data, GList **history, E2_ComboBoxFlags flags);
 GtkWidget *e2_combobox_add (GtkWidget *box, gboolean expand, guint padding,
-	gpointer activate_cb, gpointer activate_data, GList **history, E2_ComboBoxFlags flags);
+	void (*activate_cb)(GtkEntry*,gpointer), gpointer activate_data,
+	GList **history, E2_ComboBoxFlags flags);
 
 #endif //ndef __E2_COMBOBOX_H__
