@@ -1,6 +1,6 @@
-/* $Id: e2_hook.c 767 2007-11-29 10:16:54Z tpgww $
+/* $Id: e2_hook.c 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -84,7 +84,8 @@ void e2_hook_unattach_cb (E2_Duo *duo)
 @param data pointer to data for @a func
 @return the data struct for the hook func
 */
-GHook *e2_hook_register (GHookList *hook_list, gpointer func, gpointer data)
+GHook *e2_hook_register (GHookList *hook_list, gboolean (*func)(gpointer,gpointer),
+	gpointer data)
 {
 	if (!hook_list->is_setup)
 		g_hook_list_init (hook_list, sizeof (GHook));
@@ -102,8 +103,8 @@ GHook *e2_hook_register (GHookList *hook_list, gpointer func, gpointer data)
 @param use_data TRUE to find a hook with data matching @a data
 @return TRUE if the hook was found and destroyed
 */
-gboolean e2_hook_unregister (GHookList *hook_list, gpointer func, gpointer data,
-	gboolean use_data)
+gboolean e2_hook_unregister (GHookList *hook_list, gboolean (*func)(gpointer,gpointer),
+	gpointer data, gboolean use_data)
 {
 	GHook *hook = (use_data) ?
 		g_hook_find_func_data (hook_list, TRUE, func, data):
