@@ -1,6 +1,6 @@
-/* $Id: e2_hook.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_hook.h 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -25,9 +25,9 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 void e2_hook_unattach_cb (E2_Duo *duo);
 
-GHook *e2_hook_register (GHookList *hook_list, gpointer func, gpointer data);
-gboolean e2_hook_unregister (GHookList *hook_list, gpointer func, gpointer data,
-	gboolean use_data);
+GHook *e2_hook_register (GHookList *hook_list, gboolean (*func)(gpointer,gpointer), gpointer data);
+gboolean e2_hook_unregister (GHookList *hook_list, gboolean (*func)(gpointer,gpointer),
+	gpointer data, gboolean use_data);
 #define e2_hook_unregister_simple(hook_list, func) \
 	e2_hook_unregister (hook_list, func, NULL, FALSE)
 void e2_hook_list_run (GHookList *hook_list, gpointer data);
