@@ -1,6 +1,6 @@
-/* $Id: e2_menu.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_menu.h 2017 2009-11-19 20:36:36Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -28,16 +28,20 @@ This is the header file for the menu utility functions.
 #ifndef __E2_MENU_H__
 #define __E2_MENU_H__
 
-typedef enum
+typedef enum _E2_ChildMenuType
 {
-	E2_CHILD_ALL,
-	E2_CHILD_ACTIVE,
-	E2_CHILD_OUTPUT,
+	E2_CHILD_ALL,	//everything in the tasks history list into a check-menu
+	E2_CHILD_ACTIVE, //active child processes i.e. commands
+	E2_CHILD_ACTIVETASKS, //active child processes and action(s) (in Q, normally)
+	E2_CHILD_OUTPUT, //child processes which have printed to output pane
 } E2_ChildMenuType;
 
 //callbacks
-void e2_menu_destroy_cb (GtkWidget *menu, gpointer data);
-void e2_menu_control_cb (GtkWidget *menu_item, gpointer data);
+void e2_menu_action_activated_cb (GtkMenuItem *menuitem, gpointer data);
+void e2_menu_action_activated_cb2 (GtkMenuItem *menu_item, gpointer data);
+void e2_menu_selection_done_cb (GtkWidget *menu, gpointer data);
+void e2_menu_control_cb (GtkMenuItem *menu_item, gpointer data);
+gboolean e2_menu_destroy (GtkWidget *menu);
 
 //public
 void e2_menu_connect (GtkWidget *menu, gboolean  active);
@@ -47,26 +51,30 @@ void e2_menu_popup (GtkWidget *menu, gint button, guint32 time);
 	e2_menu_add (menu, label, NULL, NULL, func, data)
 #define e2_menu_add_simple(menu, label, icon, func, data) \
 	e2_menu_add (menu, label, icon, NULL, func, data) */
-GtkWidget *e2_menu_add (GtkWidget *menu, gchar *label, gchar *icon,
-	gchar *tip, void *func, gpointer data);
-
+GtkWidget *e2_menu_add (GtkWidget *menu, gchar *label, gchar *icon, gchar *tip,
+	void (*func)(), gpointer data); //func args GtkMenuItem*,gpointer
+GtkWidget *e2_menu_add_check (GtkWidget *menu, gchar *label, gboolean state,
+	void (*func)(), gpointer data); //func args GtkCheckMenuItem*,gpointer
+GtkWidget *e2_menu_add_radio (GtkWidget *menu, GSList **group, gchar *label,
+	gboolean state, void (*func)(), gpointer data); //func args GtkMenuItem*,gpointer
 GtkWidget *e2_menu_add_action (GtkWidget *menu, gchar *label, gchar *icon,
-	gchar *tip, gchar *action, gchar *arg, gpointer free);
-GtkWidget *e2_menu_add_check (GtkWidget *menu, gchar *label,
-	gboolean state, void *func, gpointer data);
-GtkWidget *e2_menu_add_radio (GtkWidget *menu, GSList **group,
-	gchar *label, gboolean state, void *func, gpointer data);
+	gchar *tip, gchar *action, gchar *arg);
 GtkWidget *e2_menu_add_separator (GtkWidget *menu);
 GtkWidget *e2_menu_add_tear_off (GtkWidget *menu);
 GtkWidget *e2_menu_add_submenu (GtkWidget *menu, gchar *label_text, gchar *icon);
-GtkWidget *e2_menu_add_toggle (GtkWidget *menu, gchar *label, gchar *icon,
-	gchar *tip, gchar *toggle, gchar *cmd);
+GtkWidget *e2_menu_add_toggle (GtkWidget *menu, gboolean destroy,
+	gchar *label, gchar *icon, gchar *tip, gchar *toggle, gchar *cmd);
 GtkWidget *e2_menu_create_options_menu (GtkWidget *controller,
-	GtkWidget *menu, E2_OptionSet *set, ...);
-void e2_menu_create_bookmarks_menu (E2_Action *action,
-	E2_OptionSet *set, GtkWidget *menu, GtkTreeIter *iter);
-void e2_menu_create_plugins_menu (GtkWidget *menu, gboolean is_selection);
-GtkWidget *e2_menu_create_child_menu (E2_ChildMenuType type, gpointer func);
+	GtkWidget *menu, E2_OptionSet *set, ...)
+#ifdef G_GNUC_NULL_TERMINATED
+    G_GNUC_NULL_TERMINATED
+#endif
+	;
+void e2_menu_add_bookmark_items (GtkWidget *menu, gboolean onetime,
+	E2_Action *action, gint pane, GtkTreeModel *mdl, GtkTreeIter *iter);
+void e2_menu_create_plugins_menu (GtkWidget *menu, gboolean onetime, gboolean is_selection);
+GtkWidget *e2_menu_create_child_menu (E2_ChildMenuType type,
+	void (*activate_cb)()); //func args (GtkMenuItem*,E2_TaskRuntime*)
 GtkWidget *e2_menu_create_filter_menu (ViewInfo *view);
 GtkWidget *e2_menu_create_custom_menu (gchar *name);
 void e2_menu_custom_option_register (void);
