@@ -1,6 +1,6 @@
-/* $Id: e2_tree.c 639 2007-09-05 03:10:56Z tpgww $
+/* $Id: e2_tree.c 1759 2009-07-04 01:18:44Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -60,7 +60,7 @@ gboolean e2_tree_find_iter_from_str (GtkTreeModel *model, gint column,
 }
 /**
 @brief starting from @a iter, check all iters at the same level as @a iter in
-  @a model, and all their descendants, to find the first iter which contains string @a search
+  @a model, and all their descendants, to find the first iter which contains @a search
 @a iter is set to matching iter if any, or is unchanged if no match is found
 @param model treemodel to be interrogated
 @param column index of the column in @a model to scan
@@ -82,7 +82,7 @@ gboolean e2_tree_find_iter_from_str_simple (GtkTreeModel *model, gint column,
 		gtk_tree_model_get (model, iter, column, &value, -1);
 		if (value == NULL)
 			continue;
-		if (g_str_equal (value, search))
+		if (!strcmp (value, search))
 		{
 			g_free (value);
 			if (with_children)
@@ -112,12 +112,12 @@ gboolean e2_tree_find_iter_from_str_simple (GtkTreeModel *model, gint column,
 }
 /**
 @brief starting from @a iter, check all iters at the same level as @a iter in
-@a model, to find the first iter which contains string @a search
-@a iter is unchanged if no match is found
+@a model, to find the first one which contains string @a search in column @a colnum
+@a iter is set to the matching iter, or is unchanged if no match is found
 @param model treemodel to be interrogated
 @param column index of the column in @a model to scan
 @param search the string to be searched for
-@param iter pointer to initial treeiter for model interrogation, set to matching iter
+@param iter pointer to treeiter for model interrogation, start
 
 @return TRUE if the string was found
 */
@@ -132,7 +132,7 @@ gboolean e2_tree_find_iter_from_str_same (GtkTreeModel *model, gint column,
 		gtk_tree_model_get (model, iter, column, &value, -1);
 		if (value != NULL)
 		{
-			if (g_str_equal (value, search))
+			if (!strcmp (value, search))
 			{
 				g_free (value);
 				return TRUE;
@@ -143,146 +143,144 @@ gboolean e2_tree_find_iter_from_str_same (GtkTreeModel *model, gint column,
 	*iter = backup;
 	return FALSE;
 }
+//#ifdef E2_TREEINCREMENT
 /**
-@brief find tree iter which matches the last "."-separated segment of string @a search
+@brief find or otherwise create an iter in @a model which corresponds to @a search
 @a search is formatted like level1[.level2.level3 ...] Each "." (if any)
 indicates a child of the previous segment
-If successful, @a iter is set to the matching iter in the model
+@a iter is set to the existing or newly-created iter in the model
 @param model treemodel to be interrogated
 @param column index of the column in @a model to scan
-@param search string describing the iter to be searched for
 @param iter pointer to treeiter for model interrogation and result
+@param search '.'-segmented string describing the iter to be found or created
 
-@return TRUE if the string was found
+@return TRUE if the iter was found, FALSE if it had to be created
 */
-gboolean e2_tree_find_lowest_iter_from_str (GtkTreeModel *model,
-	gint column, const gchar *search, GtkTreeIter *iter)
+gboolean e2_tree_get_lowest_iter_for_str (GtkTreeModel *model, gint column,
+	GtkTreeIter *iter, const gchar *search)
 {
-	if (!gtk_tree_model_get_iter_first (model, iter))
-		return FALSE;
 	gchar **split = g_strsplit (search, ".", -1);
-	if (!e2_tree_find_iter_from_str_simple (model, column, split[0], iter, FALSE))
-	{
-		g_strfreev (split);
-		return FALSE;
-	}
-//FIXME this should continue searching at iterative parent levels whenever
-// a match fails, in case there are multiple values
-	gboolean retval = TRUE;
-	GtkTreeIter child;
-	gint i = 1;
-	while (split[i] != NULL)
-	{
-		if (*split[i] == '\0')
-		{
-			retval = FALSE;	//don't have empty items in the store
-			break;
-		}
-		if (!gtk_tree_model_iter_children (model, &child, iter))
-		{
-			retval = FALSE;
-			break;	//FIXME iterate
-		}
-		gboolean found = FALSE;
-		gchar *value;
-		*iter = child;
-		do
-		{
-			gtk_tree_model_get (model, iter, column, &value, -1);
-			if (g_str_equal (value, split[i]))
-			{
-				g_free (value);
-				found = TRUE;
-				break;
-			}
-			g_free (value);
-		} while (gtk_tree_model_iter_next (model, iter));
-		if (!found)
-		{
-			retval = FALSE;
-			break;	//FIXME iterate
-		}
-		i++;
-	}
-	g_strfreev (split);
-	return retval;
-}
-#ifdef E2_TREEINCREMENT
-/**
-@brief create treestore entry which matches @a name
-@a name is formatted like level1[.level2.level3 ...] Each "." (if any)
-indicates a child of the previous segment
-@a iter is set to the newly created iter in the model
-@param model treemodel to be interrogated
-@param column index of the column in @a model to scan
-@param name segmented name string desribing the iter to be created
-@param iter pointer to treeiter for model interrogation and result
-
-@return
-*/
-//this func could be amended a bit to do the job of
-// e2_tree_find_lowest_iter_from_str() too
-void e2_tree_create_lowest_iter_from_str (GtkTreeModel *model,
-	gint column, gchar *name, GtkTreeIter *iter)
-{
-	gchar **split = g_strsplit (name, ".", -1);
 	gint i;
 	GtkTreeIter child;
 	if (gtk_tree_model_get_iter_first (model, iter)
 		&& e2_tree_find_iter_from_str_simple (model, column, split[0], iter, FALSE))
 	{	//at least the first level exists
-//FIXME this should continue searching at iterative parent levels whenever
-// a match fails, in case there are multiple values
-		i = 1;
-		while (split[i] != NULL)
+		for (i = 1; split[i] != NULL; i++)
 		{
 			if (*split[i] == '\0')
-				i++;	//ignore it FIXME eliminate it from the array
+				continue;	//ignore it
 			else if (gtk_tree_model_iter_children (model, &child, iter))
 			{
 				gboolean found = FALSE;
-				gchar *value;
 				*iter = child;
 				do
 				{
+					gchar *value;
 					gtk_tree_model_get (model, iter, column, &value, -1);
-					if (g_str_equal (value, split[i]))
-					{
-						g_free (value);
-						found = TRUE;
-						break;
-					}
+					found = (!strcmp (value, split[i]));
 					g_free (value);
-				} while (gtk_tree_model_iter_next (model, iter));
+				} while (!found && gtk_tree_model_iter_next (model, iter));
+
 				if (!found)
-					break;	//FIXME iterate
-				i++;
+					break;
 			}
 			else
-				break;	//we've run out of matching levels	FIXME iterate
+				break;
 		}
 		if (split[i] == NULL)
-			return;	//we found everything
+		{	//we found everything
+			g_strfreev (split);
+			return TRUE;
+		}
 	}
 	else
-		//we need to create all of the segments
-		i = 0;
-	//populate the new row(s)
+		i = 0;	//we need to create all of the segments
+
+	//populate new row(s)
 	GtkTreeStore *store = GTK_TREE_STORE (model);
-	while (split[i] != NULL)
+	for ( ; split[i] != NULL; i++)
 	{
-		if (i == 0)
+		if (*split[i] == '\0')
+			continue;	//ignore it
+		else if (i == 0)
 			gtk_tree_store_append (store, &child, NULL);
 		else
 			gtk_tree_store_append (store, &child, iter);
 		*iter = child;
 		gtk_tree_store_set (store, iter, column, split[i], -1);
-		i++;
 	}
 	g_strfreev (split);
+	return FALSE;
 }
-#endif	//def E2_TREEINCREMENT
-/**
+//#endif	//def E2_TREEINCREMENT
+/* *
+@brief starting from the first iter in @a model, check all iters to find the
+first whose column @a column contains the last "."-separated segment of @a search.
+@a search is formatted like level1[.level2.level3 ...] Each "." (if any)
+indicates a child of the previous segment. If successful, @a iter is set to the
+matching iter in the model. Otherwise, iter is undefined at exit
+@param model treemodel to be interrogated
+@param column index of the column in @a model to scan
+@param iter pointer to treeiter for model interrogation and result
+@param search string describing the iter to be searched for
+
+@return TRUE if the string was found
+*/
+/*gboolean e2_tree_find_lowest_iter_for_str (GtkTreeModel *model, gint column,
+	GtkTreeIter *iter, const gchar *search)
+{
+	if (!gtk_tree_model_get_iter_first (model, iter))
+	{
+//		*iter = ?;
+		return FALSE;
+	}
+
+	gchar **split = g_strsplit (search, ".", -1);
+	//get a place to start scanning
+	gboolean retval =
+	e2_tree_find_iter_from_str_simple (model, column, split[0], iter, FALSE);
+
+	if (retval)
+	{
+		gint i;
+		for (i = 1; split[i] != NULL; i++)	//we've already found 0
+		{
+			GtkTreeIter child;
+
+			if (*split[i] == '\0')
+			{
+//				retval = FALSE;	//no valid empty iter-names in the store
+//				break;
+				continue; //tolerance
+			}
+			if (!gtk_tree_model_iter_children (model, &child, iter))
+			{
+				retval = FALSE;
+				break;
+			}
+			gboolean found = FALSE;
+			*iter = child;
+			do
+			{
+				gchar *value;
+				gtk_tree_model_get (model, iter, column, &value, -1);
+				found = (!strcmp (value, split[i]));
+				g_free (value);
+			} while (!found && gtk_tree_model_iter_next (model, iter));
+
+			if (!found)
+			{
+//				*iter = child; //we found this much
+				retval = FALSE;
+				break;
+			}
+		}
+	}
+	g_strfreev (split);
+	return retval;
+}
+*/
+/* *
 @brief get string in column @a columm in the last row in @a model
 
 @param model treemodel to be interrogated
@@ -321,7 +319,7 @@ gchar *e2_tree_get_last_string (GtkTreeModel *model, gint column)
 	gtk_tree_model_get (model, &iter, column, &value, -1);
 	return value;
 } */
-/**
+/* *
 @brief determine the number of rows in treestore model @a model
 This is mainly for debugging
 @param model treemodel to be interrogated
@@ -389,12 +387,11 @@ gboolean e2_tree_ref_to_iter (GtkTreeStore *store, GtkTreeRowReference *ref,
 @param widget UNUSED the widget which activated the callback
 @param treeview the treeview to be collapsed
 
-@return TRUE (to prevent further handlers)
+@return
 */
-gboolean e2_tree_expand_all_cb (GtkWidget *widget, GtkTreeView *treeview)
+void e2_tree_expand_all_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 {
 	gtk_tree_view_expand_all (treeview);
-	return TRUE;
 }
 /**
 @brief tree collapse-all callback
@@ -402,12 +399,11 @@ gboolean e2_tree_expand_all_cb (GtkWidget *widget, GtkTreeView *treeview)
 @param widget UNUSED the widget which activated the callback
 @param treeview the treeview to be collapsed
 
-@return TRUE (to prevent further handlers)
+@return
 */
-gboolean e2_tree_collapse_all_cb (GtkWidget *widget, GtkTreeView *treeview)
+void e2_tree_collapse_all_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 {
 	gtk_tree_view_collapse_all (treeview);
-	return TRUE;
 }
 /**
 @brief find previous iter in a tree model
@@ -475,11 +471,9 @@ void e2_tree_delete (GtkTreeView *treeview)
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
 	if (gtk_tree_selection_count_selected_rows (sel) > 0)
 	{
-		GList *rowrefs = NULL;
-
 		GtkTreeModel *model;
+		GList *tmp, *rowrefs = NULL;
 		GList *selpaths = gtk_tree_selection_get_selected_rows (sel, &model);
-		GList *tmp;
 		//identify a place to goto after the deletion
 		//(current place, if we're at the start of the tree)
 		GtkTreePath *newpath = gtk_tree_path_copy (selpaths->data);
@@ -492,14 +486,17 @@ void e2_tree_delete (GtkTreeView *treeview)
 		//create treerowrefs for all selected rows
 		for (tmp = selpaths; tmp!=NULL; tmp=tmp->next)
 		{
-			if (gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) tmp->data))
+			path = (GtkTreePath *) tmp->data;
+			if (gtk_tree_model_get_iter (model, &iter, path))
 				_e2_tree_reference_treerow (model, &iter, &rowrefs);
+			gtk_tree_path_free (path);
 		}
+		g_list_free (selpaths);
 
-		for (tmp = rowrefs; tmp!=NULL; tmp=tmp->next)
+		for (tmp = rowrefs; tmp != NULL; tmp = tmp->next)
 		{
-			if ((path = gtk_tree_row_reference_get_path
-				((GtkTreeRowReference *) tmp->data)) != NULL)
+			path = gtk_tree_row_reference_get_path ((GtkTreeRowReference *) tmp->data);
+			if (path != NULL)
 			{
 				if (gtk_tree_model_get_iter (model, &iter, path))
 				{
@@ -507,8 +504,11 @@ void e2_tree_delete (GtkTreeView *treeview)
 					//this deletes all children too
 					gtk_tree_store_remove (GTK_TREE_STORE (model), &iter);
 				}
+				gtk_tree_path_free (path);
 			}
+			gtk_tree_row_reference_free ((GtkTreeRowReference *) tmp->data);
 		}
+		g_list_free (rowrefs);
 
 		//go to the new place
 		if (!gtk_tree_model_get_iter (model, &iter, newpath)
@@ -520,11 +520,6 @@ void e2_tree_delete (GtkTreeView *treeview)
 		gtk_tree_view_set_cursor (treeview, newpath,
 			gtk_tree_view_get_column (treeview, 0), FALSE);
 
-		//cleanups
-		g_list_foreach (rowrefs, (GFunc) gtk_tree_row_reference_free, NULL);
-		g_list_free (rowrefs);
-		g_list_foreach (selpaths, (GFunc) gtk_tree_path_free, NULL);
-		g_list_free (selpaths);
 		gtk_tree_path_free (newpath);
 	}
 }
@@ -597,20 +592,20 @@ GList *e2_tree_copy (GtkTreeView *treeview)
 	if (gtk_tree_selection_count_selected_rows (sel) > 0)
 	{
 		GtkTreeModel *model;
-		GList *base = gtk_tree_selection_get_selected_rows (sel, &model);
-		GtkTreeIter iter;
+		GList *rowpath;
+		GList *selpaths = gtk_tree_selection_get_selected_rows (sel, &model);
 		gint columns = gtk_tree_model_get_n_columns (model);
-		GList *selpaths;
 
 		//copy the selected rows and their descendants
-		for (selpaths = base; selpaths != NULL; selpaths = selpaths->next)
+		for (rowpath = selpaths; rowpath != NULL; rowpath = rowpath->next)
 		{
-			if (gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) selpaths->data))
+			GtkTreeIter iter;
+			if (gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) rowpath->data))
 				_e2_tree_copy_branch (model, &iter, columns, &rowscopied);
+			gtk_tree_path_free ((GtkTreePath *) rowpath->data);
 		}
 
-		g_list_foreach (base, (GFunc) gtk_tree_path_free, NULL);
-		g_list_free (base);
+		g_list_free (selpaths);
 
 		printd (DEBUG, "copied %d rows", g_list_length (rowscopied));
 	}
@@ -801,11 +796,13 @@ void e2_tree_paste (GList *rowscopied, GtkTreeView *treeview)
 		g_list_free (selpaths);
 	}
 }
-#ifdef STORECOPY
+//#ifdef STORECOPY
 /**
 @brief helper function to recursively copy a treestore
-@param model treemodel for store being copied
-@param parent pointer to treeiter which has been confirmed to have child(ren)
+@param srcmodel treemodel for store being copied
+@param srcparent pointer to source treeiter which has been confirmed to have child(ren)
+@param deststore the store to which data is being copied
+@param destparent pointer to destination store treeiter
 @param ncols no. of columns in @a model
 @param colnums array of column indices
 @param types array of GTypes corresponding to columns
@@ -813,30 +810,39 @@ void e2_tree_paste (GList *rowscopied, GtkTreeView *treeview)
 
 @return
 */
-static void _e2_tree_copy_descendants (GtkTreeModel *model, GtkTreeIter *parent, gint ncols,
-	gint colnums[], GType types[], GValue values[])
+static void _e2_tree_copy_descendants (
+	GtkTreeModel *srcmodel, GtkTreeIter *srcparent,
+	GtkTreeStore *deststore, GtkTreeIter *destparent,
+	gint ncols, gint colnums[], GType types[], GValue values[])
 {
-	gint i;
-	GtkTreeStore *newtstore = GTK_TREE_STORE (model);
 	GtkTreeIter child;
-	gtk_tree_model_iter_children (model, &child, parent);
-	do
+	if (gtk_tree_model_iter_children (srcmodel, &child, srcparent))
 	{
-		if (gtk_tree_model_iter_has_child (model, &child))
-			_e2_tree_copy_descendants (model, &child, ncols, colnums, types, values);
-		//read iter into pointers array
-		for (i = 0; i < ncols; i++)
-			gtk_tree_model_get_value (model, &child, i, &values[i]);
+		gint i;
+		GtkTreeIter dest;
+		do
+		{
+			//read iter into pointers array
+			for (i = 0; i < ncols; i++)
+				gtk_tree_model_get_value (srcmodel, &child, i, &values[i]);
 #ifdef USE_GTK2_10
-		gtk_tree_store_insert_with_valuesv (newtstore, &child, parent, -1, colnums, values, ncols);
+			gtk_tree_store_insert_with_valuesv (deststore, &dest, destparent, -1,
+				colnums, values, ncols);
+			for (i = 0; i < ncols; i++)
+				g_value_unset (&values[i]);
 #else
-		gtk_tree_store_append (newtstore, &child, parent);
-		for (i = 0; i < ncols; i++)
-			gtk_tree_store_set_value (newtstore, &child, i, &values[i]);
+			gtk_tree_store_append (deststore, &dest, destparent);
+			for (i = 0; i < ncols; i++)
+			{
+				gtk_tree_store_set_value (deststore, &dest, i, &values[i]);
+				g_value_unset (&values[i]);
+			}
 #endif
-		for (i = 0; i < ncols; i++)
-			g_value_unset (&values[i]);
-	} while (gtk_tree_model_iter_next (model, &child));
+			if (gtk_tree_model_iter_has_child (srcmodel, &child))
+				_e2_tree_copy_descendants (srcmodel, &child, deststore, &dest,
+					ncols, colnums, types, values);
+		} while (gtk_tree_model_iter_next (srcmodel, &child));
+	}
 }
 /**
 @brief copy a liststore or treestore
@@ -854,9 +860,7 @@ same as the old store.
 */
 void e2_tree_store_copy (GtkTreeModel *model, gboolean treetype, gpointer *newstore)
 {
-	GtkTreeModel *newmodel;
 	GtkTreeIter src;
-	GtkTreeIter dest;
 	gint i, ncols = gtk_tree_model_get_n_columns (model);
 	gint colnums[ncols];
 	GType types[ncols];
@@ -871,39 +875,19 @@ void e2_tree_store_copy (GtkTreeModel *model, gboolean treetype, gpointer *newst
 
 	if (treetype)
 	{
-		GtkTreeStore *newtstore;
-		newtstore = gtk_tree_store_newv (ncols, types);
+		GtkTreeStore *newtstore = gtk_tree_store_newv (ncols, types);
 		if (gtk_tree_model_get_iter_first (model, &src))
-		{
-			newmodel = GTK_TREE_MODEL (newtstore);
-			do
-			{
-				//read iter into values array
-				for (i = 0; i < ncols; i++)
-					gtk_tree_model_get_value (model, &src, i, &values[i]);
-#ifdef USE_GTK2_10
-				gtk_tree_store_insert_with_valuesv (newtstore, &dest, NULL, -1, colnums, values, ncols);
-#else
-				gtk_tree_store_append (newtstore, &dest, NULL);
-				for (i = 0; i < ncols; i++)
-					gtk_tree_store_set_value (newtstore, &dest, i, &values[i]);
-#endif
-				for (i = 0; i < ncols; i++)
-					g_value_unset (&values[i]);
-				if (gtk_tree_model_iter_has_child (model, &src))
-					_e2_tree_copy_descendants (model, NULL, ncols, colnums, types, values);
+			_e2_tree_copy_descendants (model, NULL, newtstore, NULL,
+						ncols, colnums, types, values);
 
-			} while (gtk_tree_model_iter_next (model, &src));
-		}
 		*newstore = newtstore;
 	}
 	else
 	{
-		GtkListStore *newlstore;
-		newlstore = gtk_list_store_newv (ncols, types);
+		GtkListStore *newlstore = gtk_list_store_newv (ncols, types);
 		if (gtk_tree_model_get_iter_first (model, &src))
 		{
-			newmodel = GTK_TREE_MODEL (newlstore);
+			GtkTreeIter dest;
 			do
 			{
 				for (i = 0; i < ncols; i++)
@@ -916,4 +900,4 @@ void e2_tree_store_copy (GtkTreeModel *model, gboolean treetype, gpointer *newst
 		*newstore = newlstore;
 	}
 }
-#endif
+//#endif
