@@ -1,6 +1,6 @@
-/* $Id: e2_utf8.c 881 2008-05-12 22:54:30Z tpgww $
+/* $Id: e2_utf8.c 1694 2009-05-17 07:23:29Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 Portion _maybe_ copyright (C) 2004-2005 Tarot Osuji
 
 This file is part of emelFM2.
@@ -299,16 +299,67 @@ gchar *e2_utf8_unescape (const gchar *str, gunichar c)
 	printd (DEBUG, "ret: %s", ret);
 	return ret;
 }
+/**
+@brief if a request has been logged, adjust func pointers to enable or disable file path/name encoding conversion
+*/
+void e2_utf8_set_name_conversion_if_requested (void)
+{
+	if (app.reconvert_requested)
+	{
+		printd (DEBUG, "repoint encoding conversion funcs if necessary, before action starts");
+		e2_utf8_set_name_conversion (app.pane1.view.convert || app.pane2.view.convert);
+		app.reconvert_requested = FALSE;
+	}
+}
+/**
+@brief set func pointers to en/disable file path/name encoding conversion
+@param convert TRUE if conversion to/from UTF-8 is to be done henceforth
+
+@return
+*/
+void e2_utf8_set_name_conversion (gboolean convert)
+{
+	if (convert)
+	{
+		e2_display_from_locale = g_filename_display_name;
+		e2_fname_to_locale = e2_fname_dupto_locale = e2_utf8_filename_to_locale;
+		e2_fname_from_locale = e2_fname_dupfrom_locale = e2_utf8_filename_from_locale;
+//		e2_fname_free = g_free;
+	}
+	else
+	{
+		e2_display_from_locale = e2_fname_to_locale = e2_fname_from_locale
+			= e2_utf8_not_converted;
+ 		e2_fname_dupto_locale = e2_fname_dupfrom_locale = g_strdup;
+//		e2_fname_free = e2_utf8_not_freed;
+	}
+}
 
 //inline
 gchar *e2_utf8_not_converted (const gchar *d)
 {
 	return (gchar *) d;
 }
-void e2_utf8_not_freed (gpointer dummy)
-{}
 
-#ifndef E2_FILES_UTF8ONLY
+//void e2_utf8_not_freed (gpointer dummy)
+//{}
+
+/**
+@brief cleanup @a freeme if it's not the same as @a original
+This is robust to a change of encoding-functions (hence string allocation)
+between creation and cleanup of @a freeme.
+@param freeme the string to be free'd
+@param original the string from which @a freeme was derived
+
+@return
+*/
+void e2_utf8_fname_free (gchar *freeme, const gchar *original)
+{
+	if (freeme != (gchar *)original)
+		g_free (freeme);
+}
+
+//#ifndef E2_FILES_UTF8ONLY
 //with run-time checking, a pointer may be set to this func, so no inline
 //inline
 gchar *e2_utf8_filename_from_locale (const gchar *d)
@@ -370,7 +421,7 @@ gchar *e2_utf8_filename_to_locale (const gchar *d)
 	}
 	return ret;
 }
-#endif // ndef E2_FILES_UTF8ONLY
+//#endif //ndef E2_FILES_UTF8ONLY
 
 gchar *e2_utf8_from_locale (const gchar *d)
 {
@@ -394,7 +445,8 @@ gchar *e2_utf8_from_locale_fallback (const gchar *d)
 		printd (WARN, "fallback locale string to UTF8 conversion failed: %s",
 			error->message);
 		g_error_free (error);
-		ret = g_strdup(_("Unknown"));
+		ret = g_strconcat("\357\277\275 ",	//U+FFFD inverted '?' char
+			_("unknown encoding"), NULL);
 	}
 	return ret;
 }
@@ -440,52 +492,18 @@ GB2312, GB18030, GBK, BIG5, BIG5-HKSCS and TIS-620
 (Romanian may be switching to ISO-8859-16)
 */
 
-enum
-{
-	IANA = 0,
-	OPENI18N,
-	CODEPAGE,
-	ENCODING_TYPECOUNT
-};
-
-enum
-{
-	LATIN1 = 0,
-	LATIN2,
-	LATIN3,
-//	LATIN4,
-	LATIN5,
-	LATIN6,
-	LATIN7,
-	ARABIC,
-	CYRILIC,
-	CYRILIC_TJ,
-	CYRILIC_UA,
-	GREEK,
-	HEBREW,
-	CHINESE_CN,
-	CHINESE_HK,
-	CHINESE_TW,
-	JAPANESE,
-	KOREAN,
-	VIETNAMESE,
-	THAI,
-	GEORGIAN,
-	ENCODE_COUNT
-};
-
 //up to this many locales per encoding
 #define MAX_LOCALES 10
 
 const gchar *locale_table[ENCODE_COUNT][MAX_LOCALES] =
 {
-/* LATIN1 */	{NULL},
 /* LATIN2 */	{"cs", "de", "hr", "hu", "ro", "pl", "sk", "sq", "sr", "uz"},	//this effectively sets MAX_LOCALES
 /* LATIN3 */	{"eo", "gl", "mt", NULL},
 // / * LATIN4 * /	{"et", "mi", NULL}, Estonian, Latvian, and Lithuanian essentially obsolete;
 /* LATIN5 */	{"az", "tr", NULL},
 /* LATIN6 */	{"et", "ga", "is", "kl", "mi", NULL},
 /* LATIN7 */	{"da", "fi", "lt", "lv", "no", "pl", "se", "sl", "sv", NULL},
+/* LATIN9 */	{NULL},
 /* ARABIC */	{"ar", "fa", "ur", NULL},
 /* CYRILIC */	{"be", "bg", "ky", "mk", "mn", "ru", "tt", NULL},
 /* CYRILIC_TJ */{"tg", NULL},
@@ -504,7 +522,7 @@ const gchar *locale_table[ENCODE_COUNT][MAX_LOCALES] =
 
 /*******************************************
 / * LATIN8 * /	{ "ISO-8859-14",  , },	//a.k.a celtic adds missing gaelic and welsh (cy) to latin1 mainly Old Gaelic, also ok for Albanian, Breton, Catalan, Danish, Dutch, English, Finnish, French, Gaelic, German, Irish, Italian, Norwegian, Portuguese, Swedish and Welsh.
-/ * LATIN9 * /	{ "ISO-8859-15",  , },	//latin1 with euro and some missing things
+/ * LATIN9 * /	{ "ISO-8859-15",  , },	//supersedes latin1 with euro and some missing things
 / * LATIN10 * /	{ "ISO-8859-16",  , },	//for Albanian, Croatian, Hungarian, Polish, Romanian and Slovenian, but also French, German, Italian and Irish Gaelic (new orthography). It differs from the other
 / * ARABIC * /	{ "ISO-8859-6-E",  , },	//explicit bi-directional vartiant
 / * ARABIC * /	{ "ISO-8859-6-I",  , }, //implicit bi-directional variant
@@ -515,13 +533,14 @@ const gchar *locale_table[ENCODE_COUNT][MAX_LOCALES] =
 const gchar *encoding_table [ENCODE_COUNT][ENCODING_TYPECOUNT] =
 {
 				/*  IANA          OpenI18N       Codepage */
-/* LATIN1 */	{ "ISO-8859-1",  "ISO-8859-15", "CP1252" },
+// /* LATIN1 */	{ "ISO-8859-1",  "ISO-8859-15", "CP1252" }, //superseded by latin9 (with euro symbol etc)
 /* LATIN2 */	{ "ISO-8859-2",  "ISO-8859-16", "CP1250" },
 /* LATIN3 */	{ "ISO-8859-3",  NULL,          NULL },
 // /* LATIN4 */	{ "ISO-8859-4",  "ISO-8859-13", "CP1257" },
 /* LATIN5 */	{ "ISO-8859-9",  NULL,          "CP1254" },
 /* LATIN6 */	{ "ISO-8859-10", NULL,          "CP1257" },
 /* LATIN7 */	{ "ISO-8859-13", NULL,          "CP1257" },
+/* LATIN9 */	{ "ISO-8859-1",  "ISO-8859-15", "CP1252" }, //CHECKME ISO-8859-1 is superseded
 /* ARABIC */	{ "ISO-8859-6",  NULL,          "CP1256" },
 //ISO-8859-5 for e.g. bulgarian, belarusian, russian and macedonian, not ukrainian, "never really caught on"
 /* CYRILIC */	{ "ISO-8859-5",  "KOI8-R",      "CP1251" },	//russian, macedonian
@@ -568,7 +587,7 @@ static guint get_locale_index (void)
 			}
 		}
 		if (code == ENCODE_COUNT)
-			code = LATIN1;	//default if we can't find anything supported
+			code = LATIN9;	//default if we can't find anything supported
 	}
 
 	return code;
@@ -827,7 +846,6 @@ static gboolean detect_noniso (const guchar *text)
 	}
 	return FALSE;
 }
-
 /**
 @brief detect any recognised byte-order-mark at start of @a text
 @param text the string to be processed
@@ -919,33 +937,35 @@ static const gchar *detect_charset_BOM (const guchar *text)
 	}
 	return NULL;
 }
-
 /**
-@brief detect character encoding of 0-terminated string @a text
+@brief detect (not guess) whether @a text has one of several recognized character-encodings
 @a text may be validated as UTF-8 in spite of a nominally different coding
-@a charset is set to "UNKNOWN" if nothing specific is recognised
 @param text 0-terminated string to be processed
-@param charset store for name of detected charset
-@return TRUE if @a text is found to be valid UTF-8 (which includes plain ascii)
+@param isutf8 store for T/F, whether @a text is found to be valid UTF-8 (which includes plain ascii)
+@return name of detected charset, maybe NULL
 */
-gboolean e2_utf8_detect_charset (const guchar *text, const gchar **charset)
+const gchar *e2_utf8_detect_charset (const guchar *text, gboolean *isutf8)
 {
+	const gchar *charset;
 	const guchar *start;
 	guchar c;
-	gboolean isutf8;
 
-	*charset = detect_charset_BOM (text);
-	if (*charset != NULL)	//FIXME test for short file length if UTF-32*
-		return (g_str_equal (*charset, "UTF-8"));
+	charset = detect_charset_BOM (text);
+	if (charset != NULL)	//FIXME test for short file length if UTF-32*
+	{
+		//CHECKME aliases for this encoding ?
+		*isutf8 = !strcmp (charset, "UTF-8");
+		return charset;
+	}
 
 	start = text;
-	isutf8 = FALSE;
+	*isutf8 = FALSE;
 	while ((c = *text++) != '\0')
 	{
 		if (c > 0x7F)
 		{
 			if ((c & 0xC0) == 0xC0)
-				isutf8 = g_utf8_validate ((gchar *)--text, -1, NULL);
+				*isutf8 = g_utf8_validate ((gchar *)--text, -1, NULL);
 			break;
 		}
 		else if (c == '~')
@@ -959,7 +979,7 @@ gboolean e2_utf8_detect_charset (const guchar *text, const gchar **charset)
 					c = *text++;	//maybe } or 2nd GB2312 (chinese) char
 					if (c == '}' && *text == '~')
 					{
-						*charset = "HZ";
+						charset = "HZ";
 						break;
 					}
 					else
@@ -967,7 +987,7 @@ gboolean e2_utf8_detect_charset (const guchar *text, const gchar **charset)
 						c = *text++;	//maybe } after 2nd GB2312 (chinese) char
 						if (c == '}' && *text == '~' && *(text-2) > 0x7F)
 						{
-							*charset = "HZ";
+							charset = "HZ";
 							break;
 						}
 					}
@@ -984,10 +1004,10 @@ gboolean e2_utf8_detect_charset (const guchar *text, const gchar **charset)
 				{
 					case 'B': // JIS X 0208-1983
 					case '@': // JIS X 0208-1978
-						*charset = "ISO-2022-JP";
+						charset = "ISO-2022-JP";
 						break;
 					case 'A': // GB2312-1980
-						*charset = "ISO-2022-JP-2";
+						charset = "ISO-2022-JP-2";
 						break;
 					case '(':
 						c = *text++;
@@ -995,85 +1015,84 @@ gboolean e2_utf8_detect_charset (const guchar *text, const gchar **charset)
 						{
 							case 'C': // KSC5601-1987
 							case 'D': // JIS X 0212-1990
-								*charset = "ISO-2022-JP-2";
+								charset = "ISO-2022-JP-2";
 						}
 						break;
 					case ')':
 						c = *text++;
 						if (c == 'C')
-							*charset = "ISO-2022-KR"; // KSC5601-1987
+							charset = "ISO-2022-KR"; // KSC5601-1987
 					default:
 						break;
 				}
-				if (*charset != NULL)
+				if (charset != NULL)
 					break;
 			}
 		}
 	}
 
-	if (isutf8)
+	if (*isutf8)
 	{	//validation succeeded
-		if (!g_get_charset (charset))	//use default encoding if it's utf8-compatible
-			*charset = "UTF-8";
+		if (!g_get_charset (&charset))	//use default encoding if it's utf8-compatible
+			charset = "UTF-8";
 	}
 	else	//no validation requested, or validation failed
 		if (c == '\0')	//reached end of text without detection
 	{	//it's almost certainly ASCII, assume the default encoding applies
 		//FIXME except if that's incompatible with ascii
 //		printd (DEBUG, "Specific UTF8 detection failed, reverted to DEFAULT CHARSET %s", *charset);
-		e2_utils_get_charset (charset);
-		isutf8 = TRUE;	//no conversion needed
+		e2_utils_get_charset (&charset);
+		*isutf8 = TRUE;	//no conversion needed
 	}
-	else // !isutf8 && not scanned to end of buffer
-		if (*charset == NULL)	//not detected ISO2022 or HZ
+	return charset;
+}
+/**
+@brief get encoding name for a supported language
+Sometimes this uses a simple lookup, sometimes there's some checking of @a text,
+sometimes just the user's default encoding
+@param text 0-terminated text string to evaluate
+@return string representing the encoding, or NULL
+*/
+const gchar *e2_utf8_guess_charset (const guchar *text)
+{
+	guint langcode = get_locale_index();
+	switch (langcode)
 	{
-		//next, assume that the file originates in the user's locale
-		guint code = get_locale_index();
-		switch (code)
+		case CYRILIC:
+		case CYRILIC_UA:
+		case CYRILIC_TJ:
+			return (detect_charset_cyrillic(text)); // fuzzy...
+		case CHINESE_CN:
+		case CHINESE_TW:
+		case CHINESE_HK:
+			return (detect_charset_chinese(text));
+		case JAPANESE:
+			return (detect_charset_japanese(text));
+		case KOREAN:
+			return (detect_charset_korean(text));
+		case VIETNAMESE:
+		case THAI:
+		case GEORGIAN:
+			return (encoding_table [langcode][OPENI18N]);
+		default:
 		{
-			case CYRILIC:
-			case CYRILIC_UA:
-			case CYRILIC_TJ:
-				*charset = detect_charset_cyrillic(text); // fuzzy...
-				break;
-			case CHINESE_CN:
-			case CHINESE_TW:
-			case CHINESE_HK:
-				*charset = detect_charset_chinese(text);
-				break;
-			case JAPANESE:
-				*charset = detect_charset_japanese(text);
-				break;
-			case KOREAN:
-				*charset = detect_charset_korean(text);
-				break;
-			case VIETNAMESE:
-			case THAI:
-			case GEORGIAN:
-				*charset = encoding_table [code][OPENI18N];
-				break;
-			default:
-				e2_utils_get_charset (charset);
-				if (strcmp (*charset, "UTF-8") == 0)	//CHECKME this is no use as the text isn't compliant
+			const gchar *charset;
+			e2_utils_get_charset (&charset);
+			if (charset != NULL)
+			{
+				//CHECKME c.f. _e2_fs_charset_is_utf()
+				if (strcmp (charset, "UTF-8") == 0)
 				{
+					//CHECKME relevance of this when the text isn't compliant
 					if (detect_noniso (text))
-						*charset = encoding_table [code][CODEPAGE];
+						charset = encoding_table [langcode][CODEPAGE];
 					else
-						*charset = encoding_table [code][OPENI18N];
+						charset = encoding_table [langcode][OPENI18N];
 				}
-				if (*charset == NULL)
-					*charset = encoding_table [code][IANA];
-				break;
+				if (charset == NULL)
+					return (encoding_table [langcode][IANA]);
+			}
+			return charset;
 		}
 	}
-
-	if (*charset == NULL)
-	{
-		//all checks failed, send back something ...
-//		e2_utils_get_charset (charset);
-//		printd (DEBUG, "Detection failed 2, reverted to DEFAULT CHARSET %s", *charset);
-		*charset = "UNKNOWN";	//no translation
-	}
-
-	return isutf8;
 }
