@@ -1,6 +1,6 @@
-/* $Id: e2_widget.c 841 2008-03-25 01:41:34Z tpgww $
+/* $Id: e2_widget.c 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -46,6 +46,65 @@ static void _e2_widget_destroy_ttips (E2_TipsToggle *data)
 	DEMALLOCATE (E2_TipsToggle, data);
 }
 /**
+@brief callback for "query-tooltip" signal on tipped widgets
+@param widget the widget which was moused etc
+@param x UNUSED
+@param y UNUSED
+@param keyboard_mode UNUSED
+@param tooltip where to set the tip
+@param user_data UNUSED pointer to data specified when signal was connected
+
+@return TRUE to show the tip
+*/
+static gboolean _e2_widget_show_tip_cb (GtkWidget  *widget,
+									gint        x,
+									gint        y,
+									gboolean    keyboard_mode,
+									GtkTooltip *tooltip,
+									gpointer    user_data)
+{
+	gchar *text = (gchar *) g_object_get_data (G_OBJECT (widget), "_e2-tiptext");
+	if (text != NULL && tooltip != NULL)
+	{
+		gtk_tooltip_set_text (tooltip, text);
+		return TRUE;
+	}
+	return FALSE;
+}
+/**
+@brief idle-callback to add or remove a tip, with BGL safely closed
+@param widget the widget whose tip is to be altered
+@return FALSE always, to cancel the idle
+*/
+static gboolean _e2_widget_defer_tip (GtkWidget *widget)
+{
+	gdk_threads_enter ();
+	gpointer tip = g_object_get_data (G_OBJECT (widget), "_e2-tiptext");
+	gtk_widget_set_has_tooltip (widget, (tip != NULL));
+	if (tip != NULL)
+		g_signal_connect (G_OBJECT (widget), "query-tooltip",
+			G_CALLBACK(_e2_widget_show_tip_cb), NULL);
+	else
+		g_signal_handlers_disconnect_by_func ((gpointer)widget,
+			_e2_widget_show_tip_cb, NULL);
+	gdk_threads_leave ();
+	return FALSE;
+}
+/**
+@brief set tooltip @a tiptext for @a widget
+@param widget the item to which the tip is to be applied
+@param tiptext UTF8 non-markup text, or NULL to disable tip
+@return
+*/
+void e2_widget_set_safetip (GtkWidget *widget, const gchar *tiptext)
+{
+	if (tiptext != NULL && *tiptext != '\0')
+		g_object_set_data_full (G_OBJECT (widget), "_e2-tiptext", g_strdup(tiptext), g_free);
+	else
+		g_object_set_data (G_OBJECT (widget), "_e2-tiptext", NULL);
+	g_idle_add ((GSourceFunc) _e2_widget_defer_tip, widget);
+}
+/**
 @brief setup for a button to have 2 alternate tooltips
 
 @param widget the button widget
@@ -64,13 +123,62 @@ void e2_widget_set_toggletip (GtkWidget *widget, const gchar *initialtip,
 		data->tip1 = g_strdup (initialtip);
 		data->tip2 = g_strdup (othertip);
 		data->oneactive = TRUE;
-		g_object_set_data_full (G_OBJECT (widget), "btn-toggle-data", data,
+		g_object_set_data_full (G_OBJECT (widget), "_e2-tips-data", data,
 			(GDestroyNotify) _e2_widget_destroy_ttips);
 	}
-	gtk_widget_set_tooltip_text (widget, initialtip);
+	e2_widget_set_safetip (widget, initialtip);
 }
+#else //pre-2.12 form of tip-handling
 /**
-@brief toggle the visiblity of 2 tip labels setup by e2_widget_set_toggletip()
+@brief set tooltips @a tiptext1 (main) and @a tiptext2 (private) for @a widget
+@param widget the item to which the tip is to be applied
+@param tiptext1 UTF8 non-markup text for primary tip, or NULL
+@param tiptext2 UTF8 non-markup text for private tip, or NULL
+
+@return
+*/
+void e2_widget_set_tip (GtkWidget *widget, const gchar *tiptext1, const gchar *tiptext2)
+{
+	/* gtk >= 2.12 (downstream) uses gtk_widget_set_tooltip_text(), and for
+	   gtk < 2.16.3 that synchronously accesses X server, so needs BGL closed */
+	gboolean close = (app.gtkversion >= 21200 //revised tips-code is in play from 2.12.0
+				   && app.gtkversion < 21603);//tip-setting mechanism adjusted in 2.16.3
+	if (close)
+	{
+#ifdef NATIVE_BGL
+		printd (WARN, "Closing BGL with default mutex will sometimes be recursively bad");
+		gdk_threads_enter ();
+#else
+# ifdef DEBUG_MESSAGES
+		//FIXME find a way to make errorcheck mutex block, to avoid having to abort
+		gint _lockres = pthread_mutex_trylock (&gdklock);
+		if (_lockres != 0)
+		{
+			printd (DEBUG, "%s BGL trylock result %d", __PRETTY_FUNCTION__, _lockres);
+			if (_lockres != EBUSY)
+			{
+				printd (DEBUG, "Close BGL failed, ignoring tooltip request");
+				return;
+			}
+		}
+# else
+		//recursive mutex is more tolerant
+		gdk_threads_enter ();
+# endif
+#endif
+	}
+/*	else
+		printd (DEBUG, "No closing BGL with gtk version %d", app.gtkversion);
+*/
+	gtk_tooltips_set_tip (app.tooltips, widget, tiptext1, tiptext2);
+
+	if (close)
+		gdk_threads_leave ();
+}
+#endif //def USE_GTK2_12TIPS
+
+/**
+@brief toggle the visiblity of 2 tip labels for a toggle-button
 
 @param widget the button widget
 
@@ -78,57 +186,55 @@ void e2_widget_set_toggletip (GtkWidget *widget, const gchar *initialtip,
 */
 void e2_widget_swap_tooltip (GtkWidget *widget)
 {
+#ifdef USE_GTK2_12TIPS
+	//use data setup by e2_widget_set_toggletip()
 	E2_TipsToggle *data = g_object_get_data (G_OBJECT (widget), "btn-toggle-data");
 	if (data != NULL)
 	{
-		if (data->oneactive)
+		gchar *newtip = (data->oneactive) ? data->tip2 : data->tip1;
+		if (G_LIKELY (newtip != NULL))
 		{
-			gtk_widget_set_tooltip_text (widget, data->tip2);
-			data->oneactive = FALSE;
+			gtk_widget_set_has_tooltip (widget, TRUE);
+			g_object_set_data_full (G_OBJECT (widget), "_e2-tiptext", g_strdup(newtip), g_free);
 		}
 		else
 		{
-			gtk_widget_set_tooltip_text (widget, data->tip1);
-			data->oneactive = TRUE;
+			gtk_widget_set_has_tooltip (widget, FALSE);
+			g_object_set_data (G_OBJECT (widget), "_e2-tiptext", NULL);
 		}
+		data->oneactive = !data->oneactive;
 	}
-}
 #else
-void e2_widget_set_tooltip (GtkTooltips *tooltips, GtkWidget *widget, gchar *text)
-{
-	if (text == NULL)
-		return;
-	if (tooltips == NULL)
-		tooltips = app.tooltips; //use default tooltip group
-	gtk_tooltips_set_tip (tooltips, widget, text, NULL);
-}
-void e2_widget_swap_tooltip (GtkWidget *widget)
-{
 	GtkTooltipsData *tips = gtk_tooltips_data_get (widget);
-	gchar *tmp = tips->tip_text;
-	tips->tip_text = tips->tip_private;
-	tips->tip_private = tmp;
-}
+	if (app.gtkversion >= 21200)
+		e2_widget_set_tip (widget, tips->tip_private, tips->tip_text);
+	else
+	{
+		gchar *tmp = tips->tip_text;
+		tips->tip_text = tips->tip_private;
+		tips->tip_private = tmp;
+	}
 #endif
+}
 /**
 @brief get image widget for an icon
 
 @param icon gtk-stock-item name, or localised custom-icon filename with or without path
 @param size enumerator of the desired image size
 
-@return Image widget, or NULL
+@return image widget, or NULL
 */
 GtkWidget *e2_widget_get_icon (const gchar *icon, GtkIconSize size)
 {
-	GtkWidget *image;
 	if (icon == NULL || *icon == '\0')
 		return NULL;
 
 #ifdef E2_IMAGECACHE
 	//get cached image, after adding it to the cache if not there already
 	E2_Image *cache_img = e2_cache_image_get (icon, size);
-	image = gtk_image_new_from_pixbuf (cache_img->pixbuf);
+	return (gtk_image_new_from_pixbuf (cache_img->pixbuf));
 #else
+	GtkWidget *image;
 	if (e2_utils_check_stock_icon (icon))
 		image = gtk_image_new_from_stock (icon, size);
 	else
@@ -164,8 +270,8 @@ GtkWidget *e2_widget_get_icon (const gchar *icon, GtkIconSize size)
 		if (fullname != icon)
 			g_free (fullname);
 	}
-#endif
 	return image;
+#endif
 }
 /**
 @brief create a vertically-centred GtkLabel and add it to @a table
@@ -191,8 +297,6 @@ GtkWidget *e2_widget_add_mid_label_to_table (GtkWidget *table,
 	gtk_misc_set_alignment (GTK_MISC (label), xalign, 0.5);
 //	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
 	gtk_table_attach_defaults (GTK_TABLE (table), label, left, right, top, bottom);
-	gtk_widget_show (label);
-
 	return label;
 }
 /**
@@ -232,7 +336,6 @@ GtkWidget *e2_widget_add_label (GtkWidget *box, const gchar *text,
 	gtk_misc_set_alignment (GTK_MISC (label), xalign, yalign);
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
 	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
-	gtk_widget_show (label);
 	if (box != NULL)
 		gtk_box_pack_start (GTK_BOX (box), label, exp, TRUE, pad);
 	return label;
@@ -242,13 +345,16 @@ GtkWidget *e2_widget_add_label (GtkWidget *box, const gchar *text,
 <Shift>Delete removes from cursor to end
 @param entry the entry widget
 @param event pointer to event data struct
-@param data UNUSED data specified when callback was connnected
+@param user_data UNUSED data specified when callback was connnected
 
 @return TRUE if the key was <Shift>Delete
 */
 static gboolean _e2_widget_entry_keypress_cb (GtkWidget *entry, GdkEventKey *event,
-	gpointer data)
+	gpointer user_data)
 {
+	//prevent sporadic bad latency in UI updates
+	gtk_widget_queue_draw (entry);
+
 	if (event->keyval == GDK_Delete)
 	{
 		guint modifiers = gtk_accelerator_get_default_mod_mask ();
@@ -286,9 +392,8 @@ GtkWidget *e2_widget_add_entry (GtkWidget *box, gchar *init_text,
 	g_signal_connect (G_OBJECT (entry), "key-press-event",
 		G_CALLBACK (_e2_widget_entry_keypress_cb), NULL);
 	if (box != NULL)
-		gtk_box_pack_start (GTK_BOX (box), entry, exp, TRUE, E2_PADDING);
+		gtk_box_pack_start (GTK_BOX (box), entry, exp, TRUE, E2_PADDING_XSMALL);
 	gtk_widget_grab_focus (entry);
-	gtk_widget_show (entry);
 
 	return entry;
 }
@@ -318,8 +423,9 @@ GtkWidget *e2_widget_add_entry_to_table (GtkWidget *table,
 		if (select_text)
 			gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
 	}
+	g_signal_connect (G_OBJECT (entry), "key-press-event",
+		G_CALLBACK (_e2_widget_entry_keypress_cb), NULL);
 	gtk_table_attach_defaults (GTK_TABLE (table), entry, left, right, top, bottom);
-	gtk_widget_show (entry);
 
 	return entry;
 }
@@ -329,7 +435,7 @@ GtkWidget *e2_widget_add_entry_to_table (GtkWidget *table,
 
 @param vertical TRUE for a vbox, FALSE for a hbox
 @param homogen the new box's "homogenous" parameter
-@param homogen the new box's "spacing" parameter
+@param spacing the new box's "spacing" parameter
 
 @return the widget, unshown
 */
@@ -348,7 +454,7 @@ GtkWidget *e2_widget_get_box (gboolean vertical, gboolean homogen, gint spacing)
 @param pad padding for packing the box into @a parentbox
 @param vertical TRUE for a vbox, FALSE for a hbox
 @param homogen the new box's "homogenous" parameter
-@param homogen the new box's "spacing" parameter
+@param spacing the new box's "spacing" parameter
 
 @return the new box widget
 */
@@ -357,7 +463,6 @@ GtkWidget *e2_widget_add_box (GtkWidget *parentbox, gboolean exp, guint pad,
 {
 	GtkWidget *box = e2_widget_get_box (vertical, homogen, spacing);
 	gtk_box_pack_start (GTK_BOX (parentbox), box, exp, TRUE, pad);
-	gtk_widget_show (box);
 	return box;
 }
 /**
@@ -389,6 +494,9 @@ GtkWidget *e2_widget_get_sw (GtkPolicyType h_policy, GtkPolicyType v_policy,
 	GtkShadowType shadow)
 {
  	GtkWidget *scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+	//set bars positions even for policy none, that may change later
+	gtk_scrolled_window_set_placement (GTK_SCROLLED_WINDOW (scrolled_window),
+		e2_option_int_get ("scrollbar-position"));
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 		h_policy, v_policy);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
@@ -396,13 +504,13 @@ GtkWidget *e2_widget_get_sw (GtkPolicyType h_policy, GtkPolicyType v_policy,
  	return scrolled_window;
 }
 /**
-@brief create a GtkScrolledWindow and add it to a GtkBox
+@brief create a GtkScrolledWindow and pack it into start of @a box
 
 This function creates a GtkScrolledWindow using e2_widget_get_sw_plain() and
 adds it to @a box. The parameters @a h_policy and @a v_policy are used as
 arguments for e2_widget_get_sw_plain(). @a exp and @a pad are used as arguments
 for gtk_box_pack_start() to adjust the packing of the scrolled window.
-
+The shadow type is set to GTK_SHADOW_ETCHED_IN.
 @param box the parent box to pack the scrolled window to
 @param h_policy  scrolled window horizontal scrolling policy
 @param v_policy  scrolled window vertical scrolling policy
@@ -415,8 +523,9 @@ GtkWidget *e2_widget_add_sw (GtkWidget *box, GtkPolicyType h_policy,
 	GtkPolicyType v_policy, gboolean exp, guint pad)
 {
 	GtkWidget *scrolled_window = e2_widget_get_sw_plain (h_policy, v_policy);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
+			GTK_SHADOW_ETCHED_IN);
 	gtk_box_pack_start (GTK_BOX (box), scrolled_window, exp, TRUE, pad);
-	gtk_widget_show (scrolled_window);
 	return scrolled_window;
 }
 /**
@@ -436,7 +545,12 @@ set to GTK_SHADOW_NONE.
 void e2_widget_sw_add_with_viewport (GtkWidget *sw, GtkWidget *child)
 {
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), child);
-	gtk_viewport_set_shadow_type (GTK_VIEWPORT (GTK_BIN (sw)->child),
+	gtk_viewport_set_shadow_type (
+#ifdef USE_GTK2_14
+		GTK_VIEWPORT (gtk_bin_get_child (GTK_BIN (sw))),
+#else
+		GTK_VIEWPORT (GTK_BIN (sw)->child),
+#endif
 		GTK_SHADOW_NONE);
 }
 /**
@@ -451,7 +565,7 @@ void e2_widget_sw_add_with_viewport (GtkWidget *sw, GtkWidget *child)
 GtkWidget *e2_widget_add_separator (GtkWidget *box, gboolean exp, guint pad)
 {
 	GtkWidget *sep;
-	if (box != NULL && GTK_IS_HBOX (box))
+	if (GTK_IS_HBOX (box))
 		sep = gtk_vseparator_new ();
 	else if (box == NULL || GTK_IS_VBOX (box))
 		sep = gtk_hseparator_new ();
@@ -459,7 +573,6 @@ GtkWidget *e2_widget_add_separator (GtkWidget *box, gboolean exp, guint pad)
 		return NULL;
 	if (box != NULL)
 		gtk_box_pack_start (GTK_BOX (box), sep, exp, TRUE, pad);
-	gtk_widget_show (sep);
 	return sep;
 }
 /**
@@ -481,8 +594,6 @@ GtkWidget *e2_widget_add_table (GtkWidget *box, gint rows, gint cols,
 	table = gtk_table_new (rows, cols, homogen);
 	if (box != NULL)
 		gtk_box_pack_start (GTK_BOX (box), table, exp, TRUE, pad);
-	gtk_widget_show (table);
-
 	return table;
 }
 /*UNUSED
@@ -493,14 +604,11 @@ GtkWidget *e2_widget_add_framed_table (GtkWidget *box, gchar *title, gint rows,
 
 	frame = gtk_frame_new (title);
 	gtk_box_pack_start (GTK_BOX (box), frame, exp, TRUE, pad);
-	gtk_widget_show (frame);
 
 	table = gtk_table_new (rows, cols, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (table), E2_PADDING_XSMALL);
 	gtk_table_set_row_spacings (GTK_TABLE (table), E2_PADDING);
 	gtk_container_add (GTK_CONTAINER (frame), table);
-	gtk_widget_show (table);
-
 	return table;
 }
 UNUSED
@@ -512,9 +620,6 @@ GtkWidget *e2_widget_add_framed_widget (GtkWidget *box, gchar *title,
 	frame = gtk_frame_new (title);
 	gtk_box_pack_start (GTK_BOX (box), frame, exp, TRUE, pad);
 	gtk_container_add (GTK_CONTAINER (frame), widget);
-	gtk_widget_show (widget);
-	gtk_widget_show (frame);
-
 	return frame;
 }
 */
@@ -549,7 +654,8 @@ GtkWidget *e2_widget_add_eventbox (GtkWidget *box, gboolean fill, guint pad)
 
 @return the notebook widget, unshown
 */
-GtkWidget *e2_widget_get_notebook (gpointer func, gpointer data)
+GtkWidget *e2_widget_get_notebook (
+	void (*func)(/*GtkNotebook*,GtkNotebookPage*,guint,gpointer*/), gpointer data)
 {
 	GtkWidget *notebook = gtk_notebook_new ();
 	if (func != NULL)
@@ -568,7 +674,7 @@ GtkWidget *e2_widget_get_notebook (gpointer func, gpointer data)
 @return the notebook widget, unshown
 */
 GtkWidget *e2_widget_add_notebook (GtkWidget *box, gboolean fill, guint pad,
-	gpointer func, gpointer data)
+	void (*func)(/*GtkNotebook*,GtkNotebookPage*,guint,gpointer*/), gpointer data)
 {
 	GtkWidget *notebook = e2_widget_get_notebook (func, data);
 	gtk_box_pack_start (GTK_BOX (box), notebook, fill, fill, pad);
@@ -631,8 +737,14 @@ used in @a widget. So it should be called only after a font has been assigned
 void e2_widget_get_font_pixels (GtkWidget *widget, gint *width, gint *height)
 {
 	PangoContext *context = gtk_widget_get_pango_context (widget);
+#ifdef USE_GTK2_14
+	GtkStyle *style = gtk_widget_get_style (widget);
+#endif
 	PangoFontMetrics *metrics = pango_context_get_metrics (context,
-			widget->style->font_desc, pango_context_get_language (context));
+#ifndef USE_GTK2_14
+		widget->
+#endif
+		style->font_desc, pango_context_get_language (context));
 	if (width != NULL)
 		*width = PANGO_PIXELS (pango_font_metrics_get_approximate_char_width
 			(metrics));
@@ -678,98 +790,6 @@ void e2_widget_get_font_pixels (GtkWidget *widget, gint *width, gint *height)
 */
 }
 /**
-@brief callback upon expander-widget open/close
-Show/hide the expander child widget, and if not blocked by the controller widget,
-update the associated set value, if any
-@param expander the widget that changed
-@param spec
-@param boolset pointer to data for boolean set tied to expander state, or NULL
-
-@return
-*/
-static void _e2_widget_expander_toggled_cb (GtkWidget *expander, GParamSpec *spec,
-	E2_OptionSet *boolset)
-{
-	printd (DEBUG, "_e2_widget_toggle_expander_cb");
-	gboolean value = gtk_expander_get_expanded (GTK_EXPANDER (expander));
-	GtkWidget *show = g_object_get_data (G_OBJECT (expander), "e2-show-widget");
-	if (show != NULL)
-	{
-		if (value)
-		{
-			if (!GTK_WIDGET_VISIBLE (show))
-				gtk_widget_show (show);
-		}
-		else if (GTK_WIDGET_VISIBLE (show))
-			gtk_widget_hide (show);
-
-	}
-	if (boolset != NULL)
-	{
-		GtkWidget *controller = g_object_get_data (G_OBJECT (expander),
-			"e2-controller-widget");
-		if (!GPOINTER_TO_INT (g_object_get_data (G_OBJECT (controller),
-			"e2-controller-blocked")))
-			e2_option_bool_set_direct (boolset, value);
-	}
-}
-/**
-@brief set @a expander state if it's not blocked and not already at the desired state
-This is a hook-function callack upon change of set data associated with @a expander
-@param state pointerised TRUE/FALSE, the value to apply to @a expander
-@param expander the widget to change
-
-@return TRUE always
-*/
-static gboolean _e2_widget_set_expander (gpointer state, GtkWidget *expander)
-{
-	GtkWidget *controller = g_object_get_data (G_OBJECT (expander),
-		"e2-controller-widget");
-	if (!GPOINTER_TO_INT (g_object_get_data (G_OBJECT (controller),
-		"e2-controller-blocked")))
-	{
-		gboolean value = GPOINTER_TO_INT (state);
-		gboolean current = gtk_expander_get_expanded (GTK_EXPANDER (expander));
-		if (value != current)
-			gtk_expander_set_expanded (GTK_EXPANDER (expander), value);
-	}
-	return TRUE;
-}
-/**
-@brief add expander and a child widget to @a box
-
-@param parentbox the parent box to pack the new expander into
-@param text content of label for the expander
-@param boolset pointer to data for boolean set that's tied to the expander state, or NULL
-@param controller widget which may have "e2-controller-blocked" data to prevent set value being updated when expander toggles
-@param child widget shown/hidden according to expander state, also packed into @a parentbox
-
-@return the expander widget
-*/
-GtkWidget *e2_widget_add_expander (GtkWidget *parentbox, gchar *text,
-	E2_OptionSet *boolset, GtkWidget *controller, GtkWidget *child)
-{
-	GtkWidget *expander = gtk_expander_new (text);
-	g_object_set_data (G_OBJECT (expander), "e2-controller-widget", controller);
-	g_object_set_data (G_OBJECT (expander), "e2-show-widget", child);
-	g_signal_connect (G_OBJECT (expander), "notify::expanded",
-		G_CALLBACK (_e2_widget_expander_toggled_cb), boolset);
-	if (boolset != NULL)
-		e2_option_attach_value_changed_simple (boolset, expander,
-			_e2_widget_set_expander, expander);
-
-	gtk_container_set_border_width (GTK_CONTAINER (child), E2_PADDING_SMALL);
-
-	gtk_box_pack_start (GTK_BOX (parentbox), expander, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (parentbox), child, FALSE, FALSE, 0);
-	gtk_widget_show (expander);
-	//some wm's like this to be shown before any "expander-toggle" in the toggle cb
-	if (boolset != NULL && e2_option_bool_get_direct (boolset))
-		gtk_widget_show (child);
-
-	return expander;
-}
-/**
 @brief callback upon tied check-button click
 If not blocked by the controller widget, update the associated set value
 @param button the widget that changed
@@ -777,14 +797,19 @@ If not blocked by the controller widget, update the associated set value
 
 @return
 */
-static void _e2_widget_check_changed_cb (GtkWidget *button, E2_OptionSet *boolset)
+static void _e2_widget_check_changed_cb (GtkToggleButton *button, E2_OptionSet *boolset)
 {
 	GtkWidget *controller = g_object_get_data (G_OBJECT (button),
 		"e2-controller-widget");
 	if (!GPOINTER_TO_INT (g_object_get_data (G_OBJECT (controller),
 		"e2-controller-blocked")))
 		e2_option_bool_set_direct (boolset,
-			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
+#ifdef USE_GTK2_14
+			gtk_toggle_button_get_active (button)
+#else
+			button->active
+#endif
+	);
 }
 /**
 @brief set @a button state if it's not blocked and not already at the desired state
@@ -794,7 +819,7 @@ This is a hook-function callack upon change of set data associated with @a butto
 
 @return TRUE always
 */
-static gboolean _e2_widget_set_check_state (gpointer state, GtkWidget *button)
+static gboolean _e2_widget_set_check_state_hook (gpointer state, GtkWidget *button)
 {
 	GtkWidget *controller = g_object_get_data (G_OBJECT (button),
 		"e2-controller-widget");
@@ -825,13 +850,8 @@ GtkWidget *e2_widget_add_tied_check_button
 		NULL, FALSE, 0, _e2_widget_check_changed_cb, boolset);
 	g_object_set_data (G_OBJECT (boolset->widget), "e2-controller-widget", controller);
 	e2_option_attach_value_changed_simple (boolset, boolset->widget,
-		_e2_widget_set_check_state, boolset->widget);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		boolset->widget, boolset->tip);
+		(HookFunc)_e2_widget_set_check_state_hook, boolset->widget);
+	e2_widget_set_safetip (boolset->widget, boolset->tip);
 	e2_widget_handle_depends (boolset->widget, boolset);
 
 	return boolset->widget;
@@ -843,12 +863,24 @@ GtkWidget *e2_widget_add_tied_check_button
 @param widget the widget to change
 @return TRUE always
 */
-static gboolean _e2_widget_dependent_changed_cb (gpointer state, GtkWidget *widget)
+static gboolean _e2_widget_dependent_changed_hook (gpointer state, GtkWidget *widget)
 {
 	gboolean value = GPOINTER_TO_INT (state);
-	if (value && !GTK_WIDGET_SENSITIVE (widget))
+	if (value &&
+#ifdef USE_GTK2_18
+			!gtk_widget_is_sensitive (widget)
+#else
+			!GTK_WIDGET_SENSITIVE (widget)
+#endif
+	)
 		gtk_widget_set_sensitive (widget, TRUE);
-	else if (!value && GTK_WIDGET_SENSITIVE (widget))
+	else if (!value &&
+#ifdef USE_GTK2_18
+		 	gtk_widget_is_sensitive (widget)
+#else
+			GTK_WIDGET_SENSITIVE (widget)
+#endif
+	)
 		gtk_widget_set_sensitive (widget, FALSE);
 	return TRUE;
 }
@@ -858,12 +890,24 @@ static gboolean _e2_widget_dependent_changed_cb (gpointer state, GtkWidget *widg
 @param widget the widget to change
 @return TRUE always
 */
-static gboolean _e2_widget_invdependent_changed_cb (gpointer state, GtkWidget *widget)
+static gboolean _e2_widget_invdependent_changed_hook (gpointer state, GtkWidget *widget)
 {
 	gboolean value = GPOINTER_TO_INT (state);
-	if (value && GTK_WIDGET_SENSITIVE (widget))
+	if (value &&
+#ifdef USE_GTK2_18
+		 	gtk_widget_is_sensitive (widget)
+#else
+			GTK_WIDGET_SENSITIVE (widget)
+#endif
+	)
 		gtk_widget_set_sensitive (widget, FALSE);
-	else if (!value && !GTK_WIDGET_SENSITIVE (widget))
+	else if (!value &&
+#ifdef USE_GTK2_18
+		 	!gtk_widget_is_sensitive (widget)
+#else
+			!GTK_WIDGET_SENSITIVE (widget)
+#endif
+	)
 		gtk_widget_set_sensitive (widget, TRUE);
 	return TRUE;
 }
@@ -888,7 +932,7 @@ void e2_widget_handle_depends (GtkWidget *widget, E2_OptionSet *boolset)
 				gtk_widget_set_sensitive (widget,
 					!e2_option_bool_get_direct (dep));
 				e2_option_attach_value_changed_simple (dep, widget,
-					_e2_widget_invdependent_changed_cb, widget);
+					(HookFunc)_e2_widget_invdependent_changed_hook, widget);
 			}
 		}
 		else
@@ -899,7 +943,7 @@ void e2_widget_handle_depends (GtkWidget *widget, E2_OptionSet *boolset)
 				gtk_widget_set_sensitive (widget,
 					e2_option_bool_get_direct (dep));
 				e2_option_attach_value_changed_simple (dep, widget,
-					_e2_widget_dependent_changed_cb, widget);
+					(HookFunc)_e2_widget_dependent_changed_hook, widget);
 			}
 		}
 	}
@@ -910,12 +954,12 @@ void e2_widget_handle_depends (GtkWidget *widget, E2_OptionSet *boolset)
 @brief set relations on @a label for @a widget and vice-versa
 This enables accessiblity tools to identify @a label as descriptive item for
 @a @widget etc
-@param label the label associated with @a widget
+@param label the label(etc) associated with @a widget
 @param widget the widget to be tagged
 
 @return
 */
-void e2_widget_set_label_relations (GtkLabel *label, GtkWidget *widget)
+void e2_widget_set_label_relations (GtkWidget *label, GtkWidget *widget)
 {
 	AtkObject *atklabelob = gtk_widget_get_accessible (GTK_WIDGET (label));
 	AtkObject *atkob = gtk_widget_get_accessible (widget);
