@@ -1,6 +1,6 @@
-/* $Id: e2_widget.h 874 2008-05-07 14:47:27Z tpgww $
+/* $Id: e2_widget.h 1917 2009-09-27 09:30:57Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -32,8 +32,12 @@ functions.
 #ifdef USE_GTK2_12TIPS
 void e2_widget_set_toggletip (GtkWidget *widget, const gchar *initialtip,
 	const gchar *othertip);
+void e2_widget_set_safetip (GtkWidget *widget, const gchar *tiptext);
 #else
-void e2_widget_set_tooltip (GtkTooltips *tooltips, GtkWidget *widget, gchar *text);
+void e2_widget_set_toggletip (GtkWidget *widget, const gchar *initialtip,
+	const gchar *othertip);
+void e2_widget_set_tip (GtkWidget *widget, const gchar *tiptext1, const gchar *tiptext2);
+# define e2_widget_set_safetip(w,t) e2_widget_set_tip(w,t,NULL)
 #endif
 void e2_widget_swap_tooltip (GtkWidget *widget);
 GtkWidget *e2_widget_get_icon (const gchar *icon, GtkIconSize size);
@@ -43,13 +47,9 @@ GtkWidget *e2_widget_add_mid_label (GtkWidget *box, const gchar *text, gfloat al
 GtkWidget *e2_widget_add_label (GtkWidget *box, const gchar *text,
 	gfloat xalign, gfloat yalign, gboolean exp, guint pad);
 
-//FIXME: remove
-//#define add_entry(box, init_text, fill, pad) e2_widget_add_entry (box, init_text, fill, FALSE)
 GtkWidget *e2_widget_add_entry (GtkWidget *box, gchar *init_text, gboolean exp,
 	gboolean select_text);
 
-//FIXME: remove
-//#define add_hbox(box, homogen, spacing, fill, pad) e2_widget_add_box (box, fill, pad, FALSE, homogen, spacing)
 GtkWidget *e2_widget_get_box (gboolean vertical, gboolean homogen, gint spacing);
 GtkWidget *e2_widget_add_box (GtkWidget *parent, gboolean exp, guint pad,
 	gboolean vertical, gboolean homogen, gint spacing);
@@ -75,21 +75,20 @@ GtkWidget *e2_widget_add_mid_label_to_table (GtkWidget *table, gchar *text,
 GtkWidget *e2_widget_add_frame (GtkWidget *box, gboolean fill, guint pad,
 	gchar *title, gboolean strech);
 GtkWidget *e2_widget_add_eventbox (GtkWidget *box, gboolean fill, guint pad);
-GtkWidget *e2_widget_get_notebook (gpointer func, gpointer data);
+GtkWidget *e2_widget_get_notebook (
+	void (*func)(/*GtkNotebook*,GtkNotebookPage*,guint,gpointer*/), gpointer data);
 GtkWidget *e2_widget_add_notebook (GtkWidget *box, gboolean fill, guint pad,
-	gpointer func, gpointer data);
+	void (*func)(/*GtkNotebook*,GtkNotebookPage*,guint,gpointer*/), gpointer data);
 GtkWidget *e2_widget_add_notebook_page (//GtkWidget *dialog,
 	GtkWidget *notebook, gchar *name, GtkPolicyType swpolicy);
 void e2_widget_get_font_pixels (GtkWidget *widget, int *width, int *height);
 void e2_widget_set_font (GtkWidget *widget, const gchar *font_string);
-GtkWidget *e2_widget_add_expander (GtkWidget *box, gchar *text,
-	E2_OptionSet *boolset, GtkWidget *controller, GtkWidget *child);
 GtkWidget *e2_widget_add_tied_check_button
 	(GtkWidget *box, E2_OptionSet *boolset, GtkWidget *controller);
 void e2_widget_handle_depends (GtkWidget *widget, E2_OptionSet *boolset);
 
 #ifdef E2_ASSISTED
-void e2_widget_set_label_relations (GtkLabel *label, GtkWidget *widget);
+void e2_widget_set_label_relations (GtkWidget *label, GtkWidget *widget);
 AtkObject *e2_widget_get_accessible (GtkWidget *widget, const gchar *name,
 	const gchar *desc, AtkRole role);
 #endif
